static const char file_id[] = "AnyCGPeekPoke.pl";
// .cc file generated from AnyCGPeekPoke.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "AnyCGPeekPoke.h"
#include "Galaxy.h"
#include "EventHorizon.h"
#include "MultiTarget.h"
#include "miscFuncs.h"

const char *star_nm_AnyCGPeekPoke = "AnyCGPeekPoke";

const char* AnyCGPeekPoke :: className() const {return star_nm_AnyCGPeekPoke;}

ISA_FUNC(AnyCGPeekPoke,AnyCGStar);

Block* AnyCGPeekPoke :: makeNew() const { LOG_NEW; return new AnyCGPeekPoke;}

AnyCGPeekPoke::AnyCGPeekPoke ()
{
	setDescriptor("Nondeterminate communication link, splices in a peek/poke pair");
	addPort(input.setPort("input",this,ANYTYPE));
	addPort(output.setPort("output",this,ANYTYPE));
	output.inheritTypeFrom(input);
	addState(Poke_procId.setState("Poke_procId",this,"1","assigned processor id for poke - must be manually assigned."));
	addState(blockSize.setState("blockSize",this,"1","number of tokens to be transfered per firing."));
	addState(updateRate.setState("updateRate",this,"1","Update the link at most every 'updateRate' schedule iterations."));

# line 123 "AnyCGPeekPoke.pl"
procId.setState("Peek_procId",this, "0",
		    "assigned processor id for peek - must be manually "
		    "assigned.");
}

void AnyCGPeekPoke::setup() {
# line 142 "AnyCGPeekPoke.pl"
// Try to find the multitarget by going up the target parent
    Target* multitarget = findMultiTarget(target());

    // Now if we haven't found the multitarget yet - we iterate over
    // the parent wormholes
    Block* prnt = this;
    while ((prnt = prnt->parent()) != NULL) {
	// If my parent is not atomic, it is not a wormhole
	if (!prnt->isItAtomic()) continue;

	// My parent is atomic, it is a wormhole - check its
	// target to see if it is a multitarget
	multitarget = findMultiTarget(prnt->target());
	if (multitarget) break;
    }
    
    if (!multitarget) {
	Error::abortRun(*this,"is not inside of a multitarget");
	return;
    }

    if (multitarget->nProcs() <= (int)procId || procId < 0 ) {
	StringList message;
	message << "The Peek_ProcId is invalid, it is set to "
		<< procId << " but the number of child targets is "
		<< multitarget->nProcs();
	Error::abortRun(*this,message);
	return;
    }

    if (multitarget->nProcs() <= Poke_procId || Poke_procId < 0) {
	StringList message;
	message << "The Poke_ProcId is invalid, it is set to "
		<< Poke_procId << " but the number of child targets is "
		<< multitarget->nProcs();
	Error::abortRun(*this,message);
	return;
    }

    PortHole* peekPort = findPort(output);
    PortHole* pokePort = findPort(input);
    if (!(peekPort && pokePort)) return;

    AsynchCommPair 
	pair = ((MultiTarget*)multitarget)
	    ->createPeekPokeProcId(procId,Poke_procId);

    CGStar *peek = pair.peek, *poke = pair.poke;

    if (peek == NULL || poke == NULL) return;

    PortHole* input = poke->portWithName("input");
    if (!input) {
	Error::abortRun(*this,poke->name(),
			": Poke star does not have a porthole named 'input'");
	return;
    }	

    PortHole* output = peek->portWithName("output");
    if (!output) {
	Error::abortRun(*this,peek->name(),
			": Peek star does not have a porthole named 'output'");
	return;
    }

    if (output->type() == ANYTYPE) input->inheritTypeFrom(*output);

    int numInitDelays;
    Galaxy* parentGal;
    StringList starName, linkName;
    linkName << starSymbol.lookup("Link");
    // FIXME - we do not support initial delays on peek/poke arcs
    // The peekPort and pokePort are in a disconnected state here.
    // numInitDelays = peekPort->numInitDelays();
    numInitDelays = 0;
    peekPort->disconnect();
    output->connect(*peekPort,numInitDelays);
    parentGal = (Galaxy*) peekPort->parent()->parent();
    starName.initialize();
    starName << "Peek_" << linkName ;
    parentGal->addBlock(*peek,hashstring(starName));
    peek->setTarget(multitarget->child(procId));

    // FIXME - we do not support initial delays on peek/poke arcs
    // The peekPort and pokePort are in a disconnected state here.
    // numInitDelays = pokePort->numInitDelays();
    numInitDelays = 0;
    pokePort->disconnect();
    pokePort->connect(*input,numInitDelays);
    parentGal = (Galaxy*) pokePort->parent()->parent();
    starName.initialize();
    starName << "Poke_" << linkName;
    parentGal->addBlock(*poke,hashstring(starName));
    poke->setTarget(multitarget->child(Poke_procId));

    if ((int) blockSize > 1) 
	setAsynchCommState(pair,"blockSize",blockSize.currentValue());
    if ((int) updateRate > 1)
	setAsynchCommState(pair,"updateRate",updateRate.currentValue());
    ((Galaxy*) parent())->deleteBlockAfterInit(*this);

    peek->setProcId(procId);
    poke->setProcId(Poke_procId);
}


void AnyCGPeekPoke::removeGalaxyPort (GenericPort* port)
{
# line 81 "AnyCGPeekPoke.pl"
if (port == NULL) return;
	if (port->aliasFrom()) removeGalaxyPort(port->aliasFrom());
	port->clearAliases();
	port->parent()->removePort(port->newPort());
	LOG_DEL; delete port;
}


PortHole* AnyCGPeekPoke::findPort (PortHole& startPort)
{
# line 95 "AnyCGPeekPoke.pl"
PortHole* farPort = startPort.far();
	while (farPort && farPort->asEH()) {
	    // Find the inside port
	    PortHole* temp = farPort->asEH()->ghostAsPort()->far();

	    // Now remove the port from the wormhole
	    farPort->parent()->removePort(*farPort);
	    LOG_DEL; delete farPort;

	    // Now remove the corresponding ports from the galaxies inside 
	    // the wormhole
	    removeGalaxyPort(temp->aliasFrom());

	    farPort = temp;
	}
	if (!farPort)
	    Error::abortRun(*this,startPort.name()," not connected");

	// Remove all galaxy references to the startPort
	removeGalaxyPort(startPort.aliasFrom());
	// Remove the all galaxy references to the farPort
	removeGalaxyPort(farPort->aliasFrom());

	return farPort;
}


inline Target* AnyCGPeekPoke::findMultiTarget (Target* multitarget)
{
# line 134 "AnyCGPeekPoke.pl"
while (multitarget && !multitarget->isA("MultiTarget"))
	    multitarget = (Target*) multitarget->parent();
	if (multitarget) return multitarget;
	return (Target*)NULL;
}


// prototype instance for known block list
static AnyCGPeekPoke proto;
static RegisterBlock registerBlock(proto,"PeekPoke");
