static const char file_id[] = "CG56GoertzelBase.pl";
// .cc file generated from CG56GoertzelBase.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "CG56GoertzelBase.h"
#include <math.h>

const char *star_nm_CG56GoertzelBase = "CG56GoertzelBase";

const char* CG56GoertzelBase :: className() const {return star_nm_CG56GoertzelBase;}

ISA_FUNC(CG56GoertzelBase,CG56Star);

Block* CG56GoertzelBase :: makeNew() const { LOG_NEW; return new CG56GoertzelBase;}

CodeBlock CG56GoertzelBase :: initialize (
# line 116 "CG56GoertzelBase.pl"
"; Register usage:\n"
"; x0 = real part of previous output \n"
"; x1 = WnReal\n"
"; y0 = imaginary part of previous output\n"
"; y1 = WnImag\n"
"; a = real part of this output\n"
"; b = imaginary part of this output\n"
"; r0 = address of real part of input\n"
";\n"
"; Initialization\n"
"; b = x0 = y0 = 0\n"
"	clr	a	#$addr(input),r0		; r0 = address of input\n"
"	clr	b	$ref(WnReal),x1		a,y0	; x1 = WnReal\n"
"	clr	a	b,x0		$ref(WnImag),y1	; y1 = WnImag\n"
"	move 	x:(r0)+,a			; a = real part of first input\n");

const char* CG56GoertzelBase :: loop(int len) {
# line 134 "CG56GoertzelBase.pl"
	static StringList _str_; _str_.initialize(); _str_ << 
"; Compute the output of the Goertzel filter\n"
"	do	#" << (len) << ",$label(_GoertzelLoop)\n"
"	macr	x1,x0,a		b,y0	\n"
"	macr	-y1,y0,a	#0,b		\n"
"	macr	x0,y1,b		a,x0\n"
"	macr	x1,y0,b		x:(r0)+,a\n"
"$label(_GoertzelLoop)\n"
"; Register usage:\n"
"; x0 = real coefficient\n"
"; b = imaginary coefficient\n";
	return (const char*)_str_;
}

CG56GoertzelBase::CG56GoertzelBase ()
{
	setDescriptor("Base class for Goertzel algorithm stars");
	addPort(input.setPort("input",this,FIX));
	addState(k.setState("k",this,"0","the DFT coefficient to compute (k < N)"));
	addState(N.setState("N",this,"32","length of the DFT"));
	addState(size.setState("size",this,"32","amount of data to read (N <= size)"));
	addState(WnReal.setState("WnReal",this,"0.0","internal state for the storage of the real part of the twiddle factor, \nwhich is a function of k and N",
# line 46 "CG56GoertzelBase.pl"
A_XMEM|A_NONCONSTANT|A_NONSETTABLE));
	addState(WnImag.setState("WnImag",this,"0.0","internal state for the storage of the imaginary part of the twiddle factor,\nwhich is a function of k and N",
# line 56 "CG56GoertzelBase.pl"
A_YMEM|A_NONCONSTANT|A_NONSETTABLE));

# line 62 "CG56GoertzelBase.pl"
noInternalState();
		theta = 0.0;
}

int CG56GoertzelBase::myExecTime() {
# line 161 "CG56GoertzelBase.pl"
return (4 + 3 + 4*(int(N)+1));
}

void CG56GoertzelBase::setup() {
# line 103 "CG56GoertzelBase.pl"
// FIXME: Parameters are not always resolved properly
		// before setup but should be.  For now, check parameters
		// in go method and guard against division by N = 0
		// CheckParameterValues();
		// double Nd = double(int(N));
		double Nd = double(int(N) ? int(N) : 1);
		double kd = double(int(k));
		theta = 2.0 * M_PI * kd / Nd;
		WnReal= double(cos(theta));
		WnImag  = double(sin(theta));
		input.setSDFParams(int(size), int(size)-1);
}

void CG56GoertzelBase::go() {
# line 148 "CG56GoertzelBase.pl"
int dftLength = int(N);

		// See the FIXME statement above
		CheckParameterValues();

  		// Initialize registers
		addCode(initialize);

		// Compute kth coeff discarding all but last sample
		addCode(loop(dftLength + 1));
}


void CG56GoertzelBase::CheckParameterValues ()
{
# line 72 "CG56GoertzelBase.pl"
if ( int(k) < 0 ) {
		  Error::abortRun(*this,
		  	"The value for state k must be nonnegative.");
		  return;
		}
		if ( int(N) <= 0 ) {
		  Error::abortRun(*this,
		  	"The value for state N must be positive.");
		  return;
		}
		if ( int(size) <= 0 ) {
		  Error::abortRun(*this,
		  	"The value for state size must be positive.");
		  return;
		}
		if ( int(k) >= int(N) ) {
		  Error::abortRun(*this,
		  	"The DFT coefficient k must be less than the ",
			"DFT length N.");
		  return;
		}
		if ( int(N) > int(size) ) {
		  Error::abortRun(*this,
		  	"The DFT lenght N must be less that or equal to the ",
			"number of data samples read, given by state size.");
		  return;
		}
}


// prototype instance for known block list
static CG56GoertzelBase proto;
static RegisterBlock registerBlock(proto,"GoertzelBase");
