static const char file_id[] = "CG56IIR.pl";
// .cc file generated from CG56IIR.pl by ptlang
/*
Copyright (c) 1990- The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "CG56IIR.h"
#include <minmax.h>

const char *star_nm_CG56IIR = "CG56IIR";

const char* CG56IIR :: className() const {return star_nm_CG56IIR;}

ISA_FUNC(CG56IIR,CG56Star);

Block* CG56IIR :: makeNew() const { LOG_NEW; return new CG56IIR;}

CodeBlock CG56IIR :: one (
# line 185 "CG56IIR.pl"
"; H(z) = 1 so just pass input to output\n"
"	move	$ref(signalIn),x1\n"
"	move	$ref(coeffs,1),x0\n"
"	mpyr	x1,x0,a\n"
"	move	a,$ref(signalOut)\n");

CodeBlock CG56IIR :: init (
# line 193 "CG56IIR.pl"
";        b[0] + b[1]z^-1 + ... + b[n]z^-n\n"
"; H(z) = ----------------------------------\n"
";        1  + a[1]z^-1 + ... + a[n]z^-n\n"
"; register after executing this block:\n"
"; r0 -> w[1]\n"
"; r4 -> a[2]\n"
"; x0 == w[1]\n"
"; y0 == a[1]\n"
"; (a[0] is never referenced since it is normalized in setup)\n"
"; a  == sample \n"
"; b  == 0\n"
"	move	#($addr(delays)+1),r0\n"
"	move	#($addr(coeffs)+2),r4\n"
"	move	$ref(signalIn),a\n"
"	clr	b	x:(r0),x0	y:(r4)+,y0\n");

const char* CG56IIR :: doFilter(int numLoops) {
# line 211 "CG56IIR.pl"
	static StringList _str_; _str_.initialize(); _str_ << 
"	do #" << (numLoops) << ",$label(end_loops)\n"
"; x0 == w[p]	y0 == a[p]	x1 == w[p-1]\n"
"	mac	y0,x0,a		x1,x:(r0)+	y:(r4)+,y1\n"
"; a += w[p]*a[p]	w[p]=w[p-1]	y1 = b[p]\n"
"	move	x0,x1						\n"
"; x1 = w[p]\n"
"	mac	y1,x1,b		x:(r0),x0	y:(r4)+,y0\n"
"; b += w[p]*b[p]	x0 = w[p+1] 	y0 = a[p+1]\n"
"$label(end_loops)\n";
	return (const char*)_str_;
}

CodeBlock CG56IIR :: end (
# line 223 "CG56IIR.pl"
"; y1 = b[0]\n"
"; x0 = sample + a[1]w[1] + ... + a[n]w[n]\n"
"; b  = b[0]*{ sample + a[1]w[1] + ... + a[n]w[n]} = output\n"
"; w[1] = sample + a[1]w[1] + ... + a[n]w[n]\n"
"	move	$ref(coeffs,1),y1\n"
"	move	a,x0\n"
"	macr	x0,y1,b    a,$ref(delays,1)\n"
"	move	b,$ref(signalOut)\n");

CG56IIR::CG56IIR ()
{
	setDescriptor("An infinite impulse response (IIR) filter implemented in a direct form II\nrealization.  The transfer function is of the form H(z) = G*N(1/z)/D(1/z),\nwhere N() and D() are polynomials.  The parameter \"gain\" specifies G, and\nthe fixed-point arrays \"numerator\" and \"denominator\" specify N() and D(),\nrespectively.  Both arrays start with the constant terms of the polynomial\nand decrease in powers of z (increase in powers of 1/z).  Note that the\nconstant term of D is not omitted, as is common in other programs that\nassume that it has been normalized to unity.  The denominator coefficients\nwill be scaled by 1/leading denominator coefficient; the numerator\ncoefficients will be scaled by gain/leading denominator coefficient.  An\nerror will result if, after scaling, any of the coefficients is greater\nthan 1 or less than -1.");
	addPort(signalIn.setPort("signalIn",this,FIX));
	addPort(signalOut.setPort("signalOut",this,FIX));
	addState(gain.setState("gain",this,"1","Gain."));
	addState(numerator.setState("numerator",this,".5 .25 .1","Numerator coefficients."));
	addState(denominator.setState("denominator",this,"1 .5 .3","Denominator coefficients."));
	addState(coeffs.setState("coeffs",this,"0","internal state: contains coeffs; even addresses correspond to denominator\ncoeffs., odd addresses correspond to numerator coeffs.",
# line 98 "CG56IIR.pl"
A_NONCONSTANT|A_NONSETTABLE|A_YMEM));
	addState(delays.setState("delays",this,"0","internal state: contains delays",
# line 105 "CG56IIR.pl"
A_NONCONSTANT|A_NONSETTABLE|A_XMEM));


}

int CG56IIR::myExecTime() {
# line 235 "CG56IIR.pl"
if (numState == 1) return 4;
	return (4 + 3 + 4*numLoops + 4);
}

void CG56IIR::setup() {
# line 114 "CG56IIR.pl"
int numNumer = numerator.size();
	int numDenom = denominator.size();
	numState = max(numNumer, numDenom);
	double b0, scaleDenom, scaleNumer;
	numLoops = numState - 1;

	// Set up scaling to distribute the gain through the numerator,
	// and scale both numer and denom to make b0 = 1
	if ( numDenom < 1 ) {
	    b0 = 1.0;
	}
	else if ( (b0 = double(denominator[0])) == 0.0 ) {
	    // FIXME: should sanity-check b0 more thoroughly
	    // (e.g., shouldn't even be close to zero)
	    Error::abortRun(*this, "Must have non-zero leading denominator");
	    return;
	}

	scaleDenom = 1.0 / b0;
	scaleNumer = scaleDenom * double(gain);
	coeffs.resize(2*numState);
	delays.resize(numState+1);
	for (int i = 0; i < numState; i++) {
            delays[i] = 0;
            if (i < numNumer) {
                double temp = scaleNumer*double(numerator[i]);
                if ((temp > 1) || (temp < -1)) {
		    StringList msg = "After scaling, numerator coefficient #";
		    msg << i << " has a value of " << temp
			<< " which is not is the range of (-1, 1).";
                    Error::abortRun(*this, msg);
		    return;
                }
		else {
                    coeffs[i*2 + 1] = temp;
                }
            }
	    else {
                coeffs[i*2 + 1] = 0;
            }

            if ( i < numDenom ) {
                double temp = scaleDenom * -(double(denominator[i]));
                if ((temp > 1) || (temp < -1)) {
		    StringList msg = "After scaling, denominator coefficient #";
		    msg << i << " has a value of " << temp
			<< " which is not is the range of (-1, 1).";
                    Error::abortRun(*this, msg);
		    return;
                }
		else {
                    coeffs[i*2] = temp;
                }
            }
	    else {
                coeffs[i*2] = 0;
            }
        }
}

void CG56IIR::go() {
# line 175 "CG56IIR.pl"
if (numState == 1) {
		addCode(one);
	}
	else {
		addCode(init);
		addCode(doFilter(numLoops));
		addCode(end);
	}
}

// prototype instance for known block list
static CG56IIR proto;
static RegisterBlock registerBlock(proto,"IIR");
