static const char file_id[] = "CG56Window.pl";
// .cc file generated from CG56Window.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "CG56Window.h"
#include <string.h>
#include <math.h>
#include "cephes.h"

const char *star_nm_CG56Window = "CG56Window";

const char* CG56Window :: className() const {return star_nm_CG56Window;}

ISA_FUNC(CG56Window,CG56WaveForm);

Block* CG56Window :: makeNew() const { LOG_NEW; return new CG56Window;}

# line 92 "CG56Window.pl"
#define SDFWinType_Null		(0)
#define SDFWinType_Rectangle	(1)
#define SDFWinType_Bartlett	(2)
#define SDFWinType_Hanning	(3)
#define SDFWinType_Hamming	(4)
#define SDFWinType_Blackman	(5)
#define SDFWinType_SteepBlackman	(6)
#define SDFWinType_Kaiser	(7)

CG56Window::CG56Window ()
{
	setDescriptor("Generates standard window functions:\nRectangle, Bartlett, Hanning, Hamming, Kaiser, Blackman, and SteepBlackman.");
	addState(name.setState("name",this,"Hanning","Name of the window function to generate:\nRectangle, Bartlett, Hanning, Hamming, Kaiser, Blackman, or SteepBlackman."));
	addState(length.setState("length",this,"256","Length of the window function to produce.",
# line 57 "CG56Window.pl"
A_SETTABLE|A_CONSTANT));
	addState(WindowParameters.setState("WindowParameters",this,"0","An array of numeric parameters for the window.\nFor the Kaiser window, the first entry in this state is taken as the\nbeta parameter which is proportional to the stopband attenuation of\nthe window."));

# line 103 "CG56Window.pl"
value.setAttributes(A_NONSETTABLE|A_CONSTANT);
}

void CG56Window::setup() {
# line 107 "CG56Window.pl"
const char* wn = name;

	if ( strcasecmp(wn, "Rectangle")==0 ) {
	    winType = SDFWinType_Rectangle;
	} else if ( strcasecmp(wn, "Bartlett")==0 ) {
	    winType = SDFWinType_Bartlett;
	} else if ( strcasecmp(wn, "Hanning")==0 ) {
	    winType = SDFWinType_Hanning;
	} else if ( strcasecmp(wn, "Hamming")==0 ) {
	    winType = SDFWinType_Hamming;
	} else if ( strcasecmp(wn, "Blackman")==0 ) {
	    winType = SDFWinType_Blackman;
	} else if ( strcasecmp(wn, "SteepBlackman")==0 ) {
	    winType = SDFWinType_SteepBlackman;
	} else if ( strcasecmp(wn, "Kaiser") == 0){
	    winType = SDFWinType_Kaiser;
	} else {
	    Error::abortRun(*this, "Unknown window name ", wn);
	    return;
	}

	// Don't want to risk divide by zero
	realLen = int(length);
	if ( realLen < 4 ) {
	    Error::abortRun(*this, "Window length is too small ",
			    "(should be greater than 3)");
	    return;
	}

	beta = double(WindowParameters[0]);

	double base_w = M_PI/(realLen-1);
	double sin_base_w = sin(base_w);
	double sin_2base_w = sin(2*base_w);
	double d = - (sin_base_w/sin_2base_w) * (sin_base_w/sin_2base_w);
	scale0 = scale1 = freq1 = scale2 = freq2 = 0.0;

	// Window defs taken from Jackson, Digital Filters and Signal
	//  Processing, Second Ed, chap 7.
	switch ( winType ) {
	  case SDFWinType_Rectangle:
	    scale0 = 1;
	    break;
	  case SDFWinType_Bartlett:
	    scale1 = 2.0/(realLen - 1);
	    break;
	  case SDFWinType_Hanning:
	    scale0 = .5;
	    scale1 = -.5;
	    freq1 = 2*base_w;
	    break;
	  case SDFWinType_Hamming:
	    scale0 = .54;
	    scale1 = -.46;
	    freq1 = 2*base_w;
	    break;
	  case SDFWinType_Blackman:
	    // This is a special case of SteepBlackman
	    d = -.16;		// scale0 = .42, scale1 = -.5, scale2=.08
	    // FALLTHROUGH
	  case SDFWinType_SteepBlackman:
	    // See Jackson 2ed, eqns 7.3.6 through 7.3.10
	    scale0 = (d+1)/2;
	    scale1 = -.5;
	    freq1 = 2*base_w;
	    scale2 = -d/2;
	    freq2 = 4*base_w;
	    break;
	  case SDFWinType_Kaiser:
	    alpha = double((realLen-1.0)/2.0);
	    norm = fabs(i0(beta));
	    break;
	  default:
	    Error::abortRun(*this, "Invalid window type");
	    return;
	}

	value.resize(int(length));
	window.initialize();		// initialize StringList to null string
	int i;

	switch ( winType ) {
	  case SDFWinType_Bartlett:
	    for (i = 0; i < (realLen - 1)/2; i++) {
		double val = scale1 * i;
		window << val << " ";
	    }
	    for ( ; i < realLen; i++) {
		double val = scale1 * (realLen-(i+1));
		// Outputs created at the time the star is compiled
		// Handled as a special case by Waveform::go method.
		output%(realPeriod-(i+1)) << val;
		window << val;
		if (i != realLen-1) window << " ";
	    }
	    break;
	  case SDFWinType_Kaiser:
	    for (i = 0; i < realLen; i++){
		double squared = double(pow(((i - alpha)/alpha), 2));
		double entry = beta*sqrt(1-squared);
		double val = i0(entry)/norm;
		window << val;
		if (i != realLen-1) window << " ";
	    }
	    break;
	  default:
	    for (i = 0; i < realLen; i++) {
		if (i!=0) window << " ";
		double val = scale0 + scale1 * cos(freq1*i) + scale2 * cos(freq2*i);
		window << val;
	    }
	}
	value.setInitValue(window);
	value.initialize();
	CG56WaveForm::setup();
}

// prototype instance for known block list
static CG56Window proto;
static RegisterBlock registerBlock(proto,"Window");
