static const char file_id[] = "CG56Chop.pl";
// .cc file generated from CG56Chop.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, an disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "CG56Chop.h"

const char *star_nm_CG56Chop = "CG56Chop";

const char* CG56Chop :: className() const {return star_nm_CG56Chop;}

ISA_FUNC(CG56Chop,CG56Star);

Block* CG56Chop :: makeNew() const { LOG_NEW; return new CG56Chop;}

const char* CG56Chop :: initRegisters(int x) {
# line 166 "CG56Chop.pl"
	static StringList _str_; _str_.initialize(); _str_ << 
"; Register usage:\n"
"; r0 = address of the input sample (initialized to start of input buffer)\n"
"; r1 = address of the output sample (initialized to start of output buffer)\n"
"	move #($addr(input)+(" << (x) << ")),r0\n"
"	move #$addr(output),r1	\n"
"	move #0,x0\n";
	return (const char*)_str_;
}

const char* CG56Chop :: padWithZeroes(int N) {
# line 174 "CG56Chop.pl"
	static StringList _str_; _str_.initialize(); _str_ << 
"	do	#" << (N) << ",$label(loop)	\n"
"	move	x0,$mem(output):(r1)+\n"
"$label(loop)\n";
	return (const char*)_str_;
}

const char* CG56Chop :: padAndSaveInputs(int N, int loc) {
# line 179 "CG56Chop.pl"
	static StringList _str_; _str_.initialize(); _str_ << 
"	move	#$addr(past_inputs),r4\n"
"	move	#($addr(input)+" << (loc) << "),r3\n"
"	do	#" << (N) << ",$label(loop2)\n"
"	tst	b	$mem(input):(r3)+,a	y:(r4),b\n"
"	tst	b	b,$mem(output):(r1)+	a,y:(r4)+\n"
"$label(loop2)\n";
	return (const char*)_str_;
}

const char* CG56Chop :: read_to_write(int N) {
# line 187 "CG56Chop.pl"
	static StringList _str_; _str_.initialize(); _str_ << 
"	do	#" << (N) << ",$label(loop3)	\n"
"	move	$mem(input):(r0)+,a\n"
"	move	a,$mem(output):(r1)+\n"
"$label(loop3)\n";
	return (const char*)_str_;
}

CG56Chop::CG56Chop ()
{
	setDescriptor("On each execution, this star reads a block of \"nread\" particles\nand writes them to the output with the given offset.  The number of\nparticles written is given by \"nwrite\".  The output block contains\nall or part of the input block, depending on \"offset\" and \"nwrite\".\nThe \"offset\" specifies where in the output block the first (oldest)\nparticle in the input block will lie.  If \"offset\" is positive,\nthen the first \"offset\" output particles will be either particles\nconsumed on previous firings (if \"use_past_inputs\" parameter is YES),\nor zero (otherwise).  If \"offset\" is negative, then the first \"offset\"\ninput particles will be discarded.");
	addPort(input.setPort("input",this,ANYTYPE));
	addPort(output.setPort("output",this,ANYTYPE));
	output.inheritTypeFrom(input);
	addState(nread.setState("nread",this,"128","Number of particles read."));
	addState(nwrite.setState("nwrite",this,"64","Number of particles written."));
	addState(offset.setState("offset",this,"0","start output block relative to start of input block."));
	addState(use_past_inputs.setState("use_past_inputs",this,"YES","if offset > 0, specify whether to use previously read inputs\n(otherwise use zeros)."));
	addState(past_inputs.setState("past_inputs",this,"0","internal state",
# line 70 "CG56Chop.pl"
A_NONSETTABLE|A_YMEM));


}

int CG56Chop::myExecTime() {
# line 195 "CG56Chop.pl"
int time = 3;
		computeRange();

		if (int(use_past_inputs)) time += ( padding + 3); 
		else time += ( 5 + 3*padding );

		if (numInputs > 0) time += ( 3 + 2*numInputs);

		if (hiLim < int(nwrite)) {
			int pad_end = int(nwrite) - hiLim;
			time += (pad_end + 3);
		}

		return time;
}

void CG56Chop::setup() {
# line 115 "CG56Chop.pl"
if (int(nread) <= 0){
			Error::abortRun(*this,
					"The number of samples to read ",
					"must be positive");
			return;
		}
		if (int(nwrite) <= 0){
			Error::abortRun(*this,
					"The number of samples to write ",
					"must be positive");
			return;
		}

		input.setSDFParams(int(nread),int(nread)-1);
		output.setSDFParams(int(nwrite),int(nwrite)-1);
		computeRange();
		// FIXME: We had to use a past_inputs state because
		// the CG56 domain does not support a construct like
		// input.setSDFParams(int(nread), int(nread)+int(offset)-1);
		if (int(use_past_inputs) > 0) {
			past_inputs.resize(padding);
		}
}

void CG56Chop::go() {
# line 140 "CG56Chop.pl"
addCode(initRegisters(index));
		if (padding > 0) {
			if (int(use_past_inputs)) { 
				int save_index = 0;
				if (save_index < 0)
					save_index = 0;
				else if (save_index > int(nread))
					save_index = int(nread);
				else
					save_index = int(nread) - padding;

				addCode(padAndSaveInputs(padding, save_index));
			}
			else {
				addCode(padWithZeroes(padding));
			}
		}
		if (numInputs > 0) {
			addCode(read_to_write(numInputs));
		}
		if (hiLim < int(nwrite)) {
			int pad_end = int(nwrite) - hiLim;
			addCode(padWithZeroes(pad_end));
		}
}


void CG56Chop::computeRange ()
{
# line 83 "CG56Chop.pl"
// Compute the range of output indexes that come from inputs
		// This method is called in the setup() method for the Chop
		// star, and in the go method for ChopVarOffset because
		// it resets the offset parameter
		hiLim = int(nread) + int(offset);
		if (hiLim > int(nwrite)) hiLim = int(nwrite);
		numInputs = int(nread);
		padding = int(offset);
		if (padding < 0 ) {
			padding = 0;
			numInputs = hiLim;
		}
		if (padding > int(nwrite)){
			padding = int(nwrite);
		}
		if ((int(nwrite) - padding) < numInputs) {
			numInputs = int(nwrite) - padding;
		}
		index = -int(offset);
		if (index < 0) {
			index = 0;
		}
		else if (index > int(nread)){
			Error::abortRun(*this,
					"absolute value of negative offsets ",
					"must not be larger than nread");
			return;
		}
}


// prototype instance for known block list
static CG56Chop proto;
static RegisterBlock registerBlock(proto,"Chop");
