.EQ
delim off
.EN
.\" documentation file generated from CGCTclScript.pl by ptlang
.\" Copyright (c) 1990-1996 The Regents of the University of California.
.\" All rights reserved.
.\" See the file $PTOLEMY/copyright for copyright notice,
.\" limitation of liability, and disclaimer of warranty provisions.
.NA "TclScript"
.SD
Sends input values to a Tcl script.  Gets output values from a Tcl script.
The star can communicate with Tcl either synchronously or asynchronously.
.SE
.LO "CGC Tcl/Tk library"
.DM CGC CGCStar
.SV 1.21 "2/28/96"
.AL "E. A. Lee, D. Niehaus and J. L. Pino"
.IH
.NE
\fIinput\fR (multiple), (float)
.PE
.OH
.NE
\fIoutput\fR (multiple), (float)
.PE
.SH
.NE
\fItcl_file\fR (StringState): The file from which to read the tcl script
.DF "$PTOLEMY/src/domains/cgc/tcltk/stars/tkScript.tcl"
.NE
\fIsynchronous\fR (IntState): If TRUE, tcl procedure '${uniqueSymbol}callTcl' is called on every firing
.DF "TRUE"
.NE
\fInumOutputs\fR (IntState)
.DF "0"
.NE
\fInumInputs\fR (IntState)
.DF "0"
.NE
\fItemp\fR (IntState)
.DF "0"
.ET
.EQ
delim $$
.EN
.LD
.EQ
delim off
.EN
This star reads a file containing Tcl commands and communicates with Tcl
via procedures defined in that file.  Those procedures can read the inputs
to the star and set its outputs.
It can be used in a huge
variety of ways, including using Tk to animate or control a simulation.
The Tcl file must define three Tcl procedures to communicate
between the star and the Tcl code.  One of these reads the values of the
inputs to the star (if any), another writes new values to the outputs (if
any), and the third is called by Ptolemy either on every firing of
the star (if \fIsynchronous\fR is TRUE) or on starting the simulation
(if \fIsynchronous\fR is FALSE).
.pp
The names of the three procedures are different for each instance of this star.
This allows sharing of Tcl code without name conflicts.
These unique names are constructed by prepending a unique string to a
constant suffix.  The suffixes for the three procedures mentioned
above are "setOutputs", "grabInputs", and "callTcl".
The first thing the star does is to define the Tcl variable "uniqueSymbol"
to equal the unique string.  This string specifies the prefix that makes
the name unique.  Thus the full name for the three procedures
is "${uniqueSymbol}setOutputs", "${uniqueSymbol}grabInputs", and
"${uniqueSymbol}callTcl".  The first two of these are defined internally
by the star.  The third should be defined by the user in the Tcl file
that the star reads.
.pp
Two basic mechanisms can be used to control the behavior of the star.
In the first, X events are bound to Tcl/Tk commands that read or write
data to the star.  These Tcl commands use
${uniqueSymbol}grabInputs, which returns
a list containing the current value of each of the inputs, and
${uniqueSymbol}setOutputs, which sets the value of the outputs.
The argument list for ${uniqueSymbol}setOutputs should contain a
floating point value for each output of the star.
This mechanism is entirely asychronous, in that the Tcl/Tk script
decides when these actions should be performed on the basis of X events.
.pp
In addition, the Tcl procedure ${uniqueSymbol}callTcl will be called
by the star.  If the parameter "synchronous" is TRUE, this procedure
will be called every time the star fires.
The procedure could,
for example, grab input values and computes output values,
although it can do anything the designer wishes, even ignoring the input
and output values.  If the parameter "synchronous" is FALSE, then the
Tcl script "${uniqueSymbol}callTcl" is called only once during the
initialization phase.  At that
time, it can, for example,
set up periodic calls to poll the inputs and set the outputs.
.pp
If the procedure "${uniqueSymbol}callTcl" is not defined in the given
tcl_file, an error message results.
.pp
Finally, this star can be used as a based class for other Tcl/Tk CGC
stars.  In this manner, you can also have tcl/tk code which uses custom
states.  The states are stored in a array named ${uniqueSymbol}
indexed by the state name.  For example the tcl command:
.pp
set foo [set ${uniqueSymbol}(foo)]
.pp
will set the variable "foo" to the value of the state named "foo".
.EQ
delim $$
.EN
.ES
