static const char file_id[] = "CGCFIRFix.pl";
// .cc file generated from CGCFIRFix.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "CGCFIRFix.h"

const char *star_nm_CGCFIRFix = "CGCFIRFix";

const char* CGCFIRFix :: className() const {return star_nm_CGCFIRFix;}

ISA_FUNC(CGCFIRFix,CGCFix);

Block* CGCFIRFix :: makeNew() const { LOG_NEW; return new CGCFIRFix;}

CodeBlock CGCFIRFix :: bodyDecl (
# line 224 "CGCFIRFix.pl"
"	int phase, tapsIndex, inC, i;\n"
"	int outCount = $val(interpolation) - 1;\n"
"	int inPos;\n"
"	double out, tap;\n");

CodeBlock CGCFIRFix :: body (
# line 231 "CGCFIRFix.pl"
"	/* phase keeps track of which phase of the filter coefficients is used.\n"
"	   Starting phase depends on the decimationPhase state. */\n"
"	phase = $val(decimation) - $val(decimationPhase) - 1;   \n"
"	\n"
"	/* Iterate once for each input consumed */\n"
"	for (inC = 1; inC <= $val(decimation) ; inC++) {\n"
"\n"
"		/* Produce however many outputs are required for each \n"
"		   input consumed */\n"
"		while (phase < $val(interpolation)) {\n"
"\n"
"			FIX_SetToZero($ref(Accum));\n"
"\n"
"			/* Compute the inner product. */\n"
"			for (i = 0; i < $val(phaseLength); i++) {\n"
"				tapsIndex = i * $val(interpolation) + phase;\n"
"				if (tapsIndex >= $val(tapSize))\n"
"			    		FIX_SetToZero($ref(tap));\n"
"				else\n"
"			 		FIX_Assign($ref(tap),$ref(taps,tapsIndex));\n"
"\n"
"				inPos = $val(decimation) - inC + i;\n"
"				FIX_MulAdd($ref(Accum), $ref(tap),$ref(signalIn,inPos));\n"
"			}\n"
"			FIX_Assign($ref(signalOut,outCount),$ref(Accum));\n"
"			outCount--;;\n"
"			phase += $val(decimation);\n"
"		}\n"
"		phase -= $val(interpolation);\n"
"	}\n");

CGCFIRFix::CGCFIRFix ()
{
	setDescriptor("A finite impulse response (FIR) filter with fixed-point capabilities.\nThe fixed-point coefficients are specified by the \"taps\" parameter.\nThe default coefficients give an 8th-order, linear phase lowpass filter.\nTo read coefficients from a file, replace the default coefficients\nwith \"<fileName\", preferably specifying a complete path.\nPolyphase multirate filtering is also supported.");
	addPort(signalIn.setPort("signalIn",this,FIX));
	addPort(signalOut.setPort("signalOut",this,FIX));
	addState(taps.setState("taps",this,"-.040609 -.001628 .17853 .37665 .37665 .17853 -.001628 -.040609","Filter tap values."));
	addState(decimation.setState("decimation",this,"1","Decimation ratio."));
	addState(decimationPhase.setState("decimationPhase",this,"0","Downsampler phase."));
	addState(interpolation.setState("interpolation",this,"1","Interpolation ratio."));
	addState(ArrivingPrecision.setState("ArrivingPrecision",this,"YES","Flag indicating whether or not to use the arriving particles as they are:\nYES keeps the same precision, and NO casts them to the precision specified\nby the parameter \"InputPrecision\"."));
	addState(InputPrecision.setState("InputPrecision",this,"4.14","Precision of the input in bits.\nThe input particles are only cast to this precision if the parameter\n\"ArrivingPrecision\" is set to NO."));
	addState(TapPrecision.setState("TapPrecision",this,"4.14","Precision of the taps in bits."));
	addState(AccumulationPrecision.setState("AccumulationPrecision",this,"4.14","Precision of the accumulation in bits."));
	addState(OutputPrecision.setState("OutputPrecision",this,"4.14","Precision of the output in bits."));
	addState(phaseLength.setState("phaseLength",this,"0","phaseLength",
# line 174 "CGCFIRFix.pl"
A_NONCONSTANT|A_NONSETTABLE));
	addState(tapSize.setState("tapSize",this,"0","tapSize",
# line 180 "CGCFIRFix.pl"
A_NONCONSTANT|A_NONSETTABLE));
	addState(tap.setState("tap",this,"0.0","tap",
# line 186 "CGCFIRFix.pl"
A_NONCONSTANT|A_NONSETTABLE));
	addState(Accum.setState("Accum",this,"0.0","Accum",
# line 192 "CGCFIRFix.pl"
A_NONCONSTANT|A_NONSETTABLE));


}

void CGCFIRFix::setup() {
# line 196 "CGCFIRFix.pl"
CGCFix::setup();

		int d = decimation;
		int i = interpolation;
		int dP = decimationPhase;
		signalIn.setSDFParams(d, d+1+(taps.size()/i));
		signalOut.setSDFParams(i, i-1);
		if (dP >= d) {
			Error::abortRun (*this, ": decimationPhase too large");
			return;
		}
		// The phaseLength is ceiling(taps.size/interpolation)
		// It is a protected instance variable.
		int temp = taps.size() / i;
		tapSize = taps.size();
		if ((taps.size() % i) != 0)  temp++;
		phaseLength = temp;

		// Set the precision on the fixed-point variables

		if (!int(ArrivingPrecision))
			signalIn.setPrecision(InputPrecision);
		signalOut.setPrecision(OutputPrecision);

		tap.setPrecision(TapPrecision);
		Accum.setPrecision(AccumulationPrecision);
}

void CGCFIRFix::go() {
# line 265 "CGCFIRFix.pl"
addCode(bodyDecl);
		addCode(body);
}

// prototype instance for known block list
static CGCFIRFix proto;
static RegisterBlock registerBlock(proto,"FIRFix");
