static const char file_id[] = "CGCGoertzelBase.pl";
// .cc file generated from CGCGoertzelBase.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "CGCGoertzelBase.h"
#include <math.h>

const char *star_nm_CGCGoertzelBase = "CGCGoertzelBase";

const char* CGCGoertzelBase :: className() const {return star_nm_CGCGoertzelBase;}

ISA_FUNC(CGCGoertzelBase,CGCStar);

Block* CGCGoertzelBase :: makeNew() const { LOG_NEW; return new CGCGoertzelBase;}

CodeBlock CGCGoertzelBase :: decl (
# line 113 "CGCGoertzelBase.pl"
"		double acc = 0.0;\n"
"		double d1val = $ref(d1);\n"
"		int i;\n");

CodeBlock CGCGoertzelBase :: filter (
# line 118 "CGCGoertzelBase.pl"
"		/* Run all-pole section of Goertzel's algorithm N iterations.\n"
"		   Only one multiplier (d1) in iteration.\n"
"		   Zero the IIR state for each DFT calculation; otherwise,\n"
"		   the filter output could grow without bound.\n"
"		   state1 and state2 are states and not local variables\n"
"		   ONLY to pass their values to derived stars */\n"
"		$ref(state1) = 0.0;\n"
"		$ref(state2) = 0.0;\n"
"		for (i = $val(N)-1; i >= 0; i--) {\n"
"		  acc = $ref2(input,i);\n"
"		  acc += d1val * $ref(state1);\n"
"		  acc -= $ref(state2);\n"
"		  $ref(state2) = $ref(state1);\n"
"		  $ref(state1) = acc;\n"
"		}\n");

CGCGoertzelBase::CGCGoertzelBase ()
{
	setDescriptor("Base class for Goertzel algorithm stars.");
	addPort(input.setPort("input",this,FLOAT));
	addState(k.setState("k",this,"0","the DFT coefficient to compute (k < N)"));
	addState(N.setState("N",this,"32","length of the DFT"));
	addState(size.setState("size",this,"32","amount of data to read (N <= size)"));
	addState(d1.setState("d1",this,"0.0","first-order feedback coefficient which is a function of k and N",
# line 44 "CGCGoertzelBase.pl"
A_NONCONSTANT|A_NONSETTABLE));
	addState(state1.setState("state1",this,"0.0","internal state.",
# line 51 "CGCGoertzelBase.pl"
A_NONCONSTANT|A_NONSETTABLE));
	addState(state2.setState("state2",this,"0.0","internal state.",
# line 58 "CGCGoertzelBase.pl"
A_NONCONSTANT|A_NONSETTABLE));

# line 64 "CGCGoertzelBase.pl"
theta = 0.0;
}

int CGCGoertzelBase::myExecTime() {
# line 141 "CGCGoertzelBase.pl"
return (3 + 3 + 7*int(N));
}

void CGCGoertzelBase::setup() {
# line 102 "CGCGoertzelBase.pl"
// FIXME: Parameters are not always resolved properly
                // before setup but should be.  For now, check parameters
                // in go method and guard against division by N = 0
                // CheckParameterValues();
                // double Nd = double(int(N));
		double Nd = double(int(N) ? int(N) : 1);
		double kd = int(k);
		theta = -2.0 * M_PI * kd / Nd;
		d1 = 2.0 * cos(theta);
		input.setSDFParams(int(size), int(size)-1);
}

void CGCGoertzelBase::go() {
# line 136 "CGCGoertzelBase.pl"
CheckParameterValues();
		addCode(decl);
		addCode(filter);
}


void CGCGoertzelBase::CheckParameterValues ()
{
# line 72 "CGCGoertzelBase.pl"
if ( int(k) < 0 ) {
		  Error::abortRun(*this,
			"The value for state k must be nonnegative.");
		  return;
		}
		if ( int(N) <= 0 ) {
		   Error::abortRun(*this,
			"The value for state N must be positive.");
		   return;
		}
		if ( int(size) <= 0 ) {
		   Error::abortRun(*this,
			"The value for state size must be positive.");
		   return;
		}
		if ( int(k) >= int(N) ) {
		   Error::abortRun(*this,
			"The DFT coefficient k must be less than the ",
			"DFT length N.");
		   return;
		}
		if ( int(N) > int(size) ) {
		   Error::abortRun(*this,
			"The DFT length N must be less than or equal to the ",
			"number of data samples read, given by state size.");
		   return;
		}
}


// prototype instance for known block list
static CGCGoertzelBase proto;
static RegisterBlock registerBlock(proto,"GoertzelBase");
