static const char file_id[] = "DDFSelf.pl";
// .cc file generated from DDFSelf.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "DDFSelf.h"

const char *star_nm_DDFSelf = "DDFSelf";

const char* DDFSelf :: className() const {return star_nm_DDFSelf;}

ISA_FUNC(DDFSelf,DDFStar);

Block* DDFSelf :: makeNew() const { LOG_NEW; return new DDFSelf;}

DDFSelf::DDFSelf ()
{
	setDescriptor("This is a first exploration of recursion and higher-order functions\nexpanded while the system is running.  This is in contrast to the\nhigher-order function stars like HOFChain, HOFMap, and HOFMapGr that\nexpand the system during compilation and incur no run-time penalty.\nThe DDFSelf star is still experimental, so do not expect it to be\neither efficient or bug-free.\n\nThe star \"represents\" the galaxy given by the parameter \"recurGal\",\nwhich must be above it in the hierarchy.  That is, when the Self\nstar fires, it actually invokes the galaxy that it represents.\nSince that galaxy is above the Self star in the hierarchy, it\ncontains the Self star somewhere within it.  Thus, this star\nimplements recursion.  Since the Self star takes an argument\n(\"recurGal\") that specifies the function to invoke, it is itself\na higher-order function.\n\nThe instance of the \"recurGal\" galaxy is not created until it is\nactually needed, so the number of instances (the depth of the\nrecursion) does not need to be known a-priori.  If the parameter\n\"reinitialize\" is FALSE, then the instance of the galaxy is created\nthe first time it fires and reused on subsequent firings.\nIf \"reinitialize\" is TRUE, then the galaxy is created on every firing\nand destroyed after the firing.  Inputs are sent to the instance\nof the galaxy and outputs are retrieved from it.  The inputs of the\nnamed galaxy must be named \"input#?\" and the outputs must be named\n\"output#?\", where \"?\" is replaced with an integer starting with zero.\nThis allows the inputs and outputs of this star to be matched\nunambiguously with the inputs and outputs of the referenced galaxy.");
	addPort(input.setPort("input",this,ANYTYPE));
	addPort(output.setPort("output",this,ANYTYPE,0));
	addState(recurGal.setState("recurGal",this,"","Name of galaxy to be invoked."));
	addState(reinitialize.setState("reinitialize",this,"NO","Initialize the galaxy on each firing"));

# line 92 "DDFSelf.pl"
masterGal = 0;
		myGal = 0;
}

DDFSelf::~DDFSelf() {
# line 96 "DDFSelf.pl"
LOG_DEL; delete myGal;
}

void DDFSelf::setup() {
# line 99 "DDFSelf.pl"
// If a galaxy already exists, just initialize it
	    if (myGal) {
		myGal->initialize();
	    }
	    else {
		// The following should not be required because of the
		// num { 0 } above.  At one time, this did not work.
		// output.setDDFParams(0);

		StringList msg = "DDFSelf Star \"";
		msg << name() << "\" :\n";
		int flag = 0;

		// find out which galaxy is recursive.
		if (recurGal.null()) {
			msg << "undefined name for recursion construct\n";
			Error :: abortRun(*this, msg);
			return;
		}

		Block* b = this;
		do {
		    b = b->parent();
		    if (!b || (b->isA("InterpUniverse"))) {
			msg << "Unmatched name "
			    << (const char*)recurGal
			    << " for recursion construct\n";
			Error :: abortRun(*this, msg);
			return;
		    }
		    if(!(b->isA("InterpGalaxy"))) {
			msg << "Sorry, recursion only works for InterpGalaxies";
			Error :: abortRun(*this, msg);
			return;
		    }
		    masterGal = (InterpGalaxy*)b;
		} while (strcmp(masterGal->className(), recurGal.currentValue())
			!= 0);

		// match portholes.
		int inNum = 0, outNum = 0;
		BlockPortIter next(*masterGal);
		for (int i = masterGal->numberPorts(); i > 0; i--) {
			PortHole& p = *next++;
			if (p.isItInput()) inNum++;
			else outNum++;
			DDFPortHole* dp = (DDFPortHole*) portWithName(p.name());
			if (!dp) {
				msg << "wrong galaxy portname for recursion \""
				    << p.name()
				    << "\" (should be input#? or output#?)\n";
				flag++;
			}
			else {
				dp->imagePort = &p;
			}
		}
		if (input.numberPorts() != inNum ||
		    output.numberPorts() != outNum) {
			msg << "unmatched number of ports with recursion galaxy\n";
			flag++;
		}
		if (flag) Error :: abortRun(*this, msg);
	    }
}

void DDFSelf::go() {
# line 165 "DDFSelf.pl"
if (myGal == 0) {
		// clone the recursion galaxy
		myGal = (InterpGalaxy*) masterGal->clone();
		myGal->setNameParent(masterGal->name(), this);
		sched.resetFlag();

		// make a image connection between DDFSelf and galaxy.
		BlockPortIter next(*myGal);
		for (int i = myGal->numberPorts(); i > 0; i--) {
			PortHole& p = *next++;
			DDFPortHole* dp = (DDFPortHole*) portWithName(p.name());
			dp->imagePort = &p;

			// make connection
			dp->imageConnect();
		}

		// scheduler setup and run
		sched.setGalaxy(*myGal);
		sched.setup();
		if (Scheduler::haltRequested()) {
			Error :: abortRun(*this,
					  "error in setting up the scheduler");
			return;
		}
	    } else {
		if (int(reinitialize)) myGal->initialize();
	    }

	    // move data into galaxy
	    MPHIter nexti(input);
	    for (int j = input.numberPorts(); j > 0; j--) {
	    	InDDFPort& p = *(InDDFPort*) nexti++;
	    	p.receiveData();
		if (Scheduler :: haltRequested()) return;
	    	p.moveData();
	    }

	    // scheduler run.  Unless we reinitialize, we increment the
	    // stop time by one each time through.
	    sched.setStopTime(sched.getStopTime() + 1.0);
	    sched.run();

	    // move data out of galaxy
	    int numXfred = 0;
	    MPHIter nexto(output);
	    for (j = output.numberPorts(); j > 0; j--) {
	    	OutDDFPort& p = *(OutDDFPort*) nexto++;
	    	numXfred = p.moveData();
		for (int c = numXfred; c > 0; c--) p.sendData();
	    }
}


int DDFSelf::isItSelf ()
{
# line 84 "DDFSelf.pl"
return TRUE;
}


// prototype instance for known block list
static DDFSelf proto;
static RegisterBlock registerBlock(proto,"Self");
