static const char file_id[] = "DESrcControl.pl";
// .cc file generated from DESrcControl.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "DESrcControl.h"
#include "VoiceData.h"

const char *star_nm_DESrcControl = "DESrcControl";

const char* DESrcControl :: className() const {return star_nm_DESrcControl;}

ISA_FUNC(DESrcControl,DERepeatStar);

Block* DESrcControl :: makeNew() const { LOG_NEW; return new DESrcControl;}

# line 73 "DESrcControl.pl"
extern int voiceCheck (Envelope&, NamedObj&);

DESrcControl::DESrcControl ()
{
	setDescriptor("Communicates with an\n.c MQTelephone\nstar to control\n.c DEVoiceSrc\nstars' transmissions");
	addPort(phoneIn.setPort("phoneIn",this,MESSAGE));
	addPort(phoneOut.setPort("phoneOut",this,MESSAGE));
	addPort(control.setPort("control",this,MESSAGE));
	addState(numVci.setState("numVci",this,"1","VCI of associated telephone"));

# line 77 "DESrcControl.pl"
phoneIn.triggers();
}

void DESrcControl::setup() {
# line 90 "DESrcControl.pl"
activeOrNot = FALSE;
               feedbacksLeft = 1;      //  recall one feedback event comes
                                       //  at time zero for RepeatStar
}

void DESrcControl::go() {
# line 96 "DESrcControl.pl"
Envelope inEnv;
               OutDEMPHIter nextp(control);
               OutDEPort*   oport;

               if (activeOrNot == FALSE) {     //  sources idle

                   if (!phoneIn.dataNew) {
                       feedbacksLeft--;
                       if (feedbacksLeft < 0)  {
                           Error :: abortRun (*this,
                                    "how can no. of feedback events be negative?");
                           return;
                       }
                       return;
                   }

                   phoneIn.get().getMessage(inEnv);
                   if (!voiceCheck (inEnv, *this)) return;
                   const VoiceData*  v = (const VoiceData*) inEnv.myData();
                
                   if (!v->isitSDMessage())  {
                       Error :: abortRun (*this,
                                "expected a sendDataPacket in idle state.");
                       return;
                   }

                   int  vpi, vciSrc, vciDst;
                   unsigned char* ptr = v->asVoice();

                   // read info from SDP to put into Transmit packet
                   // record specified call length
                   length = ptr[6];
                   vciDst = ptr[7];
                   vciSrc = int(numVci);
                   
                   if (vciSrc%3 == 1 || vciSrc%3 == 2)
                       vpi = vciSrc%3;
                   else
                       vpi = 3;

                   LOG_NEW; Envelope  oEnv(*new VoiceData ("TRP", &vciSrc, &vciDst, &vpi));
                   completionTime = arrivalTime;
                   
                   // send the transmit packet to all connected sources
                   while ((oport = nextp++) != 0)
                       (*oport).put(completionTime) << oEnv;

                   refireAtTime (completionTime + double(length));
		   //  keep track of the refire feedback particles
                   feedbacksLeft++;
                   activeOrNot = TRUE;

               }
               else  {
 
                   if (phoneIn.dataNew) {

                       //  the caller interrupted by MQ telephone
                       phoneIn.get().getMessage(inEnv);
                       if (!voiceCheck (inEnv, *this)) return;
                       const VoiceData*  v1 = (const VoiceData*) inEnv.myData();
                    
                       if (!v1->isitSSDMessage()) {
                           Error :: abortRun (*this,
                                    "expected a stopSendData packet!");
                           return;
                       }

                       completionTime = arrivalTime;

                       // acknowledge the interruption
                       LOG_NEW; Envelope  oEnv1 (*new VoiceData("ACK"));
                       phoneOut.put(completionTime) << oEnv1;

                       LOG_NEW; Envelope  oEnv2 (*new VoiceData("STRP"));
                       // send the stop transmit packet to all sources
                       while ((oport = nextp++) != 0)
                           (*oport).put(completionTime) << oEnv2;

                       activeOrNot = FALSE;

                   }
                   else {

		       // time-out, time for this phone to end the conversation

                       feedbacksLeft--;
                       if (feedbacksLeft < 0)  {
                           // impossible to have negative number
                           Error :: abortRun (*this,
                                    "how can no. of feedback events be negative?");
                           return;
                       }

                       feedbackIn->get();

                       if (feedbacksLeft != 0)  {
                           // this situation indicates that there are leftover events
                           // in the feedback arc, would happen if Caller A talks to
                           // Caller B, B terminates (A's feedback event is still in
                           // queue for ending the call), and another call occurs between
                           // A and C before this feedback event is processed, then 
                           // Caller A hangs up on C, i.e. the feedback event from the
                           // previous call has been processed instead of the current call!
                           // so we take suitable precautions
                           // Error :: error (*this,
                           //       "leftover feedback refirings in transmission!");
                           return;
                       }
                       
                       completionTime = arrivalTime;

                       // send ACK to phone to let it know of desire to "hang up"
                       LOG_NEW; Envelope oEnv3 (*new VoiceData ("ACK"));
                       phoneOut.put(completionTime) << oEnv3;

                       LOG_NEW; Envelope oEnv4 (*new VoiceData("STRP"));
                       // force sources to cease transmissions
                       while ((oport = nextp++) != 0)
                           (*oport).put(completionTime) << oEnv4;

                       activeOrNot = FALSE;
                   }
               }
}

// prototype instance for known block list
static DESrcControl proto;
static RegisterBlock registerBlock(proto,"SrcControl");
