static const char file_id[] = "DEVoiceSrc.pl";
// .cc file generated from DEVoiceSrc.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "DEVoiceSrc.h"
#include "VoiceData.h"
#include <ACG.h>

const char *star_nm_DEVoiceSrc = "DEVoiceSrc";

const char* DEVoiceSrc :: className() const {return star_nm_DEVoiceSrc;}

ISA_FUNC(DEVoiceSrc,DERepeatStar);

Block* DEVoiceSrc :: makeNew() const { LOG_NEW; return new DEVoiceSrc;}

# line 77 "DEVoiceSrc.pl"
extern ACG* gen;
                extern int voiceCheck (Envelope&, NamedObj&);

DEVoiceSrc::DEVoiceSrc ()
{
	setDescriptor("Generates a voice process for a single caller and recognizes\n.c VoiceData\nprompts to begin and cease packet transmission.");
	addPort(input.setPort("input",this,MESSAGE));
	addPort(output.setPort("output",this,MESSAGE));
	addState(packPeriod.setState("packPeriod",this,".016","Voice packetization period"));
	addState(silenceExpMean.setState("silenceExpMean",this,".650","Exp. mean for a silence period in seconds"));
	addState(meanPacketsSpurt.setState("meanPacketsSpurt",this,"22","Mean no. of packets per talkspurt"));

# line 82 "DEVoiceSrc.pl"
randomExp = NULL;
                randomUnif = NULL;
}

DEVoiceSrc::~DEVoiceSrc() {
# line 87 "DEVoiceSrc.pl"
if (randomExp) { LOG_DEL; delete randomExp; }
		if (randomUnif) { LOG_DEL; delete randomUnif; }
}

void DEVoiceSrc::setup() {
# line 100 "DEVoiceSrc.pl"
if (randomExp) { LOG_DEL; delete randomExp; }
                LOG_NEW; randomExp = new NegativeExpntl(double(silenceExpMean),gen);

                if (randomUnif) { LOG_DEL; delete randomUnif; }
                LOG_NEW; randomUnif = new Uniform(double(0),double(1),gen);

                talkingOrNot = FALSE;
}

void DEVoiceSrc::go() {
# line 110 "DEVoiceSrc.pl"
Envelope inEnv;

                if (talkingOrNot == FALSE)  {       //  idle state
                  
                    if (!input.dataNew)  {
                        //  leftover feedback event is possible
                        feedbackIn->get();
                        return;
                    }

                    //  expect a TRANSMIT packet to begin voice transmission
                    input.get().getMessage(inEnv);
                    if (!voiceCheck (inEnv, *this)) return;
                    const VoiceData*  v = (const VoiceData*) inEnv.myData();

                    if (!v->isitTRMessage()) {
                        Error :: abortRun (*this,
                                 "expected a Transmit packet in idling state.");
                        return;
                    }

                    unsigned char* ptr = v->asVoice();
                    //  record info to be mapped into voice packet headers
                    vciSrc = ptr[6];
                    vciDst = ptr[7];
                    vpi    = ptr[8];

                    // mark the start of the transmission with an "SOT" packet
                    completionTime = arrivalTime;
                    LOG_NEW; VoiceData* w = new VoiceData ("SOT", &vciSrc, &vciDst);
		    w -> setStamp(completionTime);
                    Envelope oEnv(*w);
                    output.put(completionTime) << oEnv;

                    // prepare to refire to begin data transmission
                    completionTime += double(packPeriod);
                    refireAtTime(completionTime);
                    talkingOrNot = TRUE;

                }
                else  {       //  talking state
  
                    if (input.dataNew) {

                        //  transmission interrupted
                        input.get().getMessage(inEnv);
                        if (!voiceCheck (inEnv, *this)) return;
                        const VoiceData*  v1 = (const VoiceData*) inEnv.myData();

                        if (!v1->isitSTRMessage()) {
                            Error :: abortRun (*this,
                                     "expected a stopTransmit packet when talking.");
                            return;
                        }

                        // send out an EOT packet to mark end of transmission
                        completionTime = arrivalTime + double(packPeriod);
                        LOG_NEW; VoiceData*  w1 = new VoiceData("EOT", &vciSrc, &vciDst);
			w1 -> setStamp(completionTime);
                        Envelope oEnv1(*w1);
                        output.put(completionTime) << oEnv1;
                        talkingOrNot  = FALSE;
            
                    }
                    else {
                  
			//  received a token on the "feedback"port caused by a
			//  call to refire this star, transmit a packet with
			//  appropriate header information
                        LOG_NEW; VoiceData*  w2 = new VoiceData();
                        unsigned char*  ptz = w2->asVoice();
                        ptz[0] = vpi;  ptz[2] = vciSrc;  ptz[3] = vciDst;

			completionTime = arrivalTime;
			w2 -> setStamp(completionTime);
                        Envelope  oEnv2(*w2);
                        output.put(completionTime) << oEnv2;
                        double d1, d2;
                        d2 = (*randomUnif)();
    
                        if (d2 < 1.0/double(int(meanPacketsSpurt)))  {
                            d1 = (*randomExp)();
                            completionTime += d1;
                        }
                        else {
                            completionTime += double(packPeriod);
                        }

                        refireAtTime(completionTime);

                    }  // packet transmission vs. interruption "if" check
                }  // testing for idle or talking state
}

// prototype instance for known block list
static DEVoiceSrc proto;
static RegisterBlock registerBlock(proto,"VoiceSrc");
