.EQ
delim off
.EN
.\" documentation file generated from DEEther.pl by ptlang
.\" Copyright (c) 1990-1995 The Regents of the University of California.
.\" All rights reserved.
.\" See the file $PTOLEMY/copyright for copyright notice,
.\" limitation of liability, and disclaimer of warranty provisions.
.NA "Ether"
.SD
This is the base class for transmitter and receiver
stars that communicate over a shared medium.
Each transmitter can communicate with any or all receivers
that have the same value for the "medium" parameter.
The communication is accomplished without graphical connections,
and the communication topology can be continually changing.
This base class implements the data structures that are shared
between the transmitters and receivers.
.SE
.LO "DE main library"
.DM DE DERepeatStar
.SV 1.8 "3/2/95"
.AL "E. A. Lee  and T. Parks"
.SH
.NE
\fImedium\fR (StringState): The name of the medium to share.
.DF "radio900MHz"
.ET
.EQ
delim $$
.EN
.LD
This star, and its derivatives
.c EtherRec
and
.c EtherSend,
demonstrate the use of shared data structures
.Ie "shared data structures"
to simulate one or more shared physical media.
Each instance of the star specifies the medium it uses by name;
all stars that use that name will share that medium.
Any ASCII string can be used as a name.
.pp
The
.c EtherSend
star is given a Particle, an address, and a duration.
It checks to see whether the medium is currently occupied by another
transmitter using the same medium.  If not, it occupies
the medium for the specified duration, and delivers a copy of
the particle to a receiver that matches the specified address.
The
.c EtherRec
star, after the "duration" time has elapsed, outputs the particle.
In this implementation, receiver addresses are arbitrary non-negative integers.
They need not be contiguous, but they must be unique.
The implementation could easily be extended to permit more
than one receiver to have the same address.
If a negative integer is specified as the address at the transmitter,
then the transmitter broadcasts a copy of the data particle to
all receivers.
.pp
When the transmitter receives a new data particle, the medium
could be occupied (by the same transmitter or a different one using
the same medium).
If this is the case, then the data particle is not transmitted to
any receiver, but instead is sent out through the "collision"
output port.
This is a pretty simple-minded implementation, and may not be rich
enough to implement practical collision detection protocols.
So it should be viewed as a concept demonstration only.
.pp
The way these stars work is to define, in the base class
.c DEEther,
a static hash table that contains an entry for each medium.
By making this hash table static, it is made available to
all derived stars.
Each entry in this table contains another hash table
with an entry for each receiver that uses the medium.
The index into the hash table is the receiver address.
In this implementation, only one receiver can occupy
a given address, but it would be easy to change the hash table
entry to a
.c SequentialList
of receivers.
This would allow any number of receivers to share the same address.
.pp
The receiver addresses are interpreted as strings.  Thus,
anything can be used.  The particular implementation of
the derived star
.c EtherSend
assumes the receivers will have non-negative integers for addresses.
But another implementation might use arbitrary ASCII strings.
.pp
The medium is reset in the
.c wrapup() method of the star.
.Ie wrapup
Note that this does not get invoked if an error occurs during the run.
Thus, if an error occurs, the star should be destroyed
and recreated.  Pigi takes care of this automatically,
but if the star is run under ptcl, then it is up to the
user to do this.
.ES
