.EQ
delim off
.EN
.\" documentation file generated from DEVirtClock.pl by ptlang
.\" Copyright (c) 1990-1995 The Regents of the University of California.
.\" All rights reserved.
.\" See the file $PTOLEMY/copyright for copyright notice,
.\" limitation of liability, and disclaimer of warranty provisions.
.NA "VirtClock"
.SD
This star reads in a NetworkCell. It identifies which virtual circuit
number the cell belongs to and then computes the virtual time stamp for
the cell by applying the (special!) virtual clock algorithm (see the full
manual page). It then outputs all cells in order of increasing
virtual time stamp.

Upon receiving a 'demand' input, the cell with the smallest timestamp is
output. An output packet is generated for every demand input unless
all of the queues are empty. Demand inputs arriving when all queues
are empty are ignored.

The number of stored cells is output after the receipt of each 'input'
or 'demand'.

When a cell arrives and the number of stored cells equals 'MaxSize'
then the cell with the biggest virtual timestamp is discarded. This
cell may or may not be the new arrival. If 'MaxSize' is zero or
negative, then infinitely many cells can be stored.
.SE
.LO "DE main library"
.DM DE DEStar
.SV 1.8 "8/9/95"
.AL "Richard Han, Paul Haskell"
.IH
.NE
\fIinput\fR (multiple), (message)
.NE
\fIdemand\fR (anytype)
.PE
.OH
.NE
\fIoutput\fR (message)
.NE
\fIdiscard\fR (message)
.NE
\fIlength\fR (int)
.PE
.SH
.NE
\fIMaxSize\fR (IntState): Maximum number of stored cells.
.DF "50"
.NE
\fIAvgRate\fR (FloatArrayState): Average rate for each input virtual circuit.
.DF "1.0 1.0"
.ET
.EQ
delim $$
.EN
.LD
The (new, improved) virtual clock buffer service discipline used by
this star works like this:
.pp
Each connection through a buffer is assigned a rate Ri.
Let Di = (1/Ri), the minimum allowable inter-cell time duration.
.pp
The virtual "current time" is the timestamp of the most recently
served cell.
.pp
The first cell from the first established connection through a
buffer is given virtual timestamp 0.0.
The first cells from other connections are stamped with the
virtual "current time".
Cells from a connection i other than the first cell are stamped with
the later (i.e. maximum) of
.ti 5
(1) the current time
.ti 5
(2) Di + the timestamp of the most recent enqueued OR served cell
from stream i.
.pp
Note that the timestamp for each new cell is AT LEAST Di time units
later than the timestamps of all other cells on the same virtual
circuit.
.pp
When a cell arrives at a full buffer, it is assigned a timestamp as
usual.
The cell with the biggest timestamp is discarded.
If the discarded cell is not the just-arrived cell, the just-arrived
cell is enqueued using the just-freed memory.
.pp
This buffer control policy (i.e. buffer service and buffer access
policies) is fair, work-conserving, and buffer-conserving.
Fairness is guaranteed because, in the presence of competing traffic,
every source gets served at exactly its average rate.
The control is work-conserving because even if only one source
is active, the buffer never remains idle when there are
cells available to serve.
The controls is buffer-conserving because cells are discarded only
when the buffer is full.
.pp
This buffer control strategy simplifies Zhang's proposed
Virtual Clock control discipline [1].
Zhang's discipline requires that network users and networks agree
on a "monitoring interval" in addition to an average service rate.
If a source is found to violate the agreed data rate over any monitoring
interval, some unspecified action must be taken.
.pp
The discipline presented here does not use a monitoring interval;
it only performs buffer admission control when the buffer is full.
Also, this discipline makes explicit the action taken when a
buffer fills.
It discards the cell with the "most future" virtual timestamp.
This control penalizes the virtual circuit that deserves it the
most, but it still guarantees that penalized virtual circuits
receive
.ul
at least
their requested bandwidth allocation.
This buffer control strategy thus eliminates the "flooding" problem
inherent in Zhang's Virtual Clock discipline.
With Zhang's discipline, one source flooding a single buffer can
prevent other sources from becoming enqueued.
With this discipline, a misbehaving source's cells receive timestamps
much larger than the current virtual time.
Thus, arrivals from other sources
can bump out cells from the misbehaving source.
Every source is
.ul
guaranteed
service at its average rate
.ul
without loss
assuming:
.ti 5
(1) the buffer size is at least as large as the number of sources
.ti 5
(2) the buffer output rate is at least as large as the sum of the
source input rates
.pp
This discipline ends up being a flavor of Earliest Due Date
scheduling with
.ti 5
(1) a specific algorithm for picking due dates at each buffer
.ti 5
(2) a specific cell discard mechanism that ensures loss fairness
.UH Reference
.ip [1]
Lixia Zhang, "VirtualClock: A New Traffic Control Algorithm for
Packet Switching Networks," \fISIGCOMM'90 Symposium\fR,
Sept. 1990, Philadelphia, \fIComputer Communications Review\fR,
vol. 20:4, Sept. 1990.
.ES
