static const char file_id[] = "DECellToImage.pl";
// .cc file generated from DECellToImage.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "DECellToImage.h"

const char *star_nm_DECellToImage = "DECellToImage";

const char* DECellToImage :: className() const {return star_nm_DECellToImage;}

ISA_FUNC(DECellToImage,DEStar);

Block* DECellToImage :: makeNew() const { LOG_NEW; return new DECellToImage;}

DECellToImage::DECellToImage ()
{
	setDescriptor("This star reads in NetworkCells containing image data and outputs whole\nimages. The current image is sent to the output when the star reads\nimage data with a higher frame id than the current image.\nFor each frame, the fraction of input data that was lost is sent to the\n\"lossPct\" output.");
	addPort(input.setPort("input",this,MESSAGE));
	addPort(output.setPort("output",this,MESSAGE));
	addPort(lossPct.setPort("lossPct",this,FLOAT));

# line 46 "DECellToImage.pl"
activeImage = 0;
		firstNC = 0;
}

DECellToImage::~DECellToImage() {
# line 51 "DECellToImage.pl"
delete firstNC;
}

void DECellToImage::setup() {
# line 55 "DECellToImage.pl"
Envelope nullEnvp;
		activeEnvp = nullEnvp;
		activeImage = 0;

		delete firstNC;
		firstNC = 0;
}

void DECellToImage::go() {
# line 64 "DECellToImage.pl"
completionTime = arrivalTime;

		// Read input image.
		Envelope inEnvp;
		input.get().getMessage(inEnvp);
		TYPE_CHECK(inEnvp, "NetworkCell");

		if (!activeImage) {
			// Here, activeEnvp is set to contain a WRITABLE
			// copy of the incoming BaseImage.
			NetworkCell* tmpNC =
				(NetworkCell*) inEnvp.writableCopy();
			if (!tmpNC->contentsType("BaseImage")) {
				LOG_DEL; delete tmpNC;
				Error::abortRun(*this, "Cell doesn't hold BaseImage.");
				return;
			}
			firstNC = tmpNC;	    // keep for later deletion
			activeImage = (BaseImage*) tmpNC->writableData();
			Envelope tmpE(*activeImage);
			activeEnvp = tmpE;
			actual = activeImage->retSize();

		} else { // If here, we have already read one envelope.
			NetworkCell* tmpNC =
				(NetworkCell*) inEnvp.writableCopy();
			if (!tmpNC->contentsType("BaseImage")) {
				LOG_DEL; delete tmpNC;
				Error::abortRun(*this,
						"Cell doesn't hold BaseImage.");
				return;
			}
			BaseImage* newPtr = (BaseImage*) tmpNC->writableData();

			if (*activeImage == *newPtr) { // same frame id's...
				activeImage->assemble(newPtr);
				actual += newPtr->retSize();
				LOG_DEL; delete newPtr;
			} else {	// Send old image to output port.
				float total = activeImage->retFullSize();
				Envelope outEnvp(*activeImage);
				output.put(completionTime) << outEnvp;
				if (total > 0.0)
					lossPct.put(completionTime) << (1.0-(actual/total));
				else
					lossPct.put(completionTime) << 0.0;

				// Set activeEnvp to contain a writable copy
				// of the new BaseImage.
				activeImage = newPtr;
				Envelope tmpE(*activeImage); activeEnvp = tmpE;
				actual = activeImage->retSize();
			}
			LOG_DEL; delete tmpNC; // we "own" it, must clean it up!
		}
}

// prototype instance for known block list
static DECellToImage proto;
static RegisterBlock registerBlock(proto,"CellToImage");
