static const char file_id[] = "DEPCMVoiceRecover.pl";
// .cc file generated from DEPCMVoiceRecover.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "DEPCMVoiceRecover.h"

const char *star_nm_DEPCMVoiceRecover = "DEPCMVoiceRecover";

const char* DEPCMVoiceRecover :: className() const {return star_nm_DEPCMVoiceRecover;}

ISA_FUNC(DEPCMVoiceRecover,DEStar);

Block* DEPCMVoiceRecover :: makeNew() const { LOG_NEW; return new DEPCMVoiceRecover;}

DEPCMVoiceRecover::DEPCMVoiceRecover ()
{
	setDescriptor("This star inputs a stream of\n.c SeqATMCell\nobjects.  All of the\ninformation bits in the objects received with correct sequence\nnumbers are sent to 'output'.\n\nIf a missing\n.c SeqATMCell\nobject is detected, then this star sends the most\nrecent 8 * 'tempSize' received bits to the 'temp' output, and the\nmost recent (8 * 'searchWindowSize' + 'numInfoBits') received bits\nto the 'window' output.\n\nThe bits output on the 'window' and 'temp' outputs can be used by\nthe 'PatternMatch' galaxy to implement lost-speech recovery.");
	addState(numInfoBits.setState("numInfoBits",this,"384","number of information bits per cell <= 384"));
	addState(searchWindowSize.setState("searchWindowSize",this,"176","number of samples <= 4*numInfoBits/8\n			and >= 3*numInfoBits/8 with which to\n			search and retrieve match in SDF domain"));
	addState(tempSize.setState("tempSize",this,"32","number of samples <= 48 which act as a guide\n			in determining missing packet samples"));
	addPort(input.setPort("input",this,MESSAGE));
	addPort(subIn.setPort("subIn",this,MESSAGE));
	addPort(temp.setPort("temp",this,INT));
	addPort(window.setPort("window",this,INT));
	addPort(output.setPort("output",this,INT));

# line 136 "DEPCMVoiceRecover.pl"
pkt1 = pkt2 = pkt3 = pkt4 = pkt5 = currentPkt = created = 0;
}

DEPCMVoiceRecover::~DEPCMVoiceRecover() {
# line 328 "DEPCMVoiceRecover.pl"
LOG_DEL; delete pkt5;
		LOG_DEL; delete pkt4;
		LOG_DEL; delete pkt3;
		LOG_DEL; delete pkt2;
		LOG_DEL; delete pkt1;
		LOG_DEL; delete currentPkt;
		LOG_DEL; delete created;
}

void DEPCMVoiceRecover::setup() {
# line 140 "DEPCMVoiceRecover.pl"
if ( numInfoBits > 384 )
			Error::abortRun( *this, "illegal SeqATMCell size" );
		if ( searchWindowSize > 4*int(numInfoBits)/8 ||
				searchWindowSize < 3*int(numInfoBits)/8 )
			Error::abortRun( *this, "illegal searchWindowSize" );
		if ( tempSize > 48 )
			Error::abortRun( *this, "illegal tempSize" );
		count = missingPkt = 0;
		int samples = numInfoBits / 8;
		startTemp = ( samples - tempSize + 5 ) * 8;
		stopTemp = ( samples + 5 ) * 8 - 1;
		int pkt5Samples = int( searchWindowSize ) - 3 *
				int( numInfoBits ) / 8;
		startPkt5 = ( samples - pkt5Samples + 5 ) * 8;
		stopPkt5 = ( samples + 5 ) * 8 - 1;
		pkt1 = pkt2 = pkt3 = pkt4 = pkt5 = currentPkt = created = 0;
		subIn.before( input );
}

void DEPCMVoiceRecover::go() {
# line 160 "DEPCMVoiceRecover.pl"
if ( input.dataNew ) {
			int i, j;
			Envelope inPkt;
			input.get().getMessage( inPkt );
			TYPE_CHECK( inPkt, "SeqATMCell" );
			currentPkt = ( SeqATMCell* ) inPkt.writableCopy();

			// if proper order, output packet
			if ( ( currentPkt->readSeq() ) == count ) {
				for ( i = 40; i < int ( 40 + numInfoBits ); i++ ) {
					if ( currentPkt->isON( i ) )
						output.put( arrivalTime ) << 1;
					else
						output.put( arrivalTime ) << 0;
				}
				count++;
				count %= 8;
				if ( pkt5 != pkt4 ) {
					LOG_DEL; delete pkt5;
				}
				pkt5 = pkt4;
				pkt4 = pkt3;
				pkt3 = pkt2;
				pkt2 = pkt1;
				pkt1 = currentPkt;
			} // end proper order

			// else if a packet is missing
			else {
				while ( pkt5 == 0 && count !=
						( currentPkt->readSeq() ) ) {
					for ( i = 0; i < int( numInfoBits ); i++ )
						output.put( arrivalTime ) << 0;
					count++;
					count %= 8;
					if ( pkt5 != pkt4 ) {
						delete pkt5;
					}
					pkt5 = pkt4;
					pkt4 = pkt3;
					pkt3 = pkt2;
					pkt2 = pkt1;
					pkt1 = 0;
					LOG_NEW; pkt1 = new SeqATMCell;
				} // end while

				// at this point there may still be missing packets
				// 1) either we were never at the beginning of
				// transmission (pkt5 != 0) or 2) pkt5 became
				// nonzero in the above loop;
				// now calculate how many are still missing
				while ( ( currentPkt->readSeq() ) != count ) {
					missingPkt++;
					count++;
					count %= 8;
				}

				// if not missing any more, output currentPkt
				if ( !missingPkt ) {
					for ( i = 40; i < int ( 40 + numInfoBits ); i++ ) {
						if ( currentPkt->isON( i ) )
							output.put( arrivalTime ) << 1;
						else
							output.put( arrivalTime ) << 0;
					}
					count++;
					count %= 8;
					if ( pkt5 != pkt4 ) {
						LOG_DEL; delete pkt5;
					}
					pkt5 = pkt4;
					pkt4 = pkt3;
					pkt3 = pkt2;
					pkt2 = pkt1;
					pkt1 = currentPkt;
				} // end if not missing

				else if ( missingPkt ) {
					// output bits to do pattern matching
					// first output temp bits;
					for ( j = startTemp; j <= stopTemp; j++ ) {
						if ( pkt1->isON( j ) )
							temp.put( arrivalTime ) << 1;
						else
							temp.put( arrivalTime ) << 0;
					}
					// now output search window
					for ( j = startPkt5; j <= stopPkt5; j++ ) {
						if ( pkt5->isON( j ) )
							window.put( arrivalTime ) << 1;
						else
							window.put( arrivalTime ) << 0;
					}
					for ( j = 40; j < numInfoBits + 40; j++ ) {
						if ( pkt4->isON( j ) )
							window.put( arrivalTime ) << 1;
						else
							window.put( arrivalTime ) << 0;
					}
					for ( j = 40; j < numInfoBits + 40; j++ ) {
						if ( pkt3->isON( j ) )
							window.put( arrivalTime ) << 1;
						else
							window.put( arrivalTime ) << 0;
					}
					for ( j = 40; j < numInfoBits + 40; j++ ) {
						if ( pkt2->isON( j ) )
							window.put( arrivalTime ) << 1;
						else
							window.put( arrivalTime ) << 0;
					}
					for ( j = 40; j < numInfoBits + 40; j++ ) {
						if ( pkt1->isON( j ) )
							window.put( arrivalTime ) << 1;
						else
							window.put( arrivalTime ) << 0;
					}
				} // end else if missingPkt
			} // end else if missing packets
		} // end if input.dataNew

		if ( subIn.dataNew ) {
			int k, n;
			Envelope subPkt;
			subIn.get().getMessage( subPkt );
			TYPE_CHECK( subPkt, "SeqATMCell" );
			created = ( SeqATMCell* ) subPkt.writableCopy();
			// output packet missingPkt times
			for ( n = 1; n <= missingPkt; n++ ) {
				for ( k = 40; k < int ( 40 + numInfoBits ); k++ ) {
					if ( created->isON( k ) )
						output.put( arrivalTime ) << 1;
					else
						output.put( arrivalTime ) << 0;
				}
				if ( pkt5 != pkt4 ) {
					LOG_DEL; delete pkt5;
				}
				pkt5 = pkt4;
				pkt4 = pkt3;
				pkt3 = pkt2;
				pkt2 = pkt1;
				pkt1 = created;
			}

			// output current packet
			for ( k = 40; k < int ( 40 + numInfoBits ); k++ ) {
				if ( currentPkt->isON( k ) )
					output.put( arrivalTime ) << 1;
				else
					output.put( arrivalTime ) << 0;
			}

			count++;
			count %= 8;
			missingPkt = 0;
			if ( pkt5 != pkt4 ) {
				LOG_DEL; delete pkt5;
			}
			pkt5 = pkt4;
			pkt4 = pkt3;
			pkt3 = pkt2;
			pkt2 = pkt1;
			pkt1 = currentPkt;
		} // end if subIn.dataNew
}

// prototype instance for known block list
static DEPCMVoiceRecover proto;
static RegisterBlock registerBlock(proto,"PCMVoiceRecover");
