static const char file_id[] = "DEPSServer.pl";
// .cc file generated from DEPSServer.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "DEPSServer.h"

const char *star_nm_DEPSServer = "DEPSServer";

const char* DEPSServer :: className() const {return star_nm_DEPSServer;}

ISA_FUNC(DEPSServer,DERepeatStar);

Block* DEPSServer :: makeNew() const { LOG_NEW; return new DEPSServer;}

# line 35 "DEPSServer.pl"
struct token {
		   Particle* pp;
		   double serviceNeeded;
		   double lastUpdate;
		    token() : pp(0) {}
		   ~token() { if (pp) pp->die();}
		};

DEPSServer::DEPSServer ()
{
	setDescriptor("This star emulates a deterministic, processor-sharing server.\nIf input events arrive when it is not busy, it delays them by the nominal service time.\nIf they arrive when it is busy, the server is shared. Hence prior arrivals that are\nstill in service will be delayed by more than the nominal service time.");
	addPort(input.setPort("input",this,ANYTYPE));
	addPort(output.setPort("output",this,ANYTYPE));
	output.inheritTypeFrom(input);
	addState(nomServiceTime.setState("nomServiceTime",this,"1.0","\"Nominal service time\""));

# line 49 "DEPSServer.pl"
delayType = TRUE;
		numberInService = 0;
}

DEPSServer::~DEPSServer() {
# line 62 "DEPSServer.pl"
// Remove any remaining tokens
		while (numberInService > 0) {
			token* t = (token*)tokensInService.getAndRemove();
			LOG_DEL; delete t;
			numberInService--;
		}
}

void DEPSServer::setup() {
# line 53 "DEPSServer.pl"
// Remove any remaining tokens
		while (numberInService > 0) {
			token* t = (token*)tokensInService.getAndRemove();
			LOG_DEL; delete t;
			numberInService--;
		}
		tokensInService.initialize();
}

void DEPSServer::go() {
# line 70 "DEPSServer.pl"
completionTime = arrivalTime;

	   // Check to see whether the serviceNeeded of the first token
	   // in the list goes to zero, and if so, output the particle.
	   // Keep trying as long as the first particle in the list is ready
	   // to be output.
	   int outputP = FALSE;
	   while(numberInService > 0) {
	      token *t = (token*)(tokensInService.head());
	      if (t->serviceNeeded
	              <= (arrivalTime - t->lastUpdate)/numberInService) {
		// This token is ready to be output
		output.put(completionTime) = *(t->pp);
		tokensInService.getAndRemove();
		LOG_DEL; delete t;
		numberInService--;
		outputP = TRUE;
	      } else {
		break;
	      }
	   }

	   // Check for false alarm, and return if nothing has changed
	   if((!input.dataNew) & (!outputP)) return;

	   // Modify the serviceNeeded for all tokensInService.
	   ListIter nextTok(tokensInService);
	   for(int i = numberInService; i > 0; i--) {
		token *t = (token*) nextTok++;
		t->serviceNeeded = t->serviceNeeded -
			(arrivalTime - t->lastUpdate)/numberInService;
		t->lastUpdate = arrivalTime;
		// Schedule a refiring.
		// Need to account for number that will be in
		// service in the next interval.
		int nISNext = numberInService;
		if ( input.dataNew ) nISNext++;
		refireAtTime(arrivalTime
				+ (t->serviceNeeded)*nISNext);

	   }

	   // Now check for new input data
	   if ( input.dataNew ) {
		// Create a token.
		Particle& pp = input.get();
		Particle* newp = pp.clone();
		LOG_NEW; token *t = new token;
		t->pp = newp;
		t->serviceNeeded = double(nomServiceTime);
		t->lastUpdate = arrivalTime;

		// Append new token to the list of tokensInService
		tokensInService.append(t);
		numberInService++;

		// Schedule a firing at the current estimate of
		// the completion time of the service.
		refireAtTime(arrivalTime + (t->serviceNeeded)*numberInService);
	   }
}

// prototype instance for known block list
static DEPSServer proto;
static RegisterBlock registerBlock(proto,"PSServer");
