static const char file_id[] = "DEPriorityQueue.pl";
// .cc file generated from DEPriorityQueue.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "DEPriorityQueue.h"

const char *star_nm_DEPriorityQueue = "DEPriorityQueue";

const char* DEPriorityQueue :: className() const {return star_nm_DEPriorityQueue;}

ISA_FUNC(DEPriorityQueue,DEStar);

Block* DEPriorityQueue :: makeNew() const { LOG_NEW; return new DEPriorityQueue;}

DEPriorityQueue::DEPriorityQueue ()
{
	setDescriptor("A priority queue.\nInputs have priorities according to the number of the input, with\n\"inData#1\" having highest priority, \"inData#2\" having the second\nhighest priority, etc.\nWhen a \"demand\" is received, outputs are produced by selecting first\nbased on priority, and then based on the time of arrival, using a\nfirst-in first-out (FIFO) policy.\nA finite total capacity of the queue can be specified by setting the\n\"capacity\" parameter to a positive integer.\nWhen the capacity has been reached, further inputs are sent to the \"overflow\"\noutput and are not stored.\nThe \"numDemandsPending\" and \"consolidateDemands\" parameters have the same\nmeaning as in other Queue stars.\nThe size of the queue is sent to the \"size\" output whenever an \"inData\"\nor \"demand\" event is processed.");
	addPort(demand.setPort("demand",this,ANYTYPE));
	addPort(inData.setPort("inData",this,ANYTYPE));
	addPort(outData.setPort("outData",this,ANYTYPE));
	outData.inheritTypeFrom(inData);
	addPort(size.setPort("size",this,INT));
	addPort(overflow.setPort("overflow",this,ANYTYPE));
	overflow.inheritTypeFrom(inData);
	addState(capacity.setState("capacity",this,"-1","Maximum total number of particles. If <0, capacity is infinite."));
	addState(numDemandsPending.setState("numDemandsPending",this,"1","Number of demands that have not yet stimulated outputs."));
	addState(consolidateDemands.setState("consolidateDemands",this,"TRUE","If FALSE, every demand eventually stimulates an output."));

# line 144 "DEPriorityQueue.pl"
// Hint to the scheduler that both demand and inData trigger
		// size outputs.  And demand triggers an output.
		inData.triggers(size);
		demand.triggers(size);
		demand.triggers(outData);

		// Hint to the scheduler that if there is any chance of
		// simultaneous events on inData and demand, then the inData
		// events should be made available.
		inData.before(demand);
}

DEPriorityQueue::~DEPriorityQueue() {
# line 160 "DEPriorityQueue.pl"
emptyQueue();
}

void DEPriorityQueue::setup() {
# line 156 "DEPriorityQueue.pl"
emptyQueue();
		infinite = (int(capacity) < 0);
}

void DEPriorityQueue::go() {
# line 163 "DEPriorityQueue.pl"
completionTime = arrivalTime;

	    while (demand.dataNew) {
		if (int(consolidateDemands))
			numDemandsPending = 1;
		else
			numDemandsPending = int(numDemandsPending) + 1;
		// get any pending simultaneous demand inputs.
		demand.getSimulEvent();
	    }

	    // Iterate over the inputs, collecting new input data
	    // All input data should be collected before any outputs are
	    // produced so that priorities of the inputs will be observed.
	    // Note that the queue size limitation will not be observed at
	    // this point.  Only at the end of the firing will the queue
	    // be truncated to capacity.
	    InDEMPHIter nextp(inData);
	    for(int i=0;i<inData.numberPorts(); i++) {
		InDEPort* p = nextp++;
		while (p->dataNew) {
		    queue.levelput(p->get().clone(),i);
		    p->getSimulEvent();
		}
	    }

	    // If the queue has data, and one or more demands are pending,
	    // produce output data.  Higher priority data will be automatically
	    // sent out before lower priority data, because of the order in
	    // which it was enqueued.
	    while (int(numDemandsPending) > 0 && queue.length() > 0 ) {
		Particle* pp = (Particle*) queue.getFirstElem();
		outData.put(completionTime) = *pp;
		pp->die();
		numDemandsPending = int(numDemandsPending) - 1;
	    }

	    // Cut queue down to size by removing lowest priority entries.
	    // Entries removed are sent to the "overflow" output.
	    if (!infinite)
		while (queue.length() > int(capacity)) {
		    Particle* junkp = (Particle*) queue.getLastElem();
		    overflow.put(completionTime) = *junkp;
		    junkp->die();
		}

	    // output the queue size
	    size.put(completionTime) << queue.length();
}


void DEPriorityQueue::emptyQueue ()
{
# line 217 "DEPriorityQueue.pl"
while (queue.length() > 0) {
				Particle* p = (Particle*) queue.getFirstElem();
				p->die();
			}
}


// prototype instance for known block list
static DEPriorityQueue proto;
static RegisterBlock registerBlock(proto,"PriorityQueue");
