static const char file_id[] = "DEQueue.pl";
// .cc file generated from DEQueue.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "DEQueue.h"
#include "DataStruct.h"

const char *star_nm_DEQueue = "DEQueue";

const char* DEQueue :: className() const {return star_nm_DEQueue;}

ISA_FUNC(DEQueue,DEStar);

Block* DEQueue :: makeNew() const { LOG_NEW; return new DEQueue;}

DEQueue::DEQueue ()
{
	setDescriptor("THIS STAR IS OBSOLETE.  USE FIFOQueue.\nFIFO queue with finite or infinite length.\nEvents on the \"demand\" input trigger \"outData\" if the Queue is not empty.\nIf the queue is empty, then a \"demand\" event enables the next future\n\"inData\" particle to pass immediately to \"outData\".\nThe first particle to arrive at \"inData\" is always passed directly\nto the output, unless \"numDemandsPending\" is initialized to 0.\nIf \"consolidateDemands\" is set to TRUE (the default), then \"numDemandsPending\"\nis not permitted to rise above unity.\nThe size of the queue is sent to the \"size\" output whenever an \"inData\"\nor \"demand\" event is processed.");
	addPort(demand.setPort("demand",this,ANYTYPE));
	addPort(inData.setPort("inData",this,ANYTYPE));
	addPort(outData.setPort("outData",this,ANYTYPE));
	outData.inheritTypeFrom(inData);
	addPort(size.setPort("size",this,INT));
	addState(capacity.setState("capacity",this,"-1","Maximum size of the queue. If <0, capacity is infinite."));
	addState(numDemandsPending.setState("numDemandsPending",this,"1","Number of demands that have not yet stimulated outputs."));
	addState(consolidateDemands.setState("consolidateDemands",this,"TRUE","If FALSE, every demand eventually stimulates an output."));

# line 140 "DEQueue.pl"
// Indicate to the scheduler that size outputs are triggered
		// by inData or demand inputs.
		inData.triggers(size);
		demand.triggers(size);

		// simultaneous inData events should be available in the
		// same firing with demand events
		inData.before(demand);

		demand.triggers(outData);
		// Note that although demand triggers outData if the Queue
		// has data, it does not trigger an immediate output if the
		// Queue is empty.  Hence, if the Queue is empty, there is
		// a delay in this path.  Although this might imply we do not
		// want to declare that demand triggers outData, actually, we
		// do.  Consider for instance a star feeding data to both
		// the demand input of the Queue and a downstream star that
		// also gets data from the demand input of the star. We want
		// the Queue to fire before the downstream star. Delay free
		// loops involving a path through demand to outData are fine,
		// but an infinitessimal delay should be put in the feedback
		// path.
}

void DEQueue::setup() {
# line 164 "DEQueue.pl"
infinite = (int(capacity) < 0);
		queue.initialize();
}

void DEQueue::go() {
# line 169 "DEQueue.pl"
completionTime = arrivalTime;

	    while (demand.dataNew) {
		if (int(consolidateDemands))
			numDemandsPending = 1;
		else
			numDemandsPending = int(numDemandsPending) + 1;
		// get any pending simultaneous demand inputs.
		demand.getSimulEvent();
	    }

	    // If the queue has data, and one or more demands are pending,
	    // produce output data
	    while (int(numDemandsPending) > 0 && queue.length() > 0 ) {
		Particle* pp = (Particle*) queue.get();
		outData.put(completionTime) = *pp;
		pp->die();
		numDemandsPending = int(numDemandsPending) - 1;
	    }

	    // If there is new inData, and the queue is not full, store it
	    while (inData.dataNew) {
		if (queue.length() == 0 && int(numDemandsPending) > 0) {
		    outData.put(completionTime) = inData.get();
		    numDemandsPending = int(numDemandsPending) - 1;
		} else if (infinite || queue.length() < int(capacity)) {
		    queue.put(inData.get().clone());
		}
		inData.getSimulEvent();
	    }

	    // output the queue size
	    size.put(completionTime) << int(queue.length());
}

// prototype instance for known block list
static DEQueue proto;
static RegisterBlock registerBlock(proto,"Queue");
