static const char file_id[] = "DEVirtClock.pl";
// .cc file generated from DEVirtClock.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "DEVirtClock.h"
#include "Error.h"
#include <minmax.h>

const char *star_nm_DEVirtClock = "DEVirtClock";

const char* DEVirtClock :: className() const {return star_nm_DEVirtClock;}

ISA_FUNC(DEVirtClock,DEStar);

Block* DEVirtClock :: makeNew() const { LOG_NEW; return new DEVirtClock;}

DEVirtClock::DEVirtClock ()
{
	setDescriptor("This star reads in a NetworkCell. It identifies which virtual circuit\nnumber the cell belongs to and then computes the virtual time stamp for\nthe cell by applying the (special!) virtual clock algorithm (see the full\nmanual page). It then outputs all cells in order of increasing\nvirtual time stamp.\n\nUpon receiving a 'demand' input, the cell with the smallest timestamp is\noutput. An output packet is generated for every demand input unless\nall of the queues are empty. Demand inputs arriving when all queues\nare empty are ignored.\n\nThe number of stored cells is output after the receipt of each 'input'\nor 'demand'.\n\nWhen a cell arrives and the number of stored cells equals 'MaxSize'\nthen the cell with the biggest virtual timestamp is discarded. This\ncell may or may not be the new arrival. If 'MaxSize' is zero or\nnegative, then infinitely many cells can be stored.");
	addPort(input.setPort("input",this,MESSAGE));
	addPort(demand.setPort("demand",this,ANYTYPE));
	addPort(output.setPort("output",this,MESSAGE));
	addPort(discard.setPort("discard",this,MESSAGE));
	addPort(length.setPort("length",this,INT));
	addState(MaxSize.setState("MaxSize",this,"50","Maximum number of stored cells."));
	addState(AvgRate.setState("AvgRate",this,"1.0 1.0","Average rate for each input virtual circuit."));

# line 202 "DEVirtClock.pl"
delayType = TRUE;

		// initialize variables
		numStreams = 0;
		vTime = 0.0;

		// initialize pointers
		vTimeStamp = 0;
		cellCount = 0;
		priList = 0;
		queues = 0;

		input.before(demand);
		input.triggers(discard);
		demand.triggers(output);
		demand.triggers(length);
}

DEVirtClock::~DEVirtClock() {
# line 254 "DEVirtClock.pl"
LOG_DEL; delete [] vTimeStamp;
		LOG_DEL; delete [] cellCount;

		if (priList) {
			while(priList->length()) {
				LOG_DEL; delete (Tag*) priList->getFirstElem();
			}
			LOG_DEL; delete priList;
		}

		if (queues) {
			for(int i = 0; i < numStreams; i++) {
				while((queues[i]).size()) {
					LOG_DEL; delete (BufCell*) (queues[i]).get();
			}	}
			LOG_DEL; delete [] queues;
		}
}

void DEVirtClock::setup() {
# line 221 "DEVirtClock.pl"
if (priList) {
			while(priList->length()) {
				LOG_DEL; delete (Tag*) priList->getFirstElem();
			}
			LOG_DEL; delete priList;
		}

		if (queues) {
			for(int i = 0; i < numStreams; i++) {
				while((queues[i]).size()) {
					LOG_DEL; delete (BufCell*) (queues[i]).get();
			}	}
			LOG_DEL; delete [] queues;
		}

		LOG_DEL; delete [] vTimeStamp;
		LOG_DEL; delete [] cellCount;

		vTime = 0.0;
		numStreams = AvgRate.size();

		LOG_NEW; priList = new PriorityQueue;
		LOG_NEW; queues = new Queue[numStreams];

		LOG_NEW; vTimeStamp = new float[numStreams];
		LOG_NEW; cellCount = new int[numStreams];
		for (int i = 0; i < numStreams; i++) {
			vTimeStamp[i] = 0.0;
			cellCount[i] = 0;
		}
}

void DEVirtClock::go() {
# line 291 "DEVirtClock.pl"
completionTime = arrivalTime;

// Handle inputs: for each input...
		InDEMPHIter nextInp(input);
		InDEPort* p;
		while ( (p = nextInp++) ) {
			while (p->dataNew) {

// Get the input Message.
				Envelope envp;
				p->get().getMessage(envp);

// Check its type and get its destination address.
				TYPE_CHECK(envp, "NetworkCell");
				const NetworkCell* ncp = (const NetworkCell*)
						envp.myData();
				const int destAddr = ncp->dest();
				if ((destAddr < 0) || (destAddr >= AvgRate.size())) {
					Error::abortRun(*this, "Illegal dest. addr.");
					return;
				}

// Timestamp for this arrival is the more recent of "current virtual
// time" or timestamp of last arrival + interarrival time.
				vTimeStamp[destAddr] = max(vTime,
						float(vTimeStamp[destAddr] +
						1.0/AvgRate[destAddr]) );

// BUFFER SERVICE CONTROL:
// Put new arrival into the two appropriate queues.
				LOG_NEW;
				Tag* tag = new Tag(destAddr, vTimeStamp[destAddr]);
				priList->levelput(tag, vTimeStamp[destAddr]);

				LOG_NEW;
				BufCell* cell = new
						BufCell(cellCount[destAddr]++, envp);
				(queues[destAddr]).put(cell);

// BUFFER ACCESS CONTROL:
// If global queue is too long, find the source "most in violation."
// That is, the source with the virtual timestamp most in the future.
// Delete some element from that queue (with the LoseOne() fcn). Modify
// the vTimeStamp[] entry (for inputs with the same destination address
// as the deleted element) so that they are not delayed unfairly by the
// deleted cell.
				if ((MaxSize > 0) && (priList->length() > MaxSize)) {
					Tag* loss = (Tag*) priList->getLastElem();
					const int la = loss->output();
					LOG_DEL; delete loss;
					vTimeStamp[la] -= 1.0/AvgRate[la];
					LoseOne(queues[la]);
				}
				length.put(completionTime) << priList->length();
				p->getSimulEvent();
		}	} // end handle inputs.

// Handle outputs.
		while (demand.dataNew) {
			if (priList->length() > 0) {
				Tag* outTag = (Tag*) priList->getFirstElem();
				vTime = outTag->vTime();
				BufCell* outCell = (BufCell*)
						(queues[outTag->output()]).get();
				output.put(completionTime) << outCell->envp();
				LOG_DEL; delete outTag;
				LOG_DEL; delete outCell;
			}
			length.put(completionTime) << priList->length();
			demand.getSimulEvent();
		}
}


void DEVirtClock::LoseOne (Queue& queue)
{
# line 282 "DEVirtClock.pl"
if (queue.size() <= 0) { return; }
			BufCell* loss = (BufCell*) queue.getTail();
			discard.put(completionTime) << loss->envp();
			LOG_DEL; delete loss;
}


// prototype instance for known block list
static DEVirtClock proto;
static RegisterBlock registerBlock(proto,"VirtClock");
