#ifndef _DEVirtClock_h
#define _DEVirtClock_h 1
// header file generated from DEVirtClock.pl by ptlang

#ifdef __GNUG__
#pragma interface
#endif

/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */
#include "DEStar.h"
#include "NetworkCell.h"
#include "DataStruct.h"
#include "PriorityQueue.h"
#include "IntState.h"
#include "FloatArrayState.h"
# line 154 "DEVirtClock.pl"
// BufCell instances store an arriving NetworkCell along with the
// number of arrivals on that virtual circuit.
		class BufCell {
		public:
			BufCell(const int i, const Envelope& e): counter(i)
					{ myEnvp = e; }
			Envelope& envp() { return myEnvp; }
			const int count() { return counter; }
		private:
			Envelope myEnvp;
			int counter;
		};

// Tag instances store the virtual circuit i.d. of an input cell and
// the virtual timestamp assigned to that cell.
		class Tag {
		public:
			Tag(const int i, const float f): port(i), tStamp(f) { }
			const int output() { return port; }
			const float vTime() { return tStamp; }
		private:
			int port;
			float tStamp;
		};

class DEVirtClock : public DEStar
{
public:
	DEVirtClock();
	/* virtual */ Block* makeNew() const;
	/* virtual*/ const char* className() const;
	/* virtual*/ int isA(const char*) const;
	/* virtual */ ~DEVirtClock();
	MultiInDEPort input;
	InDEPort demand;
	OutDEPort output;
	OutDEPort discard;
	OutDEPort length;

protected:
	/* virtual */ void setup();
	/* virtual */ void go();
	IntState MaxSize;
	FloatArrayState AvgRate;
# line 181 "DEVirtClock.pl"
// This star implements storage with TWO mechanisms: a sorted priority
// queue that contains 'Tag' instances, and a COLLECTION of FIFO
// queues (one per virtual circuit) that store 'BufCell' instances
// (and thus store input cells). The sorted priority queue is useful
// in that it determines the order in which cells from different
// virtual circuits should be served. The collection of FIFO queues
// are useful in that they simplify the implementation of different
// cell discard disciplines.

// The 'cellCount' variable counts the number of cells on each
// destination address.

		int numStreams; // Number of virtual circuits being served;
		float vTime; // virtual time for the entire buffer.
		float* vTimeStamp; // Next timestamp value for each virt. ckt.
		int* cellCount; // Number of cells so far from each virt. ckt.
		PriorityQueue* priList; // Holds all (sorted) Tag's.
		Queue* queues; // Array of Queue's that store BufCell's.
	virtual void LoseOne (Queue& queue);

};
#endif
