static const char file_id[] = "HOFBase.pl";
// .cc file generated from HOFBase.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "HOFBase.h"
#include "Galaxy.h"
#include "InterpGalaxy.h"
#include "InfString.h"
#include "KnownBlock.h"
#include "ptk.h"
#include "SimControl.h"
#include "HOFNop.h"

const char *star_nm_HOFBase = "HOFBase";

const char* HOFBase :: className() const {return star_nm_HOFBase;}

ISA_FUNC(HOFBase,HOFStar);

Block* HOFBase :: makeNew() const { LOG_NEW; return new HOFBase;}

# line 36 "HOFBase.pl"
int HOFBase::count = 0;

HOFBase::HOFBase ()
{
	setDescriptor("This is a base class containing utility methods shared by HOF stars,\nand defining their common functionality.");


}

void HOFBase::setup() {
# line 46 "HOFBase.pl"
// Since it is so easy to get into an infinite loop
	    // (just name as your block a galaxy that you are within),
	    // process Tk events, and check for a halt request.
	    // Note that this alone prevents the use of this star in ptcl,
	    // which does not have Tk.
	    // FIXME: This should be done some other way.
	    Tk_DoOneEvent(TK_DONT_WAIT|TK_ALL_EVENTS);
	    if (SimControl::haltRequested()) {
		Error::abortRun(*this, "Initialization aborted");
		return;
	    }
}


Galaxy* HOFBase::idParent ()
{
# line 64 "HOFBase.pl"
// Identify the parent galaxy
	        Block *dad = parent();
	        if (!dad || dad->isItAtomic()) {
		    Error::abortRun(*this, "No parent!");
		    return NULL;
	        }
	        return &(dad->asGalaxy());
}


void HOFBase::initConnections (MultiPortHole &mph)
{
# line 85 "HOFBase.pl"
MPHIter next(mph);
	      PortHole *p, *farsideport;
	      Block *farstar;
	      while ( (p = next++) ) {
		if ((farsideport = p->far()) &&
		    (farstar = farsideport->parent()) &&
		    (farstar->isA("HOFNop"))) {
		      ((HOFNop*)farstar)->reconnect();
		    }
	      }
}


Block* HOFBase::createBlock (Galaxy& mom, const char* blockname, const char* where_defined)
{
# line 103 "HOFBase.pl"
// Create the star and install in the galaxy
	        if (!KnownBlock::find(blockname, mom.domain())) {

		    // Failed to find star master.  Try compiling the
		    // facet specified in the where_defined argument
		    InfString command = "ptkOpenFacet ";
		    command += where_defined;
		    if(Tcl_GlobalEval(ptkInterp, (char*)command) == TCL_ERROR) {
			Error::abortRun(*this,
		            "Unable to open the where_defined facet: ",
		            where_defined);
			return NULL;
		    }
		    command = "ptkCompile ";
		    command += ptkInterp->result;
		    if(Tcl_GlobalEval(ptkInterp, (char*)command) == TCL_ERROR) {
			Error::abortRun(*this,
		            "Unable to compile the where_defined facet: ",
		            where_defined);
			return NULL;
		    }
	        }
		// Should now have a master of the star -- clone it
		Block *block = KnownBlock::clone(blockname, mom.domain());
		if (!block) {
		    Error::abortRun(*this,
			"Unable to create an instance of block: ",
			blockname);
		    return NULL;
		}
		// Set the target
		if (target()) block->setTarget(target());

		// Choose a name for the block
	        StringList instancename = "HOF_";
		instancename += blockname;
	        instancename += count++;
	        const char* instance = hashstring(instancename);

		mom.addBlock(*block,instance);
		return block;
}


int HOFBase::connectInput (PortHole* pi, GenericPort* dest)
{
# line 157 "HOFBase.pl"
GenericPort *gp = aliasPointingAt(pi);

		const char* initDelayVals;
		PortHole* farside = pi->far();
		if (!farside) {
		  Error::abortRun(*this,"Star is not fully connected");
		  return 0;
		}
		int numdelays = pi->numInitDelays();
		initDelayVals = pi->initDelayValues();
		farside->disconnect();

		farside->connect(*dest,numdelays,initDelayVals);
		if(parent()->isA("InterpGalaxy") && 
		   ((numdelays > 0) || (initDelayVals && *initDelayVals))) {
		  ((InterpGalaxy*)parent())->registerInit("C",
							  farside->parent()->name(),
							  farside->name(),
							  initDelayVals,
							  dest->parent()->name(),
							  dest->name());
		}
		fixAliases(gp,pi,dest);

		// Since output portholes are responsible for initializing
		// the geodesics, call initialize for the farside port
		farside->initialize();
		return 1;
}


int HOFBase::connectOutput (PortHole* po, GenericPort* source)
{
# line 198 "HOFBase.pl"
GenericPort *gp = aliasPointingAt(po);

		const char* initDelayVals;
		PortHole* farside = po->far();
		if (!farside) {
		  Error::abortRun(*this,"Star is not fully connected");
		  return 0;
		}
		int numdelays = po->numInitDelays();
		initDelayVals = po->initDelayValues();
		farside->disconnect();

		source->connect(*farside,numdelays,initDelayVals);
		if(parent()->isA("InterpGalaxy") && 
		   ((numdelays > 0) || (initDelayVals && *initDelayVals))) {
		  ((InterpGalaxy*)parent())->registerInit("C",
							  source->parent()->name(),
							  source->name(),
							  initDelayVals,
							  farside->parent()->name(),
							  farside->name());
		}
		fixAliases(gp,po,source);
		return 1;
}


GenericPort* HOFBase::aliasPointingAt (PortHole *po)
{
# line 234 "HOFBase.pl"
GenericPort *gp = po->aliasFrom();
	    if (gp == 0) {
	      // Check for a multiporthole alias
	      MultiPortHole *mph = po->getMyMultiPortHole();
	      if (mph != 0) gp = mph->aliasFrom();
	    }
	    return gp;
}


void HOFBase::fixAliases (GenericPort *gp, PortHole* po, GenericPort *source)
{
# line 251 "HOFBase.pl"
// Fix aliases if any
		if (gp != 0) {
		  // The following prevents a later destruction of
		  // the po porthole from undoing the alias fix below.
		  if (po) po->clearAliases();
		  // Note that this can have the effect of making a
		  // MultiPortHole have a PortHole as an alias
		  gp->setAlias(*source);
		}
}


int HOFBase::setParameter (Block* block, int instanceno, const char* parameter_name, const char* parameter_value)
{
# line 272 "HOFBase.pl"
// First step is to determine whether the parameter
	      // name is of the form xxx(n) or xxx(n)=, in which case, it only
	      // applies to this instance if n == instanceno.
	      // Use a regular express to parse the form xxx(n).

	      InfString command =
		"regexp {^ *([^( =]*) *\\( *([0-9]+) *\\) *=? *$} ";
	      command += parameter_name;
	      command += " junk parameter_name instance_index";
	      Tcl_Eval(ptkInterp, (char*)command);

	      // Check for regular expression match
	      if (!strcmp("1",ptkInterp->result)) {

		// The regular expression matched, so we have xxx(n)
		// First convert the ascii instance number to numeric
		// Can safely ignore unexpected Tcl failures,
		// since givenInstNo will end up being zero

		int givenInstNo = 0;
		command = "instance_index";
		char* instance_index =
		  Tcl_GetVar(ptkInterp,command,(int)NULL);
		Tcl_GetInt(ptkInterp, instance_index, &givenInstNo);

		// If the instance numbers don't match,
		// there is nothing more to do.
		if (!(instanceno == givenInstNo)) return 1;

		// If the instance numbers match, then reset
		// parameter_name and proceed with substitutions
		command = "parameter_name";
		parameter_name = Tcl_GetVar(ptkInterp,command,(int)NULL);
		if (!parameter_name) {
		  Error::abortRun(*this,
				  "Unexpected error setting parameter_name!");
		  return 0;
		}
		
	      }

	      // Whether the regular expression matched or not,
	      // replace references to instance number in the
	      // parameter value. Use Tcl to do this, since it has
	      // such convenient string manipulation functions.

	      command = "regsub -all instance_number {";
	      command += parameter_value;
	      command += "} ";
	      command += instanceno;
	      command += " state_value";
	      if(Tcl_Eval(ptkInterp, (char*)command)
		 == TCL_ERROR) {
		Error::abortRun(*this,
				"Substitution of instance number failed:",
				ptkInterp->result);
		return 0;
	      }

	      // Set the state
	      command = "state_value";
	      // The call to hashstring makes this string persistent
	      // Hack alert: This is really a small memory leak.
	      if(!block->setState(parameter_name,
				  hashstring(Tcl_GetVar(ptkInterp,command,(int)NULL)))) { 
		Error::abortRun(*this,
				"Bad name in parameter map: ", parameter_name);
		return 0;
	      }
	      return 1;
}


int HOFBase::restrictConnectivity (Block *block, Star *star)
{
# line 352 "HOFBase.pl"
BlockPortIter bpi(*block);
	    PortHole *p;
	    while ( (p = bpi++) ) {
	      PortHole *far = p->far();
	      if (!far) {
		// This might be a galaxy porthole, which should return something,
		// but doesn't.  Find the corresponding real porthole.
		GenericPort &gp = p->realPort();
		if (gp.isA("PortHole")) p = (PortHole*)(&gp);
		far = p->far();
	      }
	      // Ignore the porthole if far is still NULL,
	      // or if the porthole is hidden
	      if (far && !hidden(*far) && (far->parent() != star)) {
		Error::abortRun(*this,
			    "Sorry, replacement block can only be a single block");
		return FALSE;
	      }
	    }
	    return TRUE;
}


GenericPort* HOFBase::findTopGenericPort (PortHole *ph)
{
# line 384 "HOFBase.pl"
MultiPortHole *mph;
	    GenericPort *gp, *gpt;
	    if ( (mph = ph->getMyMultiPortHole()) ) {
	      gp = mph;
	    } else {
	      gp = ph;
	    }
	    // Get the top-level port that we are connected to

	    while ( (gpt = gp->aliasFrom()) ) {
	      gp = gpt;
	    }
	    // If the multiporthole did not have an aliasFrom
	    // porthole, it could still be that the original
	    // porthole did.
	    if (gp == mph) {
	      gp = ph;
	      while ( (gpt = gp->aliasFrom()) ) {
		gp = gpt;
	      }
	      // If the original porthole also did not have an aliasFrom,
	      // then make sure to return a pointer to its mph.
	      if (gp == ph) gp = mph;
	    }
	    return gp;
}


void HOFBase::connectError ()
{
# line 417 "HOFBase.pl"
Error::abortRun(*this,
			    "Star is not fully connected");
}


GenericPort* HOFBase::breakConnection (PortHole* peo)
{
# line 436 "HOFBase.pl"
PortHole* dest = peo->far();
	    if (dest == 0) {
	      connectError();
	      return NULL;
	    }
	    GenericPort *destgp = findTopGenericPort(dest);
	    if (!myblock) {
	      myblock = destgp->parent();
	      if (!restrictConnectivity(myblock,this)) return NULL;
	    } else {
	      if (myblock != destgp->parent()) {
		Error::abortRun(*this,
				"Sorry, only a single replacement block"
				" is supported at this time.");
		return NULL;
	      }
	    }
	    dest->disconnect();
	    return destgp;
}


// prototype instance for known block list
static HOFBase proto;
static RegisterBlock registerBlock(proto,"Base");
