static const char file_id[] = "HOFBusMerge.pl";
// .cc file generated from HOFBusMerge.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "HOFBusMerge.h"
#include "Galaxy.h"
#include "InterpGalaxy.h"

const char *star_nm_HOFBusMerge = "HOFBusMerge";

const char* HOFBusMerge :: className() const {return star_nm_HOFBusMerge;}

ISA_FUNC(HOFBusMerge,HOFBase);

Block* HOFBusMerge :: makeNew() const { LOG_NEW; return new HOFBusMerge;}

HOFBusMerge::HOFBusMerge ()
{
	setDescriptor("Bridge inputs to outputs and then self-destruct.\nThis star merges two input busses into a single bus.\nIf the input bus widths are M1 and M2, and the output\nbus width is N, then we require that N = M1 + M2.\nThe first M1 outputs come from the first input bus,\nwhile the next M2 outputs come from the second input bus.");
	addPort(output.setPort("output",this,ANYTYPE));
	output.inheritTypeFrom(top);
	addPort(top.setPort("top",this,ANYTYPE));
	addPort(bottom.setPort("bottom",this,ANYTYPE));
	bottom.inheritTypeFrom(top);


}

void HOFBusMerge::setup() {
# line 38 "HOFBusMerge.pl"
MPHIter nexttop(top);
	  MPHIter nextbottom(bottom);
	  // start with the top inputs
	  MPHIter *nexti = &nexttop;
	  MPHIter nexto(output);
	  PortHole *source, *sink;
	  const char *sourceDelayVals, *sinkDelayVals;
	  int numInDelays, numOutDelays;

	  PortHole *pi, *po;
	  while ((po = nexto++) != 0) {
	    if ((pi = (*nexti)++) == 0) {
	      // Out of inputs, switch to the bottom inputs.
	      nexti = &nextbottom;
	      if ((pi = (*nexti)++) == 0) {
		// Still out of inputs
		Error::abortRun(*this,"Not enough inputs for the given number of outputs");
		return;
	      }
	    }
	    if((source = pi->far()) == 0 ||
	       (sink = po->far()) == 0) {
	      Error::abortRun(*this,"Star is not fully connected");
	      return;
	    }
	    sourceDelayVals = pi->initDelayValues();
	    sinkDelayVals = po->initDelayValues();
	    numInDelays = pi->numInitDelays();
	    numOutDelays = po->numInitDelays();

	    // Get alias pointers before disconnecting
	    GenericPort *gpo = aliasPointingAt(po);
	    GenericPort *gpi = aliasPointingAt(pi);

	    source->disconnect();
	    sink->disconnect();

	    int numDelays = 0;
	    const char* delayVals = 0;
	    if ((numInDelays > 0) || (sourceDelayVals && *sourceDelayVals)) {
	      numDelays = numInDelays;
	      delayVals = sourceDelayVals;
	      if ((numOutDelays > 0) || (sinkDelayVals && *sinkDelayVals)) {
		Error::warn(*this,
			    "Cannot have delays on inputs and outputs."
			    " Using input value");
	      }
	    } else {
	      numDelays = numOutDelays;
	      delayVals = sinkDelayVals;
	    }
	    source->connect(*sink,numDelays,delayVals);
	    // Register the connection with the parent galaxy so that delays get
	    // initialized when the galaxy is reinitialized.
	    if(parent()->isA("InterpGalaxy") && 
	       ((numDelays > 0) || (delayVals && *delayVals))) {
	      ((InterpGalaxy*)parent())->registerInit("C",
						      source->parent()->name(),
						      source->name(),
						      delayVals,
						      sink->parent()->name(),
						      sink->name());
	    }
	    fixAliases(gpi,pi,sink);
	    fixAliases(gpo,po,source);

	    source->initialize();
	    sink->initialize();
	  }                        // while loop
	  // Out of outputs at this point.
	  // Check to be sure we are also out of inputs.
	  if ((nextbottom++ != 0) || (nexttop++ != 0)) {
	    Error::abortRun(*this,"Not enough outputs for the given number of inputs");
	    return;
	  }
	  // Now remove ourselves from the parent galaxy and self-destruct
	  Galaxy* mom = idParent();
	  if(!mom) return;
	  mom->deleteBlockAfterInit(*this);
}

// prototype instance for known block list
static HOFBusMerge proto;
static RegisterBlock registerBlock(proto,"BusMerge");
