static const char file_id[] = "HOFBusSplit.pl";
// .cc file generated from HOFBusSplit.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "HOFBusSplit.h"
#include "Galaxy.h"
#include "InterpGalaxy.h"

const char *star_nm_HOFBusSplit = "HOFBusSplit";

const char* HOFBusSplit :: className() const {return star_nm_HOFBusSplit;}

ISA_FUNC(HOFBusSplit,HOFBase);

Block* HOFBusSplit :: makeNew() const { LOG_NEW; return new HOFBusSplit;}

HOFBusSplit::HOFBusSplit ()
{
	setDescriptor("Bridge inputs to outputs and then self-destruct.\nThis star splits an input bus into two.\nIf the input bus width is N, and the output bus widths\nare M1 and M2, then we require that N = M1 + M2.\nThe first M1 inputs go the first output bus, while\nthe next M2 inputs go to the second output bus.");
	addPort(top.setPort("top",this,ANYTYPE));
	top.inheritTypeFrom(input);
	addPort(bottom.setPort("bottom",this,ANYTYPE));
	bottom.inheritTypeFrom(input);
	addPort(input.setPort("input",this,ANYTYPE));


}

void HOFBusSplit::setup() {
# line 38 "HOFBusSplit.pl"
MPHIter nexttop(top);
	  MPHIter nextbottom(bottom);
	  // start with the top inputs
	  MPHIter *nexto = &nexttop;
	  MPHIter nexti(input);
	  PortHole *source, *sink;
	  const char *sourceDelayVals, *sinkDelayVals;
	  int numInDelays, numOutDelays;

	  PortHole *pi, *po;
	  while ((pi = nexti++) != 0) {
	    if ((po = (*nexto)++) == 0) {
	      // Out of outputs, switch to the bottom outputs.
	      nexto = &nextbottom;
	      if ((po = (*nexto)++) == 0) {
		// Still out of outputs
		Error::abortRun(*this,"Not enough outputs for the given number of inputs");
		return;
	      }
	    }
	    if((source = pi->far()) == 0 ||
	       (sink = po->far()) == 0) {
	      Error::abortRun(*this,"Star is not fully connected");
	      return;
	    }
	    sourceDelayVals = pi->initDelayValues();
	    sinkDelayVals = po->initDelayValues();
	    numInDelays = pi->numInitDelays();
	    numOutDelays = po->numInitDelays();

	    // Get alias pointers before disconnecting
	    GenericPort *gpo = aliasPointingAt(po);
	    GenericPort *gpi = aliasPointingAt(pi);

	    source->disconnect();
	    sink->disconnect();

	    int numDelays = 0;
	    const char* delayVals = 0;
	    if ((numInDelays > 0) || (sourceDelayVals && *sourceDelayVals)) {
	      numDelays = numInDelays;
	      delayVals = sourceDelayVals;
	      if ((numOutDelays > 0) || (sinkDelayVals && *sinkDelayVals)) {
		Error::warn(*this,
			    "Cannot have delays on inputs and outputs."
			    " Using input value");
	      }
	    } else {
	      numDelays = numOutDelays;
	      delayVals = sinkDelayVals;
	    }
	    source->connect(*sink,numDelays,delayVals);
	    // Register the connection with the parent galaxy so that delays get
	    // initialized when the galaxy is reinitialized.
	    if(parent()->isA("InterpGalaxy") && 
	       ((numDelays > 0) || (delayVals && *delayVals))) {
	      ((InterpGalaxy*)parent())->registerInit("C",
						      source->parent()->name(),
						      source->name(),
						      delayVals,
						      sink->parent()->name(),
						      sink->name());
	    }
	    fixAliases(gpi,pi,sink);
	    fixAliases(gpo,po,source);

	    source->initialize();
	    sink->initialize();
	  }                        // while loop
	  // Out of inputs at this point.
	  // Check to be sure we are also out of outputs.
	  if ((nextbottom++ != 0) || (nexttop++ != 0)) {
	    Error::abortRun(*this,"Not enough inputs for the given number of outputs");
	    return;
	  }
	  // Now remove ourselves from the parent galaxy and self-destruct
	  Galaxy* mom = idParent();
	  if(!mom) return;
	  mom->deleteBlockAfterInit(*this);
}

// prototype instance for known block list
static HOFBusSplit proto;
static RegisterBlock registerBlock(proto,"BusSplit");
