static const char file_id[] = "HOFChain.pl";
// .cc file generated from HOFChain.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "HOFChain.h"
#include "Galaxy.h"

const char *star_nm_HOFChain = "HOFChain";

const char* HOFChain :: className() const {return star_nm_HOFChain;}

ISA_FUNC(HOFChain,HOFBaseHiOrdFn);

Block* HOFChain :: makeNew() const { LOG_NEW; return new HOFChain;}

HOFChain::HOFChain ()
{
	setDescriptor("Create one or more instances of the named block connected in a chain.\nThis is implemented by replacing the\n.c Chain\nstar with instances of the named blocks at setup time.\nThe replacement block(s) are connected as specified by\n\fIinput_map\fR, \fIinternal_map\fR, and \fIoutput_map\fR.\nTheir parameters are determined by \fIparameter_map\fR.\nIf \fIpipeline\fR is YES then a unit delay is put on all internal connections.");
	addPort(input.setPort("input",this,ANYTYPE));
	addPort(output.setPort("output",this,ANYTYPE));
	output.inheritTypeFrom(input);
	addState(chain_length.setState("chain_length",this,"2","The length of the chain"));
	addState(input_map.setState("input_map",this,"input","The mapping of inputs"));
	addState(output_map.setState("output_map",this,"output","The mapping of outputs"));
	addState(internal_map.setState("internal_map",this,"output input","The internal connections between blocks"));
	addState(pipeline.setState("pipeline",this,"NO","If YES, put a delay on each internal connection"));


}

void HOFChain::setup() {
# line 117 "HOFChain.pl"
HOFBaseHiOrdFn::setup();
	    // Check that all maps are the right size
	    int allsizes = input.numberPorts();
	    if (input_map.size() != allsizes ||
		output_map.size() != allsizes ||
		internal_map.size() != allsizes*2 ||
		output.numberPorts() != allsizes) {
		    Error::abortRun(*this,
			"Number of inputs, outputs, and internal connections"
			" need to be the same");
		    return;
	    }
	    // Get the parent galaxy
	    Galaxy* mom = idParent();
	    if(!mom) return;

	    // Create the first block
	    Block* block = createBlock(*mom,
				       (const char*)blockname,
				       (const char*)where_defined);
	    if (!block) return;

	    // Connect the inputs to the first block
	    int inputno = 0;
	    MPHIter nexti(input);
	    PortHole *pi;
	    while ((pi = nexti++) != 0) {
		GenericPort *dest;
		if (!(dest = block->genPortWithName(input_map[inputno++]))) {
		    Error::abortRun(*this,
			"input_map contains unrecognized name: ",
			input_map[inputno-1]);
		    return;
		}
		if(!connectInput(pi,dest)) return;
	    }

	    // Create the rest of the blocks and the internal connections
	    for (int instno = 1; instno < (int)chain_length; instno++) {
		Block* newblock;
		if(!(newblock = createBlock(*mom,
				       (const char*)blockname,
				       (const char*)where_defined)))
		   return;
		if(!(connectInternal(block,newblock))) return;

		// The previous block is now fully connected.  Initialize it.
		if(!setParams(block, instno)) return;
		block->setTarget(target());
		block->initialize();

		block = newblock;
	    }

	    // Connect the outputs to the last block
	    int outputno = 0;
	    MPHIter nexto(output);
	    PortHole *po;
	    while ((po = nexto++) != 0) {
		GenericPort *source;
		if (!(source = block->genPortWithName(output_map[outputno++]))) {
		    Error::abortRun(*this,
			"maybe output_map contains unrecognized name: ",
			output_map[outputno-1]);
		    return;
		}
		if(!connectOutput(po,source)) return;
	    }

	    // The last block is now fully connected.  Initialize it.
	    if(!setParams(block, (int)chain_length)) return;
	    block->setTarget(target());
	    block->initialize();

	    mom->deleteBlockAfterInit(*this);
}


int HOFChain::connectInternal (Block* block1, Block* block2)
{
# line 199 "HOFChain.pl"
PortHole *port1, *port2;
		int i = 0;
		int numdelays = 0;
		if ((int)pipeline) numdelays = 1;
		while (i < internal_map.size()-1) {
		    if (!(port1 = block1->portWithName(internal_map[i++]))) {
			Error::abortRun(*this,
			    "internal_map contains unrecognized name: ",
			    internal_map[i-1]);
			return 0;
		    }
		    if (!(port2 = block2->portWithName(internal_map[i++]))) {
			Error::abortRun(*this,
			    "internal_map contains unrecognized name: ",
			    internal_map[i-1]);
			return 0;
		    }
		    port1->connect(*port2,numdelays);
		}
		// No need to initialize delays because block1 has not
		// been initialized yet.
		return 1;
}


// prototype instance for known block list
static HOFChain proto;
static RegisterBlock registerBlock(proto,"Chain");
