static const char file_id[] = "HOFIfElseGr.pl";
// .cc file generated from HOFIfElseGr.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "HOFIfElseGr.h"
#include "ptk.h"
#include "SimControl.h"
#include "InfString.h"
#include "HOFDelayedMap.h"

const char *star_nm_HOFIfElseGr = "HOFIfElseGr";

const char* HOFIfElseGr :: className() const {return star_nm_HOFIfElseGr;}

ISA_FUNC(HOFIfElseGr,HOFIfElse);

Block* HOFIfElseGr :: makeNew() const { LOG_NEW; return new HOFIfElseGr;}

HOFIfElseGr::HOFIfElseGr ()
{
	setDescriptor("A variant of the IfElse star where the two possible replacement\nblocks are specified by graphically rather than textually.\nThere must be exactly two blocks connected in the positions of the\nreplacement blocks.\nThe HOFNop stars are the only exception: they may be used in addition to the\ntwo replacement blocks in order to control the order of connection.");
	addPort(trueout.setPort("trueout",this,ANYTYPE));
	trueout.inheritTypeFrom(input);
	addPort(truein.setPort("truein",this,ANYTYPE));
	addPort(falseout.setPort("falseout",this,ANYTYPE));
	falseout.inheritTypeFrom(input);
	addPort(falsein.setPort("falsein",this,ANYTYPE));

# line 54 "HOFIfElseGr.pl"
// Note that both branches must have the same output type.
	  output.inheritTypeFrom(truein);

	  // The constructor for IfElse hides blockname, where_defined,
	  // input_map, output_map, and parameter_map.  Here, we need to hide
	  // the rest.
	  true_block.clearAttributes(A_SETTABLE);
	  where_true_defined.clearAttributes(A_SETTABLE);
	  true_input_map.clearAttributes(A_SETTABLE);
	  true_output_map.clearAttributes(A_SETTABLE);
	  true_parameter_map.clearAttributes(A_SETTABLE);
	  false_block.clearAttributes(A_SETTABLE);
	  where_false_defined.clearAttributes(A_SETTABLE);
	  false_input_map.clearAttributes(A_SETTABLE);
	  false_output_map.clearAttributes(A_SETTABLE);
	  false_parameter_map.clearAttributes(A_SETTABLE);
}

void HOFIfElseGr::setup() {
# line 75 "HOFIfElseGr.pl"
HOFBaseHiOrdFn::setup();

	  Galaxy* mom = idParent();
	  if(!mom) return;

	  // Any HOF star has to call this method for all multiPortHoles
	  // to be sure that HOFNop stars are dealt with properly.
	  initConnections(trueout);
	  initConnections(truein);
	  initConnections(falseout);
	  initConnections(falsein);
	  initConnections(input);
	  initConnections(output);

	  // At this point, any HOFNop stars will have been disconnected,
	  // so there should be only one block connected to each of the
	  // true and false connections.

	  MPHIter nexti(input);
	  MPHIter nexto(output);
	  MPHIter *nextexo;
	  MPHIter *nextexi;
	  MPHIter *nextnuo;
	  MPHIter *nextnui;
	  // Choose the porthole over which to iterate
	  if (int(condition)) {
	    // Iterators for ports connected to the block we will use
	    LOG_NEW; nextexo = new MPHIter(trueout);
	    LOG_NEW; nextexi = new MPHIter(truein);
	    // Iterators for ports connected to the block we won't use
	    LOG_NEW; nextnuo = new MPHIter(falseout);
	    LOG_NEW; nextnui = new MPHIter(falsein);
	  } else {
	    LOG_NEW; nextexo = new MPHIter(falseout);
	    LOG_NEW; nextexi = new MPHIter(falsein);
	    LOG_NEW; nextnuo = new MPHIter(trueout);
	    LOG_NEW; nextnui = new MPHIter(truein);
	  }

	  PortHole *pi, *po, *pei, *peo, *dest;
	  GenericPort *sourcegp, *destgp;

	  // Now that we know which substitution block will be used, it
	  // is safe to initialize that substitution block even if it is
	  // a recursive reference to a galaxy above us.
	  // This is a violation of information hiding, but I can't figure
	  // out a cleaner way to support graphical recursion.
	  if ((peo = (*nextexo)++) &&
	      (dest = peo->far()) &&
	      (myblock = dest->parent()) &&
	      (myblock->isA("HOFDelayedMap"))) {
	    destgp = findTopGenericPort(dest);
	    destgp->parent()->setTarget(target());
	    destgp->parent()->initialize();
	    ((HOFDelayedMap*)myblock)->substitute();
	  }
	  myblock = 0;
	  nextexo->reset();

	  // Make the block connection
	  // Iterate over the inputs first
	  while ((peo = (*nextexo)++) != 0) {
	    if ((pi = nexti++) == 0) {
	      Error::abortRun(*this,
			      "Not enough inputs for the specified replacement block");
	      return;
	    }
	    if (!(destgp = breakConnection(peo))) return;
	    if (!connectInput(pi,destgp)) return;
	  }
	  // Better be out of outputs at this point
	  if ((pi = nexti++) != 0) {
	    Error::abortRun(*this, "Leftover inputs after connecting the replacement block");
	    return;
	  }
	  // Iterate over the outputs next
	  while ((pei = (*nextexi)++) != 0) {
	    if ((po = nexto++) == 0) {
	      Error::abortRun(*this,
			      "Not enough outputs for the specified replacement block");
	      return;
	    }
	    if (!(sourcegp = breakConnection(pei))) return;
	    if (!connectOutput(po, sourcegp)) return;
	  }
	  if (!myblock) {
	    Error::abortRun(*this,
			    "No connections to a replacement block!");
	    return;
	  }

	  myblock->setTarget(target());
	  myblock->initialize();
	  myblock = 0;

	  // Disconnect the block that we will not use
	  while ((peo = (*nextnuo)++) != 0) {
	    breakConnection(peo);
          }
	  while ((pei = (*nextnui)++) != 0) {
	    breakConnection(pei);
          }

	  LOG_DEL; delete nextexo;
	  LOG_DEL; delete nextexi;
	  LOG_DEL; delete nextnuo;
	  LOG_DEL; delete nextnui;

	  // delete the unused block.
	  mom->deleteBlockAfterInit(*myblock);

	  mom->deleteBlockAfterInit(*this);
}

// prototype instance for known block list
static HOFIfElseGr proto;
static RegisterBlock registerBlock(proto,"IfElseGr");
