static const char file_id[] = "HOFMap.pl";
// .cc file generated from HOFMap.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "HOFMap.h"
#include "Geodesic.h"
#include "SimControl.h"
#include "InterpGalaxy.h"

const char *star_nm_HOFMap = "HOFMap";

const char* HOFMap :: className() const {return star_nm_HOFMap;}

ISA_FUNC(HOFMap,HOFBaseHiOrdFn);

Block* HOFMap :: makeNew() const { LOG_NEW; return new HOFMap;}

HOFMap::HOFMap ()
{
	setDescriptor("Map one or more instances of the named block to the input stream(s)\nto produce the output stream(s).\nThis is implemented by replacing the Map star with the named block at\nsetup time.\nThe replacement block(s) are connected as specified by \"input_map\"\nand \"output_map\", using the existing connections to the Map star.\nTheir parameters are determined by \"parameter_map\".");
	addPort(input.setPort("input",this,ANYTYPE));
	addPort(output.setPort("output",this,ANYTYPE));
	output.inheritTypeFrom(input);
	addState(input_map.setState("input_map",this,"input","The mapping of inputs"));
	addState(output_map.setState("output_map",this,"output","The mapping of outputs"));


}

void HOFMap::setup() {
# line 130 "HOFMap.pl"
HOFBaseHiOrdFn::setup();

	    // Check that either an input_map or output_map is given
	    if (input_map.size() == 0 && output_map.size() == 0) {
		Error::abortRun(*this,
		    "Must specify at least one of input_map or output_map");
		return;
	    }
	    Galaxy* mom = idParent();
	    if(!mom) return;

	    // Any HOF star has to call this method for all multiPortHoles
	    // to be sure that HOFNop stars are dealt with properly.
	    initConnections(input);
	    initConnections(output);

	    MPHIter nexti(input);
	    MPHIter nexto(output);

	    if (!iterateOverPorts(nexti,nexto,mom,1)) return;

	    mom->deleteBlockAfterInit(*this);
}


int HOFMap::iterateOverPorts (MPHIter &nexti, MPHIter &nexto, Galaxy *mom, int firstInstanceNo)
{
# line 162 "HOFMap.pl"
PortHole *pi, *po;
	    int instanceno = firstInstanceNo;

	    // The following loop creates however many instances of the block
	    // are required to connect all the inputs and outputs.  The number
	    // of inputs and number of outputs of the Map star must be an
	    // integer multiple of the number of inputs and outputs of the block
	    while (!SimControl::haltRequested()) {

	      // Check to see whether we are out of inputs or outputs
	      if ((pi = nexti++) == 0 && input_map.size() != 0) {
		// Out of inputs.  Better be out of outputs as well.
		if (nexto++) {	
		  Error::abortRun(*this,
				  "Too many outputs for the number of inputs");
		  return 0;
		}
		break;
	      }

	      // Not out of inputs.  Had better not be out of outputs either
	      if ((po = nexto++) == 0 && output_map.size() != 0) {
		if (input_map.size() == 0) {
		  break;
		} else {
		  Error::abortRun(*this,
				  "Too many inputs for the number of outputs");
		  return 0;
		}
	      }

	      Block* block = createBlock(*mom,
					 (const char*)blockname,
					 (const char*)where_defined);
	      if (!block) return 0;

	      int i;
	      // Connect the inputs, if any.
	      for (i=0; i < input_map.size(); i++) {
		if (i != 0 && (pi = nexti++) == 0) {
		  Error::abortRun(*this,
				  "number of inputs is not a multiple of the ",
				  "input_map size");
		  return 0;
		}
		// Reconnect
		GenericPort *dest;
		if (!(dest = block->genPortWithName(input_map[i]))) {
		    Error::abortRun(*this,
			"input_map contains unrecognized name: ",
			input_map[i]);
		    return 0;
		}
		if(!connectInput(pi,dest)) return 0;
	      }

	      // Connect the outputs, if any
	      for (i=0; i < output_map.size(); i++) {
		if (i != 0 && (po = nexto++) == 0) {
		  Error::abortRun(*this,
				  "number of outputs is not a multiple of the ",
				  "output_map size");
		  return 0;
		}
		// Reconnect
		GenericPort *source;
		if (!(source = block->genPortWithName(output_map[i]))) {
		    Error::abortRun(*this,
			"maybe output_map contains unrecognized name: ",
			output_map[i]);
		    return 0;
		}
		if(!connectOutput(po,source)) return 0;
	      }
	      if(!setParams(block, instanceno++)) return 0;
	      block->setTarget(target());
	      block->initialize();
	    }
	    return 1;
}


// prototype instance for known block list
static HOFMap proto;
static RegisterBlock registerBlock(proto,"Map");
