static const char file_id[] = "HOFMapGr.pl";
// .cc file generated from HOFMapGr.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "HOFMapGr.h"
#include "ptk.h"
#include "SimControl.h"
#include "InfString.h"

const char *star_nm_HOFMapGr = "HOFMapGr";

const char* HOFMapGr :: className() const {return star_nm_HOFMapGr;}

ISA_FUNC(HOFMapGr,HOFMap);

Block* HOFMapGr :: makeNew() const { LOG_NEW; return new HOFMapGr;}

HOFMapGr::HOFMapGr ()
{
	setDescriptor("A variant of the Map star where the replacement block is specified\nby graphically connecting it.\nThere must be exactly one block connected in the position of the\nreplacement block.\nThe HOFNop stars are the only exception: they may be used in addition to the\none replacement block in order to control the order of connection.");
	addPort(exout.setPort("exout",this,ANYTYPE));
	exout.inheritTypeFrom(input);
	addPort(exin.setPort("exin",this,ANYTYPE));

# line 49 "HOFMapGr.pl"
output.inheritTypeFrom(exin);
	  input_map.clearAttributes(A_SETTABLE);
	  output_map.clearAttributes(A_SETTABLE);
	  blockname.clearAttributes(A_SETTABLE);
	  where_defined.clearAttributes(A_SETTABLE);
	  parameter_map.setInitValue("");
}

void HOFMapGr::setup() {
# line 57 "HOFMapGr.pl"
HOFBaseHiOrdFn::setup();

	  Galaxy* mom = idParent();
	  if(!mom) return;

	  // Any HOF star has to call this method for all multiPortHoles
	  // to be sure that HOFNop stars are dealt with properly.
	  initConnections(exout);
	  initConnections(exin);
	  initConnections(input);
	  initConnections(output);

	  // At this point, any HOFNop stars will have been disconnected,
	  // so there should be only one example star.

	  MPHIter nexti(input);
	  MPHIter nexto(output);
	  MPHIter nextexo(exout);
	  MPHIter nextexi(exin);

	  // Make the first block connection, and in the process, and set
	  // the input_map and output_map and blockname states.
	  PortHole *pi, *po, *pei, *peo;
	  InfString inputMap, outputMap;
	  GenericPort *sourcegp, *destgp;
	  myblock = 0;

	  // Iterate over the inputs first
	  while ((peo = nextexo++) != 0) {
	    if ((pi = nexti++) == 0) {
	      Error::abortRun(*this,
			      "Not enough inputs for the specified replacement block");
	      return;
	    }
	    if (!(destgp = breakConnection(peo))) return;
	    if (inputMap.numPieces() > 0) inputMap += " ";
	    inputMap += destgp->name();
	    if (!connectInput(pi,destgp)) return;
	  }
	  input_map.setInitValue((const char*)inputMap);
	  input_map.initialize();
	  // Iterate over the outputs next
	  while ((pei = nextexi++) != 0) {
	    if ((po = nexto++) == 0) {
	      Error::abortRun(*this,
			      "Not enough outputs for the specified replacement block");
	      return;
	    }
	    if (!(sourcegp = breakConnection(pei))) return;
	    if (outputMap.numPieces() > 0) outputMap += " ";
	    outputMap += sourcegp->name();
	    if (!connectOutput(po, sourcegp)) return;
	  }
	  if (!myblock) {
	    Error::abortRun(*this,
			    "No connections to a replacement block!");
	    return;
	  }
	  output_map.setInitValue((const char*)outputMap);
	  output_map.initialize();
	  blockname.setInitValue(myblock->name());
	  blockname.initialize();
	  
	  // Iterate over the remaining blocks.
	  if (!iterateOverPorts(nexti,nexto,mom,2)) return;

	  // Set the parameter values after all the clones have been created
	  // so that cloning takes the right default values.
	  if(!setParams(myblock, 1)) return;
	  myblock->setTarget(target());
	  myblock->initialize();

	  mom->deleteBlockAfterInit(*this);
}


Block* HOFMapGr::createBlock (Galaxy& mom, const char* /*blockname*/, const char* /*where_defined*/)
{
# line 138 "HOFMapGr.pl"
Block *block = myblock->clone();
	      if (!block) {
		Error::abortRun(*this,
				"Unable to create instance of block: ",
				myblock->name());
	      }
	      // Set the target
	      if (target()) block->setTarget(target());

	      // Choose a name for the block
	      StringList instancename = "Map_";
	      instancename += (const char*)myblock->name();
	      instancename += "__";
	      instancename += count++;
	      // The following amounts to a small memory leak
	      const char* instance = hashstring(instancename);

	      mom.addBlock(*block,instance);
	      return block;
}


// prototype instance for known block list
static HOFMapGr proto;
static RegisterBlock registerBlock(proto,"MapGr");
