static const char file_id[] = "HOFNop.pl";
// .cc file generated from HOFNop.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "HOFNop.h"
#include "Galaxy.h"
#include "InterpGalaxy.h"

const char *star_nm_HOFNop = "HOFNop";

const char* HOFNop :: className() const {return star_nm_HOFNop;}

ISA_FUNC(HOFNop,HOFBase);

Block* HOFNop :: makeNew() const { LOG_NEW; return new HOFNop;}

HOFNop::HOFNop ()
{
	setDescriptor("Bridge inputs to outputs and then self-destruct.\nThis star is used to split a bus into individual\nlines or combine individual lines into a bus.\nIt is also used to break out multi-inputs\nand multi-outputs into individual ports.");
	addPort(output.setPort("output",this,ANYTYPE));
	output.inheritTypeFrom(input);
	addPort(input.setPort("input",this,ANYTYPE));

# line 45 "HOFNop.pl"
initFlag = 0;
}

void HOFNop::setup() {
# line 48 "HOFNop.pl"
if (!initFlag) reconnect();
	  // Now remove ourselves from the parent galaxy and self-destruct
	  Galaxy* mom = idParent();
	  if(!mom) return;
	  mom->deleteBlockAfterInit(*this);
}


void HOFNop::reconnect ()
{
# line 66 "HOFNop.pl"
MPHIter nexti(input);
	    MPHIter nexto(output);
	    PortHole *source, *sink;
	    MultiPortHole *mph;
	    const char *sourceDelayVals, *sinkDelayVals;
	    int numInDelays, numOutDelays;

	    if (!initFlag) {
	      initFlag = 1;

	      // A NamedObjList is used to collect pointers to the Blocks at
	      // the other end of modified connections.  After completing all the
	      // reconnections, we go through and initialize all these objects.
	      // The reason that the blocks need to be reinitialized, and not just
	      // their portholes, is that the number of connections to the block
	      // may have changed.

	      NamedObjList objectsToReinitialize;
	      PortHole *prevpofar = 0;
	      PortHole *prevpifar = 0;
	      PortHole *pi = nexti++;
	      PortHole *po = nexto++;
	      GenericPort *gpo, *gpi;
	      while ((pi != 0) ||
		     (po != 0)) {
		if (pi == 0) {
		  // Out of inputs.
		  // Try to create a new input connection.
		  // Fail if we can't.
		  if (!prevpifar) {
		    Error::abortRun(*this, "No inputs connected!");
		    return;
		  }
		  if ((mph = prevpifar->getMyMultiPortHole()) != 0) {
		    // Far side is a MultiPortHole -- can make a new connection
		    source = &(mph->newConnection());
		  } else {
		    Error::abortRun(*this,"Not enough inputs for the given number of outputs");
		    return;
		  }
		  if ((sink = prevpofar = po->far()) == 0) {
		    Error::abortRun(*this,"Can't find sink PortHole!");
		    return;
		  }
		  numInDelays = prevpifar->numInitDelays();
		  numOutDelays = po->numInitDelays();
		  sinkDelayVals = po->initDelayValues();
		  sourceDelayVals = prevpifar->initDelayValues();
		  gpo = aliasPointingAt(po);
		  gpi = 0;
		  sink->disconnect();
		} else if (po == 0) {
		  // Out of outputs.
		  // Try to create a new output connection
		  // Fail if we can't.
		  if (!prevpofar) {
		    Error::abortRun(*this, "No outputs connected!");
		    return;
		  }
		  if ((mph = prevpofar->getMyMultiPortHole()) != 0) {
		    // Far side is a MultiPortHole -- can make a new connection
		    sink = &(mph->newConnection());
		  } else {
		    Error::abortRun(*this,"Not enough outputs for the given number of inputs");
		    return;
		  }
		  if ((source = prevpifar = pi->far()) == 0) {
		    Error::abortRun(*this,"Can't find source PortHole!");
		    return;
		  }
		  numInDelays = pi->numInitDelays();
		  numOutDelays = prevpofar->numInitDelays();
		  sourceDelayVals = pi->initDelayValues();
		  sinkDelayVals = prevpofar->initDelayValues();
		  gpi = aliasPointingAt(pi);
		  gpo = 0;
		  source->disconnect();
		} else {
		  if((source = pi->far()) == 0 ||
		     (sink = po->far()) == 0) {
		    Error::abortRun(*this,"Star is not fully connected");
		    return;
		  }
		  sourceDelayVals = pi->initDelayValues();
		  sinkDelayVals = po->initDelayValues();
		  prevpofar = po->far();
		  prevpifar = pi->far();
		  numInDelays = pi->numInitDelays();
		  numOutDelays = po->numInitDelays();
		  gpo = aliasPointingAt(po);
		  gpi = aliasPointingAt(pi);
		  source->disconnect();
		  sink->disconnect();
		}

		int numDelays = 0;
		const char* delayVals = 0;
		if ((numInDelays > 0) || (sourceDelayVals && *sourceDelayVals)) {
		  numDelays = numInDelays;
		  delayVals = sourceDelayVals;
		  if ((numOutDelays > 0) || (sinkDelayVals && *sinkDelayVals)) {
		    Error::warn(*this,
				"Cannot have delays on inputs and outputs."
				" Using input value");
		  }
		} else {
		  numDelays = numOutDelays;
		  delayVals = sinkDelayVals;
		}
		source->connect(*sink,numDelays,delayVals);
		// Register the connection with the parent galaxy so that delays get
		// initialized when the galaxy is reinitialized.
		if(parent()->isA("InterpGalaxy") && 
		   ((numDelays > 0) || (delayVals && *delayVals))) {
		  ((InterpGalaxy*)parent())->registerInit("C",
							  source->parent()->name(),
							  source->name(),
							  delayVals,
							  sink->parent()->name(),
							  sink->name());
		}

		fixAliases(gpi,pi,sink);
		fixAliases(gpo,po,source);

		// If the farside porthole belongs to a HOFStar, we can assume it
		// has not been initialized (since it would have self-destructed).
		// It should not be initialized here either, since it may be already
		// on the list of stars to be initialized, and since it self-
		// destructs when it initializes.

		// The actual initialization is postponed until after this star
		// has completed, in case they have behavior that depends on
		// the number of ports.

		NamedObj *farObj;
		if (!source->parent()->isA("HOFStar")) {
		  if (source->parent()->isItWormhole()) {
		    source->initialize();
		    sink->initialize();
		  } else {
		    farObj = source->parent();
		    objectsToReinitialize.remove(farObj);
		    objectsToReinitialize.put(*farObj);
		  }
		}
		if (!sink->parent()->isA("HOFStar")) {
		  if (sink->parent()->isItWormhole()) {
		    source->initialize();
		    sink->initialize();
		  } else {
		    farObj = sink->parent();
		    objectsToReinitialize.remove(farObj);
		    objectsToReinitialize.put(*farObj);
		  }
		}
		po = nexto++;
		pi = nexti++;
	      }
	      objectsToReinitialize.initElements();
	    }
}


// prototype instance for known block list
static HOFNop proto;
static RegisterBlock registerBlock(proto,"Nop");
