static const char file_id[] = "MDSDFFFTCxCol.pl";
// .cc file generated from MDSDFFFTCxCol.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "MDSDFFFTCxCol.h"

const char *star_nm_MDSDFFFTCxCol = "MDSDFFFTCxCol";

const char* MDSDFFFTCxCol :: className() const {return star_nm_MDSDFFFTCxCol;}

ISA_FUNC(MDSDFFFTCxCol,MDSDFStar);

Block* MDSDFFFTCxCol :: makeNew() const { LOG_NEW; return new MDSDFFFTCxCol;}

# line 83 "MDSDFFFTCxCol.pl"
/*
 * This fft routine is from ~gabriel/src/filters/fft/fft.c;
 * I am unsure of the original source.  The file contains no
 * copyright notice or description.
 * The declaration is changed to the prototype form but the
 * function body is unchanged.  (J. T. Buck)
 */

#define SWAP(a, b) temp=(a); (a)=(b); (b)=temp

/*
 * Replace data by its discrete Fourier transform, if isign is
 * input as 1, or by its inverse discrete Fourier transform, if 
 * "isign" is input as -1.  "data'"is a complex array of length "nn",
 * input as a real array data[0..2*nn-1]. "nn" MUST be an integer
 * power of 2 (this is not checked for!?)
 */

static void fft_rif(ComplexSubMatrix *inMatrix, int nn, int isign)
{
	int	mmax;
	int	m, j, istep, i;
	double	wtemp, theta;
	Complex w, wp, temp;

	j = 0;

        // put into bit-reversed order
	for (i = 0; i < nn; i++) {
		if(j > i) {
			SWAP(inMatrix->entry(j), inMatrix->entry(i));
                }
		m = nn >> 1;
		while (m >= 1 && j >= m) {
			j -= m;
			m >>= 1;
		}
		j += m;
	}
	mmax = 1;
	while (nn > mmax) {
		istep = mmax*2;
		theta = -3.1415926535897949/(isign*mmax);
		wtemp = sin(0.5*theta);
		wp = Complex(-2.0*wtemp*wtemp, sin(theta));
		w = Complex(1.0,0.0);
		for (m = 0; m < mmax; m++) {
			for (i = m; i < nn; i += istep) {
				j = i + mmax;
				temp = w * inMatrix->entry(j);
				inMatrix->entry(j) = inMatrix->entry(i) - temp;
				inMatrix->entry(i) += temp;
			}
			w  = Complex((wtemp = w.real())*wp.real() - 
	                             w.imag()*wp.imag() + w.real(),
				     w.imag()*wp.real() + wtemp*wp.imag() +
				     w.imag());
		}
		mmax = istep;
	}
}

MDSDFFFTCxCol::MDSDFFFTCxCol ()
{
	setDescriptor("Compute the 2D discrete Fourier transform of a complex input using the\nrow-column decomposition with 1D fast Fourier transform (FFT) algorithm.\n\nThis is similar to FFTCx except that it is specialized for column\nFFTs to match ordering common to image processing.");
	addPort(input.setPort("input",this,COMPLEX_MATRIX));
	addPort(output.setPort("output",this,COMPLEX_MATRIX));
	addState(order.setState("order",this,"8","Log base 2 of the column FFT transform size."));
	addState(size.setState("size",this,"256","The number of columns in the input."));
	addState(direction.setState("direction",this,"1","Equals 1 for forward, or -1 for inverse, transform."));


}

void MDSDFFFTCxCol::setup() {
# line 148 "MDSDFFFTCxCol.pl"
fftSize = 1 << int(order);
	  if (fftSize < int(size)) {
	    Error::abortRun(*this, "2^order must be >= size");
	    return;
	  }
          // column FFT
	  input.setMDSDFParams(int(size), 1);
	  output.setMDSDFParams(int(fftSize), 1);
}

void MDSDFFFTCxCol::go() {
# line 158 "MDSDFFFTCxCol.pl"
// get input and output matrices
	  ComplexSubMatrix* data = (ComplexSubMatrix*)(input.getInput());
	  ComplexSubMatrix* result = (ComplexSubMatrix*)(output.getOutput());

// copy input to output, we will work on output buffer
          int i;
          // if column, reverse order to match normal
          // index arrangement for image processing
          for(i = 0; i < int(size); i++)
	    result->entry(fftSize - i - 1) = data->entry(i);

	  // zero fill the result of the output buffer
          for(i = int(size); i < fftSize; i++)
            result->entry(fftSize - i - 1) = Complex(0.0);

          fft_rif(result, fftSize, int(direction));

          // invert reverse order for column FFT
          for(i = 0; i < int(size)/2; i++) {
            Complex temp = result->entry(i);
	    result->entry(i) = result->entry(fftSize - i - 1);
	    result->entry(fftSize - i - 1) = temp;
	  }
      
	  // generate output data.  If inverse, we must scale the result.
	  if (int(direction) != 1)
            for (i = 0; i < fftSize; i++)
              result->entry(i) /= fftSize;
	  // delete access structures
	  delete data;
	  delete result;
}

// prototype instance for known block list
static MDSDFFFTCxCol proto;
static RegisterBlock registerBlock(proto,"FFTCxCol");
