static const char file_id[] = "MDSDFQuant.pl";
// .cc file generated from MDSDFQuant.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "MDSDFQuant.h"

const char *star_nm_MDSDFQuant = "MDSDFQuant";

const char* MDSDFQuant :: className() const {return star_nm_MDSDFQuant;}

ISA_FUNC(MDSDFQuant,MDSDFStar);

Block* MDSDFQuant :: makeNew() const { LOG_NEW; return new MDSDFQuant;}

MDSDFQuant::MDSDFQuant ()
{
	setDescriptor("Quantize the input value to one of N+1 possible output levels using N\nthresholds.  For an input less than or equal to the n-th threshold, but\nlarger than all previous thresholds, the output will be the n-th level.\nIf the input is greater than all thresholds, the output is the N+1-th\nlevel.  If level is specified, there must be one more level than\nthresholds; the default value for level is 0, 1, 2, ... N.  This star\nis much slower than LinQuantIdx, so if possible, that one should be\nused instead.");
	addPort(input.setPort("input",this,FLOAT_MATRIX));
	addPort(output.setPort("output",this,FLOAT_MATRIX));
	addState(numRows.setState("numRows",this,"4","Number of rows in the input block."));
	addState(numCols.setState("numCols",this,"4","Number of columns in the input block."));
	addState(thresholds.setState("thresholds",this,"0.0","Quantization thresholds, in increasing order"));
	addState(levels.setState("levels",this,"","Output levels.  If empty, use 0, 1, 2, ..."));


}

void MDSDFQuant::setup() {
# line 56 "MDSDFQuant.pl"
input.setMDSDFParams(int(numRows),int(numCols));
		output.setMDSDFParams(int(numRows),int(numCols));

		int n = thresholds.size();
		if (levels.size() == 0) {
			// default: 0, 1, 2...
			levels.resize(n + 1);
			for (int i = 0; i <= n; i++)
				levels[i] = i;
		}
		else if (levels.size() != n+1) {
			Error::abortRun (*this,
			      "must have one more level than thresholds");
		}
}

void MDSDFQuant::go() {
# line 72 "MDSDFQuant.pl"
// get input and output matrices
		FloatSubMatrix* matrix = (FloatSubMatrix*)(input.getInput());
		FloatSubMatrix* out = (FloatSubMatrix*)(output.getOutput());

		int siz = thresholds.size();
		int numentries = int(numRows)*int(numCols);
		for (int i = 0; i < numentries; i++) {
	                int lo = 0;
	                int hi = siz;
		        int mid = (hi+lo)/2;
			double in = matrix->entry(i);
			do {
				if (in <= thresholds[mid]) {
					hi = mid;
				}
				else {
					lo = mid+1;
				}
				mid = (hi+lo)/2;
			} while (mid < siz && hi > lo);

			// now in is <= thresholds[mid] but > all smaller ones.
			// (where thresholds[siz] is infinity)

			out->entry(i) = levels[mid];
		}

		// delete access structures
		delete matrix;
		delete out;
}

// prototype instance for known block list
static MDSDFQuant proto;
static RegisterBlock registerBlock(proto,"Quant");
