static const char file_id[] = "MDSDFReadImg.pl";
// .cc file generated from MDSDFReadImg.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "MDSDFReadImg.h"
#include "SubMatrix.h"
#include <std.h>
#include <stdio.h>

const char *star_nm_MDSDFReadImg = "MDSDFReadImg";

const char* MDSDFReadImg :: className() const {return star_nm_MDSDFReadImg;}

ISA_FUNC(MDSDFReadImg,MDSDFStar);

Block* MDSDFReadImg :: makeNew() const { LOG_NEW; return new MDSDFReadImg;}

MDSDFReadImg::MDSDFReadImg ()
{
	setDescriptor("Read a sequence of images in Portable GrayMap (PGM) format from\ndifferent files and send them out as a FloatMatrix.\n\nThis star does not support frame numbers.  See ReadImg2.");
	addPort(imageOutput.setPort("imageOutput",this,FLOAT_MATRIX));
	addState(width.setState("width",this,"100","width of the image, cols of matrix"));
	addState(height.setState("height",this,"100","height of the image, rows of matrix"));
	addState(fileName.setState("fileName",this,"$PTOLEMY/src/domains/sdf/demo/ptimage","Name of file containing PGM-format image."));


}

void MDSDFReadImg::setup() {
# line 49 "MDSDFReadImg.pl"
// set the dimensions of the output
    imageOutput.setMDSDFParams(int(height),int(width));
}

void MDSDFReadImg::go() {
# line 54 "MDSDFReadImg.pl"
// Open file containing the image.
    char* nm = expandPathName(fileName);
    StringList expandedName = nm;
    delete [] nm;
    FILE* fp = fopen(expandedName, "r");
    if (fp == (FILE*) NULL) {
      Error::abortRun(*this, "cannot open '", expandedName, "' for reading.");
      return;
    }

    // Read header, skipping 1 whitespace character at end.
    char word[80];
    int fileImageWidth, fileImageHeight, maxval;
    fscanf(fp, "%s", word);
    if (strcmp(word, "P5")) { // "P5" is PGM magic number.
      Error::abortRun(*this, fileName, ": not in PGM format");
      return;
    }
    fscanf(fp, "%s", word);
    while (word[0] == '#') {
      fscanf(fp, "%*[^\n]%s", word);
    }
    sscanf(word, "%d", &fileImageWidth);
    fscanf(fp, "%s", word);
    while (word[0] == '#') {
      fscanf(fp, "%*[^\n]%s", word);
    }
    sscanf(word, "%d", &fileImageHeight);
    fscanf(fp, "%s", word);
    while (word[0] == '#') {
      fscanf(fp, "%*[^\n]%s", word);
    }
    sscanf(word, "%d", &maxval);
    if (maxval > 255) {
      Error::abortRun(*this, fileName,": not in 8-bit format.");
      return;
    }
    fscanf(fp, "%*c"); // skip one whitespace char.

    // Create a FloatMatrix object and fill it with data.
    // We use a hack for faster file reading: create a buffer to read in
    // the data as unsigned char's and then convert that in memory.
    if(fileImageWidth != int(width) ||
       fileImageHeight != int(height)) {
      Error::abortRun(*this,
		      "the width and height states do not ",
		      "match those of the file image read from ",
		      fileName);
      return;
    }
    unsigned int size = fileImageWidth * fileImageHeight;
    unsigned char *buffer = new unsigned char[size];
    unsigned char *p = buffer;
    fread( (char*)buffer,
    	   int(width) * sizeof(unsigned char),
	   int(height),
	   fp );
    FloatSubMatrix* imgData = (FloatSubMatrix*)imageOutput.getOutput();
    for(unsigned int i = 0; i < size; i++)
      imgData->entry(i) = double(*p++);
    delete [] buffer;
    delete imgData;
    fclose(fp);
}

// prototype instance for known block list
static MDSDFReadImg proto;
static RegisterBlock registerBlock(proto,"ReadImg");
