static const char file_id[] = "MDSDFReadImg2.pl";
// .cc file generated from MDSDFReadImg2.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "MDSDFReadImg2.h"
#include "SubMatrix.h"
#include <std.h>
#include <stdio.h>
#include "miscFuncs.h"
#include "StringList.h"

const char *star_nm_MDSDFReadImg2 = "MDSDFReadImg2";

const char* MDSDFReadImg2 :: className() const {return star_nm_MDSDFReadImg2;}

ISA_FUNC(MDSDFReadImg2,MDSDFStar);

Block* MDSDFReadImg2 :: makeNew() const { LOG_NEW; return new MDSDFReadImg2;}

MDSDFReadImg2::MDSDFReadImg2 ()
{
	setDescriptor("Read a sequence of images in Portable GrayMap (PGM) format from\ndifferent files and send them out as a FloatMatrix.\n\nIf present, the character '#' in the 'fileName' state is replaced with\nthe frame number to be read next.\nFor example, if the 'frameId' state is set to 2 and\nif the 'fileName' state is 'dir.#/pic#', then the files that\nare read are 'dir.2/pic2', 'dir.3/pic3', etc.");
	addPort(imageOutput.setPort("imageOutput",this,FLOAT_MATRIX));
	addPort(frameIdOutput.setPort("frameIdOutput",this,INT_MATRIX));
	addState(width.setState("width",this,"100","width of the image, cols of matrix"));
	addState(height.setState("height",this,"100","height of the image, rows of matrix"));
	addState(fileName.setState("fileName",this,"$PTOLEMY/src/domains/sdf/demo/ptimage","Name of file containing PGM-format image."));
	addState(frameId.setState("frameId",this,"0","Starting frame ID value."));


}

void MDSDFReadImg2::setup() {
# line 62 "MDSDFReadImg2.pl"
// set the dimensions of the output
    imageOutput.setMDSDFParams(int(height),int(width));
    frameIdOutput.setMDSDFParams(1,1);
}

void MDSDFReadImg2::go() {
# line 82 "MDSDFReadImg2.pl"
// Open file containing the image.
    char *expandedName = genFileName(fileName, int(frameId));
    StringList fullName = expandedName;
    delete [] expandedName;
    FILE* fp = fopen(fullName, "r");
    if (fp == 0) {
      Error::abortRun(*this, "cannot open '", fullName, "' for reading.");
      return;
    }

    // Read header, skipping 1 whitespace character at end.
    char word[80];
    int fileImageWidth, fileImageHeight, maxval;
    fscanf(fp, "%s", word);
    if (strcmp(word, "P5")) { // "P5" is PGM magic number.
      fclose(fp);
      Error::abortRun(*this, fullName, ": not in PGM format");
      return;
    }
    fscanf(fp, "%s", word);
    while (word[0] == '#') {
      fscanf(fp, "%*[^\n]%s", word);
    }
    sscanf(word, "%d", &fileImageWidth);
    fscanf(fp, "%s", word);
    while (word[0] == '#') {
      fscanf(fp, "%*[^\n]%s", word);
    }
    sscanf(word, "%d", &fileImageHeight);
    fscanf(fp, "%s", word);
    while (word[0] == '#') {
      fscanf(fp, "%*[^\n]%s", word);
    }
    sscanf(word, "%d", &maxval);
    if (maxval > 255) {
      fclose(fp);
      Error::abortRun(*this, fullName, ": not in 8-bit format.");
      return;
    }
    fscanf(fp, "%*c"); // skip one whitespace char.

    // Create a FloatMatrix object and fill it with data.
    // We use a hack for faster file reading: create a buffer to read in
    // the data as unsigned char's and then convert that in memory.
    if (fileImageWidth != int(width) ||
        fileImageHeight != int(height)) {
      fclose(fp);
      Error::abortRun(*this,
		      "the width and height states of the star do ",
		      "not match those of the image file ",
		      fullName);
      return;
    }
    unsigned int size = fileImageWidth * fileImageHeight;
    unsigned char *buffer = new unsigned char[size];
    unsigned char *p = buffer;
    fread( (char*)buffer,
    	   fileImageWidth * sizeof(unsigned char),
	   fileImageHeight,
	   fp );
    FloatSubMatrix* imgData = (FloatSubMatrix*)imageOutput.getOutput();
    for (unsigned int i = 0; i < size; i++) {
      imgData->entry(i) = double(*p++);
    }
    delete[] buffer;
    delete imgData;
    fclose(fp);

    IntSubMatrix* floatIdOut = (IntSubMatrix*)frameIdOutput.getOutput();
    floatIdOut->entry(0) = frameId;
    frameId = int(frameId) + 1;			// increment frame id
    delete floatIdOut;
}


char * MDSDFReadImg2::genFileName (const char* str, const int d)
{
# line 72 "MDSDFReadImg2.pl"
char *filename = expandPathName(str);
      char num[16];
      sprintf(num, "%d", d);
      char *newname = subCharByString(filename, '#', num);
      delete [] filename;
      return newname;
}


// prototype instance for known block list
static MDSDFReadImg2 proto;
static RegisterBlock registerBlock(proto,"ReadImg2");
