static const char file_id[] = "MDSDFShowImg2.pl";
// .cc file generated from MDSDFShowImg2.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "MDSDFShowImg2.h"
#include "SubMatrix.h"
#include <std.h>
#include <stdio.h>
#include "Error.h"

const char *star_nm_MDSDFShowImg2 = "MDSDFShowImg2";

const char* MDSDFShowImg2 :: className() const {return star_nm_MDSDFShowImg2;}

ISA_FUNC(MDSDFShowImg2,MDSDFStar);

Block* MDSDFShowImg2 :: makeNew() const { LOG_NEW; return new MDSDFShowImg2;}

MDSDFShowImg2::MDSDFShowImg2 ()
{
	setDescriptor("Accept a black-and-white input FloatMatrix image and generate\noutput in PGM format. Send the output to a user-specified command\n(by default,\n.EQ\ndelim off\n.EN\n\"xv\"\n.EQ\ndelim $$\n.EN\nis used).\n\nThe user can set the root filename of the displayed image (which will\nprobably be printed in the image display window titlebar) and\ncan choose whether or not the image file is saved or deleted.\nThe image frame number is appended to the root filename to form the\ncomplete filename of the displayed image.");
	addPort(imageInput.setPort("imageInput",this,FLOAT_MATRIX));
	addPort(frameIdInput.setPort("frameIdInput",this,INT_MATRIX));
	addState(width.setState("width",this,"100","width of the image, cols of matrix"));
	addState(height.setState("height",this,"100","height of the image, rows of matrix"));
	addState(command.setState("command",this,"xv","Program to run on PGM data"));
	addState(imageName.setState("imageName",this,"","If non-null, name for PGM file"));
	addState(saveImage.setState("saveImage",this,"n","If 'y' or 'Y', then save the file"));


}

void MDSDFShowImg2::setup() {
# line 83 "MDSDFShowImg2.pl"
// set the dimensions of the output
    imageInput.setMDSDFParams(int(height),int(width));
    frameIdInput.setMDSDFParams(1,1);
}

void MDSDFShowImg2::go() {
# line 88 "MDSDFShowImg2.pl"
// Read data from input.
    FloatSubMatrix* image = (FloatSubMatrix*)(imageInput.getInput());
    IntSubMatrix* frameId = (IntSubMatrix*)(frameIdInput.getInput());
    int frame = frameId->entry(0);

    // Set filename and save values.
    const char* saveMe = saveImage;
    int del = !((saveMe[0] == 'y') || (saveMe[0] == 'Y'));

    const char* iname = imageName;
    char* nm = 0;
    if (iname && *iname) {
      nm = expandPathName(iname);
    }
    else {
      nm = tempFileName();
    }
    StringList fileName = nm;
    fileName << "." << frame;
    delete [] nm;

    FILE* fp = fopen(fileName, "w");
    if (fp == 0) {
      Error::abortRun(*this, "cannot open '", fileName, "' for writing.");
      delete image;
      delete frameId;
      return;
    }

    // Write the PGM header and the data, and then run.
    fprintf(fp, "P5\n %d %d 255\n", int(width), int(height));

    // Reverse of the hack used in ReadImage, first copy the data to
    // a buffer of the unsigned char's, then do a block fwrite
    unsigned int size = int(width) * int(height);
    unsigned char* buffer = new unsigned char[size];
    unsigned char* p = buffer;
    for(unsigned int i = 0; i < size; i++) {
      // limit range to be between 0 and 255
      double tmp = image->entry(i);
      if(tmp < 0)
        *p++ = 0;
      else if(tmp > 255)
        *p++ = 255;
      else
        *p++ = (unsigned char)tmp;
    }
    fwrite( (const char*) buffer,
    	    sizeof(unsigned char) * int(width),
	    int(height),
	    fp );
    fclose(fp);
    delete [] buffer;

    StringList cmdbuf = "(";
    cmdbuf << (const char*) command << " " << fileName;
    if (del) {
      cmdbuf << "; rm -f " << fileName;
    }
    cmdbuf << ")&"; 			// Run command in the background
    system(cmdbuf);

    delete image;
    delete frameId;
}

// prototype instance for known block list
static MDSDFShowImg2 proto;
static RegisterBlock registerBlock(proto,"ShowImg2");
