static const char file_id[] = "SDFRfft.pl";
// .cc file generated from SDFRfft.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFRfft.h"

const char *star_nm_SDFRfft = "SDFRfft";

const char* SDFRfft :: className() const {return star_nm_SDFRfft;}

ISA_FUNC(SDFRfft,SDFStar);

Block* SDFRfft :: makeNew() const { LOG_NEW; return new SDFRfft;}

SDFRfft::SDFRfft ()
{
	setDescriptor("Compute the FFT of a real input, using the '2-channel trick'.\n\fIInputSize\fR is number of input samples and\n\fIfftSize\fR is the size of FFT.\nZero padding is used if \fIfftSize\fR > \fIInputSize\fR.\nThe inverse FFT is taken if \fIInverse\fR is non-zero.\nApply a Hamming window to the data if \fIUseWindow\fR is non-zero.\nThe output is half the size of the input.");
	addPort(input.setPort("input",this,FLOAT));
	addPort(output.setPort("output",this,COMPLEX));
	addState(InputSize.setState("InputSize",this,"256","Number of input samples to read."));
	addState(fftSize.setState("fftSize",this,"256","Size of the transform."));
	addState(Inverse.setState("Inverse",this,"0","Equals 1 for inverse."));
	addState(UseWindow.setState("UseWindow",this,"0","Equals 1 to use Hamming window"));

# line 59 "SDFRfft.pl"
Temp1 = 0;
    Temp2 = 0;
    In = 0;
}

SDFRfft::~SDFRfft() {
# line 64 "SDFRfft.pl"
delete [] Temp1;
    delete [] Temp2;
    delete [] In;
}

void SDFRfft::setup() {
# line 69 "SDFRfft.pl"
int fftlength = fftSize;
    delete [] Temp1;
    Temp1 = new Complex[ fftlength/2 ];
    delete [] Temp2;
    Temp2 = new Complex[ fftlength/2 ];
    delete [] In;
    In = new double[ fftlength ];
    input.setSDFParams( int(InputSize), int(InputSize)-1 );
    output.setSDFParams( fftlength / 2, (fftlength/2)-1 );
}

void SDFRfft::go() {
# line 92 "SDFRfft.pl"
int invflag = Inverse;
    int fftlength = fftSize;
    int i;
    nb_fft F( invflag, fftlength / 2 );

    for (i = 0; i < int(InputSize); i++ )
      In[ int(InputSize) - i - 1 ] = input%i;
    for (i = int(InputSize); i < fftlength; i++ )
      In[ i ] = 0;

    double WinNorm = 1;
    if( int(UseWindow) ) {
      double Sum = 0;
      for( i = 0; i < fftlength; i++ ) {
	double TimeArg = ( double(i) / ( double(fftlength) - 1.) ) - 0.5;
	double WV = WinVal( TimeArg );
	In[ i ] *= WV;
	Sum += WV;
      }
      // This is the 1/(DC gain) compared to rect. window
      WinNorm = fftlength / Sum;
    }

    for( i = 0; i < fftlength / 2; i++ )
      Temp1[ i ] = Complex( In[ 2 * i ], In[ ( 2 * i ) + 1 ] );

    Complex* Result = F.Compute( Temp1, Temp2 );

    double Norm = int(Inverse) ? WinNorm : WinNorm / (double(fftlength));
    for( i = 0; i < fftlength; i++ )
      Result[ i ] *= Norm;

    for( i = 0; i < fftlength/2; i++ ) {
      Complex X1 = .5 * ( Result[ i ] +
		   conj( Result[ i == 0 ? 0 : fftlength/2 - i ] ) );
      Complex X2 = Complex( 0, -.5 ) * ( Result[ i ] -
		   conj( Result[ i == 0 ? 0 : fftlength/2 - i ] ) );
      output%(fftlength/2 - i - 1) <<
		Norm * ( X1 + F.getW( -i, fftlength ) * X2 );
    }
}

// prototype instance for known block list
static SDFRfft proto;
static RegisterBlock registerBlock(proto,"Rfft");
