static const char file_id[] = "SDFTWTA.pl";
// .cc file generated from SDFTWTA.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFTWTA.h"
#include <math.h>

const char *star_nm_SDFTWTA = "SDFTWTA";

const char* SDFTWTA :: className() const {return star_nm_SDFTWTA;}

ISA_FUNC(SDFTWTA,SDFStar);

Block* SDFTWTA :: makeNew() const { LOG_NEW; return new SDFTWTA;}

SDFTWTA::SDFTWTA ()
{
	setDescriptor("Model of a TWTA amplifier");
	addState(InputBackoff.setState("InputBackoff",this,"","Input Backoff sample values in dB (for phase and amplitude values)"));
	addState(Phase.setState("Phase",this,"","Phase (degree) vs. Input Backoff (dB)"));
	addState(Amplitude.setState("Amplitude",this,"","Amplitude (dB) vs. Input Backoff (dB)"));
	addState(Backoff.setState("Backoff",this,"0","Input Backoff (dB)"));
	addPort(in.setPort("in",this,COMPLEX));
	addPort(out.setPort("out",this,COMPLEX));

# line 56 "SDFTWTA.pl"
PhaseInterp = 0;
    AmplInterp = 0;
}

SDFTWTA::~SDFTWTA() {
# line 60 "SDFTWTA.pl"
delete PhaseInterp;
    delete AmplInterp;
}

void SDFTWTA::setup() {
# line 66 "SDFTWTA.pl"
if( Phase.size() != InputBackoff.size() ||
        Amplitude.size() != InputBackoff.size() ) {
      Error::abortRun( *this, "Phase, InputBackoff, and Amplitude arrays must be the same size" );
    }
    if( InputBackoff.size() < 2 ) {
      Error::abortRun( *this, "Need at least 2 data points" );
    }
    delete AmplInterp;
    delete PhaseInterp;
    AmplInterp = new Interp( (double *)InputBackoff,
			     (double *)Amplitude, InputBackoff.size() );
    PhaseInterp = new Interp( (double *)InputBackoff,
			      (double *)Phase, InputBackoff.size() );
}

void SDFTWTA::go() {
# line 84 "SDFTWTA.pl"
Complex z = Complex(in%0);
    double bo = norm(z);
    double oldphase = arg(z);
    double bodB = 10. * log10(bo) - double(Backoff);
    double ampdB = (*AmplInterp)[ bodB ];
    // Note the square root to convert power to volts
    double voltage = power( 10., 0.05 * ampdB );
    double Iphase = (*PhaseInterp)[ bodB ] * DEGREES_TO_RADIANS;
    double phase = oldphase + Iphase;
    Complex v = Complex( cos( phase ) * voltage, sin( phase ) * voltage );
    out%0 << v;
}

// prototype instance for known block list
static SDFTWTA proto;
static RegisterBlock registerBlock(proto,"TWTA");
