.EQ
delim off
.EN
.\" documentation file generated from SDFIIRFix.pl by ptlang
.\" Copyright (c) 1990-1996 The Regents of the University of California.
.\" All rights reserved.
.\" See the file $PTOLEMY/copyright for copyright notice,
.\" limitation of liability, and disclaimer of warranty provisions.
.NA "IIRFix"
.SD
An infinite impulse response (IIR) filter implemented in direct form II,
using fixed-point arithmetic.  The transfer function is of the form
H(z) = G*N(1/z)/D(1/z), where N() and D() are polynomials.
The parameter "gain" specifies G, and the floating-point arrays
"numerator" and "denominator" specify N() and D(), respectively.
Both arrays start with the constant terms of the polynomial
and decrease in powers of z (increase in powers of 1/z).
The coefficients are rounded to the precision given by "CoefPrecision".
Before the numerator and denominator coefficients are quantized, they
are rescaled so that the leading denominator coefficient is unity.
The gain is multiplied through the numerator coefficients as well.
.SE
.LO "SDF dsp library"
.DM SDF SDFFix
.SV 1.15 "2/12/96"
.AL "E. A. Lee, A. Khazeni, K. White"
.IH
.NE
\fIsignalIn\fR (fix)
.PE
.OH
.NE
\fIsignalOut\fR (fix)
.PE
.SH
.NE
\fIgain\fR (FloatState): Gain.
.DF "1"
.NE
\fInumerator\fR (FloatArrayState): Numerator coefficients.
.DF ".5 .25 .1"
.NE
\fIdenominator\fR (FloatArrayState): Denominator coefficients.
.DF "1 .5 .3"
.NE
\fICoefPrecision\fR (PrecisionState): Precision of the coefficients in bits.
.DF "2.14"
.NE
\fIArrivingPrecision\fR (IntState): Flag indicating whether or not to use the arriving particles as they are:
YES keeps the same precision, and NO casts them to the precision specified
by the parameter "InputPrecision".
.DF "YES"
.NE
\fIInputPrecision\fR (PrecisionState): Precision of the input in bits.
The input particles are only cast to this precision if the parameter
"ArrivingPrecision" is set to NO.
.DF "2.14"
.NE
\fIAccumPrecision\fR (PrecisionState): Precision of the accumulation in bits.
.DF "2.14"
.NE
\fIStatePrecision\fR (PrecisionState): Precision of the state in bits.
.DF "2.14"
.NE
\fIOutputPrecision\fR (PrecisionState): Precision of the output in bits.
.DF "2.14"
.ET
.EQ
delim $$
.EN
.LD
.pp
This star implements an infinite impulse response filter of arbitrary order
in a direct form II [1] realization, using fixed-point arithmetic.
The parameters of the star specify $H(z)$, the $Z$-transform of an
impulse response $h(n)$.
The output of the star is the convolution of the input with $h(n)$.
.pp
Note that the numerical finite precision noise increases with the filter order.
To minimize this distortion, it is often desirable to expand the filter
into a parallel or cascade form.
.pp
Quantization is performed in several places.
First, the coefficients are quantized (rounded) to the precision
specified by \fICoefPrecision\fR.
Each time the star fires, the
input is optionally quantized (rounded) to precision specified
by \fIInputPrecision\fR.
The multiplication of the state by the coefficients preserves
full precision, but the result is quantized to \fIAccumPrecision\fR
after being added to other products.
The state variables are stored with the precision given by \fIStatePrecision\fR.
The output is quantized (rounded) to \fIOutputPrecision\fR before being
sent to the output.
.pp
The numerator "tap" coefficients and the denominator "feedback"
coefficients have a default precision of 24 bits.
The number of decimal bits is chosen as the minimum number of bits
needed to represent the number in fixed-point.
One bit is reserved for the sign, and the rest are fractional bits.
During computation of filter outputs, the precision of the filter taps
is converted to the precision contained in the "CoefPrecision" parameter.
.ID "Schafer, R. W."
.ID "Oppenheim, A. V."
.UH REFERENCES
.ip [1]
A. V. Oppenheim and R. W. Schafer, \fIDiscrete-Time Signal Processing\fR,
Prentice-Hall: Englewood Cliffs, NJ, 1989.
.SA
FIR and
Biquad.
.ES
