static const char file_id[] = "SDFBlockAllPole.pl";
// .cc file generated from SDFBlockAllPole.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFBlockAllPole.h"

const char *star_nm_SDFBlockAllPole = "SDFBlockAllPole";

const char* SDFBlockAllPole :: className() const {return star_nm_SDFBlockAllPole;}

ISA_FUNC(SDFBlockAllPole,SDFStar);

Block* SDFBlockAllPole :: makeNew() const { LOG_NEW; return new SDFBlockAllPole;}

SDFBlockAllPole::SDFBlockAllPole ()
{
	setDescriptor("This star implements an all pole filter with the denominator\ncoefficients of the transfer function externally supplied.\nFor each set of coefficients, a block of input samples is processed,\nall in one firing.");
	addPort(signalIn.setPort("signalIn",this,FLOAT));
	addPort(signalOut.setPort("signalOut",this,FLOAT));
	addPort(coefs.setPort("coefs",this,FLOAT));
	addState(blockSize.setState("blockSize",this,"128","Number of inputs that use each each coefficient set."));
	addState(order.setState("order",this,"16","Number of new coefficients to read each time."));

# line 76 "SDFBlockAllPole.pl"
taps = 0;
	    M = 0;
	    fdbkDelayLine = 0;
}

SDFBlockAllPole::~SDFBlockAllPole() {
# line 81 "SDFBlockAllPole.pl"
LOG_DEL; delete [] taps;
	    LOG_DEL; delete [] fdbkDelayLine;
}

void SDFBlockAllPole::setup() {
# line 85 "SDFBlockAllPole.pl"
// delete and remake arrays if a different size is being used.
	    if (int(order) != M) {
		LOG_DEL; delete [] taps;
		LOG_DEL; delete [] fdbkDelayLine;
		M = int(order);
		LOG_NEW; taps = new double[M];
		LOG_NEW; fdbkDelayLine = new double[M];
	    }
	    // set the SDF Params to account for the block processing
	    signalIn.setSDFParams(int(blockSize), int(blockSize)-1);
	    coefs.setSDFParams(M, M - 1);
	    signalOut.setSDFParams(int(blockSize), int(blockSize) - 1);
	    writeIndex = 0;
	    for (int i = 0; i < M; i++) {
		fdbkDelayLine[i]=0.0;
	    }
}

void SDFBlockAllPole::go() {
# line 103 "SDFBlockAllPole.pl"
double out;

	    // first read in new tap values
	    int index = 0;
	    for (int cCount = M-1; cCount >=0; cCount--)
		taps[index++] = coefs%cCount;
	
	    // Iterate for each block
	    for (int j = int(blockSize)-1; j >= 0; j--) {

		   out = 0.0;
		   // Compute the inner product.
		   for (int i = 0; i < M; i++) {
			out += taps[i]
			    * fdbkDelayLine[(writeIndex + i) % M];
		   }
		   // Get the input and add to the inner product output
		   out += double(signalIn%j);

		   // Feed output back to the delay line:
		   writeIndex--;
		   if(writeIndex < 0) writeIndex = M - 1;
		   fdbkDelayLine[writeIndex] = out;

		   // note: output%0 is the last output chronologically
		   signalOut%(j) << out;
	    }
}

// prototype instance for known block list
static SDFBlockAllPole proto;
static RegisterBlock registerBlock(proto,"BlockAllPole");
