static const char file_id[] = "SDFBlockFIR.pl";
// .cc file generated from SDFBlockFIR.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFBlockFIR.h"

const char *star_nm_SDFBlockFIR = "SDFBlockFIR";

const char* SDFBlockFIR :: className() const {return star_nm_SDFBlockFIR;}

ISA_FUNC(SDFBlockFIR,SDFStar);

Block* SDFBlockFIR :: makeNew() const { LOG_NEW; return new SDFBlockFIR;}

SDFBlockFIR::SDFBlockFIR ()
{
	setDescriptor("This star implements an FIR filter with coefficients that\nare periodically updated from the outside. For each set of\ncoefficients, a block of input samples is processed, all in one firing.");
	addPort(signalIn.setPort("signalIn",this,FLOAT));
	addPort(signalOut.setPort("signalOut",this,FLOAT));
	addPort(coefs.setPort("coefs",this,FLOAT));
	addState(blockSize.setState("blockSize",this,"128","Number of inputs that use each each coefficient set."));
	addState(order.setState("order",this,"16","Number of new coefficients to read each time."));
	addState(decimation.setState("decimation",this,"1","Decimation ratio."));
	addState(decimationPhase.setState("decimationPhase",this,"0","Downsampler phase."));
	addState(interpolation.setState("interpolation",this,"1","Interpolation ratio."));

# line 81 "SDFBlockFIR.pl"
lastM = 0;
		taps = 0;
}

SDFBlockFIR::~SDFBlockFIR() {
# line 85 "SDFBlockFIR.pl"
LOG_DEL; delete [] taps;
}

void SDFBlockFIR::setup() {
# line 88 "SDFBlockFIR.pl"
int d = decimation;
	    int i = interpolation;
	    int dP = decimationPhase;

	    if (lastM != int(order)) {
		LOG_DEL; delete [] taps;
		lastM = int(order);
		LOG_NEW; taps = new double[lastM];
	    }

	    // Then set the SDF Params to account for the block processing
	    signalIn.setSDFParams(d*int(blockSize),
		d*int(blockSize)+1+int(order)/i);
	    coefs.setSDFParams(int(order), int(order) - 1);
	    signalOut.setSDFParams(i*int(blockSize),
		i*int(blockSize) - 1);
	    if (dP >= d) {
		Error::abortRun (*this, ": decimationPhase too large");
		return;
	    }
	    // The phaseLength is ceiling(order/interpolation)
	    // It is a protected instance variable.
	    phaseLength = int(order) / i;
	    if ((int(order) % i) != 0) phaseLength++;
}

void SDFBlockFIR::go() {
# line 114 "SDFBlockFIR.pl"
int phase, tapsIndex;
	    double out, tap;

	    // first read in new tap values
	    int index = 0;
	    for (int cCount = int(order)-1; cCount >=0; cCount--)
		taps[index++] = coefs%cCount;
	
	    // Iterate for each block
	    for (int j = int(blockSize)-1; j >= 0; j--) {
	      // phase keeps track of which phase of the
	      //filter coefficients are used.
	      // Starting phase depends on the decimationPhase state.
	      phase = int(decimation) - int(decimationPhase) - 1;   

	      int outCount = int(interpolation)-1;
	
	      // Iterate once for each input consumed
	      for (int inC = 1; inC <= int(decimation); inC++) {
		// Produce however many outputs are required
		// for each input consumed
		while (phase < int(interpolation)) {
		   out = 0.0;
		   // Compute the inner product.
		   for (int i = 0; i < phaseLength; i++) {
			tapsIndex = i * int(interpolation) + phase;
			if (tapsIndex >= int(order))
			    tap = 0.0;
			else
			    tap = taps[tapsIndex];
			out += tap
			    * double(signalIn%(int(decimation) - inC + i
				+ int(decimation)*j));
		   }
		   // note: output%0 is the last output chronologically
		   signalOut%(int(interpolation)*j + outCount--) << out;
		   phase += int(decimation);
		}
		phase -= int(interpolation);
	      }
	    }
}

// prototype instance for known block list
static SDFBlockFIR proto;
static RegisterBlock registerBlock(proto,"BlockFIR");
