static const char file_id[] = "SDFBlockLattice.pl";
// .cc file generated from SDFBlockLattice.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFBlockLattice.h"

const char *star_nm_SDFBlockLattice = "SDFBlockLattice";

const char* SDFBlockLattice :: className() const {return star_nm_SDFBlockLattice;}

ISA_FUNC(SDFBlockLattice,SDFStar);

Block* SDFBlockLattice :: makeNew() const { LOG_NEW; return new SDFBlockLattice;}

SDFBlockLattice::SDFBlockLattice ()
{
	setDescriptor("A block forward lattice filter.\nIt is identical to the Lattice star except that\nthe reflection coefficients are updated each time the star fires\nby reading the \"coefs\" input.\nThe \"order\" parameter indicates how many coefficient\nshould be read.  The \"blockSize\" parameter specifies how many\ndata samples should be processed for each set of coefficients.");
	addPort(signalIn.setPort("signalIn",this,FLOAT));
	addPort(coefs.setPort("coefs",this,FLOAT));
	addState(blockSize.setState("blockSize",this,"128","Number of inputs that use each each coefficient set."));
	addState(order.setState("order",this,"16","Number of new coefficients to read each time."));
	addPort(signalOut.setPort("signalOut",this,FLOAT));

# line 54 "SDFBlockLattice.pl"
b = 0; f = 0; lastM = -1;
		reflectionCoefs = 0;
}

SDFBlockLattice::~SDFBlockLattice() {
# line 58 "SDFBlockLattice.pl"
LOG_DEL; delete [] b;
		LOG_DEL; delete [] f;
		LOG_DEL; delete [] reflectionCoefs;
}

void SDFBlockLattice::setup() {
# line 63 "SDFBlockLattice.pl"
// reallocate arrays only if size has changed,
		// or this is the first run.
		if (lastM != int(order)) {
			lastM = int(order);
			LOG_DEL; delete [] b;
			LOG_NEW; b = new double[lastM];
			LOG_DEL; delete [] f;
			LOG_NEW; f = new double[lastM+1];
			LOG_DEL; delete [] reflectionCoefs;
			LOG_NEW; reflectionCoefs = new double[lastM];
		}
		for (int i=0; i < lastM; i++)  b[i]=0.0 ;
		signalIn.setSDFParams(int(blockSize), int(blockSize)-1);
		coefs.setSDFParams(int(order), int(order)-1);
		signalOut.setSDFParams(int(blockSize), int(blockSize)-1);
}

void SDFBlockLattice::go() {
# line 81 "SDFBlockLattice.pl"
// first read in new tap values
	    int index = 0;
	    for (int cCount = int(order)-1; cCount >=0; cCount--)
		reflectionCoefs[index++] = coefs%cCount;

	    // Iterate for each block
	    for (int j = int(blockSize)-1; j >= 0; j--) {
		double k;
		
		// Forward prediction error
		f[0] = double (signalIn%j);   // f(0)=x(n)
		for (int i=1; i <= lastM; i++) {
			k = - reflectionCoefs[i-1];
			f[i] = k * b[i-1] + f[i-1];
		}
		signalOut%j << f[lastM];

		// Backward:  Compute the w's for the next round
		for (i = lastM-1; i >0 ; i--) {
			k = - reflectionCoefs[i-1];
			b[i] = k * f[i-1] + b[i-1];	
		}
		b[0] = double (signalIn%j);   // b[0]=x[n] 
	    }
}

// prototype instance for known block list
static SDFBlockLattice proto;
static RegisterBlock registerBlock(proto,"BlockLattice");
