static const char file_id[] = "SDFBlockRLattice.pl";
// .cc file generated from SDFBlockRLattice.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFBlockRLattice.h"

const char *star_nm_SDFBlockRLattice = "SDFBlockRLattice";

const char* SDFBlockRLattice :: className() const {return star_nm_SDFBlockRLattice;}

ISA_FUNC(SDFBlockRLattice,SDFStar);

Block* SDFBlockRLattice :: makeNew() const { LOG_NEW; return new SDFBlockRLattice;}

SDFBlockRLattice::SDFBlockRLattice ()
{
	setDescriptor("A block recursive (IIR) lattice filter.\nIt is identical to the RLattice star, except that the reflection\ncoefficients are updated each time the star fires by reading the\n\"coefs\" input. The \"order\" parameter indicates how many coefficient\nshould be read. The \"blockSize\" parameter specifies how many data\nsamples should be processed for each set of coefficients.");
	addPort(signalIn.setPort("signalIn",this,FLOAT));
	addPort(coefs.setPort("coefs",this,FLOAT));
	addPort(signalOut.setPort("signalOut",this,FLOAT));
	addState(blockSize.setState("blockSize",this,"128","Number of inputs that use each each coefficient set."));
	addState(order.setState("order",this,"16","Number of new coefficients to read each time."));

# line 53 "SDFBlockRLattice.pl"
w = 0; y = 0; M = -1;
		reflectionCoefs = 0;
}

SDFBlockRLattice::~SDFBlockRLattice() {
# line 57 "SDFBlockRLattice.pl"
LOG_DEL; delete [] w;
		LOG_DEL; delete [] y;
		LOG_DEL; delete [] reflectionCoefs;
}

void SDFBlockRLattice::setup() {
# line 62 "SDFBlockRLattice.pl"
// reallocate arrays only if size has changed,
		// or this is the first run.
		if (M != int(order)) {
			M = int(order);
			LOG_DEL; delete [] w;
			LOG_NEW; w = new double[M+1];
			LOG_DEL; delete [] y;
			LOG_NEW; y = new double[M+1];
			LOG_DEL; delete [] reflectionCoefs;
			LOG_NEW; reflectionCoefs = new double[M];
		}
		for (int i=0; i <= M; i++)  w[i]=0.0 ;
		signalIn.setSDFParams(int(blockSize), int(blockSize)-1);
		coefs.setSDFParams(int(order), int(order)-1);
		signalOut.setSDFParams(int(blockSize), int(blockSize)-1);
}

void SDFBlockRLattice::go() {
# line 79 "SDFBlockRLattice.pl"
// first read in new tap values
            int index = 0;
	    for (int cCount = int(order)-1; cCount >=0; cCount--)
		reflectionCoefs[index++] = coefs%cCount;

            // Iterate for each block
            for (int j = int(blockSize)-1; j >= 0; j--) {
		double k;
		
		// Forward:  Compute each of the Y values
		y[0] = double (signalIn%j);   // y(0)=x(n)
		for (int i=1; i <= M; i++) {
			k = reflectionCoefs[M-i];
			y[i] = k * w[i] + y[i-1];
		}
		signalOut%j << y[M];

		// Backward:  Compute the w's for the next round
		for (i = 1; i < M ; i++) {
			k = - reflectionCoefs[M-1-i];
			w[i] = w[i+1] + k*y[i+1];	
		}
		w[M] = y[M];
	    }
}

// prototype instance for known block list
static SDFBlockRLattice proto;
static RegisterBlock registerBlock(proto,"BlockRLattice");
