static const char file_id[] = "SDFBurg.pl";
// .cc file generated from SDFBurg.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFBurg.h"

const char *star_nm_SDFBurg = "SDFBurg";

const char* SDFBurg :: className() const {return star_nm_SDFBurg;}

ISA_FUNC(SDFBurg,SDFStar);

Block* SDFBurg :: makeNew() const { LOG_NEW; return new SDFBurg;}

SDFBurg::SDFBurg ()
{
	setDescriptor("This star uses Burg's algorithm to estimate the linear predictor coefficients\nof an input random process.\nThese coefficients are produced both in autoregressive form\n(on the \"lp\" output) and in lattice filter form (on the \"refl\" output).\nThe \"errPower\" output is the power of the prediction error as a function\nof the predictor order.");
	addPort(input.setPort("input",this,FLOAT));
	addPort(lp.setPort("lp",this,FLOAT));
	addPort(refl.setPort("refl",this,FLOAT));
	addPort(errPower.setPort("errPower",this,FLOAT));
	addState(order.setState("order",this,"8","The number of reflection coefficients to generate."));
	addState(numInputs.setState("numInputs",this,"64","The number of inputs used to estimate the model."));

# line 117 "SDFBurg.pl"
f = b = aOrig = aPrime = 0;
		N = M = 0;
}

SDFBurg::~SDFBurg() {
# line 121 "SDFBurg.pl"
LOG_DEL; delete [] f;
		LOG_DEL; delete [] b;
		LOG_DEL; delete [] aOrig;
		LOG_DEL; delete [] aPrime;
}

void SDFBurg::setup() {
# line 127 "SDFBurg.pl"
if (N != int(numInputs)) {
			LOG_DEL; delete [] f;
			LOG_DEL; delete [] b;
			N = int(numInputs);
			LOG_NEW; f = new double[N];
			LOG_NEW; b = new double[N];
		}
		if (M != int(order)) {
			LOG_DEL; delete [] aOrig;
			LOG_DEL; delete [] aPrime;
			M = int(order);
			LOG_NEW; aOrig = new double[M+1];
			LOG_NEW; aPrime = new double[M+1];
		}
		refl.setSDFParams (int(order), int(order)-1);
		lp.setSDFParams (int(order), int(order)-1);
		errPower.setSDFParams (int(order)+1, int(order));
		input.setSDFParams (int(numInputs), int(numInputs)-1);
}

void SDFBurg::go() {
# line 147 "SDFBurg.pl"
// Define pointers so that the arrays can be swapped
	    double* a = aOrig;
	    double* aP = aPrime;
	    double gamma, dsum, nsum;

	    // initialize the forward and backward predictor errors
	    // and the prediction error power estimate
	    int count = 0;
	    double ep = 0.0;	// error power estimate
	    for (int i = N-1; i >= 0; i--) {
		double x = input%i;
		f[count] = x;
		b[count++] = x;
		ep += x * x;
	    }
	    ep = ep/N;
	    // output the zeroth order prediction error, which is simply
	    // the power estimate of the input
	    errPower%M << ep;

	    // Iterate on the predictor order
	    for (int m = 1; m <= M; m++ ) {
	        // Compute the reflection coefficients, and output them
		nsum = 0.0;
		dsum = 0.0;
		for (i = m; i < N; i++) {
		    nsum += f[i]*b[i-1];
		    dsum += f[i]*f[i] + b[i-1]*b[i-1];
		}
		gamma = -2*nsum/dsum;
		refl%(M-m) << - gamma;

		// update the forward and backward predictor errors
		for (i = N-1; i >= m; i--) {
		    double tempf = f[i];
		    f[i] = tempf + gamma*b[i-1];
		    b[i] = b[i-1] + gamma*tempf;
		}

		// update the prediction error power estimate
		ep = (1 - gamma*gamma) * ep;
		errPower%(M-m) << ep;

		// Update the FIR predictor coefficient estimates
		for (i = 1; i < m; i++) {
		    aP[i] = a[i] + gamma * a[m-i];
		}
		aP[m] = gamma;

		// Swap a and aP for next order recurrence
		double* temp = a;
		a = aP;
		aP = temp;
	    }
	    // generate the lp outputs
	    for (m = 1; m <= M; m++ ) {
		lp%(M-m) << -a[m];
	    }
}

// prototype instance for known block list
static SDFBurg proto;
static RegisterBlock registerBlock(proto,"Burg");
