static const char file_id[] = "SDFDirichlet.pl";
// .cc file generated from SDFDirichlet.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFDirichlet.h"
#include <math.h>

const char *star_nm_SDFDirichlet = "SDFDirichlet";

const char* SDFDirichlet :: className() const {return star_nm_SDFDirichlet;}

ISA_FUNC(SDFDirichlet,SDFStar);

Block* SDFDirichlet :: makeNew() const { LOG_NEW; return new SDFDirichlet;}

SDFDirichlet::SDFDirichlet ()
{
	setDescriptor("This star computes the normalized Dirichlet kernel (also called the aliased\nsinc function):\n\nd(x) = sin(N x / 2) / (N sin(x/2))\n\nThe value of the normalized Dirichlet kernel at x = 0 is always 1, and the\nnormalized Dirichlet kernel oscillates between -1 and +1.\nThe normalized Dirichlet kernel is periodic in x with a period of either\n2*pi when N is odd or 4*pi when N is even.");
	addPort(input.setPort("input",this,FLOAT));
	addPort(output.setPort("output",this,FLOAT));
	addState(N.setState("N",this,"10","Length of the Dirichlet kernel."));


}

void SDFDirichlet::go() {
# line 54 "SDFDirichlet.pl"
const double DELTA = 1.0e-9;
		double x = input%0;
		double sinInDenom = sin(x/2);
		double length = double(int(N));
		double dirichletValue;

		if ( ( -DELTA < sinInDenom ) && ( sinInDenom < DELTA ) )
		  // Use L'Hopital's Rule when sin(x/2) is approx. 0
		  dirichletValue = cos(length * x / 2) / cos(x / 2);
		else
		  // Otherwise, compute it using the definition
		  dirichletValue = sin(length * x / 2) / (length * sinInDenom);

		output%0 << dirichletValue;
}

// prototype instance for known block list
static SDFDirichlet proto;
static RegisterBlock registerBlock(proto,"Dirichlet");
