static const char file_id[] = "SDFFFTCx.pl";
// .cc file generated from SDFFFTCx.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFFFTCx.h"

const char *star_nm_SDFFFTCx = "SDFFFTCx";

const char* SDFFFTCx :: className() const {return star_nm_SDFFFTCx;}

ISA_FUNC(SDFFFTCx,SDFStar);

Block* SDFFFTCx :: makeNew() const { LOG_NEW; return new SDFFFTCx;}

# line 84 "SDFFFTCx.pl"
/*
 * This fft routine is from ~gabriel/src/filters/fft/fft.c;
 * I am unsure of the original source.  The file contains no
 * copyright notice or description.
 * The declaration is changed to the prototype form but the
 * function body is unchanged.  (J. T. Buck)
 */

#define SWAP(a, b) tempr=(a); (a)=(b); (b)=tempr

/*
 * Replace data by its discrete Fourier transform, if isign is
 * input as 1, or by its inverse discrete Fourier transform, if 
 * "isign" is input as -1.  "data'"is a complex array of length "nn",
 * input as a real array data[0..2*nn-1]. "nn" MUST be an integer
 * power of 2 (this is not checked for!?)
 */

static void fft_rif(double *data, int nn, int isign)
{
	int	n;
	int	mmax;
	int	m, j, istep, i;
	double	wtemp, wr, wpr, wpi, wi, theta;
	double	tempr, tempi;

	data--;
	n = nn << 1;
	j = 1;

	for (i = 1; i < n; i += 2) {
		if(j > i) {
			SWAP(data[j], data[i]);
			SWAP(data[j+1], data[i+1]);
		}
		m= n >> 1;
		while (m >= 2 && j >m) {
			j -= m;
			m >>= 1;
		}
		j += m;
	}
	mmax = 2;
	while (n > mmax) {
		istep = 2*mmax;
		theta = -6.28318530717959/(isign*mmax);
		wtemp = sin(0.5*theta);
		wpr = -2.0*wtemp*wtemp;
		wpi = sin(theta);
		wr = 1.0;
		wi = 0.0;
		for (m = 1; m < mmax; m += 2) {
			for (i = m; i < n; i += istep) {
				j = i + mmax;
				tempr = wr*data[j] - wi*data[j+1];
				tempi = wr*data[j+1] + wi*data[j];
				data[j] = data[i] - tempr;
				data[j+1] = data[i+1] - tempi;
				data[i] += tempr;
				data[i+1] += tempi;
			}
			wr = (wtemp=wr)*wpr - wi*wpi+wr;
			wi = wi*wpr + wtemp*wpi + wi;
		}
		mmax = istep;
	}
}

SDFFFTCx::SDFFFTCx ()
{
	setDescriptor("Compute the discrete Fourier transform of a complex input using the\nfast Fourier transform (FFT) algorithm.  The star reads \"size\" (default\n256) complex samples, zero pads the data if necessary, and then takes an\nFFT of length 2 ^ \"order\", where \"size\" <= 2 ^ \"order\".  (The default\nvalue of \"order\" is 8.)  The parameter \"direction\" (default 1) is 1 for\nthe forward and -1 for the inverse FFT.");
	addPort(input.setPort("input",this,COMPLEX));
	addPort(output.setPort("output",this,COMPLEX));
	addState(order.setState("order",this,"8","Log base 2 of the transform size."));
	addState(size.setState("size",this,"256","Number of input samples to read."));
	addState(direction.setState("direction",this,"1","Equals 1 for forward, or -1 for inverse, transform."));

# line 155 "SDFFFTCx.pl"
data = 0;
}

SDFFFTCx::~SDFFFTCx() {
# line 158 "SDFFFTCx.pl"
LOG_DEL; delete [] data;
}

void SDFFFTCx::setup() {
# line 161 "SDFFFTCx.pl"
fftSize = 1 << int(order);
		if (fftSize < int(size)) {
			Error::abortRun(*this, "2^order must be >= size");
			return;
		}
		LOG_DEL; delete [] data;
		LOG_NEW; data = new double[2*fftSize];
		input.setSDFParams (int(size), int(size)-1);
		output.setSDFParams (fftSize, fftSize-1);
}

void SDFFFTCx::go() {
# line 172 "SDFFFTCx.pl"
// load up the array
		double* p = data;

		// note: particle at maximum delay is the first one
		for (int i = int(size)-1; i >= 0; i--) {
			Complex t = Complex(input%i);
			*p++ = t.real();
			*p++ = t.imag();
		}
		for (i = size; i < fftSize; i++) {
			*p++ = 0.0;
			*p++ = 0.0;
		}
		fft_rif (data, fftSize, int(direction));
		// generate output data.  If inverse, we scale the result.
		if (int(direction) != 1)
			for (i = 0; i < 2*fftSize; i++)
				data[i] /= fftSize;

		// send the data through the output port
		p = data;
		for (i = fftSize-1; i >= 0; i--) {
			output%i << Complex(p[0], p[1]);
			p += 2;
		}
}

// prototype instance for known block list
static SDFFFTCx proto;
static RegisterBlock registerBlock(proto,"FFTCx");
