static const char file_id[] = "SDFFIR.pl";
// .cc file generated from SDFFIR.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFFIR.h"

const char *star_nm_SDFFIR = "SDFFIR";

const char* SDFFIR :: className() const {return star_nm_SDFFIR;}

ISA_FUNC(SDFFIR,SDFStar);

Block* SDFFIR :: makeNew() const { LOG_NEW; return new SDFFIR;}

SDFFIR::SDFFIR ()
{
	setDescriptor("A finite impulse response (FIR) filter.\nCoefficients are specified by the \"taps\" parameter.\nThe default coefficients give an 8th order, linear phase lowpass\nfilter. To read coefficients from a file, replace the default\ncoefficients with \"< fileName\", preferably specifying a complete path.\nPolyphase multirate filtering is also supported.");
	addPort(signalIn.setPort("signalIn",this,FLOAT));
	addPort(signalOut.setPort("signalOut",this,FLOAT));
	addState(taps.setState("taps",this,"-.040609 -.001628 .17853 .37665 .37665 .17853 -.001628 -.040609","Filter tap values."));
	addState(decimation.setState("decimation",this,"1","Decimation ratio."));
	addState(decimationPhase.setState("decimationPhase",this,"0","Downsampler phase."));
	addState(interpolation.setState("interpolation",this,"1","Interpolation ratio."));


}

void SDFFIR::setup() {
# line 118 "SDFFIR.pl"
int d = decimation;
		int i = interpolation;
		int dP = decimationPhase;
		signalIn.setSDFParams(d, d+1+(taps.size()/i));
		signalOut.setSDFParams(i, i-1);
		if (dP >= d) {
			Error::abortRun (*this, ": decimationPhase too large");
			return;
		}
		// The phaseLength is ceiling(taps.size/interpolation)
		// It is a protected instance variable.
		phaseLength = taps.size() / i;
		if ((taps.size() % i) != 0) phaseLength++;
}

void SDFFIR::go() {
# line 133 "SDFFIR.pl"
int phase, tapsIndex;
	    int Interp = interpolation;
	    int Decim = decimation;
	    int outCount = Interp-1;
	    double out, tap;
	
	    // phase keeps track of which phase of the filter coefficients are used.
	    // Starting phase depends on the decimationPhase state.
	    phase = Decim - int(decimationPhase) - 1;   
	
	    // Interpterate once for each input consumed
	    for (int inC = 1; inC <= Decim; inC++) {
		// Produce however many outputs are required
		// for each input consumed
		while (phase < Interp) {
		   out = 0.0;
		   // Compute the inner product.
		   for (int i = 0; i < phaseLength; i++) {
			tapsIndex = i * Interp + phase;
			if (tapsIndex >= taps.size())
			    tap = 0.0;
			else
			    tap = taps[tapsIndex];
			out += tap * double(signalIn%(Decim - inC + i));
		   }
		   // note: output%0 is the last output chronologically
		   signalOut%(outCount--) << out;
		   phase += Decim;
		}
		phase -= Interp;
	    }
}

// prototype instance for known block list
static SDFFIR proto;
static RegisterBlock registerBlock(proto,"FIR");
