static const char file_id[] = "SDFFIRCx.pl";
// .cc file generated from SDFFIRCx.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFFIRCx.h"

const char *star_nm_SDFFIRCx = "SDFFIRCx";

const char* SDFFIRCx :: className() const {return star_nm_SDFFIRCx;}

ISA_FUNC(SDFFIRCx,SDFStar);

Block* SDFFIRCx :: makeNew() const { LOG_NEW; return new SDFFIRCx;}

SDFFIRCx::SDFFIRCx ()
{
	setDescriptor("A complex finite impulse response (FIR) filter.\nCoefficients are specified by the \"taps\" parameter.\nDefault coefficients give an 8th order, linear phase, lowpass filter.\nTo read coefficients from a file, use the syntax: \"< fileName\",\npreferably specifying a complete path.\nReal and imaginary parts should be paired with parentheses, e.g. (1.0, 0.0).\nPolyphase multirate filtering is also supported.");
	addPort(signalIn.setPort("signalIn",this,COMPLEX));
	addPort(signalOut.setPort("signalOut",this,COMPLEX));
	addState(taps.setState("taps",this,"(-.040609,0.0) (-.001628,0.0) (.17853,0.0) (.37665,0.0)" "(.37665,0.0) (.17853,0.0) (-.001628,0.0) (-.040609,0.0)","Filter tap values."));
	addState(decimation.setState("decimation",this,"1","Decimation ratio."));
	addState(decimationPhase.setState("decimationPhase",this,"0","Downsampler phase."));
	addState(interpolation.setState("interpolation",this,"1","Interpolation ratio."));


}

void SDFFIRCx::setup() {
# line 119 "SDFFIRCx.pl"
int d = decimation;
		int i = interpolation;
		int dP = decimationPhase;
		signalIn.setSDFParams(d, d+1+(taps.size()/i));
		signalOut.setSDFParams(i, i-1);
		if (dP >= d) {
			Error::abortRun (*this, ": decimationPhase too large");
			return;
		}
		// The phaseLength is ceiling(taps.size/interpolation)
		// It is a protected instance variable.
		phaseLength = taps.size() / i;
		if ((taps.size() % i) != 0) phaseLength++;
}

void SDFFIRCx::go() {
# line 134 "SDFFIRCx.pl"
int phase, tapsIndex;
	    int outCount = int(interpolation)-1;
	    Complex out, tap;
	
	    // phase keeps track of which phase of the filter coefficients are used.
	    // Starting phase depends on the decimationPhase state.
	    phase = int(decimation) - int(decimationPhase) - 1;   
	
	    // Iterate once for each input consumed
	    for (int inC = 1; inC <= int(decimation); inC++) {
		// Produce however many outputs are required for each input consumed
		while (phase < int(interpolation)) {
		   out = 0.0;
		   // Compute the inner product.
		   for (int i = 0; i < phaseLength; i++) {
			tapsIndex = i * int(interpolation) + phase;
			if (tapsIndex >= taps.size())
			    tap = 0.0;
			else
			    tap = taps[tapsIndex];
			out += tap * Complex(signalIn%(int(decimation) - inC + i));
		   }
		   // note: output%0 is the last output chronologically
		   signalOut%(outCount--) << out;
		   phase += int(decimation);
		}
		phase -= int(interpolation);
	    }
}

// prototype instance for known block list
static SDFFIRCx proto;
static RegisterBlock registerBlock(proto,"FIRCx");
