static const char file_id[] = "SDFFIRFix.pl";
// .cc file generated from SDFFIRFix.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFFIRFix.h"

const char *star_nm_SDFFIRFix = "SDFFIRFix";

const char* SDFFIRFix :: className() const {return star_nm_SDFFIRFix;}

ISA_FUNC(SDFFIRFix,SDFFix);

Block* SDFFIRFix :: makeNew() const { LOG_NEW; return new SDFFIRFix;}

SDFFIRFix::SDFFIRFix ()
{
	setDescriptor("A finite impulse response (FIR) filter with fixed-point capabilities.\nThe fixed-point coefficients are specified by the \"taps\" parameter.\nThe default coefficients give an 8th-order, linear phase lowpass filter.\nTo read coefficients from a file, replace the default coefficients\nwith \"<fileName\", preferably specifying a complete path.\nPolyphase multirate filtering is also supported.");
	addPort(signalIn.setPort("signalIn",this,FIX));
	addPort(signalOut.setPort("signalOut",this,FIX));
	addState(taps.setState("taps",this,"-.040609 -.001628 .17853 .37665 .37665 .17853 -.001628 -.040609","Filter tap values."));
	addState(decimation.setState("decimation",this,"1","Decimation ratio."));
	addState(decimationPhase.setState("decimationPhase",this,"0","Downsampler phase."));
	addState(interpolation.setState("interpolation",this,"1","Interpolation ratio."));
	addState(ArrivingPrecision.setState("ArrivingPrecision",this,"YES","Flag indicating whether or not to use the arriving particles as they are:\nYES keeps the same precision, and NO casts them to the precision specified\nby the parameter \"InputPrecision\"."));
	addState(InputPrecision.setState("InputPrecision",this,"4.14","Precision of the input in bits.\nThe input particles are only cast to this precision if the parameter\n\"ArrivingPrecision\" is set to NO."));
	addState(TapPrecision.setState("TapPrecision",this,"4.14","Precision of the taps in bits."));
	addState(AccumulationPrecision.setState("AccumulationPrecision",this,"4.14","Precision of the accumulation in bits."));
	addState(OutputPrecision.setState("OutputPrecision",this,"4.14","Precision of the output in bits."));


}

void SDFFIRFix::setup() {
# line 174 "SDFFIRFix.pl"
SDFFix::setup();

                int d = decimation;
                int i = interpolation;
                int dP = decimationPhase;
                signalIn.setSDFParams(d, d+1+(taps.size()/i));
                signalOut.setSDFParams(i, i-1);
                if (dP >= d) {
                  Error::abortRun (*this, ": decimationPhase too large");
                  return;
                }
                // The phaseLength is ceiling(taps.size/interpolation)
                // It is a protected instance variable.
                phaseLength = taps.size() / i;
                if ((taps.size() % i) != 0) phaseLength++;

		// Set the precision on the fixed-point variables
		fixIn = Fix( ((const char *) InputPrecision) );
		if ( fixIn.invalid() )
		  Error::abortRun( *this, "Invalid InputPrecision" );

		tap = Fix( ((const char *) TapPrecision) );
		if ( tap.invalid() )
		  Error::abortRun( *this, "Invalid TapPrecision" );

		Accum = Fix( ((const char *) AccumulationPrecision) );
		if ( Accum.invalid() )
		  Error::abortRun( *this, "Invalid AccumulationPrecision" );

		out = Fix( ((const char *) OutputPrecision) );
		if ( out.invalid() )
		  Error::abortRun( *this, "Invalid OutputPrecision" );

		// Set the overflow handlers for assignments/computations
		fixIn.set_ovflow( ((const char *) OverflowHandler) );
		if ( fixIn.invalid() )
		  Error::abortRun( *this, "Invalid OverflowHandler" );
		tap.set_ovflow( ((const char *) OverflowHandler) );
		Accum.set_ovflow( ((const char *) OverflowHandler) );
		out.set_ovflow( ((const char *) OverflowHandler) );

		// Set all fixed-point assignments/computations to use rounding
		fixIn.set_rounding( int(RoundFix) );
		tap.set_rounding( int(RoundFix) );
		Accum.set_rounding( int(RoundFix) );
		out.set_rounding( int(RoundFix) );
}

void SDFFIRFix::go() {
# line 222 "SDFFIRFix.pl"
int phase, tapsIndex;
            int Interp = interpolation;
            int Decim = decimation;
            int outCount = Interp - 1;

            // phase keeps track of which phase of filter coefficients are used
            // Starting phase depends on the decimationPhase state.
            phase = Decim - int(decimationPhase) - 1;

            // Interpolate once for each input consumed
            for (int inC = 1; inC <= Decim; inC++) {
                // Produce however many outputs are required
		// for each input consumed
                while (phase < Interp) {
                   Accum = 0.0;
                   // Compute the inner product.
                   for ( int i = 0; i < phaseLength; i++ ) {
                        tapsIndex = i * Interp + phase;
                        if (tapsIndex >= taps.size())
                            tap = 0.0;
                        else 
                            tap = taps[tapsIndex];
        
                        if ( int(ArrivingPrecision) )
                            Accum += tap * Fix(signalIn%(Decim - inC + i));
                        else {
                            fixIn = Fix(signalIn%(Decim - inC + i));
                            Accum += tap * fixIn;
			}
                   }
                   out = Accum;
                   // note: output%0 is the last output chronologically
                   signalOut%(outCount--) << out;
                   phase += Decim;
                }
                phase -= Interp;
            }
}

// prototype instance for known block list
static SDFFIRFix proto;
static RegisterBlock registerBlock(proto,"FIRFix");
