static const char file_id[] = "SDFGAL.pl";
// .cc file generated from SDFGAL.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFGAL.h"

const char *star_nm_SDFGAL = "SDFGAL";

const char* SDFGAL :: className() const {return star_nm_SDFGAL;}

ISA_FUNC(SDFGAL,SDFStar);

Block* SDFGAL :: makeNew() const { LOG_NEW; return new SDFGAL;}

SDFGAL::SDFGAL ()
{
	setDescriptor("Gradient adaptive lattice filter.");
	addPort(input.setPort("input",this,FLOAT));
	addPort(residual.setPort("residual",this,FLOAT));
	addState(order.setState("order",this,"1","Lattice filter order."));
	addState(timeConstant.setState("timeConstant",this,"1.0","Adaptation time constant."));

# line 58 "SDFGAL.pl"
f = b = k = sigma = 0;
	previousOrder = -1;
}

SDFGAL::~SDFGAL() {
# line 64 "SDFGAL.pl"
LOG_DEL; delete [] f;
	LOG_DEL; delete [] b;
	LOG_DEL; delete [] k;
	LOG_DEL; delete [] sigma;
}

void SDFGAL::setup() {
# line 72 "SDFGAL.pl"
// Reallocate arrays only if size had changed.
	if (previousOrder != order)
	{
	    // Delete left-over arrays.
	    LOG_DEL; delete [] f;
	    LOG_DEL; delete [] b;
	    LOG_DEL; delete [] k;
	    LOG_DEL; delete [] sigma;

	    // Allocate new arrays.
	    LOG_NEW; f = new double[order+1];
	    LOG_NEW; b = new double[order+1];
	    LOG_NEW; k = new double[order+1];
	    LOG_NEW; sigma = new double[order+1];

	    previousOrder = order;
	}

	// Zero out old values.
	for(int m = 0; m <= order; m++)
	{
	    f[m] = b[m] = k[m] = sigma[m] = 0.0;
	}

	double tc = timeConstant;
	beta = (tc - 1.0) / (tc + 1.0);
	alpha = 1.0 - beta;
}

void SDFGAL::go() {
# line 103 "SDFGAL.pl"
// update forward errors
        f[0] = input%0;
        for(int m = 1; m <= order; m++)
        {
            f[m] = f[m-1] - k[m]*b[m-1];
        }

	// update backward errors and reflection coefficients
	for(m = order; m > 0; m--)
	{
	    b[m] = b[m-1] - k[m]*f[m-1];
	    sigma[m] *= beta;
	    sigma[m] += (1.0 - beta) * (f[m-1]*f[m-1] + b[m-1]*b[m-1]);
	    if (sigma[m] != 0.0)
	    {
		k[m] += alpha * (f[m]*b[m-1] + b[m]*f[m-1]) / sigma[m];
		if (k[m] > 1.0) k[m] = 1.0;
		if (k[m] < -1.0) k[m] = -1.0;
	    }
	}

    	b[0] = input%0;
    	residual%0 << f[order];
}

// prototype instance for known block list
static SDFGAL proto;
static RegisterBlock registerBlock(proto,"GAL");
