static const char file_id[] = "SDFGLA.pl";
// .cc file generated from SDFGLA.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFGLA.h"
#include "Matrix.h"

const char *star_nm_SDFGLA = "SDFGLA";

const char* SDFGLA :: className() const {return star_nm_SDFGLA;}

ISA_FUNC(SDFGLA,SDFStar);

Block* SDFGLA :: makeNew() const { LOG_NEW; return new SDFGLA;}

# line 72 "SDFGLA.pl"
// Code for NN function which is to find the index of nearest neighbor
     // in the codebook for the training vector, and find the squared error
     // (squared distance) between the training vector and its nearest neighbor.

     static void NN(int& index, double& distance,
                    const double* trnVector, const double* codebook,
		    const double* halfCodewordPower,
		    int sizeCodebook, int dimension) {

	// If X=input vector, and Yi=i_th codeword.
	// Find the nearest neighbor codeword Yi to maximize X'*Yi-Ai
	// (' means transpose ), where Ai=||Yi||^2/2 and should have
	// already been stored in the array halfCodewordPower[numCodewords].

	index = 0;
	distance = 0;
	for ( int j=0; j<dimension; j++ ) {
	    distance += trnVector[j]*codebook[j];
	}
	distance -= halfCodewordPower[0];

	double sum = 0.0;
	for ( int i = 1; i < sizeCodebook; i++ ) {
	    for ( int j = 0; j < dimension; j++ )
	        sum += trnVector[j]*codebook[i*dimension+j];
	    sum -= halfCodewordPower[i];
	    if ( sum > distance ) {
	       distance = sum;
	       index = i;
	    }
	    sum = 0.0;
	}

	// Distance we compute here is the X'*Yi-||Yi||^2/2, but actually the 
	// squared distance is ||X-Yi||^2 = ||X||^2+||Yi||^2-2*X'*Yi. So, we
	// need to calculate the true squared distance before return. 
	distance *= -2;
	for (int k = 0; k < dimension; k++) {
	  distance += trnVector[k] * trnVector[k];
	}
     }	// end of NN function

SDFGLA::SDFGLA ()
{
	setDescriptor("Use the Generalized Lloyd Algorithm (GLA) to yield a codebook from input  \ntraining vectors.  Note that each input matrix will be viewed as a\nrow vector in row by row. Each row of output matrix represent a\ncodeword of the codebook.");
	addPort(input.setPort("input",this,FLOAT_MATRIX_ENV));
	addPort(output.setPort("output",this,FLOAT_MATRIX_ENV));
	addState(dimension.setState("dimension",this,"16","The dimension for the input training vector."));
	addState(sizeTrnSet.setState("sizeTrnSet",this,"1024","The number of input training vectors for yielding the codebook."));
	addState(sizeCodebook.setState("sizeCodebook",this,"64","The number of desired codewords for the codebook."));
	addState(thresholdRatio.setState("thresholdRatio",this,"0","A number between 0 to 1. When equal to 0, the resulting codebook will satisfy the necessary conditions for optimality."));
	addState(maxIteration.setState("maxIteration",this,"100","The upper limit on the iterations in the Lloyd algorithm."));
	addState(saveFile.setState("saveFile",this,"","File to save the final codebook."));

# line 124 "SDFGLA.pl"
codebook = 0;
    trnSet = 0;
    numPart = 0;
    centroid = 0;
    halfCodewordPower = 0;
}

void SDFGLA::wrapup() {
# line 325 "SDFGLA.pl"
const char* sf = saveFile;
    if (sf != 0 && *sf != 0) {
      char* saveFileName = expandPathName(sf);
      // open the file for writing
      FILE* fp = fopen(saveFileName, "w");
      if ( fp ) {
         for (int i = 0; i < int(sizeCodebook); i++) {
           for (int j = 0; j < int(dimension); j++) {
             fprintf(fp, "%e ", codebook[i*int(dimension)+j]);
	   }
           fprintf(fp,"\n");
         }
      }
      else {
         Error::warn(*this, "Cannot open '", saveFile,
		     "' for writing: codebook not saved.");
      }
      fclose(fp);
      delete [] saveFileName;
    }
}

SDFGLA::~SDFGLA() {
# line 132 "SDFGLA.pl"
delete [] codebook;
    delete [] trnSet;
    delete [] numPart;
    delete [] centroid;
    delete [] halfCodewordPower;
}

void SDFGLA::setup() {
# line 140 "SDFGLA.pl"
if ( int(sizeCodebook) > int(sizeTrnSet) ) {
	Error::abortRun(*this,
			"The size of codebook must be smaller than ",
			"the size of training set");
	return;
    }

    input.setSDFParams(int(sizeTrnSet), int(sizeTrnSet)-1);

    //  Allocate an array for storing the generated codebook
    delete [] codebook;
    codebook = new double[int(sizeCodebook)*int(dimension)];

    //  Allocate an array for storing the training vectors
    delete [] trnSet;
    trnSet = new double[int(sizeTrnSet)*int(dimension)];

    //  Allocate an array to counter how many input training vectors
    //  are closest to the i_th codeword
    delete [] numPart;
    numPart = new int[int(sizeCodebook)];

    //  Allocate an array to store the centroids of the codebook
    delete [] centroid;
    centroid = new double[int(sizeCodebook)*int(dimension)];

    // halfCodewordPower[i] store the value of Ai=||Yi||^2/2.
    delete [] halfCodewordPower;
    halfCodewordPower = new double[int(sizeCodebook)];
}

void SDFGLA::go() {
# line 172 "SDFGLA.pl"
//  Get the input training vectors and store them in the 2-dimension array
    Envelope inpkt;
    FloatMatrix zerovector(1, int(dimension));
    zerovector = 0;
    int i;
    for (i = 0; i < int(sizeTrnSet); i++) {
      const FloatMatrix* vectorp;
      (input%(int(sizeTrnSet)-1-i)).getMessage(inpkt);
      vectorp = (const FloatMatrix *)inpkt.myData();
      // check for "null" matrix inputs, caused by delays: treat as zero matrix
      if (inpkt.empty()) {
        vectorp = &zerovector;
      }
      if ( vectorp->numRows()*vectorp->numCols() != int(dimension) ) {
	Error::message(*this,
		       "The number of elements in input vector(matrix) ",
		       "must equal to the parameter 'dimension'.");
	return;
      }
      int rowloc = i * int(dimension);
      for (int j = 0; j < int(dimension); j++) {
	trnSet[rowloc + j] = vectorp->entry(j);
      }
    }
 
    // Initialize the initial codebook using random code, ie, we take 
    // every Kth training vector among the input traing vectors, where
    // K = sizeTrnSet/sizeCodebook.
    const int K = int(sizeTrnSet)/int(sizeCodebook);
    for (i = 0; i < int(sizeCodebook); i++) {
      int rowloc = i * int(dimension);
      for (int j = 0; j < int(dimension); j++) {
        codebook[rowloc + j] = trnSet[K * rowloc + j];
      }
    }

    //  numPart[i] stores the number of how many input training vectors
    //  are closest to the i_th codeword in the codebook after using the 
    //  nearest neighbor condition to partition the training set.
    for (i = 0; i < int(sizeCodebook); i++) {
      numPart[i] = 0;
    }

    //  The centroid[int(sizeCodebook)*int(dimension)] array stores
    //  the centroid of each partitioned cluster of training set.
    for (i = 0; i < int(sizeCodebook); i++) {
      int rowloc = i * int(dimension);
      for (int j = 0; j < int(dimension); j++) {
        centroid[rowloc + j] = 0;
      }
    }

    // Set halfCodewordPower[i] for the initial codebook.
    for (i = 0; i < int(sizeCodebook); i++ ) {
      double sum = 0.0;
      int rowloc = i * int(dimension);
      for ( int j=0; j<int(dimension); j++ ) {
        double entry;
	entry = codebook[rowloc + j];
        sum += entry * entry;
      }
      halfCodewordPower[i] = sum/2;
    }

    //  Run Lloyd iteration for the initial codebook.

    int count = 0;
    int index = 0;
    double distance = 0;
    double oldDistortion = 0;
    double distortion = 0;

    for (i = 0; i < int(sizeTrnSet); i++) {
      NN(index, distance, &(trnSet[i*int(dimension)]), codebook,
         halfCodewordPower, int(sizeCodebook), int(dimension));
      (numPart[index])++;
      distortion += distance;
      for (int j = 0; j < int(dimension); j++) {
        centroid[index*int(dimension)+j] += trnSet[i*int(dimension)+j];
      }
    }

    count++;

    do {
      int i;
      //  Reset the improved codebook to the centroid of partitioned cluser set
      for (i = 0; i < int(sizeCodebook); i++) {
	int rowloc = i * int(dimension);
	//  Update ith code word
	//  Avoid division-by-zero error when numPart[i] is not zero
        if ( numPart[i] ) {
          for (int j = 0; j < int(dimension); j++) {
            codebook[rowloc + j] = centroid[rowloc + j] / numPart[i];
	  }
        }
      }

      //  Reset the contents of array numPart to 0.
      for (i = 0; i < int(sizeCodebook); i++) {
	numPart[i] = 0;
      }

      //  Reset the contents of array centroid to 0.
      for (i = 0; i < int(sizeCodebook); i++) {
	int rowloc = i * int(dimension);
        for (int j = 0; j < int(dimension); j++) {
          centroid[rowloc + j] = 0;
	}
      }

      //  Precompute Ai=||Yi||^2/2 for the given codebook and store them in 
      //  the array halfCodewordPower.
      for (i = 0; i < int(sizeCodebook); i++ ) {
	double power = 0.0;
	int rowloc = i * int(dimension);
        for ( int j = 0; j < int(dimension); j++ ) {
	  double entry = codebook[rowloc + j];
	  power += entry * entry;
        }
        halfCodewordPower[i] = power/2;
      }

      //  Reset the value of oldDistortion and Distortion.
      oldDistortion = distortion;
      distortion = 0;
	
      // Lloyd iteration :
      // For a given codebook, partition the training set into cluster sets
      // using the nearest neighbor condition, and find the average distortion
      // for the codebook. Then find centroid of each partitioned cluster set.
      for (i = 0; i < int(sizeTrnSet); i++) {
	NN(index, distance, &(trnSet[i*int(dimension)]), codebook,
	   halfCodewordPower, int(sizeCodebook), int(dimension));
	(numPart[index])++;
	distortion += distance;
	int t_rowloc = i * int(dimension);
	int c_rowloc = index * int(dimension);
	for (int j = 0; j < int(dimension); j++) {
	  centroid[c_rowloc + j] += trnSet[t_rowloc + j];
        }
      }
    } while ( (oldDistortion-distortion)/oldDistortion>double(thresholdRatio)
	      && (++count)<int(maxIteration) );

    // allocate dynamic memory for output which will be automatically deleted
    FloatMatrix& result = *(new FloatMatrix(int(sizeCodebook),int(dimension)));
    for (i = 0; i < int(sizeCodebook)*int(dimension); i++)
        result.entry(i) = codebook[i];
    output%0 << result;
}

// prototype instance for known block list
static SDFGLA proto;
static RegisterBlock registerBlock(proto,"GLA");
