static const char file_id[] = "SDFGoertzelBase.pl";
// .cc file generated from SDFGoertzelBase.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFGoertzelBase.h"
#include <math.h>

const char *star_nm_SDFGoertzelBase = "SDFGoertzelBase";

const char* SDFGoertzelBase :: className() const {return star_nm_SDFGoertzelBase;}

ISA_FUNC(SDFGoertzelBase,SDFStar);

Block* SDFGoertzelBase :: makeNew() const { LOG_NEW; return new SDFGoertzelBase;}

SDFGoertzelBase::SDFGoertzelBase ()
{
	setDescriptor("Base class for Goertzel algorithm stars.");
	addPort(input.setPort("input",this,FLOAT));
	addState(k.setState("k",this,"0","the DFT coefficient to compute (k < N)"));
	addState(N.setState("N",this,"32","length of the DFT"));
	addState(size.setState("size",this,"32","amount of data to read (N <= size)"));
	addState(d1.setState("d1",this,"0.0","first-order feedback coefficient which is a function of k and N",
# line 44 "SDFGoertzelBase.pl"
A_NONCONSTANT|A_NONSETTABLE));
	addState(state1.setState("state1",this,"0.0","internal state.",
# line 51 "SDFGoertzelBase.pl"
A_NONCONSTANT|A_NONSETTABLE));
	addState(state2.setState("state2",this,"0.0","internal state.",
# line 58 "SDFGoertzelBase.pl"
A_NONCONSTANT|A_NONSETTABLE));

# line 64 "SDFGoertzelBase.pl"
theta = 0.0;
}

void SDFGoertzelBase::setup() {
# line 68 "SDFGoertzelBase.pl"
if ( int(k) < 0 ) {
		  Error::abortRun(*this,
			"The value for state k must be nonnegative.");
		  return;
		}
		if ( int(N) <= 0 ) {
		   Error::abortRun(*this,
			"The value for state N must be positive.");
		   return;
		}
		if ( int(size) <= 0 ) {
		   Error::abortRun(*this,
			"The value for state size must be positive.");
		   return;
		}
		if ( int(k) >= int(N) ) {
		   Error::abortRun(*this,
			"The DFT coefficient k must be less than the ",
			"DFT length N.");
		   return;
		}
		if ( int(N) > int(size) ) {
		   Error::abortRun(*this,
			"The DFT length N must be less than or equal to the ",
			"number of data samples read, given by state size.");
		   return;
		}
		double kd = int(k);
		double Nd = int(N);
		theta = -2.0 * M_PI * kd / Nd;
		d1 = 2.0 * cos(theta);
		input.setSDFParams(int(size), int(size)-1);
}

void SDFGoertzelBase::go() {
# line 102 "SDFGoertzelBase.pl"
// Run all-pole section of Goertzel's algorithm N iterations.
		// Only one multiplier (d1) in iteration.
		// Zero the IIR state for each DFT calculation; otherwise,
		// the filter output could grow without bound.
		// state1 and state2 are states and not local variables
		// ONLY to pass their values to derived stars
		state1 = 0.0;
		state2 = 0.0;
		double acc = 0.0;
		double d1val = d1;
		for (int i = int(N)-1; i >= 0; i--) {
		  acc = double(input%i);
		  acc += d1val * double(state1);
		  acc -= double(state2);
		  state2 = double(state1);
		  state1 = acc;
		}
}

// prototype instance for known block list
static SDFGoertzelBase proto;
static RegisterBlock registerBlock(proto,"GoertzelBase");
