static const char file_id[] = "SDFIIR.pl";
// .cc file generated from SDFIIR.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFIIR.h"
#include <minmax.h>

const char *star_nm_SDFIIR = "SDFIIR";

const char* SDFIIR :: className() const {return star_nm_SDFIIR;}

ISA_FUNC(SDFIIR,SDFStar);

Block* SDFIIR :: makeNew() const { LOG_NEW; return new SDFIIR;}

SDFIIR::SDFIIR ()
{
	setDescriptor("An infinite impulse response (IIR) filter implemented in a direct form II\nrealization.\nThe transfer function is of the form H(z) = G*N(1/z)/D(1/z),\nwhere N() and D() are polynomials.\nThe parameter \"gain\" specifies G, and the floating-point arrays\n\"numerator\" and \"denominator\" specify N() and D(), respectively.\nBoth arrays start with the constant terms of the polynomial\nand decrease in powers of z (increase in powers of 1/z).\nNote that the constant term of D is not omitted, as is common in\nother programs that assume that it has been normalized to unity.");
	addPort(signalIn.setPort("signalIn",this,FLOAT));
	addPort(signalOut.setPort("signalOut",this,FLOAT));
	addState(gain.setState("gain",this,"1","Gain."));
	addState(numerator.setState("numerator",this,".5 .25 .1","Numerator coefficients."));
	addState(denominator.setState("denominator",this,"1 .5 .3","Denominator coefficients."));
	addState(state.setState("state",this,"0","State.",
# line 97 "SDFIIR.pl"
A_NONCONSTANT|A_NONSETTABLE));


}

void SDFIIR::setup() {
# line 107 "SDFIIR.pl"
int numNumer = numerator.size();
	int numDenom = denominator.size();
	numState = max(numNumer,numDenom); 
	double b0, scaleDenom, scaleNumer;

	// Set up scaling to distribute the gain through the numerator,
	// and scale both numer and denom to make b0=1
	if ( numDenom < 1 ) {
	    b0 = 1.0;
	} else {
	    if ( (b0 = denominator[0]) == 0.0 ) {
		// XXX: should sanity-check b0 more thoroughly
		// (e.g., shouldn't even be close to zero)
		Error::abortRun(*this, 
		  "Must have non-zero leading denominator");
		return;
	    }
	}
	scaleDenom = 1.0 / b0;
	scaleNumer = scaleDenom * double(gain);

	// Set up the state vector.  The state vector includes
	// both the delay states and the coefficients in the appropriate
	// order:
	// S(0) A(0) -1 S(1) A(1) -B(1) ... S(n-1) A(n-1) -B(n-1) Sn An -Bn
	// state[0] and state[2] are never referenced
	state.resize(numState*3);
	for ( int i=0; i < numState; i++) {
	    state[i*3+0] = 0;
	    state[i*3+1] = i < numNumer ? scaleNumer * numerator[i] : 0;
	    state[i*3+2] = i < numDenom ? scaleDenom * -denominator[i] : 0;
	}
	stateEnd = ((double*)state) + 3*numState;
}

void SDFIIR::go() {
# line 142 "SDFIIR.pl"
register double s0 = signalIn%0;

	if ( numState == 1 ) {
	    // actually, this means no state; just feed through
	    signalOut%0 << state[1] * s0;
	} else {
	    register double *v = & state[3];
	    register double y;
	    double s;

	    s = *v++;
	    y = s * *v++;
	    s0 += s * *v++;

	    for ( ; v < stateEnd; ) {
		double sTmp = *v;
		*v++ = s;
		s = sTmp;
		y += s * *v++;
		s0 += s * *v++;
	    }
	    state[3] = s0;
	    y += s0 * state[1];
	    signalOut%0 << y;
	}
}

// prototype instance for known block list
static SDFIIR proto;
static RegisterBlock registerBlock(proto,"IIR");
