static const char file_id[] = "SDFIIRFix.pl";
// .cc file generated from SDFIIRFix.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFIIRFix.h"
#include <minmax.h>

const char *star_nm_SDFIIRFix = "SDFIIRFix";

const char* SDFIIRFix :: className() const {return star_nm_SDFIIRFix;}

ISA_FUNC(SDFIIRFix,SDFFix);

Block* SDFIIRFix :: makeNew() const { LOG_NEW; return new SDFIIRFix;}

SDFIIRFix::SDFIIRFix ()
{
	setDescriptor("An infinite impulse response (IIR) filter implemented in direct form II,\nusing fixed-point arithmetic.  The transfer function is of the form\nH(z) = G*N(1/z)/D(1/z), where N() and D() are polynomials.\nThe parameter \"gain\" specifies G, and the floating-point arrays\n\"numerator\" and \"denominator\" specify N() and D(), respectively.\nBoth arrays start with the constant terms of the polynomial\nand decrease in powers of z (increase in powers of 1/z).\nThe coefficients are rounded to the precision given by \"CoefPrecision\".\nBefore the numerator and denominator coefficients are quantized, they\nare rescaled so that the leading denominator coefficient is unity.\nThe gain is multiplied through the numerator coefficients as well.");
	addPort(signalIn.setPort("signalIn",this,FIX));
	addPort(signalOut.setPort("signalOut",this,FIX));
	addState(gain.setState("gain",this,"1","Gain."));
	addState(numerator.setState("numerator",this,".5 .25 .1","Numerator coefficients."));
	addState(denominator.setState("denominator",this,"1 .5 .3","Denominator coefficients."));
	addState(CoefPrecision.setState("CoefPrecision",this,"2.14","Precision of the coefficients in bits."));
	addState(ArrivingPrecision.setState("ArrivingPrecision",this,"YES","Flag indicating whether or not to use the arriving particles as they are:\nYES keeps the same precision, and NO casts them to the precision specified\nby the parameter \"InputPrecision\"."));
	addState(InputPrecision.setState("InputPrecision",this,"2.14","Precision of the input in bits.\nThe input particles are only cast to this precision if the parameter\n\"ArrivingPrecision\" is set to NO."));
	addState(AccumPrecision.setState("AccumPrecision",this,"2.14","Precision of the accumulation in bits."));
	addState(StatePrecision.setState("StatePrecision",this,"2.14","Precision of the state in bits."));
	addState(OutputPrecision.setState("OutputPrecision",this,"2.14","Precision of the output in bits."));

# line 146 "SDFIIRFix.pl"
fdbckCoefs = 0;
		fwdCoefs = 0;
		state = 0;
}

SDFIIRFix::~SDFIIRFix() {
# line 151 "SDFIIRFix.pl"
LOG_DEL; delete [] fdbckCoefs;
	    LOG_DEL; delete [] fwdCoefs;
	    LOG_DEL; delete [] state;
}

void SDFIIRFix::setup() {
# line 156 "SDFIIRFix.pl"
SDFFix::setup();

	    int numNumer = numerator.size();
	    int numDenom = denominator.size();
	    numState = max(numNumer,numDenom);
	    double b0, scaleDenom, scaleNumer;

	    // Set up scaling to distribute the gain through the numerator,
	    // and scale both numer and denom to make b0=1
	    if ( numDenom < 1 ) {
	        b0 = 1.0;
	    } else {
	        if ( (b0 = denominator[0]) == 0.0 ) {
		    // XXX: should sanity-check b0 more thoroughly
		    // (e.g., shouldn't even be close to zero)
		    Error::abortRun(*this,
		      "Must have non-zero leading denominator coefficient");
		    return;
	        }
	    }
	    scaleDenom = 1.0 / b0;
	    scaleNumer = scaleDenom * double(gain);

	    LOG_DEL; delete [] fdbckCoefs;
	    LOG_NEW; fdbckCoefs = new Fix[numState];
	    LOG_DEL; delete [] fwdCoefs;
	    LOG_NEW; fwdCoefs = new Fix[numState];
	    LOG_DEL; delete [] state;
	    LOG_NEW; state = new Fix[numState];

	    // Local variables
	    Fix coef( (const char*) CoefPrecision );
	    if ( coef.invalid() )
	      Error::abortRun( *this, "Invalid CoefPrecision" );

	    Fix st( (const char*) StatePrecision );
	    if ( st.invalid() )
	      Error::abortRun( *this, "Invalid StatePrecision" );

	    coef.set_rounding( int(RoundFix) );
	    st.set_rounding( int(RoundFix) );

	    // Set up the state and coefficient vectors
	    // The first location in state and fdbckCoefs is wasted.
	    for ( int i=0; i < numState; i++) {
		st = 0.0;
	        state[i] = st;
	        coef = i<numNumer ? scaleNumer * numerator[i] : 0.0;
	        fwdCoefs[i] = coef;
	        coef = i<numDenom ? scaleDenom * -denominator[i] : 0.0;
	        fdbckCoefs[i] = coef;
	    }

	    // Set the precision of various fixed-point variables
	    fixIn = Fix( (const char*) InputPrecision );
	    if ( fixIn.invalid() )
	      Error::abortRun( *this, "Invalid InputPrecision" );

	    fdbckAccum = Fix ( (const char*) AccumPrecision );
	    if ( fdbckAccum.invalid() )
	      Error::abortRun( *this, "Invalid AccumPrecision" );
	    fwdAccum = Fix ( (const char*) AccumPrecision );

	    out = Fix ( (const char*) OutputPrecision );
	    if ( out.invalid() )
	      Error::abortRun( *this, "Invalid OutputPrecision" );

	    // Set the overflow handlers for assignments/computations
	    fixIn.set_ovflow( OverflowHandler );
	    if ( fixIn.invalid() )
	      Error::abortRun( *this, "Invalid OverflowHandler" );
	    fdbckAccum.set_ovflow( OverflowHandler );
	    fwdAccum.set_ovflow( OverflowHandler );
	    out.set_ovflow( OverflowHandler );

	    // Set all fixed-point assignments/computations to use rounding
            fixIn.set_rounding( int(RoundFix) );
	    fdbckAccum.set_rounding( int(RoundFix) );
	    fwdAccum.set_rounding( int(RoundFix) );
	    out.set_rounding( int(RoundFix) );
}

void SDFIIRFix::go() {
# line 238 "SDFIIRFix.pl"
if ( numState == 1 ) {
                // Actually, this means no state; just feed through
		if ( int(ArrivingPrecision) )
		    out = fwdCoefs[0] * Fix(signalIn%0);
		else {
		    fixIn = Fix(signalIn%0);
		    out = fwdCoefs[0] * fixIn;
		}
		checkOverflow(out);
		signalOut%0 << out;
            } else {
		fdbckAccum = 0.0;
		fwdAccum = 0.0;
		for ( int i=1; i < numState; i++) {
		    fdbckAccum += state[i] * fdbckCoefs[i];
		    checkOverflow(fdbckAccum);
		    fwdAccum += state[i] * fwdCoefs[i];
		    checkOverflow(fwdAccum);
		}
		if ( int(ArrivingPrecision) )
		    fdbckAccum += Fix(signalIn%0);
		else {
		    fixIn = Fix(signalIn%0);
		    fdbckAccum += fixIn;
		}
		checkOverflow(fdbckAccum);
		for ( i=numState-1; i > 1; i--) {
		    state[i] = state[i-1];
		}
		state[1] = fdbckAccum;
		out = fdbckAccum * fwdCoefs[0] + fwdAccum;
		checkOverflow(out);
		signalOut%0 << out;
	     }
}

// prototype instance for known block list
static SDFIIRFix proto;
static RegisterBlock registerBlock(proto,"IIRFix");
