static const char file_id[] = "SDFKalman_M.pl";
// .cc file generated from SDFKalman_M.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFKalman_M.h"

const char *star_nm_SDFKalman_M = "SDFKalman_M";

const char* SDFKalman_M :: className() const {return star_nm_SDFKalman_M;}

ISA_FUNC(SDFKalman_M,SDFStar);

Block* SDFKalman_M :: makeNew() const { LOG_NEW; return new SDFKalman_M;}

SDFKalman_M::SDFKalman_M ()
{
	setDescriptor("Implements the Kalman filter.");
	addPort(input.setPort("input",this,FLOAT_MATRIX_ENV));
	addPort(StateTransitionMatrixAtTimeN.setPort("StateTransitionMatrixAtTimeN",this,FLOAT_MATRIX_ENV));
	addPort(MeasurementMatrixAtTimeN.setPort("MeasurementMatrixAtTimeN",this,FLOAT_MATRIX_ENV));
	addPort(ProcessNoiseCorrMatrixAtTimeN.setPort("ProcessNoiseCorrMatrixAtTimeN",this,FLOAT_MATRIX_ENV));
	addPort(MeasurementNoiseCorrMatrixAtTimeN.setPort("MeasurementNoiseCorrMatrixAtTimeN",this,FLOAT_MATRIX_ENV));
	addPort(output.setPort("output",this,FLOAT_MATRIX_ENV));
	addState(stateDimension.setState("stateDimension",this,"5","The number of elements in the state vector."));
	addState(inputDimension.setState("inputDimension",this,"1","The number of elements in the observation vector."));
	addState(InitialState.setState("InitialState",this,"0.0 [5]","The initial value of the state vector."));
	addState(InitialCorrMatrix.setState("InitialCorrMatrix",this,".1 0 [5] .1 0 [5] .1 0 [5] .1 0 [5] .1","The initial value of the correlation matrix of the error."));
	addState(InitialStateTransitionMatrix.setState("InitialStateTransitionMatrix",this,"1 0 [5] 1 0 [5] 1 0 [5] 1 0 [5] 1","The state transition matrix at time 0. PHI(1,0)"));
	addState(InitialProcessNoiseCorrMatrix.setState("InitialProcessNoiseCorrMatrix",this,"0.0 [25]","Correlation matrix of process noise vector at time 0. Q(0)"));

# line 106 "SDFKalman_M.pl"
stateVector = 0;
    gainMatrix = 0;
    innovationsVector = 0;
    innovationsCorrMatrix = 0;
    predStateVector = 0;
    predStateErrCorrMatrix = 0;
    presentErrCorrMatrix = 0;
}

SDFKalman_M::~SDFKalman_M() {
# line 115 "SDFKalman_M.pl"
delete stateVector;
    delete gainMatrix;
    delete innovationsVector;
    delete innovationsCorrMatrix;
    delete predStateVector;
    delete predStateErrCorrMatrix;
    delete presentErrCorrMatrix;
}

void SDFKalman_M::setup() {
# line 124 "SDFKalman_M.pl"
stateDim = int(stateDimension);
    inputDim = int(inputDimension);

    // delete previously allocation arrays
    delete stateVector;
    delete gainMatrix;
    delete innovationsVector;
    delete innovationsCorrMatrix;
    delete predStateVector;
    delete predStateErrCorrMatrix;
    delete presentErrCorrMatrix;

    // initialize storage for intermediate and state variables
    stateVector = new FloatMatrix(stateDim,1,InitialState);  // x(n|y(n))
    gainMatrix = new FloatMatrix(stateDim,inputDim);         // G(n)
    innovationsVector = new FloatMatrix(inputDim, 1);        // alpha(n)
    innovationsCorrMatrix = new FloatMatrix(inputDim,inputDim); // sigma(n)
    predStateVector = new FloatMatrix(stateDim,1);           // x(n+1|y(n))
    predStateErrCorrMatrix = new FloatMatrix(stateDim,stateDim);// K(n+1,n)
    presentErrCorrMatrix = new FloatMatrix(stateDim,stateDim,InitialCorrMatrix);  // K(n)

    // initialize working variables
    FloatMatrix initialStateTransitionMatrix(stateDim,stateDim, InitialStateTransitionMatrix);
    FloatMatrix initialProcessNoiseCorrMatrix(stateDim,stateDim, InitialProcessNoiseCorrMatrix);

    // calculate zero-th step variables

    // calculate K(1,0) = PHI(1,0)*K(0)*(PHI(1,0)->transpose)  +  Q(0)
    (*predStateErrCorrMatrix) = initialStateTransitionMatrix * 
                                (*presentErrCorrMatrix) * 
                                (~initialStateTransitionMatrix) + 
                                initialProcessNoiseCorrMatrix;
      
    // calculate x(1|y(0)), the one step prediction
    //  x(1|y(0)) = PHI(1,0)*x(0)
    multiply(initialStateTransitionMatrix,*stateVector,*predStateVector);
}

void SDFKalman_M::go() {
# line 162 "SDFKalman_M.pl"
Envelope pkt1;
    Envelope pkt2;
    Envelope pkt3;
    Envelope pkt4;
    Envelope pkt5;
    // inputs to the system
    //    const FloatMatrix& inputVector;                        // y(n)
    //    const FloatMatrix& stateTransitionMatrix;              // PHI(n+1,n)
    //    const FloatMatrix& processNoiseCorrMatrix;             // Q(n)
    //    const FloatMatrix& measurementNoiseCorrMatrix;         // R(n)
    //    const FloatMatrix& measurementMatrix;                  // C(n)

    (input%0).getMessage(pkt1);
    const FloatMatrix& inputVector = *(const FloatMatrix *)pkt1.myData();

    (StateTransitionMatrixAtTimeN%0).getMessage(pkt2);
    const FloatMatrix& stateTransitionMatrix = *(const FloatMatrix *)pkt2.myData();

    (ProcessNoiseCorrMatrixAtTimeN%0).getMessage(pkt3);
    const FloatMatrix& processNoiseCorrMatrix = *(const FloatMatrix *)pkt3.myData();

    (MeasurementNoiseCorrMatrixAtTimeN%0).getMessage(pkt4);
    const FloatMatrix& measurementNoiseCorrMatrix = *(const FloatMatrix *)pkt4.myData();

    (MeasurementMatrixAtTimeN%0).getMessage(pkt5);
    const FloatMatrix& measurementMatrix = *(const FloatMatrix *)pkt5.myData();

    // calculate sigma(n) = C(n)*K(n,n-1)*(C(n)->transpose)  +  R(n)
    *innovationsCorrMatrix=(measurementMatrix * (*predStateErrCorrMatrix) *
                            (~measurementMatrix)) + 
                           measurementNoiseCorrMatrix;

    // calculate G(n)=PHI(n+1,n)*K(n,n-1)*(C(n)->transpose)*(sigma(n)->inverse)
    *gainMatrix = stateTransitionMatrix * (*predStateErrCorrMatrix) *
                  (~measurementMatrix) * (innovationsCorrMatrix->inverse());

    // calculate innovations vector: alpha(n) = y(n) - C(n)*x(n|y(n-1))
    *innovationsVector = inputVector - 
                         (measurementMatrix * (*predStateVector));

    // calculate one step prediction
    //  x(n+1|y(n)) = PHI(n+1,n)*x(n|y(n-1)) + G(n)*alpha(n)
    *predStateVector = (stateTransitionMatrix * (*predStateVector)) +
                       ((*gainMatrix) * (*innovationsVector));

    // calculate new state: x(n|y(n)) = PHI(n,n+1)*x(n+1|y(n))
    // makes the assumption that PHI(n,n+1) = PHI(n+1,n)
    multiply(stateTransitionMatrix,*predStateVector,*stateVector);

    // calculate correlation matrix of error in new state: K(n)
    // K(n) = K(n,n-1) - PHI(n,n+1)
    // makes the assumption that PHI(n,n+1) = PHI(n+1,n)
    *presentErrCorrMatrix = (*predStateErrCorrMatrix) -
                            (stateTransitionMatrix * (*gainMatrix) * 
                             measurementMatrix * (*predStateErrCorrMatrix));

    // calculate predStateErrCorrMatrix: K(n+1,n) 
    // K(n+1,n) = PHI(n+1,n)*K(n)*(PHI(n+1,n)->transpose)  + Q(n)
    *predStateErrCorrMatrix = (stateTransitionMatrix * 
                               (*presentErrCorrMatrix) *
                               (~stateTransitionMatrix)) +
                              processNoiseCorrMatrix;

    FloatMatrix* outMatrix = new FloatMatrix(*stateVector);
    output%0 << *outMatrix;
}

// prototype instance for known block list
static SDFKalman_M proto;
static RegisterBlock registerBlock(proto,"Kalman_M");
