static const char file_id[] = "SDFLMS.pl";
// .cc file generated from SDFLMS.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFLMS.h"
#include "miscFuncs.h"
#include <stdio.h>

const char *star_nm_SDFLMS = "SDFLMS";

const char* SDFLMS :: className() const {return star_nm_SDFLMS;}

ISA_FUNC(SDFLMS,SDFFIR);

Block* SDFLMS :: makeNew() const { LOG_NEW; return new SDFLMS;}

SDFLMS::SDFLMS ()
{
	setDescriptor("An adaptive filter using the Least-Mean Square (LMS) algorithm.\nThe initial filter coefficients are given by the \"taps\" parameter.\nThe default initial coefficients give an 8th-order, linear phase\nlowpass filter.\nTo read initial coefficients from a file, replace the default\ncoefficients with \"< fileName\", preferably specifying a complete path.\nThis star supports decimation, but not interpolation.");
	addPort(error.setPort("error",this,FLOAT));
	addState(stepSize.setState("stepSize",this,"0.01","Adaptation step size."));
	addState(errorDelay.setState("errorDelay",this,"1","Delay in the update loop."));
	addState(saveTapsFile.setState("saveTapsFile",this,"","File to save final tap values."));

# line 77 "SDFLMS.pl"
// remove interpolation as a settable parameter
		interpolation.clearAttributes(A_SETTABLE);
		// taps are no longer constant
		taps.clearAttributes(A_CONSTANT);
}

void SDFLMS::wrapup() {
# line 110 "SDFLMS.pl"
const char* sf = saveTapsFile;
		if (sf != NULL && *sf != 0) {
			char* saveFileName = expandPathName(sf);
			// open the file for writing
			FILE* fp;
			if (!(fp = fopen(saveFileName,"w"))) {
				// File cannot be opened
				Error::warn(*this,
				 "Cannot open saveTapsFile for writing: ",
				 saveFileName, ". Taps not saved.");
			} else {
				for (int i = 0; i < taps.size(); i++)
					fprintf(fp, "%d %g\n", i, taps[i]);
			}
			fclose(fp);
			LOG_DEL; delete [] saveFileName;
		}
}

void SDFLMS::setup() {
# line 83 "SDFLMS.pl"
// First check to be sure that interpolation is 1.
		interpolation = 1;

		// Next run the FIR start routine
		SDFFIR :: setup();

		// Then reset the signalIn number of samples in the past
		// to account for the error delay.
		signalIn.setSDFParams(int(decimation),
				      int(decimation) + 1 + int(errorDelay)
				      + taps.size());
}

void SDFLMS::go() {
# line 96 "SDFLMS.pl"
// First update the taps
		double e = double(error%0);
		int index = int(errorDelay)*int(decimation) + int(decimationPhase);

		for (int i = 0; i < taps.size(); i++) {
			taps[i] = taps[i] +
				e * double(signalIn%(index)) * double(stepSize);
			index++;
		}
		
		// Then run FIR filter
		SDFFIR :: go();
}

// prototype instance for known block list
static SDFLMS proto;
static RegisterBlock registerBlock(proto,"LMS");
