static const char file_id[] = "SDFLMSCx.pl";
// .cc file generated from SDFLMSCx.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFLMSCx.h"
#include "miscFuncs.h"

const char *star_nm_SDFLMSCx = "SDFLMSCx";

const char* SDFLMSCx :: className() const {return star_nm_SDFLMSCx;}

ISA_FUNC(SDFLMSCx,SDFFIRCx);

Block* SDFLMSCx :: makeNew() const { LOG_NEW; return new SDFLMSCx;}

SDFLMSCx::SDFLMSCx ()
{
	setDescriptor("A complex adaptive filter using the LMS adaptation algorithm.\nThe initial coefficients are given by the \"taps\" parameter.\nThe default initial coefficients give an 8th order, linear phase\nlowpass filter.  To read initial coefficients from a file,\nreplace the default coefficients with \"< fileName\",\npreferably specifying a complete path.\nThis star supports decimation, but not interpolation.");
	addPort(error.setPort("error",this,COMPLEX));
	addState(stepSize.setState("stepSize",this,"0.01","Adaptation step size."));
	addState(errorDelay.setState("errorDelay",this,"1","Delay in the update loop."));
	addState(saveTapsFile.setState("saveTapsFile",this,"","File to save final tap values."));

# line 72 "SDFLMSCx.pl"
// remove interpolation as a settable parameter
		interpolation.clearAttributes(A_SETTABLE);
		// taps are no longer constant
		taps.clearAttributes(A_CONSTANT);
}

void SDFLMSCx::wrapup() {
# line 103 "SDFLMSCx.pl"
const char* sf = saveTapsFile;
		if (sf != NULL && *sf != 0) {
			char* saveFileName = expandPathName(sf);
			// open the file for writing
			FILE* fp = fopen(saveFileName,"w");
			if (!fp) {
				Error::warn(*this, 
					"Cannot open file for writing: ",
					saveFileName,". Taps not saved.");
			} else {
				for (int i = 0; i < taps.size(); i++)
					fprintf(fp, "(%g ,%g)\n",
							taps[i].real(),
							taps[i].imag());
			}
			fclose(fp);
			delete [] saveFileName;
		}
}

void SDFLMSCx::setup() {
# line 78 "SDFLMSCx.pl"
// force interpolation to 1
		interpolation = 1;
		// Next run the ComplexFIR start routine
		SDFFIRCx :: setup();

		// Then reset the signalIn number of samples in the past
		// to account for the error delay.
		signalIn.setSDFParams(int(decimation),
				      int(decimation) + 1 + int(errorDelay)
				      + taps.size());
}

void SDFLMSCx::go() {
# line 90 "SDFLMSCx.pl"
// First update the taps
		Complex e = Complex(error%0) * double(stepSize);
		int index = int(errorDelay)*int(decimation) + int(decimationPhase);
		// adjustment is e times the conjugate of the output
		for (int i = 0; i < taps.size(); i++) {
			taps[i] = taps[i] + e * conj(Complex(signalIn%index));
			index++;
		}
		
		// Then run FIR filter
		SDFFIRCx :: go();
}

// prototype instance for known block list
static SDFLMSCx proto;
static RegisterBlock registerBlock(proto,"LMSCx");
