static const char file_id[] = "SDFLMSOscDet.pl";
// .cc file generated from SDFLMSOscDet.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFLMSOscDet.h"

const char *star_nm_SDFLMSOscDet = "SDFLMSOscDet";

const char* SDFLMSOscDet :: className() const {return star_nm_SDFLMSOscDet;}

ISA_FUNC(SDFLMSOscDet,SDFLMS);

Block* SDFLMSOscDet :: makeNew() const { LOG_NEW; return new SDFLMSOscDet;}

SDFLMSOscDet::SDFLMSOscDet ()
{
	setDescriptor("This filter tries to lock onto the strongest sinusoidal component in\nthe input signal, and outputs the current estimate of the cosine\nof the frequency of the strongest component.  It is a three-tap\nLeast-Mean Square (LMS) filter whose first and third coefficients are\nfixed at one.  The second coefficient is adapted.  It is a normalized\nversion of the Direct Adaptive Frequency Estimation Technique.");
	addPort(cosOmega.setPort("cosOmega",this,FLOAT));
	addState(initialOmega.setState("initialOmega",this,"PI/4","The initial guess at the angle being estimated in radians."));

# line 68 "SDFLMSOscDet.pl"
// taps state: not constant, length three, and not settable
		taps.clearAttributes(A_CONSTANT);
		taps.clearAttributes(A_SETTABLE);

		// decimation is not supported
		decimation.clearAttributes(A_SETTABLE);
		decimationPhase.clearAttributes(A_SETTABLE);
}

void SDFLMSOscDet::setup() {
# line 77 "SDFLMSOscDet.pl"
// we don't support decimation: make sure that it's 1
		decimation = 1;

		// initialize the taps of the three-tap LMS FIR filter
		taps.resize(3);
		taps[0] =  1.0;
		taps[1] = -2.0 * cos(double(initialOmega));
		taps[2] =  1.0;

		// call the LMS FIR filter setup method
		SDFLMS :: setup();
}

void SDFLMSOscDet::go() {
# line 90 "SDFLMSOscDet.pl"
// 1. Update the second tap = -2 a1[k]
		//    update:        a1[k] = a1[k] + 2 e[n] x[n-1]
		//    second tap: -2 a1[k] = -2 a1[k] - 4 mu e[n] x[n-1]
		//    new tap:      newtap = newtap - 4 mu e[n] x[n-1]
		double mu = double(stepSize);
		double e = double(error%0);

		// Since we don't support decimation,
		// index = int(errorDelay) + 1, a constant
		int index = int(errorDelay)*int(decimation) +
			    int(decimationPhase) + 1;
		double xnMinus1 = double(signalIn%(index));

		double newSecondTap = taps[1] - 4 * mu * e * xnMinus1;
		taps[1] = newSecondTap;

		// 2. Compute the estimate of cos(w), a1
		cosOmega%0 << (-newSecondTap/2);

		// 3. Run the FIR filter
		SDFFIR :: go();
}

// prototype instance for known block list
static SDFLMSOscDet proto;
static RegisterBlock registerBlock(proto,"LMSOscDet");
