static const char file_id[] = "SDFLMSPlotCx.pl";
// .cc file generated from SDFLMSPlotCx.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFLMSPlotCx.h"

const char *star_nm_SDFLMSPlotCx = "SDFLMSPlotCx";

const char* SDFLMSPlotCx :: className() const {return star_nm_SDFLMSPlotCx;}

ISA_FUNC(SDFLMSPlotCx,SDFLMSCx);

Block* SDFLMSPlotCx :: makeNew() const { LOG_NEW; return new SDFLMSPlotCx;}

SDFLMSPlotCx::SDFLMSPlotCx ()
{
	setDescriptor("This star is exactly like the LMSCx star, except that, in addition\nto the functions of LMSCx, it makes a plot of the tap coefficients,\nwith a separate plot for the magnitude and phase.\nIt can produce two types of plots: a plot of the final tap values\nor a plot that traces the time evolution of each tap value.\nThe time evolution is obtained if the value of the parameter \"trace\" is \"YES\".");
	addState(graphOptsMag.setState("graphOptsMag",this,"","options for magnitude graph"));
	addState(graphOptsPhase.setState("graphOptsPhase",this,"","options for phase graph"));
	addState(graphTitleMag.setState("graphTitleMag",this,"LMS filter tap magnitudes","title for magnitude graph"));
	addState(graphTitlePhase.setState("graphTitlePhase",this,"LMS filter tap phases","title for phase graph"));
	addState(trace.setState("trace",this,"NO","if YES, plot a trace of tap values as they adapt"));

# line 76 "SDFLMSPlotCx.pl"
// remove saveTapsFile as a settable param
		saveTapsFile.clearAttributes(A_SETTABLE);
		prevPhase = outPhase = 0;
}

void SDFLMSPlotCx::wrapup() {
# line 131 "SDFLMSPlotCx.pl"
// don't call SDFLMSCx::wrapup: no final file to write.
		if (!int(trace)) {
			// plot final taps
			for (int i = 0; i < taps.size(); i++) {
				double mag = abs(taps[i]);
				double phase = (mag <= 0) ? 0 : arg(taps[i]);
				magGraph.addPoint (mag);
				phaseGraph.addPoint (phase);
			}
		}
		// end the plot
		magGraph.terminate();
		phaseGraph.terminate();
}

SDFLMSPlotCx::~SDFLMSPlotCx() {
# line 81 "SDFLMSPlotCx.pl"
LOG_DEL; delete [] prevPhase;
		LOG_DEL; delete [] outPhase;
}

void SDFLMSPlotCx::setup() {
# line 85 "SDFLMSPlotCx.pl"
SDFLMSCx::setup();
		LOG_DEL; delete [] prevPhase;
		LOG_DEL; delete [] outPhase;
		int nPlots = 1;
		if (int(trace)) {
			nPlots = taps.size();
			LOG_NEW; prevPhase = new double[nPlots];
			LOG_NEW; outPhase = new double[nPlots];
			for (int i = 0; i < nPlots; i++)
				prevPhase[i] = outPhase[i] = 0.0;
			index = 0;
			// enforce a restriction imposed by xgraph program
			if (nPlots > 64) {
				Error::abortRun(*this,
				     ": maximum of 64 taps for trace plot");
				return;
			}
		}
		magGraph.initialize(this, nPlots, graphOptsMag,
			graphTitleMag, "");
		phaseGraph.initialize(this, nPlots, graphOptsPhase,
			graphTitlePhase, "");
}

void SDFLMSPlotCx::go() {
# line 110 "SDFLMSPlotCx.pl"
SDFLMSCx::go();
		if (int(trace)) {
			for (int i = 0; i < taps.size(); i++) {
				double mag = abs(taps[i]);
				double phase = (mag <= 0) ? 0 : arg(taps[i]);

				// handle phase wrapping for continuous plot

				double phaseChange = phase - prevPhase[i];
				if (phaseChange < -M_PI) phaseChange += 2*M_PI;
				if (phaseChange > M_PI) phaseChange -= 2*M_PI;
				outPhase[i] += phaseChange;
				prevPhase[i] = phase;
				magGraph.addPoint(i+1, index, mag);
				phaseGraph.addPoint(i+1, index, outPhase[i]);
			}
			index++;
		}
}

// prototype instance for known block list
static SDFLMSPlotCx proto;
static RegisterBlock registerBlock(proto,"LMSPlotCx");
