static const char file_id[] = "SDFLevDur.pl";
// .cc file generated from SDFLevDur.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFLevDur.h"

const char *star_nm_SDFLevDur = "SDFLevDur";

const char* SDFLevDur :: className() const {return star_nm_SDFLevDur;}

ISA_FUNC(SDFLevDur,SDFStar);

Block* SDFLevDur :: makeNew() const { LOG_NEW; return new SDFLevDur;}

SDFLevDur::SDFLevDur ()
{
	setDescriptor("This star uses the Levinson-Durbin algorithm to compute the linear predictor\ncoefficients of a random process, given its autocorrelation function as an\ninput.\nThese coefficients are produced both in autoregressive form\n(on the \"lp\" output) and in lattice filter form (on the \"refl\" output).\nThe \"errPower\" output is the power of the prediction error as a function\nof the predictor order.");
	addPort(autocor.setPort("autocor",this,FLOAT));
	addPort(lp.setPort("lp",this,FLOAT));
	addPort(refl.setPort("refl",this,FLOAT));
	addPort(errPower.setPort("errPower",this,FLOAT));
	addState(order.setState("order",this,"8","The order of the recursion."));

# line 123 "SDFLevDur.pl"
aOrig = aPrime = r = 0;
		ORD = 0;
}

SDFLevDur::~SDFLevDur() {
# line 127 "SDFLevDur.pl"
LOG_DEL; delete [] aOrig;
		LOG_DEL; delete [] aPrime;
		LOG_DEL; delete [] r;
}

void SDFLevDur::setup() {
# line 132 "SDFLevDur.pl"
// delete and remake arrays if the size has changed.
		if (int(order) != ORD) {
			LOG_DEL; delete [] aOrig;
			LOG_DEL; delete [] aPrime;
			LOG_DEL; delete [] r;
			ORD = int(order);
			LOG_NEW; aOrig = new double[ORD+1];
			LOG_NEW; aPrime = new double[ORD+1];
			LOG_NEW; r = new double[ORD+1];
		}
		refl.setSDFParams (ORD, ORD-1);
		lp.setSDFParams (ORD, ORD-1);
		errPower.setSDFParams (ORD+1, ORD);
		autocor.setSDFParams (2*ORD, ORD);
}

void SDFLevDur::go() {
# line 148 "SDFLevDur.pl"
// Define pointers so that the arrays can be swapped
	    double* a = aOrig;
	    double* aP = aPrime;
	    double gamma;
	    int m;

	    // for convenience, read the autocorrelation lags into a vector
	    for (int i = 0; i <= ORD; i++) {
		r[i] = double(autocor%(ORD-i));
	    }
	    // Initial prediction error is simply the zero-lag of
	    // of the autocorrelation, or the signal power estimate.
	    double P = r[0];

	    // Output the zeroth order prediction error power, which is
	    // simply the power of the input process
	    errPower%ORD << r[0];

	    // First coefficient is always unity
	    a[0] = 1.0;
	    aP[0] = 1.0;

	    // The order recurrence
	    for (int M = 0; M < ORD; M++ ) {

		// Compute the new reflection coefficient
		double deltaM = 0.0;
		for (m = 0; m < M+1; m++) {
		    deltaM += a[m]*r[M+1-m];
		}
		// Compute and output the reflection coefficient
		// (which is also equal to the last AR parameter)
		aP[M+1] = gamma = -deltaM/P;
		refl%(ORD-M-1) << - double(gamma);

		for (m = 1; m < M+1; m++) {
		    aP[m] = a[m] + gamma*a[M+1-m];
		}

		// Update the prediction error power
		P = P*(1.0 - gamma*gamma);
		errPower%(ORD-M-1) << double(P);

		// Swap a and aP for next order recurrence
		double* temp = a;
		a = aP;
		aP = temp;
	    }
	    // generate the lp outputs
	    for (m = 1; m <= ORD; m++ ) {
		lp%(ORD-m) << -a[m];
	    }
}

// prototype instance for known block list
static SDFLevDur proto;
static RegisterBlock registerBlock(proto,"LevDur");
