static const char file_id[] = "SDFMUSIC_M.pl";
// .cc file generated from SDFMUSIC_M.pl by ptlang
/*
Copyright (c) 1993-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFMUSIC_M.h"

const char *star_nm_SDFMUSIC_M = "SDFMUSIC_M";

const char* SDFMUSIC_M :: className() const {return star_nm_SDFMUSIC_M;}

ISA_FUNC(SDFMUSIC_M,SDFStar);

Block* SDFMUSIC_M :: makeNew() const { LOG_NEW; return new SDFMUSIC_M;}

SDFMUSIC_M::SDFMUSIC_M ()
{
	setDescriptor("This star is used to estimate the frequencies of some specified number\nof sinusoids in a signal. The output is the eigenspectrum of a signal\nsuch that the locations of the peaks of the eigenspectrum correspond\nto the frequencies of sinusoids in the signal.  The input is the right\nsingular vectors in the form generated by the SVD_M star.  The MUSIC\nalgorithm is used, where MUSIC stands for \"multiple signal classification.\"");
	addPort(rsvec.setPort("rsvec",this,FLOAT_MATRIX_ENV));
	addPort(output.setPort("output",this,FLOAT));
	addState(numRows.setState("numRows",this,"4","The number of rows in the right singular matrix V."));
	addState(numCols.setState("numCols",this,"4","The number of columns in the right singular matrix V."));
	addState(numSignals.setState("numSignals",this,"1","The number of unique signals we are trying to locate."));
	addState(resolution.setState("resolution",this,"256","The number of points in the frequency domain."));

# line 113 "SDFMUSIC_M.pl"
e = F = 0;
    Vn = VnVnt = 0;
}

SDFMUSIC_M::~SDFMUSIC_M() {
# line 107 "SDFMUSIC_M.pl"
delete e;
    delete Vn;
    delete VnVnt;
    delete F;
}

void SDFMUSIC_M::setup() {
# line 117 "SDFMUSIC_M.pl"
nrows = int(numRows);
    ncols = int(numCols);
    numNoise = ncols - 2*int(numSignals);

    // delete previously allocated arrays
    delete e;
    delete Vn;
    delete VnVnt;
    delete F;

    // allocate new arrays
    e = new ComplexMatrix(nrows,1);
    Vn = new FloatMatrix(nrows,numNoise);
    VnVnt = new FloatMatrix(nrows,nrows);
    F = new ComplexMatrix(1,1);

    // set the number of outputs
    output.setSDFParams(int(resolution)+1);
}

void SDFMUSIC_M::go() {
# line 137 "SDFMUSIC_M.pl"
// assumes that the singular values and the associated right singular
    // vectors have been sorted in descending order

    // load matrix of right singular vectors of the noise subspace
    Envelope inputPkt;
    (rsvec%0).getMessage(inputPkt);

    // check for empty input
    if(inputPkt.empty()) {
      FloatMatrix& V = *(new FloatMatrix(int(numRows),int(numCols)));
      V = 0.0;
      for(int row = 0; row < nrows; row++)
        for(int col = 0; col < numNoise; col++)
          (*Vn)[row][col] = V[row][col+(2*int(numSignals))];
      delete &V;
    }
    else {
      const FloatMatrix& V = *(const FloatMatrix *)inputPkt.myData();
      for(int row = 0; row < nrows; row++)
        for(int col = 0; col < numNoise; col++)
          (*Vn)[row][col] = V[row][col+(2*int(numSignals))];
    }
      
    // compute VnVnt = Vn * transpose(Vn)
    *VnVnt = (*Vn) * (~(*Vn));

    double w = -(M_PI);         // set omega to scan from -PI to PI
    for(int count = 0; count <= int(resolution); count++) {
      // compute e(w), the frequency-searching vector, given an w
      //  and eh(w), the hermitian transpose of e(w)
      for(int row = 0; row < nrows; row++)
        (*e)[row][0] = Complex(cos(row*w),sin(row*w));

      // calculate the real part of F(w) = eh(w)*VnVnt*e(w)
      *F = (e->hermitian()) * ComplexMatrix(*VnVnt) * (*e);

      // calculate the amplitude of S(w)
      result = 1/abs((*F)[0][0]);
      output%(count) << result;
      w += 2*(M_PI)/int(resolution);
    }
}

// prototype instance for known block list
static SDFMUSIC_M proto;
static RegisterBlock registerBlock(proto,"MUSIC_M");
