static const char file_id[] = "SDFDCTImage.pl";
// .cc file generated from SDFDCTImage.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFDCTImage.h"

const char *star_nm_SDFDCTImage = "SDFDCTImage";

const char* SDFDCTImage :: className() const {return star_nm_SDFDCTImage;}

ISA_FUNC(SDFDCTImage,SDFStar);

Block* SDFDCTImage :: makeNew() const { LOG_NEW; return new SDFDCTImage;}

SDFDCTImage::SDFDCTImage ()
{
	setDescriptor("This star takes a float matrix input particle,\ncomputes the discrete cosine transform (DCT),\nand outputs a float matrix.");
	addPort(input.setPort("input",this,FLOAT_MATRIX_ENV));
	addPort(output.setPort("output",this,FLOAT_MATRIX_ENV));
	addPort(originalW.setPort("originalW",this,INT));
	addPort(originalH.setPort("originalH",this,INT));
	addState(BlockSize.setState("BlockSize",this,"8","Block size of the DCT transform."));

# line 51 "SDFDCTImage.pl"
cosData = 0;
}

SDFDCTImage::~SDFDCTImage() {
# line 60 "SDFDCTImage.pl"
LOG_DEL; delete [] cosData;
}

void SDFDCTImage::setup() {
# line 54 "SDFDCTImage.pl"
blocksize = int(BlockSize);
		delete [] cosData;
		LOG_NEW; cosData = new float[blocksize*blocksize];
		cosSet();
}

void SDFDCTImage::go() {
# line 167 "SDFDCTImage.pl"
// Read input image.
                Envelope pkt;
                (input%0).getMessage(pkt);
                const FloatMatrix& inImage = *(const FloatMatrix*)pkt.myData();

		if (pkt.empty()) {
		  Error::abortRun(*this, "Input is a dummyMessage.");
		  return;
		}

		int inw = inImage.numCols();
		int inh = inImage.numRows();

		//Round output width and height to be multiples of "blocksize"
		int outw;
		int outh;
		if ((inw%blocksize) != 0)
		  outw = blocksize * (inw/blocksize +1);
		else 
		  outw = inw;
		if ((inh%blocksize) != 0)
		  outh = blocksize * (inh/blocksize +1);
		else 
		  outh = inh;

		//Allocate output image.
		LOG_NEW;
		FloatMatrix& outImage = *(new FloatMatrix(outh,outw));

		// Do transform.
		doDCT(outImage, inImage);

		// Send output.
		output%0 << outImage;
		originalW%0 << inw;
		originalH%0 << inh;
}


void SDFDCTImage::cosSet ()
{
# line 67 "SDFDCTImage.pl"
int i, j;
			double lambda = M_PI / (2.0 * blocksize);
			double t1 = 1.0 / sqrt(double(blocksize));
			double t2 = sqrt(2.0 / double(blocksize));
			for(j = 0; j < blocksize; j++) {
				cosData[j] = t1;
				for(i = 1; i < blocksize; i++) {
					cosData[j+i*blocksize] = float(t2 *
							cos(double((2*j+1)*i*lambda)) );
				}
			}
}


void SDFDCTImage::doDCT (FloatMatrix& outImage, const FloatMatrix& inImage)
{
# line 98 "SDFDCTImage.pl"
register int ndx, ndx2, cntr;
	int ii, jj, i, j;

	int outw = outImage.numCols();
	int outh = outImage.numRows();
	int inw = inImage.numCols();
	int inh = inImage.numRows();

	// Do the HORIZONTAL transform. From input to out...
	for(ii = 0; ii < inh; ii++) {
	  for(jj = 0; jj < outw-blocksize; jj += blocksize) {
	    ndx = ii*outw + jj;
	    ndx2 = ii*inw + jj;
	    for(j = 0; j < blocksize; j++) {
	      double sum = 0.0;
	      for(cntr = 0; cntr < blocksize; cntr++) {
		sum += fwdD(j, cntr) * inImage.entry(ndx2+cntr);
	      }
	      outImage.entry(ndx+j) = sum;
            }
	  }

	// Handle last (maybe partial) block at end of each row (i.e. zero-pad).
	  ndx = ii*outw + jj;
	  ndx2 = ii*inw + jj;
	  for(j = 0; j < blocksize; j++) {
	    double sum = 0.0;
	    for(cntr = 0; cntr < 1 + ((inw-1)%blocksize); cntr++) {
	      sum += fwdD(j, cntr) * inImage.entry(ndx2+cntr);
	    }
	    outImage.entry(ndx+j) = sum;
	  }
	}

// Handle rows at the bottom of the array.
	for(ii = inh; ii < outh; ii++) {
	  ndx = ii*outw;
	  for(j = 0; j < outw; j++) {
	    outImage.entry(ndx+j) = 0.0;
	  }
	} // end for(ii)


// Do the VERTICAL transform. From out to tmpbuf and copy back...
	LOG_NEW; float* tmpbuf = new float[outh + blocksize];
	for(jj = 0; jj < outw; jj++) {
	  for(ii = 0; ii < outh; ii += blocksize) {
	    ndx = ii*outw + jj;
	    for(i = 0; i < blocksize; i++) {
	      double sum = 0.0;
	      for(cntr = 0; cntr < blocksize; cntr++) {
		sum += fwdD(i, cntr) * outImage.entry(ndx + cntr*outw);
	      }
	      tmpbuf[ii+i] = sum;
	    }
	  }

// Copy data back to main buffer;
	  for(i = 0; i < outh; i++) {
	    outImage.entry(i*outw+jj) = tmpbuf[i];
	  }
	} // end for(jj)

	LOG_DEL; delete [] tmpbuf;
}


// prototype instance for known block list
static SDFDCTImage proto;
static RegisterBlock registerBlock(proto,"DCTImage");
