static const char file_id[] = "SDFDCTImageInv.pl";
// .cc file generated from SDFDCTImageInv.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFDCTImageInv.h"

const char *star_nm_SDFDCTImageInv = "SDFDCTImageInv";

const char* SDFDCTImageInv :: className() const {return star_nm_SDFDCTImageInv;}

ISA_FUNC(SDFDCTImageInv,SDFStar);

Block* SDFDCTImageInv :: makeNew() const { LOG_NEW; return new SDFDCTImageInv;}

SDFDCTImageInv::SDFDCTImageInv ()
{
	setDescriptor("This star takes a float matrix input and does inverse\ndiscrete cosine transform (DCT) coding and outputs a float matrix.");
	addPort(input.setPort("input",this,FLOAT_MATRIX_ENV));
	addPort(originalW.setPort("originalW",this,INT));
	addPort(originalH.setPort("originalH",this,INT));
	addPort(output.setPort("output",this,FLOAT_MATRIX_ENV));
	addState(BlockSize.setState("BlockSize",this,"8","Block size of the inverse DCT transform."));

# line 52 "SDFDCTImageInv.pl"
cosData = 0;
}

SDFDCTImageInv::~SDFDCTImageInv() {
# line 61 "SDFDCTImageInv.pl"
LOG_DEL; delete [] cosData;
}

void SDFDCTImageInv::setup() {
# line 55 "SDFDCTImageInv.pl"
blocksize = int(BlockSize);
		delete [] cosData;
		LOG_NEW; cosData = new float[blocksize*blocksize];
		cosSet();
}

void SDFDCTImageInv::go() {
# line 158 "SDFDCTImageInv.pl"
// Read input image.
                Envelope pkt;
                (input%0).getMessage(pkt);
                const FloatMatrix& inImage = *(const FloatMatrix*)pkt.myData();

		if (pkt.empty()) {
		  Error::abortRun(*this, "Input is a dummyMessage.");
		  return;
		}

		int outw = int(originalW%0);
		int outh = int(originalH%0);

		//Allocate output image.
		LOG_NEW;
		FloatMatrix& outImage = *(new FloatMatrix(outh,outw));

		// Need to allocate a new matrix for inImage
		// because the doInvDCT function modifies the matrix data!!
		LOG_NEW;
		FloatMatrix& tmpImage = *(new FloatMatrix(inImage));

		// Do inverse transform.
		doInvDCT(outImage, tmpImage);

		LOG_DEL; delete &tmpImage;

// Since we used writableCopy(), we own "dctimage".
		output%0 << outImage;
}


void SDFDCTImageInv::cosSet ()
{
# line 68 "SDFDCTImageInv.pl"
int i, j;
			double lambda = M_PI / (2.0 * blocksize);
			double t1 = 1.0 / sqrt(double(blocksize));
			double t2 = sqrt(2.0 / double(blocksize));
			for(j = 0; j < blocksize; j++) {
				cosData[j] = float(t1);
				for(i = 1; i < blocksize; i++) {
					cosData[j+blocksize*i] = float( t2 *
							cos(double((2*j+1)*i*lambda)) );
			}	}
}


void SDFDCTImageInv::doInvDCT (FloatMatrix& out, FloatMatrix& in)
{
# line 96 "SDFDCTImageInv.pl"
register int ndx, ndx2, cntr;
	int ii, jj, i, j;

	int outw = out.numCols();
	int outh = out.numRows();
	int inw = in.numCols();
	int inh = in.numRows();

	// Do the VERTICAL transform. image to tmpbuf and copy back...
	LOG_NEW; float* tmpbuf = new float[(inw>inh ? inw : inh) + blocksize];
	for(jj = 0; jj < inw; jj++) {
		for(ii = 0; ii < inh; ii += blocksize) {
			ndx = ii*inw + jj;
			for(i = 0; i < blocksize; i++) {
				double sum = 0.0;
				for(cntr = 0; cntr < blocksize; cntr++) {
					sum += invD(i, cntr) *
					       in.entry(ndx + cntr*inw);
				}
				tmpbuf[ii+i] = sum;
			}
		}

		// Copy data back to main buffer;
		for(i = 0; i < inh; i++) {
			in.entry(i*inw + jj) = tmpbuf[i];
		}
	} // end for(jj)

	// Do HORIZONTAL xform. From indata to tmpbuf to outdata...
	for(ii = 0; ii < outh; ii++) {
		for(jj = 0; jj < inw; jj += blocksize) {
			ndx = ii*inw + jj;
			for(j = 0; j < blocksize; j++) {
				double sum = 0.0;
				for(cntr = 0; cntr < blocksize; cntr++) {
				  sum += invD(j, cntr) * in.entry(ndx+cntr);
				}
				tmpbuf[jj+j] = sum;
			}
		}

		// Copy to "outdata".
		ndx2 = ii*outw;
		for(j = 0; j < outw; j++) {
		  if (tmpbuf[j] < 0.5) {
		    out.entry(ndx2+j) = (unsigned char) 0;
		  }
		  else if (tmpbuf[j] > 254.5) {
		    out.entry(ndx2+j) = (unsigned char) 255;
		  }
		  else {
		    out.entry(ndx2+j) = (unsigned char) (tmpbuf[j] + 0.5);
		  }
		}
	} // end for(ii)

	LOG_DEL; delete [] tmpbuf;
}


// prototype instance for known block list
static SDFDCTImageInv proto;
static RegisterBlock registerBlock(proto,"DCTImageInv");
