static const char file_id[] = "SDFDCTImgCde.pl";
// .cc file generated from SDFDCTImgCde.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFDCTImgCde.h"

const char *star_nm_SDFDCTImgCde = "SDFDCTImgCde";

const char* SDFDCTImgCde :: className() const {return star_nm_SDFDCTImgCde;}

ISA_FUNC(SDFDCTImgCde,SDFStar);

Block* SDFDCTImgCde :: makeNew() const { LOG_NEW; return new SDFDCTImgCde;}

# line 71 "SDFDCTImgCde.pl"
const float StartOfBlock = 524288.0;
    const float StartOfRun = 1048576.0;

SDFDCTImgCde::SDFDCTImgCde ()
{
	setDescriptor("This star takes a Float Matrix which represents a DCT image, \ninserts \"start of block\" markers, run-length encodes it, \nand outputs the modified image. \nFor the run-length encoding, all values with absolute value less than \"Thresh\"\nare set to 0.0, to help improve compression.\n\nRunlengths are coded with a \"start of run\" symbol and then an (integer) \nrun-length.\n\"HiPri\" DCT coefficients per block are sent to \"hiport\", the high-priority \noutput. \nThe remainder of the coefficients are sent to \"loport\", the low-priority \noutput.");
	addPort(inport.setPort("inport",this,FLOAT_MATRIX_ENV));
	addPort(hiport.setPort("hiport",this,FLOAT_MATRIX_ENV));
	addPort(loport.setPort("loport",this,FLOAT_MATRIX_ENV));
	addPort(originalW.setPort("originalW",this,INT));
	addPort(originalH.setPort("originalH",this,INT));
	addPort(compr.setPort("compr",this,FLOAT));
	addState(BlockSize.setState("BlockSize",this,"8","Block size of the input DCT transformed image."));
	addState(Thresh.setState("Thresh",this,"12.0","Threshold for run-length coding."));
	addState(HiPri.setState("HiPri",this,"3","Number of DCT coefficients per block sent to 'hiport'."));

# line 81 "SDFDCTImgCde.pl"
outDc = 0;
    outAc = 0;
}

SDFDCTImgCde::~SDFDCTImgCde() {
# line 203 "SDFDCTImgCde.pl"
delete [] outDc;
    delete [] outAc;
}

void SDFDCTImgCde::setup() {
# line 86 "SDFDCTImgCde.pl"
thresh = float(fabs(double(Thresh)));
}

void SDFDCTImgCde::go() {
# line 167 "SDFDCTImgCde.pl"
// Read input.
    Envelope inEnvp;
    (inport%0).getMessage(inEnvp);
    const FloatMatrix& inImage = *(const FloatMatrix *) inEnvp.myData();

    if (inEnvp.empty()) {
      Error::abortRun(*this, "Input is a dummyMessage.");
      return;
    }

    // Do processing and send out.
    doRunLen(inImage);

    // Copy the data to the output images.
    FloatMatrix& dcImage = *(new FloatMatrix(1,indxDc));
    for(int i = 0; i < indxDc; i++) {
      dcImage.entry(i) = outDc[i];
    }

    FloatMatrix& acImage = *(new FloatMatrix(1,indxAc));
    for(int j = 0; j < indxAc; j++) {
      acImage.entry(j) = outAc[j];
    }

    float comprRatio = dcImage.numRows() * dcImage.numCols() +
                       acImage.numRows() * acImage.numCols();
    comprRatio /= inImage.numRows()*inImage.numCols();
    compr%0 << comprRatio;

    hiport%0 << dcImage;
    loport%0 << acImage;
    originalW%0 << inImage.numCols();
    originalH%0 << inImage.numRows();
}


void SDFDCTImgCde::doRunLen (const FloatMatrix& inImage)
{
# line 95 "SDFDCTImgCde.pl"
// Initialize.
      const int bSize = int(BlockSize);
      const int size = inImage.numCols() * inImage.numRows();
      const int blocks = size / (bSize*bSize);

      // Temporary storage for one block.
      float* tmpFloatPtr = new float[size];
      float* tmpPtr = tmpFloatPtr;
      for (int i = 0; i < size; i++) {
	tmpPtr[i] = float(inImage.entry(i));
      }

      // The biggest runlen blowup we can have is the string "01010101...".
      // This gives a blowup of 50%, so with StartOfBlock and StartOfRun
      // markers, 1.70 should be ok.
      delete [] outDc;
      delete [] outAc;
      outDc = new float[int(1.70*size + 1)];
      outAc = new float[int(1.70*size + 1)];
      indxDc = 0; indxAc = 0;
      for (int blk = 0; blk < blocks; blk++) {
	// High priority coefficients.
	for(i = 0; i < HiPri; i++) {
	  outDc[indxDc++] = *tmpPtr++;
	}
	
	// Low priority coefficients--start with block header.
	outAc[indxAc++] = StartOfBlock;
	outAc[indxAc++] = float(blk);
	
	int zeroRunLen = 0;
	for(; i < bSize*bSize; i++) {
	  if(zeroRunLen) {
	    if (larger(*tmpPtr, thresh)) {
	      outAc[indxAc++] = float(zeroRunLen);
	      zeroRunLen = 0;
	      outAc[indxAc++] = *tmpPtr;
	    } else {
	      zeroRunLen++;
	    }
	  } else {
	    if (larger(*tmpPtr, thresh)) {
	      outAc[indxAc++] = *tmpPtr;
	    } else {
	      outAc[indxAc++] = StartOfRun;
	      zeroRunLen++;
	    }
	  }
	  tmpPtr++;
				}
	// Handle zero-runs that last till end of the block.
	if(zeroRunLen) {
	  outAc[indxAc++] = float(zeroRunLen);
	}
      }

      // Delete tmpFloatPtr and not tmpPtr since tmpPtr has been incremented
      delete [] tmpFloatPtr;
}


// prototype instance for known block list
static SDFDCTImgCde proto;
static RegisterBlock registerBlock(proto,"DCTImgCde");
