static const char file_id[] = "SDFDCTImgCdeInv.pl";
// .cc file generated from SDFDCTImgCdeInv.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFDCTImgCdeInv.h"

const char *star_nm_SDFDCTImgCdeInv = "SDFDCTImgCdeInv";

const char* SDFDCTImgCdeInv :: className() const {return star_nm_SDFDCTImgCdeInv;}

ISA_FUNC(SDFDCTImgCdeInv,SDFStar);

Block* SDFDCTImgCdeInv :: makeNew() const { LOG_NEW; return new SDFDCTImgCdeInv;}

# line 57 "SDFDCTImgCdeInv.pl"
// Numbers unlikely to come from a DCT...
	  const float StartOfBlock = 524288.0;
	  const float StartOfRun = 1048576.0;

SDFDCTImgCdeInv::SDFDCTImgCdeInv ()
{
	setDescriptor("This star reads two coded DCT images (one high priority and one low-priority),\nwhich are represented by two input FloatMatrix particle. \ninverts the run-length encoding, and outputs the resulting image.\nProtection is built in to avoid crashing even if some of the coded\ninput data is affected by loss.");
	addPort(hiport.setPort("hiport",this,FLOAT_MATRIX_ENV));
	addPort(loport.setPort("loport",this,FLOAT_MATRIX_ENV));
	addPort(originalW.setPort("originalW",this,INT));
	addPort(originalH.setPort("originalH",this,INT));
	addPort(output.setPort("output",this,FLOAT_MATRIX_ENV));
	addState(BlockSize.setState("BlockSize",this,"8","Block size of the DCT transformed image."));
	addState(HiPri.setState("HiPri",this,"1","Number of DCT coeffs per block to 'highport' input."));


}

void SDFDCTImgCdeInv::go() {
# line 135 "SDFDCTImgCdeInv.pl"
// Read input images.
    Envelope hiEnvp;
    (hiport%0).getMessage(hiEnvp);
    const FloatMatrix& hiImage = *(const FloatMatrix *) hiEnvp.myData();

    Envelope loEnvp;
    (loport%0).getMessage(loEnvp);
    const FloatMatrix& loImage = *(const FloatMatrix *) loEnvp.myData();

    if ( hiEnvp.empty() || loEnvp.empty() ) {
      Error::abortRun(*this, "Input is a dummyMessage.");
      return;
    }

    // Do the conversion.
    FloatMatrix& outImage = invRunLen(hiImage, loImage,
                                      int(originalW%0),int(originalH%0));

    // Send output.
    output%0 << outImage;
}


FloatMatrix& SDFDCTImgCdeInv::invRunLen (const FloatMatrix& hiImage, const FloatMatrix& loImage, int width, int height)
{
# line 69 "SDFDCTImgCdeInv.pl"
// Initialize.
      const int bSize = int(BlockSize);
      const int fullFrame = width * height;

      // Do DC image first.
      int i, j, k, blk;

      float* outPtr = new float[fullFrame];
      for(k = 0; k < fullFrame; k++) { outPtr[k] = 0.0; }
      
      i = 0;
      for(j = 0; j < fullFrame; j += bSize * bSize) {
	for(k = 0; k < HiPri; k++) {
	  outPtr[j + k] = hiImage.entry(i++);
	}
      }
      
      // While still low priority input data left...
      const int size = loImage.numRows() * loImage.numCols();
      
      i = 0;
      while (i < size) {

	// Process each block, which starts with "StartOfBlock".
	while ((i < size) && (loImage.entry(i) != StartOfBlock)) {
	  i++;
	}
	if (i < size-2) {
	  i++;
	  blk = int(loImage.entry(i++));
	  blk *= bSize*bSize;
	  if ((blk >= 0) && (blk < fullFrame)) {
	    blk += HiPri;
	    k = 0;
	    while ((i < size) && (k < bSize*bSize - HiPri)
		   && (loImage.entry(i) != StartOfBlock)) {
	      if (loImage.entry(i) == StartOfRun) {
		i++;
		if (i < size) {
		  int runLen = int(loImage.entry(i++));
		  for(int l = 0; l < runLen; l++ ) {
		    outPtr[blk+k] = 0.0;
		    k++;
		  }
		}
	      } else {
		outPtr[blk+k] = loImage.entry(i++);
		k++;
	      }
	    }
	  } // if (blk OK)
	}
      } // end while (indx < size)
      
      // Copy the data to return.
      FloatMatrix& outImage = *(new FloatMatrix(height,width));
      for(i = 0; i < fullFrame; i++) {
	outImage.entry(i) = outPtr[i];
      }
      delete [] outPtr;

      return outImage;
}


// prototype instance for known block list
static SDFDCTImgCdeInv proto;
static RegisterBlock registerBlock(proto,"DCTImgCdeInv");
