static const char file_id[] = "SDFDPCMImage.pl";
// .cc file generated from SDFDPCMImage.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFDPCMImage.h"

const char *star_nm_SDFDPCMImage = "SDFDPCMImage";

const char* SDFDPCMImage :: className() const {return star_nm_SDFDPCMImage;}

ISA_FUNC(SDFDPCMImage,SDFStar);

Block* SDFDPCMImage :: makeNew() const { LOG_NEW; return new SDFDPCMImage;}

SDFDPCMImage::SDFDPCMImage ()
{
	setDescriptor("This star implements differential pulse code modulation of an image.\nIf the \"past\" input is not a Float Matrix, pass the \"input\"\ndirectly to the \"output\". Otherwise, subtract the \"past\" from the\n\"input\" (with leakage factor alpha) and send the result to \"output\".");
	addPort(input.setPort("input",this,FLOAT_MATRIX_ENV));
	addPort(past.setPort("past",this,FLOAT_MATRIX_ENV));
	addPort(output.setPort("output",this,FLOAT_MATRIX_ENV));
	addState(alpha.setState("alpha",this,"1.0","Leak value for error-recovery."));


}

void SDFDPCMImage::setup() {
# line 42 "SDFDPCMImage.pl"
leak = float(double(alpha));
}

void SDFDPCMImage::go() {
# line 75 "SDFDPCMImage.pl"
// Read data from input.
    Envelope curEnvp, pastEnvp;
    (input%0).getMessage(curEnvp);
    (past%0).getMessage(pastEnvp);
    const FloatMatrix& inImage = *(const FloatMatrix *) curEnvp.myData();
    const FloatMatrix& pastImage = *(const FloatMatrix *) pastEnvp.myData();

    if (pastEnvp.empty()) {
      output%0 << curEnvp;
      return;
    }
    
    if(!inputsOk(inImage, pastImage)) {
      Error::abortRun(*this, "Problem with input images.");
      return;
    }

    int width = inImage.numCols();
    int height = inImage.numRows();

    // Handle real valid inputs...
    LOG_NEW; FloatMatrix& diffImage = *(new FloatMatrix(height,width));
    for(int travel = width*height - 1; travel >= 0; travel--) {
      diffImage.entry(travel) = 
	quant(inImage.entry(travel), pastImage.entry(travel));
    }

    output%0 << diffImage;
}


int SDFDPCMImage::inputsOk (const FloatMatrix& one, const FloatMatrix& two)
{
# line 68 "SDFDPCMImage.pl"
int retval = (one.numRows() == two.numRows());
      retval &= (one.numCols() == two.numCols());
      return(retval);
}


// prototype instance for known block list
static SDFDPCMImage proto;
static RegisterBlock registerBlock(proto,"DPCMImage");
