static const char file_id[] = "SDFDPCMImageInv.pl";
// .cc file generated from SDFDPCMImageInv.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFDPCMImageInv.h"

const char *star_nm_SDFDPCMImageInv = "SDFDPCMImageInv";

const char* SDFDPCMImageInv :: className() const {return star_nm_SDFDPCMImageInv;}

ISA_FUNC(SDFDPCMImageInv,SDFStar);

Block* SDFDPCMImageInv :: makeNew() const { LOG_NEW; return new SDFDPCMImageInv;}

SDFDPCMImageInv::SDFDPCMImageInv ()
{
	setDescriptor("This star inverts differential pulse code modulation of an image.\nIf the \"past\" input is not a GrayImage or has size 0, pass the \"diff\"\ndirectly to the \"output\". Otherwise, add the \"past\" to the \"diff\"\n(with leakage factor alpha) and send the result to \"output\".");
	addPort(diff.setPort("diff",this,FLOAT_MATRIX_ENV));
	addPort(past.setPort("past",this,FLOAT_MATRIX_ENV));
	addPort(output.setPort("output",this,FLOAT_MATRIX_ENV));
	addState(alpha.setState("alpha",this,"1.0","Leak value to aid error recovery."));


}

void SDFDPCMImageInv::setup() {
# line 43 "SDFDPCMImageInv.pl"
leak = float(double(alpha));
}

void SDFDPCMImageInv::go() {
# line 68 "SDFDPCMImageInv.pl"
// Read data from input.
    Envelope diffEnvp, pastEnvp;
    (diff%0).getMessage(diffEnvp);
    (past%0).getMessage(pastEnvp);
    const FloatMatrix& diffImage = *(const FloatMatrix *) diffEnvp.myData();
    const FloatMatrix& pastImage = *(const FloatMatrix *) pastEnvp.myData();

    // Resynchronize if 'past' of wrong type.
    if (pastEnvp.empty()) {
      output%0 << diffEnvp;
      return;
    }
    
    if(!inputsOk(diffImage, pastImage)) {
      Error::abortRun(*this, "Problem with input images.");
      return;
    }

    int width = diffImage.numCols();
    int height = diffImage.numRows();

    LOG_NEW; FloatMatrix& outImage = *(new FloatMatrix(height,width));

    for(int travel = width*height - 1; travel >= 0; travel--) {
      outImage.entry(travel) = 
	quant(diffImage.entry(travel), pastImage.entry(travel));
    }

    // Send the outputs on their way.
    output%0 << outImage;
}


int SDFDPCMImageInv::inputsOk (const FloatMatrix& one, const FloatMatrix& two)
{
# line 61 "SDFDPCMImageInv.pl"
int retval = (one.numRows() == two.numRows());
      retval &= (one.numCols() == two.numCols());
      return(retval);
}


// prototype instance for known block list
static SDFDPCMImageInv proto;
static RegisterBlock registerBlock(proto,"DPCMImageInv");
