static const char file_id[] = "SDFDisplayImage.pl";
// .cc file generated from SDFDisplayImage.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFDisplayImage.h"
#include "Matrix.h"
#include <std.h>
#include <stdio.h>
#include "Error.h"
#include "StringList.h"

const char *star_nm_SDFDisplayImage = "SDFDisplayImage";

const char* SDFDisplayImage :: className() const {return star_nm_SDFDisplayImage;}

ISA_FUNC(SDFDisplayImage,SDFStar);

Block* SDFDisplayImage :: makeNew() const { LOG_NEW; return new SDFDisplayImage;}

SDFDisplayImage::SDFDisplayImage ()
{
	setDescriptor("Accept a black-and-white input grayimage represented by a float matrix \nand generate output in PGM format.\nSend the output to a user-specified command (by default, \"xv\" is used).\n\nThe user can set the root filename of the displayed image (which will\nprobably be printed in the image display window titlebar) and\ncan choose whether or not the image file is saved or deleted.\nThe image frame number is appended to the root filename to form the\ncomplete filename of the displayed image.");
	addPort(dataInput.setPort("dataInput",this,FLOAT_MATRIX_ENV));
	addPort(frameIdInput.setPort("frameIdInput",this,INT));
	addState(command.setState("command",this,"xv","Program to run on PGM data"));
	addState(imageName.setState("imageName",this,"","If non-null, name for PGM file"));
	addState(saveImage.setState("saveImage",this,"n","If 'y' or 'Y', then save the file"));


}

void SDFDisplayImage::go() {
# line 58 "SDFDisplayImage.pl"
// Read data from input.
                Envelope pkt;
                (dataInput%0).getMessage(pkt);
                const FloatMatrix& matrix = *(const FloatMatrix*)pkt.myData();

		int height = matrix.numRows();
		int width  = matrix.numCols();

		// Create an array to read into image data.
		unsigned char* data = new unsigned char[height*width];

		for (int i=0; i<height*width ; i++) {
			if (matrix.entry(i) < 0)
				data[i] = 0;
			else if(matrix.entry(i) > 255)
				data[i] = 255;
			else data[i] = int(matrix.entry(i)); // convert to int
		}

		// Set filename and save values.
		const char* saveMe = saveImage;
		int del = !((saveMe[0] == 'y') || (saveMe[0] == 'Y'));

		const char* iname = imageName;
		char* nm = 0;
		if (iname && iname[0]) {
		  nm = expandPathName(iname);
		}
		else {
		  nm = tempFileName();
		}
		StringList fileName = nm;
		fileName << "." << int(frameIdInput%0);
		delete [] nm;

		FILE* fptr = fopen(fileName, "w");
		if (fptr == (FILE*) NULL) {
		  Error::abortRun(*this, "cannot create: ", fileName);
		  return;
		}

		// Write PGM header and image data (row by row)
		fprintf( fptr,
			 "P5\n %d %d 255\n",
			 width,
			 height );
		fwrite( data,
			width * sizeof(unsigned char),
			height,
			fptr );

		delete [] data;
		fclose(fptr);

		// Display the image using an external viewer
		StringList cmdbuf = "(";
		cmdbuf << ((const char *) command) << " " << fileName;
		if (del) {
		  cmdbuf << "; rm -f " << fileName;
		}
		cmdbuf << ")&";		// Run command in the background
		system(cmdbuf);
}

// prototype instance for known block list
static SDFDisplayImage proto;
static RegisterBlock registerBlock(proto,"DisplayImage");
