static const char file_id[] = "SDFDisplayRGB.pl";
// .cc file generated from SDFDisplayRGB.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFDisplayRGB.h"
#include "Matrix.h"
#include <std.h>
#include <stdio.h>
#include "Error.h"

const char *star_nm_SDFDisplayRGB = "SDFDisplayRGB";

const char* SDFDisplayRGB :: className() const {return star_nm_SDFDisplayRGB;}

ISA_FUNC(SDFDisplayRGB,SDFStar);

Block* SDFDisplayRGB :: makeNew() const { LOG_NEW; return new SDFDisplayRGB;}

SDFDisplayRGB::SDFDisplayRGB ()
{
	setDescriptor("Accept three ColorImages (Red, Green, and Blue) from three input\nfloat matrix and generate a color image file in Portable Pixmap (PPM) format.\nSend the filename to a user-specified command \n(by default,\n.EQ\ndelim off\n.EN\n\"xv\"\n.EQ\ndelim $$\n.EN\nis used).\n\nThe user can set the root filename of the displayed image (which will\nprobably be printed in the image display window title bar) and\ncan choose whether or not the image file is saved or deleted.\nThe frameId of the received image is appended to the root filename\nto produce the full filename of the displayed image.");
	addPort(rinput.setPort("rinput",this,FLOAT_MATRIX_ENV));
	addPort(ginput.setPort("ginput",this,FLOAT_MATRIX_ENV));
	addPort(binput.setPort("binput",this,FLOAT_MATRIX_ENV));
	addPort(frameIdIn.setPort("frameIdIn",this,INT));
	addState(command.setState("command",this,"xv","Program to run on PGM data"));
	addState(imageName.setState("imageName",this,"","If non-null, name for PGM file"));
	addState(saveColor.setState("saveColor",this,"n","If 'y' or 'Y', then save the file"));


}

void SDFDisplayRGB::go() {
# line 70 "SDFDisplayRGB.pl"
// Read data from inputs.
                Envelope rpkt,gpkt,bpkt;
                (rinput%0).getMessage(rpkt);
                (ginput%0).getMessage(gpkt);
                (binput%0).getMessage(bpkt);
                const FloatMatrix& rimage = *(const FloatMatrix*)rpkt.myData();
                const FloatMatrix& gimage = *(const FloatMatrix*)gpkt.myData();
                const FloatMatrix& bimage = *(const FloatMatrix*)bpkt.myData();

		int height = rimage.numRows();
		int width  = rimage.numCols();

		// Create an array to read into image data.
		unsigned char* rgbfp = new unsigned char[3*height*width];

		int i, j, temp1, temp2, temp3;
		for (i = 0; i < height; i++) {
			temp1 = i*width;
			for (j = 0; j < width; j++){
				temp2 = j+temp1;
				temp3 = 3*temp2;
				rgbfp[temp3] = (unsigned char)(rimage.entry(temp2));
				rgbfp[temp3+1] = (unsigned char)(gimage.entry(temp2));
				rgbfp[temp3+2] = (unsigned char)(bimage.entry(temp2));
			}
		}

		// Set filename and save values.
		const char* saveMe = saveColor;
		int del = !((saveMe[0] == 'y') || (saveMe[0] == 'Y'));

		const char* iname = imageName;
		char* nm = 0;
		if (iname && iname[0]) {
		  nm = expandPathName(iname);
		}
		else {
		  nm = tempFileName();
		}
		StringList fileName = nm;
		fileName << "." << int(frameIdIn%0);
		delete [] nm;

		FILE* fptr = fopen(fileName, "w");
		if (fptr == (FILE*) NULL) {
			Error::abortRun(*this, fileName, ": cannot create");
			return;
		}

		// Write the PPM header and then the data.
		fprintf(fptr, "P6\n %d %d 255\n", width, height);
		fwrite((const char*) rgbfp, sizeof(unsigned char),
				unsigned(3*width*height), fptr);
		fclose(fptr);

		delete [] rgbfp;

		// Build up Unix command to display the image
		StringList buf = "(";
		buf << (const char*) command << " " << fileName;
		if (del) {
			buf << "; rm -f " << fileName;
		}
		buf << ")&";		// Run command in the background
		system(buf);
}

// prototype instance for known block list
static SDFDisplayRGB proto;
static RegisterBlock registerBlock(proto,"DisplayRGB");
