static const char file_id[] = "SDFDisplayVideo.pl";
// .cc file generated from SDFDisplayVideo.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFDisplayVideo.h"
#include <std.h>
#include <stdio.h>

const char *star_nm_SDFDisplayVideo = "SDFDisplayVideo";

const char* SDFDisplayVideo :: className() const {return star_nm_SDFDisplayVideo;}

ISA_FUNC(SDFDisplayVideo,SDFStar);

Block* SDFDisplayVideo :: makeNew() const { LOG_NEW; return new SDFDisplayVideo;}

SDFDisplayVideo::SDFDisplayVideo ()
{
	setDescriptor("Accept a stream of black-and-white images from input Float Matrix,\nsave the images to files, and display the resulting files as a\nmoving video sequence.\nThis star requires that programs from the \"Utah Raster Toolkit\"\nbe in your path.\nAlthough this toolkit is not included with Ptolemy, it is available for free.\nSee this star's long description (with the \"look-inside\" or\n\"manual\" commands in the Ptolemy menu) for information on how\nto get the toolkit.\n\nThe user can set the root filename of the displayed images\n(which probably will be printed in the display window titlebar)\nwith the ImageName parameter. If no filename is set, a default\nwill be chosen.\n\nThe Save parameter can be set to \"YES\" or \"NO\" to choose whether\nthe created image files should be saved or deleted. Each images\nframe number is appended to the root filename to form the images\ncomplete filename.\n\nThe ByFields parameter can be set to either \"YES\" or \"NO\" to choose\nwhether the input images should be treated as interlaced fields\nthat make up a frame or as entire frames.\nIf the inputs are fields, then the first field should contain\nframe lines 1, 3, 5, etc. and the second field should contain\nlines 0, 2, 4, 6, etc");
	addPort(inData.setPort("inData",this,FLOAT_MATRIX_ENV));
	addPort(inFrameId.setPort("inFrameId",this,INT));
	addState(ImageName.setState("ImageName",this,"","If non-null, name for run-length encoded file."));
	addState(Save.setState("Save",this,"NO","If true (YES), then save the file."));
	addState(ByFields.setState("ByFields",this,"NO","If true (YES), then inputs are interlaced fields."));

# line 124 "SDFDisplayVideo.pl"
tempFrame = 0;
}

void SDFDisplayVideo::wrapup() {
# line 185 "SDFDisplayVideo.pl"
// Display the video here.
    if ( allFileNames.length() > 0 ) {
      StringList cmd = "getx11 -m ";
      cmd << allFileNames;
      system(cmd);
    }
}

SDFDisplayVideo::~SDFDisplayVideo() {
# line 181 "SDFDisplayVideo.pl"
CleanUp();
}

void SDFDisplayVideo::setup() {
# line 128 "SDFDisplayVideo.pl"
CleanUp();

    fieldNum = 0;
    firstTime = TRUE;

    delete [] tempFrame;
    tempFrame = 0;

    // Set rootName
    const char* t = ImageName;
    char *nm = 0;
    if (t && t[0]) {
      nm = expandPathName(t);
    }
    else {
      nm = tempFileName();
    }
    rootName = nm;
    delete [] nm;

    // Set tmpFile
    nm = tempFileName();
    tmpFile = nm;
    delete [] nm;
}

void SDFDisplayVideo::go() {
# line 261 "SDFDisplayVideo.pl"
// Read data from input.
    Envelope envp;
    (inData%0).getMessage(envp);
    const FloatMatrix& imD = *(const FloatMatrix*)envp.myData();

    if (envp.empty()) {
      Error::abortRun(*this, "Input is a dummyMessage.");
      return;
    }

    // Do size check.
    if (firstTime) {
      DoFirstTime(imD);
    }
    else {
      if (SizeCheck(imD)) {
	Error::abortRun(*this, "Different input image sizes.");
	return;
      }
    }

    // Process the input data.
    if (ByFields) {
      if (!fieldNum) {
	for(int i = 0; i < height/2; i++) {
	  const int t2 = i*width;
	  const int t1 = 2*t2 + width;
	  for(int j = 0; j < width; j++) {
	    tempFrame[t1 + j] = (unsigned char)(imD.entry(t2 + j));
	  }
	}
	fieldNum = 1;
      } else {
	for(int i = 0; i < height/2; i++) {
	  const int t2 = i*width;
	  const int t1 = 2*t2;
	  for(int j = 0; j < width; j++) {
	    tempFrame[t1 + j] = (unsigned char)(imD.entry(t2 + j));
	  }
	}
	fieldNum = 0;
	DataToFile(tempFrame, int(inFrameId%0));
      }

    } else { // Data is NOT by fields.
      for (int i=0; i<height*width ; i++) {
	tempFrame[i] = (unsigned char)(imD.entry(i));
      }
      DataToFile(tempFrame, int(inFrameId%0));
    }
}


void SDFDisplayVideo::CleanUp ()
{
# line 160 "SDFDisplayVideo.pl"
StringList cmd = "rm -f";
      int shellout = FALSE;
      if ( tmpFile.length() > 0 ) {
	cmd << " " << tmpFile;
	shellout = TRUE;
      }
      tmpFile.initialize();
      if ((allFileNames.length() > 0) && !int(Save)) {
	cmd << " " << allFileNames;
	shellout = TRUE;
      }
      if (shellout) {
	system(cmd);
      }
      allFileNames.initialize();	// Clear the file list.
      LOG_DEL; delete [] tempFrame;
      tempFrame = 0;
}


void SDFDisplayVideo::DoFirstTime (const FloatMatrix& img)
{
# line 200 "SDFDisplayVideo.pl"
width = img.numCols();
      height = img.numRows();
      if (ByFields) {
	height *= 2;
      }
      delete [] tempFrame;
      tempFrame = new unsigned char[width*height];

      firstTime = FALSE;
}


int SDFDisplayVideo::SizeCheck (const FloatMatrix& img)
{
# line 219 "SDFDisplayVideo.pl"
int err = (img.numCols() != width);
      if (ByFields) { err |= (img.numRows() != height/2); }
      else { err |= (img.numRows() != height); }
      return err;
}


void SDFDisplayVideo::DataToFile (const unsigned char* data, const int id)
{
# line 232 "SDFDisplayVideo.pl"
// Open file, write data, close file.
      FILE* fptr = fopen(tmpFile, "w");
      if (fptr == (FILE*) NULL) {
	Error::abortRun(*this, "cannot create: ", tmpFile);
	return;
      }

      fwrite((const char*) data, sizeof(unsigned char),
	     (unsigned) (width*height), fptr);
      fclose(fptr);

      // Translate data.
      StringList fileName = rootName;
      fileName << id;

      StringList cmd = "rawtorle -n 1";
      cmd << " -w " << width;
      cmd << " -h " << height;
      cmd << " < "  << tmpFile;
      cmd << " | rleflip -v -o " << fileName;
      system(cmd);

      // Add file to list.
      allFileNames << " " << fileName;
}


// prototype instance for known block list
static SDFDisplayVideo proto;
static RegisterBlock registerBlock(proto,"DisplayVideo");
