static const char file_id[] = "SDFDither.pl";
// .cc file generated from SDFDither.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFDither.h"
#include "Matrix.h"

const char *star_nm_SDFDither = "SDFDither";

const char* SDFDither :: className() const {return star_nm_SDFDither;}

ISA_FUNC(SDFDither,SDFStar);

Block* SDFDither :: makeNew() const { LOG_NEW; return new SDFDither;}

# line 56 "SDFDither.pl"
extern "C" {
	    extern int strcasecmp(const char*,const char*);
	}

SDFDither::SDFDither ()
{
	setDescriptor("Do digital halftoning (dither) of input image for monochrome printing.\nInput image should be in a float matrix.\nThe possible dither methods are: Err-Diffusion, Clustered, Dispersed, Own.\nIf you specify \"Own\", then you can use your own dither mask.");
	addPort(input.setPort("input",this,FLOAT_MATRIX_ENV));
	addPort(output.setPort("output",this,FLOAT_MATRIX_ENV));
	addState(ditherType.setState("ditherType",this,"Err-Diffusion","Name of the detector to be used."));
	addState(ownMask.setState("ownMask",this,"","Dither mask to be used when specify \"Own\" in ditherType."));
	addState(rowsOwnMask.setState("rowsOwnMask",this,"4","Number of rows for ownMask."));
	addState(colsOwnMask.setState("colsOwnMask",this,"4","Number of columns for ownMask."));

# line 62 "SDFDither.pl"
mask = 0;
}

SDFDither::~SDFDither() {
# line 65 "SDFDither.pl"
delete [] mask;
}

void SDFDither::setup() {
# line 68 "SDFDither.pl"
delete [] mask;

	const char *dt = ditherType;

	if ( strcasecmp( dt, "Err-Diffusion") == 0 ) {
            // It doesn't need dither mask for error diffusion.
	}
	else if ( strcasecmp(dt, "Clustered") == 0 ) {
            // Dither mask for clustered dither.
            rowsMask = 4; colsMask = 4;
            mask = new double[rowsMask*colsMask];
            mask[0] = 75; mask[1] = 30; mask[2] = 45; mask[3] =105;
            mask[4] =150; mask[5] = 15; mask[6] =225; mask[7] =240;
            mask[8] =135; mask[9] = 90; mask[10]=210; mask[11]=180;
            mask[12]=120; mask[13]= 60; mask[14]=195; mask[15]=165;
	}
	else if ( strcasecmp(dt, "Dispersed") == 0 ) {
            // Dither mask for dispersed dither.
            rowsMask=4; colsMask=4;
            mask = new double[rowsMask*colsMask];
            mask[0] = 15; mask[1] =135; mask[2] = 45; mask[3] =165;
            mask[4] =195; mask[5] = 75; mask[6] =225; mask[7] =105;
            mask[8] = 60; mask[9] =180; mask[10]= 30; mask[11]=150;
            mask[12]=240; mask[13]=120; mask[14]=210; mask[15]= 90;
	}
	else if ( strcasecmp( dt, "Own")==0 ) {
            // Use dither mask specified by user
            int size = ownMask.size();
            rowsMask = int(rowsOwnMask); colsMask = int(colsOwnMask);
            //  check if ownMask doesn't have enough elements.
            if ( size != rowsMask * colsMask ) {
	       Error::abortRun(*this,
			       "The number of elements in 'ownMask' ",
			       "doesn't match the specified dimension.");
	       return;
            }
            mask = new double[rowsMask*colsMask];
            double *usermask = (double *) ownMask;
	    for (int i = 0; i < rowsMask*colsMask; i++ ) {
	      mask[i] = usermask[i];
	    }
	}
	else {
	    Error::abortRun(*this, "Unknown dither type ", dt);
	    return;
	}
}

void SDFDither::go() {
# line 116 "SDFDither.pl"
Envelope pkt;
        (input%0).getMessage(pkt);
        const FloatMatrix& matrix = *(const FloatMatrix*)pkt.myData();

        if(pkt.empty()) {
          // input empty, send out a zero matrix
          output%0 << double(0);
        }
        else {
          // valid input matrix
          int height = matrix.numRows();
          int width  = matrix.numCols();

          FloatMatrix& result = *(new FloatMatrix(matrix));

          const char *dt = ditherType;

	  if ( strcasecmp( dt, "Err-Diffusion")==0 ) {
	    // Do the error diffusion

	    for (int i = 0; i < height; i++) {
	      for (int j = 0; j < width; j++) {
	        double newValue, value;
		double oldValue = result[i][j];
		// The threshold is at 128. 0 for black, and 255 for white.
		if (oldValue < 128) {
		  newValue = 0;
		}
		else {
		  newValue = 255;
		}
		result[i][j] = newValue;

		// Propagate error to neighbor dot
		double error = oldValue - newValue;
		if ((j+1)<width) {
		  value = result[i][j+1]+error*7/16;
		  result[i][j+1]=bound(value);
		}
		if ((i+1)<height && (j-1)>=0) {
		  value = result[i+1][j-1]+error*3/16;
		  result[i+1][j-1]=bound(value);
		} 
		if ((i+1)<height) {
		  value = result[i+1][j]+error*5/16;
		  result[i+1][j]=bound(value);
		}
		if ((i+1)<height && (j+1)<width) {
		  value = result[i+1][j+1]+error*1/16;
		  result[i+1][j+1]=bound(value);
		}
	      }
            }
	  } else {
	    // Do clustered dither or dispersed dither depending on mask
	    for (int i = 0; i < height; i++) {
	      for (int j = 0; j < width; j++) {
		// compare with threshold in the mask,then determine B/W
		if (255-result[i][j] >= 
                    mask[(i%rowsMask)*colsMask+(j%colsMask)])
		  result[i][j] = 0;
		else 
		  result[i][j] = 255;
	      }
	    } 
	  }
          output%0 << result;
	} // end of valid input matrix.
}


double SDFDither::bound (double value)
{
# line 191 "SDFDither.pl"
// Bound the value of the matrix into 0~255
	  if (value < 0) value = 0;
	  if (value > 255) value = 255;
	  return value;
}


// prototype instance for known block list
static SDFDither proto;
static RegisterBlock registerBlock(proto,"Dither");
