static const char file_id[] = "SDFMotionCmpInv.pl";
// .cc file generated from SDFMotionCmpInv.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFMotionCmpInv.h"

const char *star_nm_SDFMotionCmpInv = "SDFMotionCmpInv";

const char* SDFMotionCmpInv :: className() const {return star_nm_SDFMotionCmpInv;}

ISA_FUNC(SDFMotionCmpInv,SDFStar);

Block* SDFMotionCmpInv :: makeNew() const { LOG_NEW; return new SDFMotionCmpInv;}

SDFMotionCmpInv::SDFMotionCmpInv ()
{
	setDescriptor("For NULL inputs (zero size matrices) on 'mvHorzIn' and/or 'mvVertIn' \n(motion-vector-in), copy the 'diffIn' input unchanged to the output \nand discard the 'pastIn' input. (A NULL input usually indicates the \nfirst frame of a sequence.)\n\nFor non-NULL 'mvHorzIn' and 'mvVertIn' inputs, perform inverse motion\ncompensation and write the result to 'output'.");
	addPort(diffIn.setPort("diffIn",this,FLOAT_MATRIX_ENV));
	addPort(mvHorzIn.setPort("mvHorzIn",this,FLOAT_MATRIX_ENV));
	addPort(mvVertIn.setPort("mvVertIn",this,FLOAT_MATRIX_ENV));
	addPort(pastIn.setPort("pastIn",this,FLOAT_MATRIX_ENV));
	addPort(output.setPort("output",this,FLOAT_MATRIX_ENV));
	addState(BlockSize.setState("BlockSize",this,"8","Size of blocks on which to do inverse motion comp."));


}

void SDFMotionCmpInv::go() {
# line 167 "SDFMotionCmpInv.pl"
// Read data from inputs and initialize
    Envelope difEnvp, pastEnvp;
    (diffIn%0).getMessage(difEnvp);
    (pastIn%0).getMessage(pastEnvp);
    const FloatMatrix& diffImg = *(const FloatMatrix *)difEnvp.myData();
    const FloatMatrix& pastImg = *(const FloatMatrix *)pastEnvp.myData();

    Envelope mvHorzEnvp,mvVertEnvp;
    (mvHorzIn%0).getMessage(mvHorzEnvp);
    (mvVertIn%0).getMessage(mvVertEnvp);
    const FloatMatrix& mvHorzImg = *(const FloatMatrix *)mvHorzEnvp.myData();
    const FloatMatrix& mvVertImg = *(const FloatMatrix *)mvVertEnvp.myData();
     
    // Handle the first input image.
    if ( pastEnvp.empty()  ||
	 mvHorzImg.numRows()*mvHorzImg.numCols() == 0  ||
	 mvVertImg.numRows()*mvVertImg.numCols() == 0  ) {
      output%0 << difEnvp;
      return;
    }

    if(!sizesMatch(diffImg, pastImg, mvHorzImg, mvVertImg)) {
      Error::abortRun(*this, "Size mismatch");
      return;
    }

    // If we reached here, we have a motion-compensated image.
    // Create output image. 
    int height = diffImg.numRows();
    int width  = diffImg.numCols();
    LOG_NEW;
    FloatMatrix& outImage = *(new FloatMatrix(height,width));

    // Do the inverse motion compensation and send output.
    doInvMC(outImage, pastImg, diffImg, mvHorzImg,
	    mvVertImg, width, height, int(BlockSize));

    output%0 << outImage;
}


void SDFMotionCmpInv::doInvMC (FloatMatrix& out, const FloatMatrix& prev, const FloatMatrix& diff, const FloatMatrix& horzImg, const FloatMatrix& vertImg, const int width, const int height, const int blocksize)
{
# line 53 "SDFMotionCmpInv.pl"
int ii, jj;
      int count = 0;
      for(ii = 0; ii < height; ii += blocksize) {
	for(jj = 0; jj < width; jj += blocksize) {
	  char horz = (char)horzImg.entry(count);
	  char vert = (char)vertImg.entry(count);
	  if(BlockIsLost(diff, ii, jj, blocksize, width)) {
	    DoLostBlock(out, prev, horz, vert, 
			ii, jj, blocksize, width);
	  } else {
	    DoOneBlock(out, prev, diff, horz, vert, 
		       ii, jj, blocksize, width);
	  }
	  count++;
	}
      }
}


int SDFMotionCmpInv::BlockIsLost (const FloatMatrix& ptr, const int ii, const int jj, const int blocksize, const int width)
{
# line 78 "SDFMotionCmpInv.pl"
int i, j, indx;
      for(i = 0; i < blocksize; i++) {
	indx = jj + (i+ii)*width;
	for(j = 0; j < blocksize; j++) {
	  if (ptr.entry(indx + j) != 0) {
	    return 0;
	  }
	}
      }
      return 1;
}


void SDFMotionCmpInv::DoOneBlock (FloatMatrix& out, const FloatMatrix& prev, const FloatMatrix& diff, const char horz, const char vert, const int ii, const int jj, const int blocksize, const int width)
{
# line 97 "SDFMotionCmpInv.pl"
int i, j, index, mcindex;
      for(i = 0; i < blocksize; i++) {
	index = jj + (i+ii)*width;
	// NOTE THE MINUS '-' SIGNS BELOW. These are because the
	// motion vector points FROM the old block TO the current.
	mcindex = index - int(horz) - (width * int(vert));
	for(j = 0; j < blocksize; j++) {
	  out.entry(index+j) = 
	        quant(int(diff.entry(index+j)), int(prev.entry(mcindex+j)));
	}
      }
}


void SDFMotionCmpInv::DoLostBlock (FloatMatrix& out, const FloatMatrix& prev, const char horz, const char vert, const int ii, const int jj, const int blocksize, const int width)
{
# line 117 "SDFMotionCmpInv.pl"
int i, j, index, mcindex;
      for(i = 0; i < blocksize; i++) {
	index = jj + (i+ii)*width;
	// NOTE THE MINUS '-' SIGNS BELOW. These are because the
	// motion vector points FROM the old block TO the current.
	mcindex = index - int(horz) - (width * int(vert));
	for(j = 0; j < blocksize; j++) {
	  out.entry(index+j) = prev.entry(mcindex+j);
	}
      }
}


int SDFMotionCmpInv::sizesMatch (const FloatMatrix& im1, const FloatMatrix& im2, const FloatMatrix& mvH, const FloatMatrix& mvV)
{
# line 151 "SDFMotionCmpInv.pl"
int w = im1.numCols();
      int h = im1.numRows();
      int retval = (w == im2.numCols());
      retval &= (h == im2.numRows());

      int bs = int(BlockSize);
      retval &= (w == bs*mvH.numCols());
      retval &= (h == bs*mvH.numRows());
      retval &= (w == bs*mvV.numCols());
      retval &= (h == bs*mvV.numRows());

      return(retval);
}


// prototype instance for known block list
static SDFMotionCmpInv proto;
static RegisterBlock registerBlock(proto,"MotionCmpInv");
