static const char file_id[] = "SDFRGBToYUV.pl";
// .cc file generated from SDFRGBToYUV.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFRGBToYUV.h"
#include "Matrix.h"
#include "Error.h"

const char *star_nm_SDFRGBToYUV = "SDFRGBToYUV";

const char* SDFRGBToYUV :: className() const {return star_nm_SDFRGBToYUV;}

ISA_FUNC(SDFRGBToYUV,SDFStar);

Block* SDFRGBToYUV :: makeNew() const { LOG_NEW; return new SDFRGBToYUV;}

SDFRGBToYUV::SDFRGBToYUV ()
{
	setDescriptor("Read three float matrices that describe a color image in Red-Green-Blue (RGB)\nformat and output three float matrices that describe an image in YUV format.\nNo downsampling is performed on the U and V signals.");
	addPort(input1.setPort("input1",this,FLOAT_MATRIX_ENV));
	addPort(input2.setPort("input2",this,FLOAT_MATRIX_ENV));
	addPort(input3.setPort("input3",this,FLOAT_MATRIX_ENV));
	addPort(output1.setPort("output1",this,FLOAT_MATRIX_ENV));
	addPort(output2.setPort("output2",this,FLOAT_MATRIX_ENV));
	addPort(output3.setPort("output3",this,FLOAT_MATRIX_ENV));
	addState(CCIR_601.setState("CCIR_601",this,"YES","TRUE means that the RGB space will be converted to shifted-and-scaled YUV\ncolor space defined by the CCIR 601 standard, which is used in the MPEG\nand H.261 standards; otherwise, the RGB space will be converted according\nto the usual YUV definition."));


}

void SDFRGBToYUV::go() {
# line 93 "SDFRGBToYUV.pl"
// Read inputs.
                Envelope rpkt,gpkt,bpkt;
                (input1%0).getMessage(rpkt);
                (input2%0).getMessage(gpkt);
                (input3%0).getMessage(bpkt);

		// Change into YUV format
                const FloatMatrix& redI = *(const FloatMatrix*)rpkt.myData();
                const FloatMatrix& greenI = *(const FloatMatrix*)gpkt.myData();
                const FloatMatrix& blueI = *(const FloatMatrix*)bpkt.myData();

		int height = redI.numRows();
		int width  = redI.numCols();

		if ((greenI.numCols() != width) ||
		    (greenI.numRows() != height) ||
		    (blueI.numCols() != width) ||
		    (blueI.numRows() != height)) {
			Error::abortRun(*this,
					"Input image sizes don't match.");
			return;
		}

		LOG_NEW;
		FloatMatrix& yI = *(new FloatMatrix(redI));
		LOG_NEW;
		FloatMatrix& uI = *(new FloatMatrix(greenI));
		LOG_NEW;
		FloatMatrix& vI = *(new FloatMatrix(blueI));

		int i, j, temp1, temp2;
		double rvalue, gvalue, bvalue;
		double yvalue, uvalue, vvalue;
		for ( i = 0; i < height; i++ ) {
			temp1 = i*width;
			for ( j = 0; j < width; j++ ) {
				temp2 = j + temp1;
				rvalue = yI.entry(temp2);
				gvalue = uI.entry(temp2);
				bvalue = vI.entry(temp2);
				yvalue =  0.299  * rvalue +
					  0.587  * gvalue +
					  0.114  * bvalue;
				uvalue = -0.1687 * rvalue +
					 -0.3313 * gvalue +
					  0.5    * bvalue;
				vvalue =  0.5    * rvalue +
					 -0.4187 * gvalue +
					 -0.0813 * bvalue;
				if ( int(CCIR_601) ) {
				  yvalue = (219.0*yvalue)/255.0 +  16;
				  uvalue = (224.0*uvalue)/255.0 + 128;
				  vvalue = (224.0*vvalue)/255.0 + 128;
				}
				else {
				  uvalue += 128;
				  vvalue += 128;
				}
				yI.entry(temp2) = quant(yvalue);
				uI.entry(temp2) = quant(uvalue);
				vI.entry(temp2) = quant(vvalue);
			}
		}

		// Write whole frame to output here...
		output1%0 << yI;
		output2%0 << uI;
		output3%0 << vI;
}

// prototype instance for known block list
static SDFRGBToYUV proto;
static RegisterBlock registerBlock(proto,"RGBToYUV");
