static const char file_id[] = "SDFRankImage.pl";
// .cc file generated from SDFRankImage.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFRankImage.h"

const char *star_nm_SDFRankImage = "SDFRankImage";

const char* SDFRankImage :: className() const {return star_nm_SDFRankImage;}

ISA_FUNC(SDFRankImage,SDFStar);

Block* SDFRankImage :: makeNew() const { LOG_NEW; return new SDFRankImage;}

# line 53 "SDFRankImage.pl"
// Function to sort float's.
	    // These types and casts are required to satisfy cfront's
	    // ridiculously strict rules.
	    extern "C" {
		static int sortUC(const void* aV, const void* bV) {
		    float* a = (float*) aV;
		    float* b = (float*) bV;
		    if (*a < *b) return -1;
		    else if (*a == *b) return 0;
		    else return 1;
		}
	    }

SDFRankImage::SDFRankImage ()
{
	setDescriptor("Accept an input grayimage represented by a float matrix, rank \nfilter the image, and send the result to the output.  A common \nexample of a rank filter is the median filter, e.g. SDFMedianImage, \nwhich is derived from this star. Pixels at the image boundaries \nare copied and not rank filtered.");
	addPort(inData.setPort("inData",this,FLOAT_MATRIX_ENV));
	addPort(outData.setPort("outData",this,FLOAT_MATRIX_ENV));
	addState(FilterWidth.setState("FilterWidth",this,"3","Size of the rank filter window in each dimension,\nwhich must be odd."));
	addState(RankOrder.setState("RankOrder",this,"0","Which ordered sample is chosen from each neighborhood of pixels,\n0, 1, ..., FilterWidth^2 - 1."));

# line 73 "SDFRankImage.pl"
buf = 0;
}

SDFRankImage::~SDFRankImage() {
# line 106 "SDFRankImage.pl"
delete [] buf;
}

void SDFRankImage::setup() {
# line 77 "SDFRankImage.pl"
size = int(FilterWidth);
		if (size <= 0) {
		    Error::error(*this, "FilterWidth must be positive.");
		    return;
		}
		int oddsize = 1 + 2*(size/2);
		if ( size != oddsize ) {
		    Error::error(*this, "FilterWidth must be odd.");
		    return;
		}
		halfsize = (size - 1)/2;
		neighborhoodSize = size * size;

		rankIndex = int(RankOrder);
		if (rankIndex < 0) {
		    Error::error(*this, "RankOrder must be non-negative.");
		    return;
		}
		if (rankIndex >= neighborhoodSize) {
		    Error::error(*this, "RankOrder is too large.");
		    return;
		}

		delete [] buf;
		buf = (float *) new float[neighborhoodSize];

		width = height = 0;
}

void SDFRankImage::go() {
# line 133 "SDFRankImage.pl"
// Read data from input.
                Envelope pkt;
                (inData%0).getMessage(pkt);
                const FloatMatrix& inImage = *(const FloatMatrix*)pkt.myData();

		width = inImage.numCols();
		height = inImage.numRows();

		// Median filtering -- don't filter boundary pixels for now
		LOG_NEW;
		FloatMatrix& outImage = *(new FloatMatrix(height,width));

		for (int i = 0; i < height; i++) {
		    if ( (halfsize <= i) && (i < height - halfsize) ) {
		        int j = 0;
			for ( ; j < halfsize; j++) {	// copy boundary pixels
			    outImage[i][j] = inImage[i][j];
			}
			for ( ; j < width - halfsize; j++) {	// med. filter
			    outImage[i][j] = retRankedElement(inImage,i,j);
			}
			for ( ; j < width; j++) {	// copy boundary pixels
			    outImage[i][j] = inImage[i][j];
			}
		    }
		    else {
		        int j = 0;
			for ( ; j < width; j++) {	// copy boundary pixels
			    outImage[i][j] = inImage[i][j];
			}
		    }
		}

		// Send the output on its way.
		outData%0 << outImage;
}


float SDFRankImage::retRankedElement (const FloatMatrix& image, int i, int j)
{
# line 114 "SDFRankImage.pl"
// Select size x size neighbor around pixel at [i][j]
		    // and write 2-D data to 1-D buffer buf by copying
		    // pixel values at [i+k][j+l] where
		    //   k = -halfsize ... 0 ... halfsize
		    //   l = -halfsize ... 0 ... halfsize
		    float *bufptr = buf;
		    for (int k = -halfsize; k <= halfsize; k++)
			for (int l = -halfsize; l <= halfsize; l++)
			    *bufptr++ = image[i+k][j+l];

		    // Sort data
		    qsort(buf, neighborhoodSize, sizeof(float), sortUC);

		    // Return ranked data elemented 
		    return buf[ rankIndex ];
}


// prototype instance for known block list
static SDFRankImage proto;
static RegisterBlock registerBlock(proto,"RankImage");
