static const char file_id[] = "SDFReadImage.pl";
// .cc file generated from SDFReadImage.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFReadImage.h"
#include "Matrix.h"
#include <std.h>
#include <stdio.h>
#include "miscFuncs.h"
#include "StringList.h"
#include "Error.h"

const char *star_nm_SDFReadImage = "SDFReadImage";

const char* SDFReadImage :: className() const {return star_nm_SDFReadImage;}

ISA_FUNC(SDFReadImage,SDFStar);

Block* SDFReadImage :: makeNew() const { LOG_NEW; return new SDFReadImage;}

SDFReadImage::SDFReadImage ()
{
	setDescriptor("Read a sequence of images in Portable GrayMap (PGM) format from\ndifferent files and send them out in a float matrix.\n\nIf present, the character '#' in the 'fileName' state is replaced with\nthe frame number to be read next.\nFor example, if the 'frameId' state is set to 2 and\nif the 'fileName' state is 'dir.#/pic#', then the files that\nare read are 'dir.2/pic2', 'dir.3/pic3', etc.");
	addPort(dataOutput.setPort("dataOutput",this,FLOAT_MATRIX_ENV));
	addPort(frameIdOutput.setPort("frameIdOutput",this,INT));
	addState(fileName.setState("fileName",this,"$PTOLEMY/src/domains/sdf/demo/ptimage","Name of file containing PGM-format image."));
	addState(frameId.setState("frameId",this,"0","Starting frame ID value."));


}

void SDFReadImage::go() {
# line 63 "SDFReadImage.pl"
// Open file containing the image.
		char *expandedName = genFileName(fileName, int(frameId));
		StringList fullName = expandedName;
		delete [] expandedName;
		FILE *fp = fopen(fullName, "r");
		if (fp == (FILE*) NULL) {
			Error::abortRun(*this, "File not opened: ", fullName);
			return;
		}

		// Read header, skipping 1 whitespace character at end.
		char word[80];
		int width, height, maxval;
		fscanf(fp, "%s", word);
		if (strcmp(word, "P5")) { // "P5" is PGM magic number.
			Error::abortRun(*this, fullName, ": not in PGM format");
			return;
		}
		fscanf(fp, "%s", word);
		while (word[0] == '#') {
			fscanf(fp, "%*[^\n]%s", word);
		}
		sscanf(word, "%d", &width);
		fscanf(fp, "%s", word);
		while (word[0] == '#') {
			fscanf(fp, "%*[^\n]%s", word);
		}
		sscanf(word, "%d", &height);
		fscanf(fp, "%s", word);
		while (word[0] == '#') {
			fscanf(fp, "%*[^\n]%s", word);
		}
		sscanf(word, "%d", &maxval);
		if (maxval > 255) {
			Error::abortRun(*this, fullName,
					": not in 8-bit format.");
			return;
		}
		fscanf(fp, "%*c");		// skip one whitespace char.

		// Create an array to read into image data.
		unsigned char* data = (unsigned char*) new char[height*width];
		fread( (unsigned char*)data,
		       width * sizeof(unsigned char),
		       height,
		       fp );
		fclose(fp);

		// Create image matrix and fill it with data.
		LOG_NEW;
		FloatMatrix& result = *(new FloatMatrix(height,width));

		for ( int i=0; i<height; i++) 
		  for ( int j=0; j<width; j++)
                    result[i][j] = float(data[i*width+j]);

		delete [] data;      

		// Write the new frame to output...
		dataOutput%0 << result;
		frameIdOutput%0 << int(frameId);

		frameId = int(frameId) + 1;		// increment frame id
}


char* SDFReadImage::genFileName (const char* str, const int d)
{
# line 53 "SDFReadImage.pl"
char num[16];
			sprintf(num, "%d", d);
			char* filename = expandPathName(str);
			char* newfilename = subCharByString(filename, '#', num);
			delete [] filename;
			return newfilename;
}


// prototype instance for known block list
static SDFReadImage proto;
static RegisterBlock registerBlock(proto,"ReadImage");
