static const char file_id[] = "SDFReadRGB.pl";
// .cc file generated from SDFReadRGB.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFReadRGB.h"
#include "Matrix.h"
#include <std.h>
#include <stdio.h>
#include "Error.h"
#include "StringList.h"
#include "miscFuncs.h"

const char *star_nm_SDFReadRGB = "SDFReadRGB";

const char* SDFReadRGB :: className() const {return star_nm_SDFReadRGB;}

ISA_FUNC(SDFReadRGB,SDFStar);

Block* SDFReadRGB :: makeNew() const { LOG_NEW; return new SDFReadRGB;}

SDFReadRGB::SDFReadRGB ()
{
	setDescriptor("Read an image in Portable Pixmap (PPM) format from a file and\nseparate the colors into three different images\(em\na red, green, and blue image.\nEach image is represented in a float matrix.\n\nIf present, the character '#' in the 'fileName' state is replaced with\nthe frame number to be read next.\nFor example, if the 'frameId' state is set to 2 and\nif the 'fileName' state is 'dir.#/pic#', then the files that\nare read are 'dir.2/pic2', 'dir.3/pic3', etc.");
	addPort(output1.setPort("output1",this,FLOAT_MATRIX_ENV));
	addPort(output2.setPort("output2",this,FLOAT_MATRIX_ENV));
	addPort(output3.setPort("output3",this,FLOAT_MATRIX_ENV));
	addPort(frameIdOut.setPort("frameIdOut",this,INT));
	addState(fileName.setState("fileName",this,"$PTOLEMY/src/domains/sdf/demo/ppimage","Name of file containing PPM-format image."));
	addState(frameId.setState("frameId",this,"0","Starting frame ID value."));


}

void SDFReadRGB::go() {
# line 68 "SDFReadRGB.pl"
// Open file containing the image.
		char* expandedName = genFileName(fileName, int(frameId));
		StringList fullName = expandedName;
		delete [] expandedName;
		FILE* fp = fopen(fullName, "r");
		if (fp == (FILE*) NULL) {
			Error::abortRun(*this, "File not opened: ", fullName);
			return;
		}

		// Read header, skipping 1 whitespace character at end.
		char word[80];
		int width, height, maxval;
		fscanf(fp, "%s", word);
		if (strcmp(word, "P6")) { // "P6" is PPM magic number.
			Error::abortRun(*this, fullName, ": not in PPM format");
			return;
		}
		fscanf(fp, "%s", word);
		while (word[0] == '#') {
			fscanf(fp, "%*[^\n]%s", word);
		}
		sscanf(word, "%d", &width);
		fscanf(fp, "%s", word);
		while (word[0] == '#') {
			fscanf(fp, "%*[^\n]%s", word);
		}
		sscanf(word, "%d", &height);
		fscanf(fp, "%s", word);
		while (word[0] == '#') {
			fscanf(fp, "%*[^\n]%s", word);
		}
		sscanf(word, "%d", &maxval);
		if (maxval > 255) {
			Error::abortRun(*this, fullName, ": not in 8-bit format");
			return;
		}
		fscanf(fp, "%*c"); // skip one whitespace char.

// Create new images and fill them with data.
		LOG_NEW;
		unsigned char* rgbfp = new unsigned char[3*width*height];
		fread((char*)rgbfp, sizeof(unsigned char), 3*width*height, fp);
		fclose(fp);

		LOG_NEW;
		FloatMatrix& rColor = *(new FloatMatrix(height,width));
		LOG_NEW;
		FloatMatrix& gColor = *(new FloatMatrix(height,width));
		LOG_NEW;
		FloatMatrix& bColor = *(new FloatMatrix(height,width));

		int i, j, temp1, temp2, temp3;
		for (i = 0; i < height; i++) {
			temp1 = i*width;
			for (j = 0; j < width; j++){
				temp2 = j + temp1;
				temp3 = 3*temp2;
				rColor.entry(temp2) = rgbfp[temp3];
				gColor.entry(temp2) = rgbfp[temp3+1];
				bColor.entry(temp2) = rgbfp[temp3+2];
		}	}
		LOG_DEL; delete [] rgbfp;

// Write whole frame to output here...
		output1%0 << rColor;
		output2%0 << gColor;
		output3%0 << bColor;
		frameIdOut%0 << int(frameId);

		frameId = int(frameId) + 1; //increment frame id
}


char* SDFReadRGB::genFileName (const char* str, const int d)
{
# line 58 "SDFReadRGB.pl"
char num[16];
			sprintf(num, "%d", d);
			char* filename = expandPathName(str);
			char* newfilename = subCharByString(filename, '#', num);
			delete [] filename;
			return newfilename;
}


// prototype instance for known block list
static SDFReadRGB proto;
static RegisterBlock registerBlock(proto,"ReadRGB");
