static const char file_id[] = "SDFRunLenImg.pl";
// .cc file generated from SDFRunLenImg.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFRunLenImg.h"

const char *star_nm_SDFRunLenImg = "SDFRunLenImg";

const char* SDFRunLenImg :: className() const {return star_nm_SDFRunLenImg;}

ISA_FUNC(SDFRunLenImg,SDFStar);

Block* SDFRunLenImg :: makeNew() const { LOG_NEW; return new SDFRunLenImg;}

SDFRunLenImg::SDFRunLenImg ()
{
	setDescriptor("Accept a Float Matrix and run-length encode it. All values closer than\n\"Thresh\" to \"meanVal\" are set to \"meanVal\" to help improve compression.\n\nRun lengths are coded with a start symbol of \"meanVal\" and then a\nrun-length between 1 and 255. Runs longer than 255 must be coded in\nseparate pieces.");
	addPort(input.setPort("input",this,FLOAT_MATRIX_ENV));
	addPort(output.setPort("output",this,FLOAT_MATRIX_ENV));
	addPort(originalW.setPort("originalW",this,INT));
	addPort(originalH.setPort("originalH",this,INT));
	addPort(compression.setPort("compression",this,FLOAT));
	addState(Thresh.setState("Thresh",this,"0","Threshold for run-length coding."));
	addState(meanVal.setState("meanVal",this,"0","Center value for thresholding."));


}

void SDFRunLenImg::setup() {
# line 60 "SDFRunLenImg.pl"
thresh = int(Thresh);
}

void SDFRunLenImg::go() {
# line 115 "SDFRunLenImg.pl"
// Read input.
    Envelope inEnvp;
    (input%0).getMessage(inEnvp);
    const FloatMatrix& inImage = *(const FloatMatrix *) inEnvp.myData();

    if (inEnvp.empty()) {
      Error::abortRun(*this, "Input is a dummyMessage.");
      return;
    }

    // Do processing and send out.
    FloatMatrix& outData = doRunLen(inImage);
    const float comprRatio = float(outData.numRows() * outData.numCols()) /
      float(inImage.numRows() * inImage.numCols());
    compression%0 << comprRatio;
    
    output%0 << outData;
    originalW%0 << inImage.numCols();
    originalH%0 << inImage.numRows();
}


FloatMatrix& SDFRunLenImg::doRunLen (const FloatMatrix& inImage)
{
# line 78 "SDFRunLenImg.pl"
// Do the run-length coding.
      const int size = inImage.numRows() * inImage.numCols();

      // The biggest blowup we can have is the string "01010101...".
      // This gives a blowup of 33%, so 1.34 is ok.
      LOG_NEW;
      unsigned char* ptr2 = new unsigned char[int(1.34*size + 1)];
      
      int indx1 = 0, indx2 = 0;
      while (indx1 < size) {
	while ( indx1 < size   && 
	        gt((unsigned char)inImage.entry(indx1),thresh) ) {
	  ptr2[indx2++] = (unsigned char)inImage.entry(indx1++);
	}
	if (indx1 < size) {
	  int thisRun = 1; indx1++;
	  while ( (indx1 < size) && (thisRun < 255) &&
		  (!gt((unsigned char)inImage.entry(indx1),thresh)) ) {
	    indx1++; thisRun++;
	  }
	  ptr2[indx2++] = (unsigned char) int(meanVal);
	  ptr2[indx2++] = (unsigned char) thisRun;
	}
      }

      // Copy the data to return.
      LOG_NEW; FloatMatrix& outData = *(new FloatMatrix(1,indx2));
      for(indx1 = 0; indx1 < indx2; indx1++) {
	outData.entry(indx1) = ptr2[indx1];
      }
      LOG_DEL; delete [] ptr2;

      return outData;
}


// prototype instance for known block list
static SDFRunLenImg proto;
static RegisterBlock registerBlock(proto,"RunLenImg");
