static const char file_id[] = "SDFYUVToRGB.pl";
// .cc file generated from SDFYUVToRGB.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFYUVToRGB.h"
#include "Matrix.h"
#include "Error.h"

const char *star_nm_SDFYUVToRGB = "SDFYUVToRGB";

const char* SDFYUVToRGB :: className() const {return star_nm_SDFYUVToRGB;}

ISA_FUNC(SDFYUVToRGB,SDFStar);

Block* SDFYUVToRGB :: makeNew() const { LOG_NEW; return new SDFYUVToRGB;}

SDFYUVToRGB::SDFYUVToRGB ()
{
	setDescriptor("Read three float matrices that describe a color image in YUV format and\noutput three float matrices that describe an image in RGB format.");
	addPort(input1.setPort("input1",this,FLOAT_MATRIX_ENV));
	addPort(input2.setPort("input2",this,FLOAT_MATRIX_ENV));
	addPort(input3.setPort("input3",this,FLOAT_MATRIX_ENV));
	addPort(output1.setPort("output1",this,FLOAT_MATRIX_ENV));
	addPort(output2.setPort("output2",this,FLOAT_MATRIX_ENV));
	addPort(output3.setPort("output3",this,FLOAT_MATRIX_ENV));
	addState(CCIR_601.setState("CCIR_601",this,"YES","TRUE means that the YUV color space is the shifted-and-scaled YUV space\ndefined by the CCIR 601 standard, which is used in the MPEG and H.261\nstandards; otherwise, the usual YUV space is meant."));


}

void SDFYUVToRGB::go() {
# line 89 "SDFYUVToRGB.pl"
// Read inputs.
                Envelope ypkt,upkt,vpkt;
                (input1%0).getMessage(ypkt);
                (input2%0).getMessage(upkt);
                (input3%0).getMessage(vpkt);

		// Change into RGB format
                const FloatMatrix& yI = *(const FloatMatrix*)ypkt.myData();
                const FloatMatrix& uI = *(const FloatMatrix*)upkt.myData();
                const FloatMatrix& vI = *(const FloatMatrix*)vpkt.myData();

		int height = yI.numRows();
		int width  = yI.numCols();

		if ((uI.numCols() != width) ||
		    (uI.numRows() != height) ||
		    (vI.numCols() != width) ||
		    (vI.numRows() != height)) {
			Error::abortRun(*this,
					"Input image sizes don't match.");
			return;
		}

		LOG_NEW;
		FloatMatrix& rI = *(new FloatMatrix(yI));
		LOG_NEW;
		FloatMatrix& gI = *(new FloatMatrix(uI));
		LOG_NEW;
		FloatMatrix& bI = *(new FloatMatrix(vI));

		int i, j, temp1, temp2;
		double rvalue, gvalue, bvalue;
		double yvalue, uvalue, vvalue;
		for (i = 0; i < height; i++) {
			temp1 = i * width;
			for (j = 0; j < width; j++){
				temp2 = j + temp1;
				yvalue = rI.entry(temp2);
				uvalue = gI.entry(temp2);
				vvalue = bI.entry(temp2);
				if ( int(CCIR_601) ) {
				  yvalue = 255.0*(yvalue -  16)/219.0;
				  uvalue = 255.0*(uvalue - 128)/224.0;
				  vvalue = 255.0*(vvalue - 128)/224.0;
				}
				else {
				  uvalue -= 128;
				  vvalue -= 128;
				}
				rvalue = yvalue + 1.4020*vvalue;
				gvalue = yvalue - 0.3441*uvalue - 0.7141*vvalue;
				bvalue = yvalue + 1.7720*uvalue;
				rI.entry(temp2) = quant(rvalue);
				gI.entry(temp2) = quant(gvalue);
				bI.entry(temp2) = quant(bvalue);
			}
		}

		// Write whole frame to output here...
		output1%0 << rI;
		output2%0 << gI;
		output3%0 << bI;
}

// prototype instance for known block list
static SDFYUVToRGB proto;
static RegisterBlock registerBlock(proto,"YUVToRGB");
