static const char file_id[] = "SDFAddFix_M.pl";
// .cc file generated from SDFAddFix_M.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFAddFix_M.h"
#include "Matrix.h"

const char *star_nm_SDFAddFix_M = "SDFAddFix_M";

const char* SDFAddFix_M :: className() const {return star_nm_SDFAddFix_M;}

ISA_FUNC(SDFAddFix_M,SDFStar);

Block* SDFAddFix_M :: makeNew() const { LOG_NEW; return new SDFAddFix_M;}

SDFAddFix_M::SDFAddFix_M ()
{
	setDescriptor("Add two fixed-point matrices.");
	addPort(Ainput.setPort("Ainput",this,FIX_MATRIX_ENV));
	addPort(Binput.setPort("Binput",this,FIX_MATRIX_ENV));
	addPort(output.setPort("output",this,FIX_MATRIX_ENV));
	addState(KeepArrivingPrecision.setState("KeepArrivingPrecision",this,"YES","Flag that indicates whether or not to keep the precision of the\narriving matrix: YES keeps the same precision, and NO casts\nthem to the precision specified by the parameter \"InputPrecision\"."));
	addState(InputPrecision.setState("InputPrecision",this,"4.14","Precision of the input in bits.  The input matrix is only cast to\nthis precision if the parameter \"ArrivingPrecision\" is set to NO."));
	addState(OutputPrecision.setState("OutputPrecision",this,"4.14","Precision of the output matrix in bits."));
	addState(OverflowHandler.setState("OverflowHandler",this,"saturate","The type of overflow that happens if the result of any addition\noverflows.  The options are: \"saturate\"(default), \"zero-saturate\",\n\"wrapped\", \"warning\"."));
	addState(numRows.setState("numRows",this,"2","The number of rows in the input matrices."));
	addState(numCols.setState("numCols",this,"2","The number of columns in the input matrices."));


}

void SDFAddFix_M::setup() {
# line 81 "SDFAddFix_M.pl"
IP = InputPrecision;
          OP = OutputPrecision;
          OV = OverflowHandler;
          in_IntBits = Fix::get_intBits(IP);
          in_len = Fix::get_length(IP);
          out_IntBits = Fix::get_intBits(OP);
          out_len = Fix::get_length(OP);
}

void SDFAddFix_M::go() {
# line 90 "SDFAddFix_M.pl"
// get inputs
          Envelope Apkt;
          (Ainput%0).getMessage(Apkt);
          const FixMatrix& Amatrix = *(const FixMatrix *)Apkt.myData();

          Envelope Bpkt;
          (Binput%0).getMessage(Bpkt);
          const FixMatrix& Bmatrix = *(const FixMatrix *)Bpkt.myData();

          // check for "null" matrix inputs, caused by delays
          if(Apkt.empty() && Bpkt.empty()) {
            // both empty, return a zero matrix with the given dimensions
            FixMatrix& result = *(new FixMatrix(int(numRows),int(numCols),
                                                out_len,out_IntBits));
            result = 0.0;
            output%0 << result;
          }
          else if(Apkt.empty()) {
            // Amatrix is empty but Bmatrix is not, return just Bmatrix
            // with a new output precision
            FixMatrix& result = *(new FixMatrix(int(numRows),int(numCols),
                                                out_len,out_IntBits));
            result = Bmatrix;
            output%0 << result;
          }
          else if(Bpkt.empty()) {
            // Bmatrix is empty but Amatrix is not, return just Amatrix
            // with a new output precision
            FixMatrix& result = *(new FixMatrix(int(numRows),int(numCols),
                                                out_len,out_IntBits));
            result = Amatrix;
            output%0 << result;
          }
          else {
            // Amatrix and Bmatrix both valid

            // just check that A's dimensions match the state info.
            // the operator + on matrices will check that A matches B
            if((Amatrix.numRows() != int(numRows)) ||
               (Amatrix.numCols() != int(numCols))) {
              Error::abortRun(*this,"Dimension size of FixMatrix inputs do ",
                                    "not match the given state parameters.");
              return;
            }

            // do matrix addition, much more complicated than the normal
            // case because of all the precision and overflow handling
            FixMatrix& result = *(new FixMatrix(int(numRows),int(numCols),
                                                out_len,out_IntBits));
            for(int i = 0; i < int(numRows) * int(numCols); i++) {
              (result.entry(i)).set_ovflow(OV); // set overflow function
              if(int(KeepArrivingPrecision))
                result.entry(i) = Amatrix.entry(i) + Bmatrix.entry(i);
              else {
                Fix newA = Fix(in_len,in_IntBits,Amatrix.entry(i));
                Fix newB = Fix(in_len,in_IntBits,Bmatrix.entry(i));
                result.entry(i) = newA + newB;
              }
            }
            output%0 << result;
          }
}

// prototype instance for known block list
static SDFAddFix_M proto;
static RegisterBlock registerBlock(proto,"AddFix_M");
