static const char file_id[] = "SDFAvgSqrErr.pl";
// .cc file generated from SDFAvgSqrErr.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFAvgSqrErr.h"
#include "Matrix.h"

const char *star_nm_SDFAvgSqrErr = "SDFAvgSqrErr";

const char* SDFAvgSqrErr :: className() const {return star_nm_SDFAvgSqrErr;}

ISA_FUNC(SDFAvgSqrErr,SDFStar);

Block* SDFAvgSqrErr :: makeNew() const { LOG_NEW; return new SDFAvgSqrErr;}

SDFAvgSqrErr::SDFAvgSqrErr ()
{
	setDescriptor("Find the average squared error between two input sequences of matrices.");
	addPort(input1.setPort("input1",this,FLOAT_MATRIX_ENV));
	addPort(input2.setPort("input2",this,FLOAT_MATRIX_ENV));
	addPort(output.setPort("output",this,FLOAT));
	addState(numRows.setState("numRows",this,"1","The number of rows for input matrix."));
	addState(numCols.setState("numCols",this,"2","The number of columns for input matrix."));
	addState(numInputs.setState("numInputs",this,"8","The number of input matrices to average."));


}

void SDFAvgSqrErr::setup() {
# line 48 "SDFAvgSqrErr.pl"
input1.setSDFParams(int(numInputs),int(numInputs)-1);
    input2.setSDFParams(int(numInputs),int(numInputs)-1);
}

void SDFAvgSqrErr::go() {
# line 52 "SDFAvgSqrErr.pl"
Envelope inpkt1;
    Envelope inpkt2;
    FloatMatrix& tempmatrix = *(new FloatMatrix(int(numRows), int(numCols)));
    double sqrErr = 0.0;
    int maxi = int(numRows) * int(numCols);
    for (int i = 0; i < int(numInputs); i++) {
      // get input
      (input1%(int(numInputs)-1-i)).getMessage(inpkt1);
      (input2%(int(numInputs)-1-i)).getMessage(inpkt2);
 
      // check for "null" matrix inputs, caused by delays
      // if input1 is empty, treat it as a zero matrix
      const FloatMatrix& matrix1 = *(const FloatMatrix *)inpkt1.myData();
      int empty1 = inpkt1.empty();
      if ( ! empty1 ) {
        if ((matrix1.numRows() != int(numRows)) ||
            (matrix1.numCols() != int(numCols))) {
	  delete &tempmatrix;
          Error::abortRun(*this,
		  "Input1 matrix doesn't match the specified dimension");
          return;
        }
      }

      // if input2 is empty, treat it as a zero matrix
      const FloatMatrix& matrix2 = *(const FloatMatrix *)inpkt2.myData();
      int empty2 = inpkt2.empty();
      if ( ! empty2 ) {
        if ((matrix2.numRows() != int(numRows)) ||
            (matrix2.numCols() != int(numCols))) {
	  delete &tempmatrix;
          Error::abortRun(*this,
			"Input2 matrix doesn't match the specified dimension");
          return;
        }
      }

      // If at least one matrix is not empty, then adjust squared error
      if ( !empty1 || !empty2 ) {
        if ( empty1 && !empty2 ) {
          for (int i = 0; i < maxi; i++) {
	    sqrErr += matrix2.entry(i) * matrix2.entry(i);
	  }
        }
        else if ( !empty1 && empty2 ) {
          for (int i=0; i < maxi; i++) {
	    sqrErr += matrix1.entry(i) * matrix1.entry(i);
	  }
        }
        else {
          tempmatrix = matrix1;
	  tempmatrix -= matrix2;
          for (int i = 0; i < maxi; i++) {
	    sqrErr += tempmatrix.entry(i) * tempmatrix.entry(i);
	  }
        }
      }
    }
    delete &tempmatrix;
    sqrErr /= int(numInputs);
    output%0 << sqrErr;
}

// prototype instance for known block list
static SDFAvgSqrErr proto;
static RegisterBlock registerBlock(proto,"AvgSqrErr");
