static const char file_id[] = "SDFIntToFix_M.pl";
// .cc file generated from SDFIntToFix_M.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFIntToFix_M.h"
#include "Matrix.h"

const char *star_nm_SDFIntToFix_M = "SDFIntToFix_M";

const char* SDFIntToFix_M :: className() const {return star_nm_SDFIntToFix_M;}

ISA_FUNC(SDFIntToFix_M,SDFStar);

Block* SDFIntToFix_M :: makeNew() const { LOG_NEW; return new SDFIntToFix_M;}

SDFIntToFix_M::SDFIntToFix_M ()
{
	setDescriptor("Take an input IntMatrix and convert it to an FixMatrix.  This\nis done by constructing a new FixMatrix with the given precision.\nEach entry of the IntMatrix is cast using the given masking function\nand then copied to the new matrix.");
	addPort(input.setPort("input",this,INT_MATRIX_ENV));
	addPort(output.setPort("output",this,FIX_MATRIX_ENV));
	addState(FixPrecision.setState("FixPrecision",this,"16.0","Precision of the entries in the FixMatrix.  The integer values\nof the IntMatrix are converted to this precision."));
	addState(masking.setState("masking",this,"truncate","Masking method.  This parameter is used to specify the way the integer\nnumber is masked for casting to the fixed-point notation.  The keywords are:\n\"truncate\" (default), \"round\"."));


}

void SDFIntToFix_M::setup() {
# line 53 "SDFIntToFix_M.pl"
Precision = FixPrecision;
    Masking = masking;
    intBits = Fix::get_intBits(FixPrecision);
    length = Fix::get_length(FixPrecision);
}

void SDFIntToFix_M::go() {
# line 59 "SDFIntToFix_M.pl"
// get input
    Envelope inpkt;
    (input%0).getMessage(inpkt);
    const IntMatrix& matrix = *(const IntMatrix *)inpkt.myData();

    // check for "null" matrix inputs, caused by delays
    if(inpkt.empty()) {
      // input empty, just send it back out
      output%0 << inpkt;
    }
    else {
      // valid input matrix

      // do conversion using copy constructor
      if(strcmp(Masking, "truncate") == 0) {
        FixMatrix& result = *(new FixMatrix(matrix,intBits,length,Fix::mask_truncate));
        output%0 << result;
      }
      else if(strcmp(Masking, "round") == 0) {
        FixMatrix& result = *(new FixMatrix(matrix,intBits,length,Fix::mask_truncate_round));
        output%0 << result;
      }
      else
        Error::abortRun(*this, ": not a valid function for masking");
    }
}

// prototype instance for known block list
static SDFIntToFix_M proto;
static RegisterBlock registerBlock(proto,"IntToFix_M");
