static const char file_id[] = "SDFMpyFix_M.pl";
// .cc file generated from SDFMpyFix_M.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFMpyFix_M.h"
#include "Matrix.h"

const char *star_nm_SDFMpyFix_M = "SDFMpyFix_M";

const char* SDFMpyFix_M :: className() const {return star_nm_SDFMpyFix_M;}

ISA_FUNC(SDFMpyFix_M,SDFStar);

Block* SDFMpyFix_M :: makeNew() const { LOG_NEW; return new SDFMpyFix_M;}

SDFMpyFix_M::SDFMpyFix_M ()
{
	setDescriptor("Multiply two fixed-point matrices A and B to produce matrix C.\nMatrix A has dimensions (numRows,X). Matrix B has dimensions (X,numCols).\nMatrix C has dimensions (numRows,numCols). The user need only specify\nnumRows and numCols. An error will be generated automatically if the\nnumber of columns in A does not match the number of rows in B.");
	addPort(Ainput.setPort("Ainput",this,FIX_MATRIX_ENV));
	addPort(Binput.setPort("Binput",this,FIX_MATRIX_ENV));
	addPort(output.setPort("output",this,FIX_MATRIX_ENV));
	addState(KeepArrivingPrecision.setState("KeepArrivingPrecision",this,"YES","Flag that indicates whether or not to keep the precision of the\narriving matrix: YES keeps the same precision, and NO casts\nthem to the precision specified by the parameter \"InputPrecision\"."));
	addState(InputPrecision.setState("InputPrecision",this,"4.14","Precision of the input in bits.  The input matrix is only cast to\nthis precision if the parameter \"ArrivingPrecision\" is set to NO."));
	addState(OutputPrecision.setState("OutputPrecision",this,"4.14","Precision of the output matrix in bits."));
	addState(OverflowHandler.setState("OverflowHandler",this,"saturate","The type of overflow that happens if the result of any addition\noverflows.  The options are: \"saturate\"(default), \"zero-saturate\",\n\"wrapped\", \"warning\"."));
	addState(numRows.setState("numRows",this,"2","The number of rows in Matrix A and Matrix C."));
	addState(numCols.setState("numCols",this,"2","The number of columns in Matrix B and Matrix C"));


}

void SDFMpyFix_M::setup() {
# line 87 "SDFMpyFix_M.pl"
IP = InputPrecision;
          OP = OutputPrecision;
          OV = OverflowHandler;
          in_IntBits = Fix::get_intBits(IP);
          in_len = Fix::get_length(IP);
          out_IntBits = Fix::get_intBits(OP);
          out_len = Fix::get_length(OP);
}

void SDFMpyFix_M::go() {
# line 96 "SDFMpyFix_M.pl"
// get inputs
          Envelope Apkt;
          (Ainput%0).getMessage(Apkt);
          const FixMatrix& Amatrix = *(const FixMatrix *)Apkt.myData();

          Envelope Bpkt;
          (Binput%0).getMessage(Bpkt);
          const FixMatrix& Bmatrix = *(const FixMatrix *)Bpkt.myData();

          // check for "null" matrix inputs, caused by delays
          if(Apkt.empty() || Bpkt.empty()) {
            // if either empty, return a zero matrix with the given dimensions
            FixMatrix& result = *(new FixMatrix(int(numRows),int(numCols),
                                                out_len,out_IntBits));
            result = 0.0;
            output%0 << result;
          }
          else {
            // Amatrix and Bmatrix both valid

            if((Amatrix.numRows() != int(numRows)) ||
               (Bmatrix.numCols() != int(numCols))) {
              Error::abortRun(*this,"Dimension size of FixMatrix inputs do ",
                                    "not match the given state parameters.");
              return;
            }

            // do matrix multiplication, much more complicated than the normal
            // case because of all the precision and overflow handling
            FixMatrix& result = *(new FixMatrix(int(numRows),int(numCols),
                                                out_len,out_IntBits));

            for(int i = 0; i < int(numRows) * int(numCols); i++)
              (result.entry(i)).set_ovflow(OV); // set overflow function

            if(int(KeepArrivingPrecision)) 
              multiply(Amatrix,Bmatrix,result);
            else {
              FixMatrix newA = FixMatrix(Amatrix.numRows(),Amatrix.numCols(),
                                         in_len,in_IntBits);
              FixMatrix newB = FixMatrix(Bmatrix.numRows(),Bmatrix.numCols(),
                                         in_len,in_IntBits);
              newA = Amatrix;
              newB = Bmatrix;
              multiply(Amatrix,Bmatrix,result);
            }
            output%0 << result;
          }
}

// prototype instance for known block list
static SDFMpyFix_M proto;
static RegisterBlock registerBlock(proto,"MpyFix_M");
