static const char file_id[] = "SDFMpy_M.pl";
// .cc file generated from SDFMpy_M.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFMpy_M.h"
#include "Matrix.h"

const char *star_nm_SDFMpy_M = "SDFMpy_M";

const char* SDFMpy_M :: className() const {return star_nm_SDFMpy_M;}

ISA_FUNC(SDFMpy_M,SDFStar);

Block* SDFMpy_M :: makeNew() const { LOG_NEW; return new SDFMpy_M;}

SDFMpy_M::SDFMpy_M ()
{
	setDescriptor("Multiply two floating-point matrices A and B to produce matrix C.\nMatrix A has dimensions (numRows,X). Matrix B has dimensions (X,numCols).\nMatrix C has dimensions (numRows,numCols). The user need only specify\nnumRows and numCols. An error will be generated automatically if the\nnumber of columns in A does not match the number of rows in B.");
	addPort(Ainput.setPort("Ainput",this,FLOAT_MATRIX_ENV));
	addPort(Binput.setPort("Binput",this,FLOAT_MATRIX_ENV));
	addPort(output.setPort("output",this,FLOAT_MATRIX_ENV));
	addState(numRows.setState("numRows",this,"2","The number of rows in Matrix A and Matrix C."));
	addState(numCols.setState("numCols",this,"2","The number of columns in Matrix B and Matrix C"));


}

void SDFMpy_M::go() {
# line 46 "SDFMpy_M.pl"
// get inputs
    Envelope Apkt;
    (Ainput%0).getMessage(Apkt);
    const FloatMatrix& Amatrix = *(const FloatMatrix *)Apkt.myData();

    Envelope Bpkt;
    (Binput%0).getMessage(Bpkt);
    const FloatMatrix& Bmatrix = *(const FloatMatrix *)Bpkt.myData();

    // check for "null" matrix inputs, caused by delays
    if(Apkt.empty() || Bpkt.empty()) {
      // if either empty, return a zero matrix with the given dimensions
      FloatMatrix& result = *(new FloatMatrix(int(numRows),int(numCols)));
      result = 0.0;
      output%0 << result;
    }
    else {
      // Amatrix and Bmatrix both valid

      if((Amatrix.numRows() != int(numRows)) ||
         (Bmatrix.numCols() != int(numCols))) {
        Error::abortRun(*this,"Dimension size of FloatMatrix inputs do ",
                              "not match the given state parameters.");
        return;
      }

      // do matrix multiplication
      FloatMatrix& result = *(new FloatMatrix(int(numRows),int(numCols)));
      // we could write 
      //   result = Amatrix * Bmatrix;
      // but the following is faster
      multiply(Amatrix,Bmatrix,result);

      output%0 << result;
    }
}

// prototype instance for known block list
static SDFMpy_M proto;
static RegisterBlock registerBlock(proto,"Mpy_M");
