static const char file_id[] = "SDFMxCom_M.pl";
// .cc file generated from SDFMxCom_M.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFMxCom_M.h"
#include "Matrix.h"

const char *star_nm_SDFMxCom_M = "SDFMxCom_M";

const char* SDFMxCom_M :: className() const {return star_nm_SDFMxCom_M;}

ISA_FUNC(SDFMxCom_M,SDFStar);

Block* SDFMxCom_M :: makeNew() const { LOG_NEW; return new SDFMxCom_M;}

SDFMxCom_M::SDFMxCom_M ()
{
	setDescriptor("Accept input matrices and create a matrix output. Each input \nmatrix represents a decomposed submatrix of output matrix in \nrow by row. Note that for one output image, we will need a total\n(numRows/numRowsSubMx)*(numCols/numColsSubMx) input matrices.");
	addPort(input.setPort("input",this,FLOAT_MATRIX_ENV));
	addPort(output.setPort("output",this,FLOAT_MATRIX_ENV));
	addState(numRows.setState("numRows",this,"100","number of rows for output matrix"));
	addState(numCols.setState("numCols",this,"100","number of columns for output matrix"));
	addState(numRowsSubMx.setState("numRowsSubMx",this,"4","number of rows for decomposed submatrix (ie, input matrix)"));
	addState(numColsSubMx.setState("numColsSubMx",this,"4","number of columns for decomposed submatrix (ie, input matrix)"));


}

void SDFMxCom_M::setup() {
# line 59 "SDFMxCom_M.pl"
numHeight = int(numRows)/int(numRowsSubMx);
    numWidth  = int(numCols)/int(numColsSubMx);
    numBlocks = numHeight * numWidth; 
    // check if output matrix can't be properly decomposed into the desired
    // submatrix size.
    if ( int(numRows) != numHeight*int(numRowsSubMx) || 
         int(numCols) != numWidth*int(numColsSubMx) ) {
      Error::abortRun(*this,
		      "The size of desired output matrix cannot be ",
                      "properly decomposed into the specified ",
                      "submatrix size.");
      return;
    }
    input.setSDFParams(numBlocks, numBlocks-1);
}

void SDFMxCom_M::go() {
# line 75 "SDFMxCom_M.pl"
Envelope pkt;
    FloatMatrix& result = *(new FloatMatrix(int(numRows), int(numCols)));
    int numsubrows = int(numRowsSubMx);
    int numsubcols = int(numColsSubMx);

    for (int h = 0; h < numHeight; h++) {
      for (int w = 0; w < numWidth; w++) {

	// treat "null" matrix inputs (caused by delays) as zero matrices
	(input%(numBlocks-1-(h*numWidth+w))).getMessage(pkt);
	if ( pkt.empty() ) {
	  for (int i = 0; i < numsubrows; i++)
	    for (int j = 0; j < numsubcols; j++)
	      result[h*numsubrows + i][w*numsubcols + j] = 0.0;
	}
	else {
	  const FloatMatrix& matrix = *(const FloatMatrix*)pkt.myData();
	  if ( matrix.numRows() * matrix.numCols() !=
	       numsubrows * numsubcols ) {
	    Error::message(*this,
			   "Input matrix does not have the same number of ",
			   "elements as the specified submatrix.");
	    return;
	  } 
	  // FIXME: matrix[i][j] is not the right way to index because
	  // the above "if" tests the total number of matrix elements
	  else {
	    for (int i = 0; i < numsubrows; i++)
	      for (int j = 0; j < numsubcols; j++)
	        result[h*numsubrows + i][w*numsubcols + j] = matrix[i][j];
	  }
	}
      }
    }
    output%0 << result;
}

// prototype instance for known block list
static SDFMxCom_M proto;
static RegisterBlock registerBlock(proto,"MxCom_M");
