static const char file_id[] = "SDFMxDecom_M.pl";
// .cc file generated from SDFMxDecom_M.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFMxDecom_M.h"
#include "Matrix.h"

const char *star_nm_SDFMxDecom_M = "SDFMxDecom_M";

const char* SDFMxDecom_M :: className() const {return star_nm_SDFMxDecom_M;}

ISA_FUNC(SDFMxDecom_M,SDFStar);

Block* SDFMxDecom_M :: makeNew() const { LOG_NEW; return new SDFMxDecom_M;}

SDFMxDecom_M::SDFMxDecom_M ()
{
	setDescriptor("Decompose a portion of input matrix into a sequence of submatrices.\nThe desired portion of input matrix is specified by state parameters \n\"startRow\",\"startCol\",\"numRows\", and \"numCols\".\nThen output each submatrix with dimension \"numRowsSubMx\"*\"numColsSubMx\"\nin row by row.\nNote that for one input matrix, there will be totally (numRows/numRowsSubMx)\n*(numCols/numColsSubMx) output matrices.");
	addState(startRow.setState("startRow",this,"0","The starting row in the input matrix"));
	addState(startCol.setState("startCol",this,"0","The starting column in the input matrix"));
	addState(numRows.setState("numRows",this,"100","number of rows from input matrix"));
	addState(numCols.setState("numCols",this,"100","number of columns from input matrix"));
	addState(numRowsSubMx.setState("numRowsSubMx",this,"4","number of rows for decomposed submatrix (ie, output matrix)"));
	addState(numColsSubMx.setState("numColsSubMx",this,"4","number of columns for decomposed submatrix (ie, output matrix)"));
	addPort(input.setPort("input",this,FLOAT_MATRIX_ENV));
	addPort(output.setPort("output",this,FLOAT_MATRIX_ENV));


}

void SDFMxDecom_M::setup() {
# line 76 "SDFMxDecom_M.pl"
numHeight = int(numRows)/int(numRowsSubMx);
    numWidth  = int(numCols)/int(numColsSubMx);
    numBlocks = numHeight*numWidth; 
/*
 * check if input matrix can't be properly decomposed into the desired 
 * submatrix size.
 */
    if ( int(numRows) != numHeight*int(numRowsSubMx) || 
         int(numCols)  != numWidth*int(numColsSubMx) ) {
      Error::abortRun(*this,"The size of the desired portion of input "
			   ,"matrix can't be properly decomposed into the "
			   ,"specified submatrix size.");
      return;
    }
    output.setSDFParams(numBlocks, numBlocks-1);
}

void SDFMxDecom_M::go() {
# line 93 "SDFMxDecom_M.pl"
Envelope pkt;
    (input%0).getMessage(pkt);
    const FloatMatrix& matrix = *(const FloatMatrix*)pkt.myData();
    if ( int(startRow)+int(numRows) > matrix.numRows() ) {
      Error::abortRun(*this,"The desired portion of input matrix extend"
                           ,"beyond the rows of the input matrix.");
      return;
    }
    if ( int(startCol)+int(numCols) > matrix.numCols() ) {
      Error::abortRun(*this,"The desired portion of input matrix extend"
                           ,"beyond the columns of the input matrix.");
      return;
    }
    for (int h=0; h<numHeight; h++) 
      for (int w=0; w<numWidth; w++) {
        FloatMatrix& result = *(new FloatMatrix(int(numRowsSubMx),int(numColsSubMx)));
        for (int i=0; i<int(numRowsSubMx); i++)
          for (int j=0; j<int(numColsSubMx); j++)
	    result[i][j] = matrix[int(startRow)+h*int(numRowsSubMx)+i][int(startCol)+w*int(numColsSubMx)+j];
	    output%(numBlocks-1-(h*numWidth+w)) << result;
      }
}

// prototype instance for known block list
static SDFMxDecom_M proto;
static RegisterBlock registerBlock(proto,"MxDecom_M");
