static const char file_id[] = "SDFSubMxFix_M.pl";
// .cc file generated from SDFSubMxFix_M.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFSubMxFix_M.h"
#include "Matrix.h"

const char *star_nm_SDFSubMxFix_M = "SDFSubMxFix_M";

const char* SDFSubMxFix_M :: className() const {return star_nm_SDFSubMxFix_M;}

ISA_FUNC(SDFSubMxFix_M,SDFStar);

Block* SDFSubMxFix_M :: makeNew() const { LOG_NEW; return new SDFSubMxFix_M;}

SDFSubMxFix_M::SDFSubMxFix_M ()
{
	setDescriptor("Find a submatrix of the input matrix");
	addPort(input.setPort("input",this,FIX_MATRIX_ENV));
	addPort(output.setPort("output",this,FIX_MATRIX_ENV));
	addState(startRow.setState("startRow",this,"0","Starting row of the input matrix"));
	addState(startCol.setState("startCol",this,"0","Starting column of the input matrix"));
	addState(numRows.setState("numRows",this,"1","The number of rows for the desired submatrix"));
	addState(numCols.setState("numCols",this,"1","The number of columns for the desired submatrix"));
	addState(UseArrivingPrecision.setState("UseArrivingPrecision",this,"YES","Flag indicated whether or not to use the arriving particles as they are:\nYES keeps the same precision, and NO casts them to the\nprecision specified by the parameter \"NewInputPrecision\"."));
	addState(NewInputPrecision.setState("NewInputPrecision",this,"2.14","New precision of the input in bits.  The input particles are only cast\nto this precision if the parameter \"UseArrivingPrecision\" is set to NO."));
	addState(OutputPrecision.setState("OutputPrecision",this,"2.14","Precision of the output in bits.  This is the precision that will hold\nthe result of the arithmetic operation on the inputs."));
	addState(OverflowHandler.setState("OverflowHandler",this,"saturate","Overflow characteristic for the output.  If the result\nof the product cannot be fit into the precision of the output, overflow\noccurs and the overflow is taken care of by the method specified by this\nparameter.  The keywords for overflow handling methods are :\n\"saturate\"(default), \"zero_saturate\", \"wrapped\", \"warning\"."));


}

void SDFSubMxFix_M::setup() {
# line 92 "SDFSubMxFix_M.pl"
IP = NewInputPrecision;
    OP = OutputPrecision;
    OV = OverflowHandler;
    in_IntBits = Fix::get_intBits (IP);
    in_len = Fix::get_length (IP);
    out_IntBits = Fix::get_intBits (OP);
    out_len = Fix::get_length (OP);
}

void SDFSubMxFix_M::go() {
# line 102 "SDFSubMxFix_M.pl"
// get input
    Envelope inpkt;
    (input%0).getMessage(inpkt);
    const FixMatrix& matrix = *(const FixMatrix *)inpkt.myData();

    if ( int(startRow)+int(numRows) > matrix.numRows() ) {
      Error::abortRun(*this,"Submatrix rows extend beyond "
                           ,"the rows of input matrix.");
      return;
    }
    if ( int(startCol)+int(numCols) > matrix.numCols() ) {
      Error::abortRun(*this,"Submatrix columns extend beyond "
                           ,"the columns of input matrix.");
      return;
    }

    // check for "null" matrix inputs, caused by delays
    if(inpkt.empty()) {
      // input empty, just send it back out
      output%0 << inpkt;
    }
    else {
      // valid input matrix

      FixMatrix& result = *(new FixMatrix(matrix.numRows(),matrix.numCols(),
                                          out_len, out_IntBits));

      for (int i=0; i<int(numRows);i++)
	 for (int j=0; j<int(numCols);j++) {     
           if(int(UseArrivingPrecision))
             result[i][j] = matrix[int(startRow)+i][int(startCol)+j];
           else
             result[i][j] = Fix(in_len, in_IntBits, matrix[int(startRow)+i][int(startCol)+j]);
           result[i][j].set_ovflow(OV);
         }  
      output%0 << result;
    }  // end of else
}

// prototype instance for known block list
static SDFSubMxFix_M proto;
static RegisterBlock registerBlock(proto,"SubMxFix_M");
