static const char file_id[] = "SDFSub_M.pl";
// .cc file generated from SDFSub_M.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFSub_M.h"
#include "Matrix.h"

const char *star_nm_SDFSub_M = "SDFSub_M";

const char* SDFSub_M :: className() const {return star_nm_SDFSub_M;}

ISA_FUNC(SDFSub_M,SDFStar);

Block* SDFSub_M :: makeNew() const { LOG_NEW; return new SDFSub_M;}

SDFSub_M::SDFSub_M ()
{
	setDescriptor("Subtract floating-point matrix B from A.");
	addPort(Ainput.setPort("Ainput",this,FLOAT_MATRIX_ENV));
	addPort(Binput.setPort("Binput",this,FLOAT_MATRIX_ENV));
	addPort(output.setPort("output",this,FLOAT_MATRIX_ENV));
	addState(numRows.setState("numRows",this,"2","The number of rows in Matrix A and B"));
	addState(numCols.setState("numCols",this,"2","The number of columns in Matrix A and B"));


}

void SDFSub_M::go() {
# line 40 "SDFSub_M.pl"
// get inputs
    Envelope Apkt;
    (Ainput%0).getMessage(Apkt);
    const FloatMatrix& Amatrix = *(const FloatMatrix *)Apkt.myData();

    Envelope Bpkt;
    (Binput%0).getMessage(Bpkt);
    const FloatMatrix& Bmatrix = *(const FloatMatrix *)Bpkt.myData();

    // check for "null" matrix inputs, caused by delays
    if(Apkt.empty() && Bpkt.empty()) {
      // both empty, return a zero matrix with the given dimensions
      FloatMatrix& result = *(new FloatMatrix(int(numRows),int(numCols)));
      result = 0.0;
      output%0 << result;
    }
    else if(Apkt.empty()) {
      // Amatrix is empty but Bmatrix is not, return -Bmatrix
      FloatMatrix& result = *(new FloatMatrix(int(numRows),int(numCols)));
      result = -Bmatrix;
      output%0 << result;
    }
    else if(Bpkt.empty()) {
      // Bmatrix is empty but Amatrix is not, return Amatrix
      // NOTE: must resend the Envelope and not an input matrix to keep
      // the reference count mechanism working
      output%0 << Apkt;
    }
    else {
      // Amatrix and Bmatrix both valid

      // just check that A's dimensions match the state info.
      // the operator - on matrices will check that A matches B
      if((Amatrix.numRows() != int(numRows)) ||
         (Amatrix.numCols() != int(numCols))) {
        Error::abortRun(*this,"Dimension size of FloatMatrix inputs do ",
                              "not match the given state parameters.");
        return;
      }

      // do matrix subtraction
      FloatMatrix& result = *(new FloatMatrix(int(numRows),int(numCols)));
      result = Amatrix - Bmatrix;
      output%0 << result;
    }
}

// prototype instance for known block list
static SDFSub_M proto;
static RegisterBlock registerBlock(proto,"Sub_M");
