static const char file_id[] = "SDFTableCx_M.pl";
// .cc file generated from SDFTableCx_M.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFTableCx_M.h"
#include "Matrix.h"

const char *star_nm_SDFTableCx_M = "SDFTableCx_M";

const char* SDFTableCx_M :: className() const {return star_nm_SDFTableCx_M;}

ISA_FUNC(SDFTableCx_M,SDFStar);

Block* SDFTableCx_M :: makeNew() const { LOG_NEW; return new SDFTableCx_M;}

SDFTableCx_M::SDFTableCx_M ()
{
	setDescriptor("This star implements a lookup table indexed by an integer-valued input,\nand the output will be a matrix.\nThe input must lie between 0 and N-1, inclusive, where N is the number\nof matrices in the table.\nThe \"complexTable\" parameter specifies the entries of matrices in the table.\nNote that the entries of each matrix in the table should be given in row \nmajor ordering.\nThe first matrix in the table is indexed by a zero-valued input.\nAn error occurs if the input value is out of bounds.");
	addPort(input.setPort("input",this,INT));
	addPort(output.setPort("output",this,COMPLEX_MATRIX_ENV));
	addState(numRows.setState("numRows",this,"1","The number of rows for each matrix in the table."));
	addState(numCols.setState("numCols",this,"1","The number of columns for each matrix in the table."));
	addState(complexTable.setState("complexTable",this,"(1.0,1.0)   (1.0,-1.0)   (-1.0,1.0)   (-1.0,-1.0)","The table containing matrices. Each matrix with dimension numRows*numCols is given in row major ordering."));


}

void SDFTableCx_M::setup() {
# line 65 "SDFTableCx_M.pl"
int size = complexTable.size();
    dimension = int(numRows)*int(numCols);
//  check if table doesn't have enough elements.
    if ( size != size/dimension*dimension ) {
      Error::abortRun(*this,"The number of elements in state parameter ",
                            "'complexTable' doesn't match the specified ",
                            "dimension.");
      return;
    }
    numMatrices = size/dimension;
}

void SDFTableCx_M::go() {
# line 77 "SDFTableCx_M.pl"
int index = int(input%0);
    if ( index < 0 || index >= numMatrices )
      Error::abortRun(*this, ": out of bounds input value");
    else {
      ComplexMatrix& result = *(new ComplexMatrix(int(numRows),int(numCols)));
      for ( int i=0; i<dimension; i++)
        result.entry(i) = complexTable[index*dimension+i];
      output%0 << result;
    }
}

// prototype instance for known block list
static SDFTableCx_M proto;
static RegisterBlock registerBlock(proto,"TableCx_M");
