static const char file_id[] = "SDFTableInt_M.pl";
// .cc file generated from SDFTableInt_M.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFTableInt_M.h"
#include "Matrix.h"

const char *star_nm_SDFTableInt_M = "SDFTableInt_M";

const char* SDFTableInt_M :: className() const {return star_nm_SDFTableInt_M;}

ISA_FUNC(SDFTableInt_M,SDFStar);

Block* SDFTableInt_M :: makeNew() const { LOG_NEW; return new SDFTableInt_M;}

SDFTableInt_M::SDFTableInt_M ()
{
	setDescriptor("This star implements a lookup table indexed by an integer-valued input,\nand the output will be a matrix.\nThe input must lie between 0 and N-1, inclusive, where N is the number\nof matrices in the table.\nThe \"intTable\" parameter specifies the entries of matrices in the table.\nNote that the entries of each matrix in the table should be given in row \nmajor ordering.\nThe first matrix in the table is indexed by a zero-valued input.\nAn error occurs if the input value is out of bounds.");
	addPort(input.setPort("input",this,INT));
	addPort(output.setPort("output",this,INT_MATRIX_ENV));
	addState(numRows.setState("numRows",this,"1","The number of rows for each matrix in the table."));
	addState(numCols.setState("numCols",this,"2","The number of columns for each matrix in the table."));
	addState(intTable.setState("intTable",this,"1 1   1 -1   -1 1   -1 -1","The table containing matrices. Each matrix with dimension numRows*numCols is given in row major ordering."));


}

void SDFTableInt_M::setup() {
# line 65 "SDFTableInt_M.pl"
int size = intTable.size();
    dimension = int(numRows)*int(numCols);
//  check if table doesn't have enough elements.
    if ( size != size/dimension*dimension ) {
      Error::abortRun(*this,"The number of elements in state parameter ",
                            "'intTable' doesn't match the specified ",
                            "dimension.");
      return;
    }
    numMatrices = size/dimension;
}

void SDFTableInt_M::go() {
# line 77 "SDFTableInt_M.pl"
int index = int(input%0);
    if ( index < 0 || index >= numMatrices )
      Error::abortRun(*this, ": out of bounds input value");
    else {
      IntMatrix& result = *(new IntMatrix(int(numRows),int(numCols)));
      for ( int i=0; i<dimension; i++)
        result.entry(i) = intTable[index*dimension+i];
      output%0 << result;
    }
}

// prototype instance for known block list
static SDFTableInt_M proto;
static RegisterBlock registerBlock(proto,"TableInt_M");
