static const char file_id[] = "SDFChop.pl";
// .cc file generated from SDFChop.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFChop.h"

const char *star_nm_SDFChop = "SDFChop";

const char* SDFChop :: className() const {return star_nm_SDFChop;}

ISA_FUNC(SDFChop,SDFStar);

Block* SDFChop :: makeNew() const { LOG_NEW; return new SDFChop;}

SDFChop::SDFChop ()
{
	setDescriptor("On each execution, this star reads a block of \"nread\" particles\nand writes them to the output with the given offset.  The number of\nparticles written is given by \"nwrite\".  The output block contains\nall or part of the input block, depending on \"offset\" and \"nwrite\".\nThe \"offset\" specifies where in the output block the first (oldest)\nparticle in the input block will lie.  If \"offset\" is positive,\nthen the first \"offset\" output particles will be either particles\nconsumed on previous firings (if \"use_past_inputs\" parameter is YES),\nor zero (otherwise).  If \"offset\" is negative, then the first \"offset\"\ninput particles will be discarded.");
	addPort(input.setPort("input",this,ANYTYPE));
	addPort(output.setPort("output",this,ANYTYPE));
	output.inheritTypeFrom(input);
	addState(nread.setState("nread",this,"128","Number of particles read."));
	addState(nwrite.setState("nwrite",this,"64","Number of particles written."));
	addState(offset.setState("offset",this,"0","Start of output block relative to start of input block."));
	addState(use_past_inputs.setState("use_past_inputs",this,"YES","If offset > 0, specify whether to use previously read inputs (otherwise use zeros)."));


}

void SDFChop::setup() {
# line 160 "SDFChop.pl"
if (int(nread) <= 0) {
		    Error::abortRun(*this, "The number of samples to read ",
				    "must be positive");
		    return;
		}
		if (int(nwrite) <= 0) {
		    Error::abortRun(*this, "The number of samples to write ",
				    "must be positive");
		    return;
		}

		if (int(use_past_inputs))
		    input.setSDFParams(int(nread),int(nread)+int(offset)-1);
		else
		    input.setSDFParams(int(nread),int(nread)-1);
		output.setSDFParams(int(nwrite),int(nwrite)-1);

		computeRange();
}

void SDFChop::go() {
# line 180 "SDFChop.pl"
int inputIndex = inidx;
		for (int i = 0; i < int(nwrite); i++) {
		    if (i > hiLim || i < loLim) {
			(output%i).initialize();
		    }
		    else {
			output%i = input%inputIndex;
			inputIndex++;
		    }
		}
}


void SDFChop::computeRange ()
{
# line 144 "SDFChop.pl"
// Compute the range of output indexes that come from inputs
		// This method is called in the setup() method for the Chop
		// star, and in the go method for ChopVarOffset because
		// it resets the offset parameter
		hiLim = int(nwrite) - int(offset) - 1;
		if (hiLim >= int(nwrite)) hiLim = int(nwrite) - 1;
		else if (int(use_past_inputs)) hiLim = int(nwrite) - 1;

		loLim = int(nwrite) - int(nread) - int(offset);
		if (loLim < 0) loLim = 0;

		inidx = int(nread) - int(nwrite) + int(offset);
		if (inidx < 0) inidx = 0;
}


// prototype instance for known block list
static SDFChop proto;
static RegisterBlock registerBlock(proto,"Chop");
