static const char file_id[] = "SDFDeScrambler.pl";
// .cc file generated from SDFDeScrambler.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFDeScrambler.h"

const char *star_nm_SDFDeScrambler = "SDFDeScrambler";

const char* SDFDeScrambler :: className() const {return star_nm_SDFDeScrambler;}

ISA_FUNC(SDFDeScrambler,SDFStar);

Block* SDFDeScrambler :: makeNew() const { LOG_NEW; return new SDFDeScrambler;}

SDFDeScrambler::SDFDeScrambler ()
{
	setDescriptor("Descramble the input bit sequence using a feedback shift register.\nThe taps of the feedback shift register are given by the \"polynomial\"\nparameter.  This is a self-synchronizing descrambler that will exactly\nreverse the operation of the Scrambler star if the polynomials are the same.\nThe low-order bit of the polynomial should always be set. For more information,\nsee the documentation for the SDF Scrambler star and Lee and Messerschmitt,\nDigital Communication, Second Edition, Kluwer Academic Publishers, 1994,\npp. 595-603.");
	addPort(input.setPort("input",this,INT));
	addPort(output.setPort("output",this,INT));
	addState(polynomial.setState("polynomial",this,"0440001","generator polynomial for the maximal length shift register"));
	addState(shiftReg.setState("shiftReg",this,"1","the shift register"));


}

void SDFDeScrambler::setup() {
# line 56 "SDFDeScrambler.pl"
// The user interface should check that generator polynomial
	  // specification does not exceed the integer precision of the machine
	  mask = int(polynomial);
	  // To avoid sign extension problems, the high-order bit must be zero
	  if (mask < 0) {
	    Error::abortRun(*this,
			"Sorry, the polynomial must be a positive integer.");
	    return;
	  }
	  if (!(mask & 1)) {
	    Error::warn(*this,
			"The low-order bit of the polynomial is not set.",
			"Input will have no effect on the shift register.");
	  }
}

void SDFDeScrambler::go() {
# line 72 "SDFDeScrambler.pl"
int reg = int(shiftReg) << 1;
	  // Put the input in the low-order bit: zero = 0, nonzero = 1
	  if ( int(input%0) ) {
	    reg++;
	  }
	  int masked = mask & reg;
	  // Now we need to find the parity of "masked"
	  int parity = 0;
	  // Calculate the parity of the masked word
	  while (masked > 0) {
	    // toggle parity if the low-order bit is one
	    parity = parity ^ (masked & 1);
	    masked = masked >> 1;
	  }
	  output%0 << parity;
	  shiftReg = reg;
}

// prototype instance for known block list
static SDFDeScrambler proto;
static RegisterBlock registerBlock(proto,"DeScrambler");
