static const char file_id[] = "SDFIntegrator.pl";
// .cc file generated from SDFIntegrator.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFIntegrator.h"

const char *star_nm_SDFIntegrator = "SDFIntegrator";

const char* SDFIntegrator :: className() const {return star_nm_SDFIntegrator;}

ISA_FUNC(SDFIntegrator,SDFStar);

Block* SDFIntegrator :: makeNew() const { LOG_NEW; return new SDFIntegrator;}

SDFIntegrator::SDFIntegrator ()
{
	setDescriptor("This is an integrator with leakage, limits, and reset.\nWith the default parameters, input samples are simply accumulated,\nand the running sum is the output.  To prevent any resetting in the\nmiddle of a run, connect a Const source with value 0 to the \"reset\"\ninput.  Otherwise, whenever a non-zero is received on this input,\nthe accumulated sum is reset to the current input (i.e. no feedback).\n\nLimits are controlled by the \"top\" and \"bottom\" parameters.\nIf top <= bottom, no limiting is performed (this is the default).\nOtherwise, the output is kept between \"bottom\" and \"top\".\nIf \"saturate\" = YES, saturation is performed.  If \"saturate\" = NO,\nwrap-around is performed (this is the default).\nLimiting is performed before output.\n\nLeakage is controlled by the \"feedbackGain\" state (default 1.0).\nThe output is the data input plus feedbackGain*state, where state\nis the previous output.");
	addPort(data.setPort("data",this,FLOAT));
	addPort(reset.setPort("reset",this,INT));
	addPort(output.setPort("output",this,FLOAT));
	addState(feedbackGain.setState("feedbackGain",this,"1.0","The gain on the feedback path."));
	addState(top.setState("top",this,"0.0","The upper limit."));
	addState(bottom.setState("bottom",this,"0.0","The lower limit."));
	addState(saturate.setState("saturate",this,"YES","Saturate if YES, wrap around otherwise."));
	addState(state.setState("state",this,"0.0","An internal state.",
# line 76 "SDFIntegrator.pl"
A_NONCONSTANT|A_SETTABLE));


}

void SDFIntegrator::setup() {
# line 82 "SDFIntegrator.pl"
spread = double(top) - double(bottom);
}

void SDFIntegrator::go() {
# line 85 "SDFIntegrator.pl"
double t;
	    if (int(reset%0) != 0) {
		t = double(data%0);
	    } else {
		t = double(data%0) + double(feedbackGain) * double(state);
	    }
	    if (spread > 0.0) {
		    // Limiting is in effect

		    // Take care of the top
		if (t > double(top))
		    if (int(saturate)) t = double(top);
		    else do t -= spread; while (t > double(top));

		    // Take care of the bottom
		if (t < double(bottom))
		    if (int(saturate)) t = double(bottom);
		    else do t += spread; while (t < double(bottom));
	    }
	    output%0 << t;
	    state = t;
}

// prototype instance for known block list
static SDFIntegrator proto;
static RegisterBlock registerBlock(proto,"Integrator");
