static const char file_id[] = "SDFLinQuantIdx.pl";
// .cc file generated from SDFLinQuantIdx.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFLinQuantIdx.h"

const char *star_nm_SDFLinQuantIdx = "SDFLinQuantIdx";

const char* SDFLinQuantIdx :: className() const {return star_nm_SDFLinQuantIdx;}

ISA_FUNC(SDFLinQuantIdx,SDFStar);

Block* SDFLinQuantIdx :: makeNew() const { LOG_NEW; return new SDFLinQuantIdx;}

SDFLinQuantIdx::SDFLinQuantIdx ()
{
	setDescriptor("The input is quantized to the number of levels given by the \"levels\"\nparameter plus 1.  The quantization levels are uniformly spaced between \"low\"\nand \"high\" inclusive.  Rounding down is performed, so that output level\nwill equal \"high\" only if the input level equals or exceeds \"high\".\nIf the input is below \"low\", then the quantized output will equal \"low\".\nThe quantized value is output to the \"amplitude\" port, while the index\nof the quantization level is output to the \"stepNumber\" port.  This\ninteger output is useful for stars that need an integer input, such as\nThor stars.");
	addPort(input.setPort("input",this,FLOAT));
	addPort(amplitude.setPort("amplitude",this,FLOAT));
	addPort(stepNumber.setPort("stepNumber",this,INT));
	addState(levels.setState("levels",this,"128","number of levels to quantize to"));
	addState(low.setState("low",this,"-3.0","lower limit of signal excursion"));
	addState(high.setState("high",this,"3.0","upper limit of signal excursion"));
	addState(height.setState("height",this,"1.0","height of each quantization step, which is determined by the states\nhigh, low, and levels.",
# line 61 "SDFLinQuantIdx.pl"
A_NONSETTABLE|A_CONSTANT));


}

void SDFLinQuantIdx::setup() {
# line 65 "SDFLinQuantIdx.pl"
if (int(levels) <= 0 ) {
		    Error::abortRun(*this, "levels must be positive");
		}
		else if (double(high) <= double(low)) {
		    Error::abortRun(*this,
				    "quantization range incorrectly ",
				    "specified: high <= low");
		}
		else {
		    height = (double(high) - double(low))/(int(levels) - 0);
		}
}

void SDFLinQuantIdx::go() {
# line 78 "SDFLinQuantIdx.pl"
double in = double(input%0);
		double highvalue = double(high);
		double lowvalue = double(low);

	    	if ( in >= highvalue ) {
		    amplitude%0 << highvalue;
                    stepNumber%0 << int(levels) - 1;
		}
		else if ( in <= lowvalue ) {
		    amplitude%0 << lowvalue;
                    stepNumber%0 << 0;
		}
		else {
		    int step = int((in - lowvalue)/double(height));
		    stepNumber%0 << step;
        	    amplitude%0 << double(lowvalue + step * double(height));
		}
}

// prototype instance for known block list
static SDFLinQuantIdx proto;
static RegisterBlock registerBlock(proto,"LinQuantIdx");
