static const char file_id[] = "SDFMaxMin.pl";
// .cc file generated from SDFMaxMin.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFMaxMin.h"

const char *star_nm_SDFMaxMin = "SDFMaxMin";

const char* SDFMaxMin :: className() const {return star_nm_SDFMaxMin;}

ISA_FUNC(SDFMaxMin,SDFStar);

Block* SDFMaxMin :: makeNew() const { LOG_NEW; return new SDFMaxMin;}

# line 71 "SDFMaxMin.pl"
#define FABS(a)	( ((a) > 0.0) ? (a) : -(a) )

SDFMaxMin::SDFMaxMin ()
{
	setDescriptor("Finds maximum or minimum, value or magnitude, of a fixed number of data\nvalues on the input.  If you want to use this star to operate over\nmultiple data streams, then preceed this star with a Commutator and\nset the state N accordingly.");
	addPort(input.setPort("input",this,FLOAT));
	addPort(output.setPort("output",this,FLOAT));
	addPort(index.setPort("index",this,INT));
	addState(N.setState("N",this,"10","default samples"));
	addState(MAX.setState("MAX",this,"YES","output maximum value else minimum is the output"));
	addState(compareMagnitude.setState("compareMagnitude",this,"NO","default is not to compare magnitude"));
	addState(outputMagnitude.setState("outputMagnitude",this,"NO","default is not to output magnitude"));


}

void SDFMaxMin::setup() {
# line 75 "SDFMaxMin.pl"
if ( int(N) <= 0 ) {
		    Error::abortRun(*this,
				    "Number of samples, N, must be positive.");
		    return;
		}
		input.setSDFParams(int(N), int(N)-1);
}

void SDFMaxMin::go() {
# line 84 "SDFMaxMin.pl"
int maxflag = int(MAX);
		int cmpMagFlag = int(compareMagnitude);
		int i = int(N) - 1;
		double value = double(input%i);
		double valueCmp = cmpMagFlag ? FABS(value) : value;
		int valueIndex = i;

		// Walk through the input samples
		while ( i-- ) {
		    double current = double(input%i);
		    double currentCmp = cmpMagFlag ? FABS(current) : current;
		    int minChangeFlag = ( currentCmp < valueCmp );
		    // Logical exclusive OR between maxflag and minChangeFlag
		    // but we cannot use the bitwise xor ^
		    if ( (maxflag && !minChangeFlag) ||
			 (!maxflag && minChangeFlag) ) {
		      value = current;
		      valueCmp = currentCmp;
		      valueIndex = i;
		    }
		}

		// Output the value or the magnitude of the value
		if ( int(outputMagnitude) ) {
		    value = FABS(value);
		}

		// Send the maximum/minimum value to the output port
		output%0 << value;

		// Adjust the index due to the LIFO nature of input data
		index%0 << ( int(N) - valueIndex - 1);
}

// prototype instance for known block list
static SDFMaxMin proto;
static RegisterBlock registerBlock(proto,"MaxMin");
