static const char file_id[] = "SDFPcwzLinear.pl";
// .cc file generated from SDFPcwzLinear.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFPcwzLinear.h"

const char *star_nm_SDFPcwzLinear = "SDFPcwzLinear";

const char* SDFPcwzLinear :: className() const {return star_nm_SDFPcwzLinear;}

ISA_FUNC(SDFPcwzLinear,SDFStar);

Block* SDFPcwzLinear :: makeNew() const { LOG_NEW; return new SDFPcwzLinear;}

SDFPcwzLinear::SDFPcwzLinear ()
{
	setDescriptor("This star implements a piecewise linear mapping from the\ninput to the output.  The mapping is given by a sequence\nof (x,y) pairs, which specify the breakpoints in the function.\nThe sequence of x values must be increasing. The function\nimplemented by the star can be represented by drawing straight\nlines between the (x,y) pairs, in sequence.  The default\nmapping is the \"tent\" map, in which inputs between -1.0\nand 0.0 are linearly mapped into the range -1.0 to 1.0.\nInputs between 0.0 and 1.0 are mapped into the same range,\nbut with the opposite slope, 1.0 to -1.0. If the input is\noutside the range specified in the \"x\" values of the breakpoints,\nthen the appropriate extreme value will be used for the output.\nThus, for the default map, if the input is -2.0, the output\nwill be -1.0.  If the input is +2.0, the output will again be -1.0.");
	addPort(input.setPort("input",this,FLOAT));
	addPort(output.setPort("output",this,FLOAT));
	addState(breakpoints.setState("breakpoints",this,"(-1.0, -1.0) (0.0, 1.0) (1.0, -1.0)","\"Endpoints and breakpoints in the mapping\""));


}

void SDFPcwzLinear::begin() {
# line 50 "SDFPcwzLinear.pl"
// Check to make sure that x values are increasing
	  double previous = breakpoints[0].real();
	  for(int i = 1; i < breakpoints.size(); i++) {
	    if (breakpoints[i].real() <= previous) {
	      Error::abortRun(*this,"Breakpoints are not increasing in x");
	      return;
	    } else {
	      previous = breakpoints[i].real();
	    }
	  }
}

void SDFPcwzLinear::go() {
# line 62 "SDFPcwzLinear.pl"
// FIXME: Linear search on X values is expensive.
	  double in = double(input%0);
	  if (in < breakpoints[0].real()) {
	    output%0 << breakpoints[0].imag();
	    return;
	  }
	  for (int i = 1; i < breakpoints.size(); i++) {
	    if (in < breakpoints[i].real()) {
	      double x1 = breakpoints[i-1].real();
	      double y1 = breakpoints[i-1].imag();
	      double x2 = breakpoints[i].real();
	      double y2 = breakpoints[i].imag();
	      output%0 << y1 + (y2 - y1)*(in - x1)/(x2 - x1);
	      return;
	    }
	  }
	  output%0 << breakpoints[breakpoints.size() - 1].imag();
}

// prototype instance for known block list
static SDFPcwzLinear proto;
static RegisterBlock registerBlock(proto,"PcwzLinear");
