static const char file_id[] = "SDFPlay.pl";
// .cc file generated from SDFPlay.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFPlay.h"
#include "miscFuncs.h"
#include "paths.h"
#include <std.h>

const char *star_nm_SDFPlay = "SDFPlay";

const char* SDFPlay :: className() const {return star_nm_SDFPlay;}

ISA_FUNC(SDFPlay,SDFStar);

Block* SDFPlay :: makeNew() const { LOG_NEW; return new SDFPlay;}

# line 90 "SDFPlay.pl"
const int BIAS = 0x84;
		const int CLIP = 32635;

		static unsigned char ulaw( int sample )
	{
		static int exp_lut[256] = {0,0,1,1,2,2,2,2,3,3,3,3,3,3,3,3,
                               4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
                               5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
                               5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
                               6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
                               6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
                               6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
                               6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
                               7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                               7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                               7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                               7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                               7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                               7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                               7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                               7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7};
		int sign, exponent, mantissa;
		unsigned char ulawbyte;

		/* Get the sample into sign-magnitude. */
		/* this code assumes a twos complement representation */
		sign = (sample >> 8) & 0x80;
		if ( sign != 0 ) sample = -sample;
		if ( sample > CLIP ) sample = CLIP;

		/* Convert from 16 bit linear to ulaw. */
		sample = sample + BIAS;
		exponent = exp_lut[( sample >> 7 ) & 0xFF];
		mantissa = ( sample >> ( exponent + 3 ) ) & 0x0F;
		ulawbyte = ~ ( sign | ( exponent << 4 ) | mantissa );
		return ulawbyte;
	}

SDFPlay::SDFPlay ()
{
	setDescriptor("Play an input stream on the SparcStation speaker.\nThe \"gain\" state (default 1.0) multiplies the input stream\nbefore it is mu-law compressed and written.\nThe inputs should be in the range of -32000.0 to 32000.0.\nThe file is played at a fixed sampling rate of 8000 samples/second.\nWhen the wrapup method is called, a file of 8-bit mu-law\nsamples is handed to a program named \"ptplay\" which plays the file.\nThe \"ptplay\" program must be in your path.");
	addPort(input.setPort("input",this,FLOAT));
	addState(gain.setState("gain",this,"1.0","Output gain."));
	addState(saveFile.setState("saveFile",this,"","File to save the output mu-law samples."));

# line 62 "SDFPlay.pl"
strm = NULL;
		delFile = FALSE;
		fileName = 0;
}

void SDFPlay::wrapup() {
# line 138 "SDFPlay.pl"
if (!strm) return;
		fclose (strm);
		strm = NULL;
		StringList cmd;

		if (delFile) cmd += "( ";
		cmd += "ptplay ";

		cmd += fileName;
		if (delFile) {
			cmd += "; /bin/rm -f ";
			cmd += fileName;
			cmd += ")";
		}
		cmd += "&";
		system (cmd);
		// no longer need to clean up
		delFile = FALSE;
}

SDFPlay::~SDFPlay() {
# line 158 "SDFPlay.pl"
if (strm) fclose (strm);
		if (delFile) unlink (fileName);
		LOG_DEL; delete [] fileName;
		fileName = 0;
}

void SDFPlay::setup() {
# line 67 "SDFPlay.pl"
// check for required program.
		if (progNotFound("ptplay",
				 "Sound files cannot be played without it."))
			return;

		// if name is empty, use a temp file.
		const char* sf = saveFile;
		delete [] fileName;
		if (sf == NULL || *sf == 0) {
			fileName = tempFileName();
			delFile = TRUE;
		}
		else {
			fileName = expandPathName(sf);
		}

		// should check if file already exists here
		if ((strm = fopen(fileName, "w")) == NULL) {
			Error::abortRun (*this, "Can't open file ", fileName);
		}
}

void SDFPlay::go() {
# line 130 "SDFPlay.pl"
if (!strm) return;
		// add gain, cvt to integer
		int data = int(double(input%0) * double(gain));
		putc (ulaw(data), strm);
}

// prototype instance for known block list
static SDFPlay proto;
static RegisterBlock registerBlock(proto,"Play");
