static const char file_id[] = "SDFQuant.pl";
// .cc file generated from SDFQuant.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFQuant.h"

const char *star_nm_SDFQuant = "SDFQuant";

const char* SDFQuant :: className() const {return star_nm_SDFQuant;}

ISA_FUNC(SDFQuant,SDFStar);

Block* SDFQuant :: makeNew() const { LOG_NEW; return new SDFQuant;}

SDFQuant::SDFQuant ()
{
	setDescriptor("Quantize the input value to one of N+1 possible output levels using\nN thresholds.  For an input less than or equal to the n-th threshold,\nbut larger than all previous thresholds, the output will be the n-th\nlevel.  If the input is greater than all thresholds, the output is\nthe N+1-th level.  If the input is less than all thresholds, then the\noutput is the zeroth level.\n\nIf level is specified, there must be one more level than thresholds.\nThe default value for level is 0, 1, 2, ... N.  This star takes on the\norder of log N steps to find the right level, whereas the linear\nquantizer star LinQuantIdx takes a constant amount of time.  Therefore,\nif you doing linear quantization, use the LinQuantIdx star.");
	addPort(input.setPort("input",this,FLOAT));
	addPort(output.setPort("output",this,FLOAT));
	addState(thresholds.setState("thresholds",this,"0.0","Quantization thresholds, in increasing order"));
	addState(levels.setState("levels",this,"","Output levels.  If empty, use 0, 1, 2, ..."));
	addState(quantizationLevel.setState("quantizationLevel",this,"0","The quantization level",
# line 59 "SDFQuant.pl"
A_NONSETTABLE|A_NONCONSTANT));


}

void SDFQuant::setup() {
# line 62 "SDFQuant.pl"
int n = thresholds.size();
		quantizationLevel = 0;

		if (levels.size() == 0) {	// set to default: 0, 1, 2...
			levels.resize(n + 1);
			for (int i = 0; i <= n; i++)
				levels[i] = i;
		}
		else if (levels.size() != n+1) {
			Error::abortRun (*this,
			      "must have one more level than thresholds");
		}
}

void SDFQuant::go() {
# line 76 "SDFQuant.pl"
double in = double(input%0);
		int lo = 0;
		int siz = thresholds.size();
		int hi = siz;
		int mid = (hi+lo)/2;
		do {
			if (in <= thresholds[mid]) {
				hi = mid;
			} else {
				lo = mid+1;
			}
			mid = (hi+lo)/2;
		} while (mid < siz && hi > lo);

		// now in is <= thresholds[mid] but > all smaller ones.
		// (where thresholds[siz] is infinity)

		quantizationLevel = mid;
		output%0 << levels[mid];
}

// prototype instance for known block list
static SDFQuant proto;
static RegisterBlock registerBlock(proto,"Quant");
