static const char file_id[] = "SDFReadPCM.pl";
// .cc file generated from SDFReadPCM.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFReadPCM.h"
#include "SimControl.h"

const char *star_nm_SDFReadPCM = "SDFReadPCM";

const char* SDFReadPCM :: className() const {return star_nm_SDFReadPCM;}

ISA_FUNC(SDFReadPCM,SDFStar);

Block* SDFReadPCM :: makeNew() const { LOG_NEW; return new SDFReadPCM;}

# line 54 "SDFReadPCM.pl"
// This routine is by
		// Craig Reese: IDA/Supercomputing Research Center
		// 29 September 1989
		//
		// References:
		// 1) CCITT Recommendation G.711  (very difficult to follow)
		// 2) MIL-STD-188-113,"Interoperability and Performance
		//	Standards for Analog-to_Digital Conversion Techniques,"
		//     17 February 1987
		//
		// Input: 8 bit ulaw sample
		// Output: signed 16 bit linear sample

		int ulaw_to_linear(unsigned char ulawbyte)
		{
			static int exp_lut[8] =
			{ 0, 132, 396, 924, 1980, 4092, 8316, 16764 };
			int sign, exponent, mantissa, sample;

			ulawbyte = ~ ulawbyte;
			sign = ( ulawbyte & 0x80 );
			exponent = ( ulawbyte >> 4 ) & 0x07;
			mantissa = ulawbyte & 0x0F;
			sample = exp_lut[exponent] +
				( mantissa << ( exponent + 3 ) );
			if ( sign != 0 ) sample = -sample;
			return sample;
		}

SDFReadPCM::SDFReadPCM ()
{
	setDescriptor("Read a binary mu-law encoded PCM file.  Return one sample on each firing.\nThe file format that is read is the same as the one written by the Play star.\nThe simulation can be halted on end-of-file, or the file contents can be\nperiodically repeated, or the file contents can be padded with zeros.");
	addPort(output.setPort("output",this,INT));
	addState(fileName.setState("fileName",this,"/dev/null","Input file."));
	addState(haltAtEnd.setState("haltAtEnd",this,"NO","Halt the run at the end of the input file."));
	addState(periodic.setState("periodic",this,"YES","Make output periodic or zero-padded."));


}

void SDFReadPCM::wrapup() {
# line 125 "SDFReadPCM.pl"
input.close();
}

SDFReadPCM::~SDFReadPCM() {
# line 51 "SDFReadPCM.pl"
input.close();
}

void SDFReadPCM::setup() {
# line 85 "SDFReadPCM.pl"
input.close();
		// open input file
		input.open(fileName);
		if (!input) {
			Error::abortRun(*this, "can't open file ",
			    fileName);
		}
}

void SDFReadPCM::go() {
# line 95 "SDFReadPCM.pl"
int x = 0;
                // initialize ch to 0, otherwise we could get garbage
                // on EOF:sdf/263: SDFReadPCM does an uninitialized memory read
		unsigned char ch = 0;
		if (input.eof()) {
			if (haltAtEnd) {	// halt the run
				SimControl::requestHalt();
				return;
			} else if (periodic) {	// close and re-open file
				input.close();
				input.open(fileName);
				if (!input) {
					Error::abortRun(*this,
					     "can't re-open file ",
					     fileName);
					return;
				}
				input.get(ch);
				x = ulaw_to_linear(ch);
			}
		} else if (input) {		// get next value
			input.get(ch);
			x = ulaw_to_linear(ch);
		} else {
			Error::abortRun(*this, "error in input file");
			return;
		}
		output%0 << x;
}

// prototype instance for known block list
static SDFReadPCM proto;
static RegisterBlock registerBlock(proto,"ReadPCM");
