static const char file_id[] = "SDFRectCx.pl";
// .cc file generated from SDFRectCx.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFRectCx.h"

const char *star_nm_SDFRectCx = "SDFRectCx";

const char* SDFRectCx :: className() const {return star_nm_SDFRectCx;}

ISA_FUNC(SDFRectCx,SDFStar);

Block* SDFRectCx :: makeNew() const { LOG_NEW; return new SDFRectCx;}

SDFRectCx::SDFRectCx ()
{
	setDescriptor("Generate a rectangular pulse of height \"height\" (default 1.0).\nand width \"width\" (default 8).  If \"period\" is greater than zero,\nthen the pulse is repeated with the given period.");
	addPort(output.setPort("output",this,COMPLEX));
	addState(bandwidth.setState("bandwidth",this,"1.0e9","Bandwidth of the signal."));
	addState(Te.setState("Te",this,"30.0*10^-6","Duration time"));
	addState(Fe.setState("Fe",this,"3.0e9","emission frequency."));
	addState(Fsimu.setState("Fsimu",this,"8.0e6","Simulation Frequency."));
	addState(vn.setState("vn",this,"150.0","Target velocity."));
	addState(Tp.setState("Tp",this,"1.0e-3","Pulse period."));
	addState(Np.setState("Np",this,"16","Pulse number."));
	addState(Fpor.setState("Fpor",this,"3.0e9","Carrier frequency."));
	addState(c.setState("c",this,"3.0e8","Light speed."));
	addState(SNRn.setState("SNRn",this,"10.0","Signal to noise ratio."));
	addState(SqrPthn.setState("SqrPthn",this,"1.0","Square root of Noise power."));
	addState(width.setState("width",this,"240","Width of the rectangular pulse."));
	addState(period.setState("period",this,"1024","If greater than zero, the period of the pulse stream."));
	addState(sdelay.setState("sdelay",this,"0","Delay for the target."));
	addState(count.setState("count",this,"0","Internal state",
# line 116 "SDFRectCx.pl"
A_SETTABLE|A_NONCONSTANT));
	addState(dopplercount.setState("dopplercount",this,"0","Internal state",
# line 123 "SDFRectCx.pl"
A_SETTABLE|A_NONCONSTANT));


}

void SDFRectCx::setup() {
# line 127 "SDFRectCx.pl"
count = 0;
		dopplercount = 0;
}

void SDFRectCx::go() {
# line 131 "SDFRectCx.pl"
double real = 0.0, imag = 0.0;

		if ( int(count) < int(width) ) {
		  double k_minus_kd = int(count) - double(sdelay);

		  // Definition arg1= B.(k-kd)^2/(Te.Fsimu^2)
		  double arg1 = k_minus_kd * k_minus_kd;
		  arg1 *= double(bandwidth);
		  arg1 /= double(Te) * double(Fsimu) * double(Fsimu);

		  // Definition arg2 = 2.fpor.(k-kd)
	          double arg2 = 2.0 * double(Fpor) * k_minus_kd / double(Fsimu);

		  // Definition doppler effect doparg
		  double doparg = 4.0 * double(vn) * double(Fe) * double(Tp);
		  doparg *= double(dopplercount)/double(c);

	          double arg = M_PI * (arg1 + arg2 + doparg) ;
		  double amp = double(SqrPthn) * pow(10.0,(double(SNRn)/20.0));

	          real = amp * cos(arg);
	          imag = amp * sin(arg);
		}

		output%0 << Complex(real, imag);

		count = int(count) + 1;
		if ( int(period) > 0 && int(count) >= int(period) ) {
		  count = 0;
		  dopplercount = int(dopplercount) + 1;
		  if ( int(Np) > 0 && int(dopplercount) >= int(Np))
		    dopplercount = 0;
		}
}

// prototype instance for known block list
static SDFRectCx proto;
static RegisterBlock registerBlock(proto,"RectCx");
