static const char file_id[] = "SDFScrambler.pl";
// .cc file generated from SDFScrambler.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFScrambler.h"

const char *star_nm_SDFScrambler = "SDFScrambler";

const char* SDFScrambler :: className() const {return star_nm_SDFScrambler;}

ISA_FUNC(SDFScrambler,SDFStar);

Block* SDFScrambler :: makeNew() const { LOG_NEW; return new SDFScrambler;}

SDFScrambler::SDFScrambler ()
{
	setDescriptor("Scramble the input bit sequence using a feedback shift register.\nThe taps of the feedback shift register are given by the \"polynomial\"\nparameter, which should be a positive integer.\nThe n-th bit of this integer indicates whether the n-th tap of the\ndelay line is fed back.\nThe low-order bit is called the 0-th bit, and should always be set.\nThe next low-order bit indicates whether the output of the first delay\nshould be fed back, etc.\nThe default \"polynomial\" is an octal number defining the V.22bis scrambler.\nFor an explanation of scramblers, see\nLee and Messerschmitt, Digital Communication, Second Edition,\nKluwer Academic Publishers, 1994, pp. 595-603.");
	addPort(input.setPort("input",this,INT));
	addPort(output.setPort("output",this,INT));
	addState(polynomial.setState("polynomial",this,"0440001","generator polynomial for the maximal length shift register"));
	addState(shiftReg.setState("shiftReg",this,"1","the shift register"));


}

void SDFScrambler::setup() {
# line 156 "SDFScrambler.pl"
// The user interface should check that generator polynomial
	  // specification does not exceed the integer precision of the machine
	  mask = int(polynomial);
	  // To avoid sign extension problems, the high-order bit must be zero
	  if (mask < 0) {
	    Error::abortRun(*this,
			"Sorry, the polynomial must be a positive integer.");
	    return;
	  }
	  if (!(mask & 1)) {
	    Error::warn(*this,
			"The low-order bit of the polynomial is not set.",
			"Input will have no effect on the shift register.");
	  }
}

void SDFScrambler::go() {
# line 172 "SDFScrambler.pl"
int reg = int(shiftReg) << 1;
	  int masked = mask & reg;
	  // Now we need to find the parity of "masked"
	  int parity = 0;
	  // Calculate the parity of the masked word
	  while (masked > 0) {
	    // toggle parity if the low-order bit is one
	    parity = parity ^ (masked & 1);
	    masked = masked >> 1;
	  }
	  // Exclusive-or with the input
	  //   nonzero input: exclusive or with 0x01
	  //   zero input: no exclusive or
	  if ( int(input%0) ) {
	    parity = parity ^ 0x01;
	  }
	  output%0 << parity;
	  // Put the parity bit into the shift register
	  shiftReg = reg | parity;
}

// prototype instance for known block list
static SDFScrambler proto;
static RegisterBlock registerBlock(proto,"Scrambler");
