static const char file_id[] = "SDFWaterfall.pl";
// .cc file generated from SDFWaterfall.pl by ptlang
/*
Copyright (c) 1990-1995 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFWaterfall.h"

const char *star_nm_SDFWaterfall = "SDFWaterfall";

const char* SDFWaterfall :: className() const {return star_nm_SDFWaterfall;}

ISA_FUNC(SDFWaterfall,SDFXgraph);

Block* SDFWaterfall :: makeNew() const { LOG_NEW; return new SDFWaterfall;}

SDFWaterfall::SDFWaterfall ()
{
	setDescriptor("Plot a series of traces in the style of a \"waterfall\" plot.\nThis is three-dimensional plot used to show the evolution\nof signals or spectra.  Optionally, each plot can be made opaque, so\nthat lines that would appear behind the plot are eliminated.\nThe star is derived from Xgraph.");
	addState(traceLength.setState("traceLength",this,"128","Number of samples per trace."));
	addState(xOffset.setState("xOffset",this,"4","Amount of right shift of each successive trace, in number of samples."));
	addState(yOffset.setState("yOffset",this,"4.0","Amount of upward shift of each successive trace, in vertical units."));
	addState(showHiddenLines.setState("showHiddenLines",this,"NO","Flag that turns on or off hidden-line elimination."));
	addState(showZeroPlane.setState("showZeroPlane",this,"YES","Flag that turns on or off the display of the plane where y=0."));

# line 67 "SDFWaterfall.pl"
highestValue = 0;
}

void SDFWaterfall::wrapup() {
# line 169 "SDFWaterfall.pl"
if(showZeroPlane) {
			graph.addPoint(2,double(xInit),0.0);
			graph.addPoint(2,
			  double(xInit)+int(traceLength)*double(xUnits),0.0);
			graph.addPoint(2,
			  double(xInit)+int(traceLength)*double(xUnits)
			  + nTracesSoFar*int(xOffset)*double(xUnits),
			  nTracesSoFar*double(yOffset));
			graph.addPoint(2,
			  double(xInit)
			  + nTracesSoFar*int(xOffset)*double(xUnits),
			  nTracesSoFar*double(yOffset));
			graph.addPoint(2,double(xInit),0.0);
		}
		SDFXgraph::wrapup();
}

SDFWaterfall::~SDFWaterfall() {
# line 68 "SDFWaterfall.pl"
LOG_DEL; delete [] highestValue;
}

void SDFWaterfall::setup() {
# line 70 "SDFWaterfall.pl"
graph.initialize(this, 2, options, title, saveFile, ignore);
		// allow access to one past sample
		input.setSDFParams(1,1);
		traceCount = 0;
		nTracesSoFar = 0;
		graph.setIgnore(0);
		xval = double(xInit);
		LOG_DEL; delete [] highestValue;
		LOG_NEW; highestValue = new double [int(traceLength)];
		inBreak = FALSE;
}

void SDFWaterfall::go() {
# line 142 "SDFWaterfall.pl"
if (traceCount >= int(traceLength)) {
			traceCount = 0;
			if (nTracesSoFar >= int(ignore)) inBreak = TRUE;
			nTracesSoFar++;
			xval = double(xInit)
				+ nTracesSoFar*int(xOffset)*double(xUnits);
		}
		double in;
		if (nTracesSoFar >= int(ignore)) {
		    in = double(input%0);
		    if (nTracesSoFar - int(ignore) == 0) visiblePoint(in);
		    else if (traceCount >= int(traceLength) - int(xOffset)) {
			// In this case, prevent connection
		        if(inBreak) graph.newTrace();
			inBreak = FALSE;
			visiblePoint(in);
		    } else {
			if (double(yOffset)*nTracesSoFar + in
			     >= highestValue[traceCount]||int(showHiddenLines))
			    visiblePoint(in);
			else
			    hiddenPoint(in);
		    }
		}
		traceCount++;
}


void SDFWaterfall::visiblePoint (const double pointVal)
{
# line 87 "SDFWaterfall.pl"
double x,y,in0,in1,hv0,hv1,d;
		    if(inBreak) graph.newTrace();
		    if(inBreak && traceCount != 0) {
			// Plot opening point
			in0 = double(yOffset)*nTracesSoFar + pointVal;
			in1 = double(yOffset)*nTracesSoFar + double(input%1);
			hv0 = highestValue[traceCount];	// current highest val
			hv1 = highestValue[traceCount-1]; // old highest val
			// Compute denominator (can't be zero)
			d = (in0-in1) - (hv0-hv1);
			// Compute x coordinate
			x = xval - double(xUnits) + (hv1-in1)*double(xUnits)/d;
			// Compute the y coordinate
			y = in1 + (hv1-in1)*(in0-in1)/d;
			graph.addPoint(1,x,y);
		    }
		    graph.addPoint(1,xval,
			double(yOffset)*nTracesSoFar + pointVal);
		    inBreak = FALSE;
		    xval += double(xUnits);
		    if(traceCount >= int(xOffset))
			highestValue[traceCount-int(xOffset)]
			    = double(yOffset)*nTracesSoFar + pointVal;
}


void SDFWaterfall::hiddenPoint (const double pointVal)
{
# line 118 "SDFWaterfall.pl"
double x,y,in0,in1,hv0,hv1,d;
		    if(!inBreak) {
			// Plot closing point
			in0 = double(yOffset)*nTracesSoFar + pointVal;
			in1 = double(yOffset)*nTracesSoFar + double(input%1);
			hv0 = highestValue[traceCount];	// current highest val
			hv1 = highestValue[traceCount-1]; // old highest val
			// Compute denominator (can't be zero)
			d = (hv0-hv1) - (in0-in1);
			// Compute x coordinate
			x = xval - double(xUnits) + (in1-hv1)*double(xUnits)/d;
			// Compute the y coordinate
			y = hv1 + (in1-hv1)*(hv0-hv1)/d;
			graph.addPoint(1,x,y);
		    }
		    inBreak=TRUE;
		    xval += double(xUnits);
		    if((traceCount >= int(xOffset))
			&& (traceCount < int(traceLength) - int(xOffset)))
			    highestValue[traceCount-int(xOffset)]
				= highestValue[traceCount];
}


// prototype instance for known block list
static SDFWaterfall proto;
static RegisterBlock registerBlock(proto,"Waterfall");
