static const char file_id[] = "VHDLIntegrator.pl";
// .cc file generated from VHDLIntegrator.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "VHDLIntegrator.h"

const char *star_nm_VHDLIntegrator = "VHDLIntegrator";

const char* VHDLIntegrator :: className() const {return star_nm_VHDLIntegrator;}

ISA_FUNC(VHDLIntegrator,VHDLStar);

Block* VHDLIntegrator :: makeNew() const { LOG_NEW; return new VHDLIntegrator;}

CodeBlock VHDLIntegrator :: integrate (
# line 83 "VHDLIntegrator.pl"
"if ($ref(reset) /= 0)\n"
"  $temp(result,float) $assign(result,float) $ref(data);\n"
"else\n"
"  $temp(result,float) $assign(result,float) $ref(data) + $val(feedbackGain) * $ref(state);\n"
"end if;\n");

CodeBlock VHDLIntegrator :: limitWithSat (
# line 90 "VHDLIntegrator.pl"
"-- Limiting is in effect\n"
"-- Take care of the top\n"
"if ($temp(result,float) > $val(top)) $temp(write,float) = $val(top);\n"
"-- Take care of the bottom\n"
"if ($temp(result,float) < $val(bottom)) $temp(write,float) = $val(bottom);\n");

CodeBlock VHDLIntegrator :: limitWithoutSat (
# line 97 "VHDLIntegrator.pl"
"-- Limiting is in effect\n"
"-- Take care of the top\n"
"while ($temp(write,float) > $val(top)) loop\n"
"  $temp(write,float) := $temp(write,float) - ($val(top) - $val(bottom));\n"
"end loop;\n"
"-- Take care of the bottom\n"
"while ($temp(write,float) < $val(bottom)) loop\n"
"  $temp(write,float) := $temp(write,float) + ($val(top) - $val(bottom));\n"
"end loop;\n");

CodeBlock VHDLIntegrator :: write (
# line 108 "VHDLIntegrator.pl"
"	    $ref(state) $assign(state) $temp(write,float);\n"
"	    $ref(output) $assign(output) $temp(write,float);\n");

VHDLIntegrator::VHDLIntegrator ()
{
	setDescriptor("An integrator with leakage, limits, and reset.\nWith the default parameters, input samples are simply accumulated,\nand the running sum is the output.  To prevent any resetting in the\nmiddle of a run, connect a d.c. source with value 0.0 to the \"reset\"\ninput.  Otherwise, whenever a non-zero is received on this input,\nthe accumulated sum is reset to the current input (i.e. no feedback).\n\nLimits are controlled by the \"top\" and \"bottom\" parameters.\nIf top <= bottom, no limiting is performed (default).  Otherwise,\nthe output is kept between \"bottom\" and \"top\".  If \"saturate\" = YES,\nsaturation is performed.  If \"saturate\" = NO, wrap-around is performed\n(default).  Limiting is performed before output.\n\nLeakage is controlled by the \"feedbackGain\" state (default 1.0).\nThe output is the data input plus feedbackGain*state, where state\nis the previous output.");
	addPort(data.setPort("data",this,FLOAT));
	addPort(reset.setPort("reset",this,INT));
	addPort(output.setPort("output",this,FLOAT));
	addState(feedbackGain.setState("feedbackGain",this,"1.0","The gain on the feedback path."));
	addState(top.setState("top",this,"0.0","The upper limit."));
	addState(bottom.setState("bottom",this,"0.0","The lower limit."));
	addState(saturate.setState("saturate",this,"YES","Saturate if YES, wrap around otherwise."));
	addState(state.setState("state",this,"0.0","An internal state.",
# line 75 "VHDLIntegrator.pl"
A_NONCONSTANT | A_SETTABLE));


}

void VHDLIntegrator::setup() {
# line 81 "VHDLIntegrator.pl"
spread = double(top) - double(bottom);
}

void VHDLIntegrator::go() {
# line 113 "VHDLIntegrator.pl"
addCode(integrate);
	    if (spread > 0.0) 
		if (int(saturate))
		    addCode(limitWithSat);
		else
		    addCode(limitWithoutSat);
	    addCode(write);
}

// prototype instance for known block list
static VHDLIntegrator proto;
static RegisterBlock registerBlock(proto,"Integrator");
